#include "precomp.h"

# 29 "./../src/optimize.c"
typedef struct Cross_Linklet_Info 
{

  Scheme_Object * get_import ; 
  Scheme_Hash_Tree * import_keys ; 
  Scheme_Hash_Tree * rev_import_keys ; 
  Scheme_Hash_Tree * linklets ; 
  Scheme_Hash_Tree * import_next_keys ; 
  Scheme_Hash_Tree * inline_variants ; 
  Scheme_Hash_Tree * import_syms ; 

  int used_import_shape ; 
}

# 41 "./../src/optimize.c"
Cross_Linklet_Info ; 








struct Optimize_Info 
{
  Scheme_Type type ; 
  short flags ; 
  struct Optimize_Info * next ; 
  struct Scheme_Linklet * linklet ; 
  int init_kclock ; 


  Cross_Linklet_Info * cross ; 


  Scheme_Hash_Tree * * imports_used ; 


  int size ; 
  int vclock ; 







  int aclock ; 


  int kclock ; 

  int sclock ; 
  int psize ; 
  short inline_fuel , flatten_fuel ; 
  char letrec_not_twice , enforce_const , unsafe_mode , use_psize , has_nonleaf ; 
  Scheme_Hash_Table * top_level_consts ; 

  int maybe_values_argument ; 


  int single_result , preserves_marks ; 
  int escapes ; 



  int lambda_depth ; 
  int used_toplevel ; 

  Scheme_Hash_Table * uses ; 

  Scheme_IR_Local * transitive_use_var ; 


  Scheme_Object * context ; 
  Scheme_Logger * logger ; 
  Scheme_Hash_Tree * types ; 
  int no_types ; 
}

# 105 "./../src/optimize.c"
; 

typedef struct Optimize_Info_Sequence {
  int init_flatten_fuel , min_flatten_fuel ; 
}

# 109 "./../src/optimize.c"
Optimize_Info_Sequence ; 

static Scheme_Object * optimize_expr (Scheme_Object * expr , Optimize_Info * info , int context ) ; 

static int get_rator_flags (Scheme_Object * rator , int num_args , Optimize_Info * info ) ; 
Scheme_Object * lookup_constant_proc (Optimize_Info * info , Scheme_Object * le , int argc ) ; 
static void merge_lambda_arg_types (Scheme_Lambda * lam1 , Scheme_Lambda * lam2 ) ; 
static void check_lambda_arg_types_registered (Scheme_Lambda * lam , int app_count ) ; 
static int lambda_body_size_plus_info (Scheme_Lambda * lam , int check_assign , 
Optimize_Info * info , int * is_leaf ) ; 
static int lambda_has_top_level (Scheme_Lambda * lam ) ; 

static Scheme_Object * make_sequence_2 (Scheme_Object * a , Scheme_Object * b ) ; 

__xform_nongcing__ static int wants_local_type_arguments (Scheme_Object * rator , int argpos ) ; 

static void add_types_for_f_branch (Scheme_Object * t , Optimize_Info * info , int fuel ) ; 

static void register_use (Scheme_IR_Local * var , Optimize_Info * info ) ; 
static Scheme_Object * optimize_info_lookup (Scheme_Object * var ) ; 
static Scheme_Object * optimize_info_propagate_local (Scheme_Object * var ) ; 
static void optimize_info_used_top (Optimize_Info * info ) ; 
static Scheme_Object * optimize_get_predicate (Optimize_Info * info , Scheme_Object * var , int ignore_no_types ) ; 
static void add_type (Optimize_Info * info , Scheme_Object * var , Scheme_Object * pred ) ; 
static void merge_types (Optimize_Info * src_info , Optimize_Info * info , Scheme_Hash_Tree * skip_vars ) ; 

static Scheme_Object * expr_implies_predicate (Scheme_Object * expr , Optimize_Info * info ) ; 
static Scheme_Object * do_expr_implies_predicate (Scheme_Object * expr , Optimize_Info * info , 
int * _involves_k_cross , int fuel , 
Scheme_Hash_Tree * ignore_vars ) ; 
static int produces_local_type (Scheme_Object * rator , int argc ) ; 
static int optimize_any_uses (Optimize_Info * info , Scheme_IR_Let_Value * at_irlv , int n ) ; 
static void optimize_uses_of_mutable_imply_early_alloc (Scheme_IR_Let_Value * at_irlv , int n ) ; 
static void propagate_used_variables (Optimize_Info * info ) ; 
static int env_uses_toplevel (Optimize_Info * frame ) ; 
static Scheme_IR_Local * clone_variable (Scheme_IR_Local * var ) ; 
static void increment_use_count (Scheme_IR_Local * var , int as_rator ) ; 

static Optimize_Info * optimize_info_create (Scheme_Linklet * linklet , 
int enforce_const , int can_inline , int unsafe_mode ) ; 
static Optimize_Info * optimize_info_add_frame (Optimize_Info * info , int flags ) ; 
static void optimize_info_done (Optimize_Info * info , Optimize_Info * parent ) ; 

static void register_transitive_uses (Scheme_IR_Local * var , Optimize_Info * info ) ; 

static void optimize_info_seq_init (Optimize_Info * info , Optimize_Info_Sequence * info_seq ) ; 
static void optimize_info_seq_step (Optimize_Info * info , Optimize_Info_Sequence * info_seq ) ; 
static void optimize_info_seq_done (Optimize_Info * info , Optimize_Info_Sequence * info_seq ) ; 

static int ir_propagate_ok (Scheme_Object * o , Optimize_Info * info , int used_once , Scheme_IR_Local * once_var ) ; 

static Scheme_Object * estimate_closure_size (Scheme_Object * e ) ; 
static Scheme_Object * no_potential_size (Scheme_Object * value ) ; 

static Scheme_Object * optimize_lets (Scheme_Object * form , Optimize_Info * info , int context ) ; 

static Scheme_Object * optimize_clone (int single_use , Scheme_Object * obj , Optimize_Info * info , Scheme_Hash_Tree * var_map , int as_rator ) ; 

static Scheme_Object * get_import_shape (Optimize_Info * info , Scheme_IR_Toplevel * var ) ; 
static Scheme_Object * get_import_inline (Optimize_Info * info , Scheme_IR_Toplevel * var , int argc , int case_ok ) ; 
static void register_import_used (Optimize_Info * info , Scheme_IR_Toplevel * expr ) ; 
static void record_optimize_shapes (Optimize_Info * info , Scheme_Linklet * linklet , Scheme_Object * * _import_keys ) ; 
static Scheme_Object * get_value_shape (Scheme_Object * v , int imprecise ) ; 

__xform_nongcing__ static int relevant_predicate (Scheme_Object * pred ) ; 
__xform_nongcing__ static int predicate_implies (Scheme_Object * pred1 , Scheme_Object * pred2 ) ; 
__xform_nongcing__ static int predicate_implies_not (Scheme_Object * pred1 , Scheme_Object * pred2 ) ; 
static int single_valued_expression (Scheme_Object * expr , Optimize_Info * info , int fuel ) ; 
static int single_valued_noncm_expression (Scheme_Object * expr , Optimize_Info * info , int fuel ) ; 
static int noncm_expression (Scheme_Object * expr , Optimize_Info * info , int fuel ) ; 
static Scheme_Object * optimize_ignored (Scheme_Object * e , Optimize_Info * info , 
int expected_vals , int maybe_omittable , 
int fuel ) ; 
static Scheme_Object * equivalent_exprs (Scheme_Object * a , Scheme_Object * b , 
Optimize_Info * a_info , Optimize_Info * b_info , int context ) ; 
static int movable_expression (Scheme_Object * expr , Optimize_Info * info , 
int cross_lambda , int cross_k , int cross_s , 
int check_space , int fuel ) ; 
Scheme_Object * optimize_apply_values (Scheme_Object * f , Scheme_Object * e , 
Optimize_Info * info , 
int e_single_result , 
int context ) ; 








static int lambda_body_size (Scheme_Object * o , int less_args ) ; 

typedef struct Scheme_Once_Used {
  Scheme_Object so ; 
  Scheme_Object * expr ; 
  Scheme_IR_Local * var ; 
  int vclock ; 
  int aclock ; 
  int kclock ; 
  int sclock ; 
  int spans_k ; 
  int moved ; 
}

# 211 "./../src/optimize.c"
Scheme_Once_Used ; 

static Scheme_Once_Used * make_once_used (Scheme_Object * val , Scheme_IR_Local * var , 
int vclock , int aclock , int kclock , int sclock , int spans_k ) ; 

static Scheme_Hash_Tree * empty_eq_hash_tree ; 


static void register_traversers (void ) ; 


void scheme_init_optimize () 
{
  /* No conversion */

# 224 "./../src/optimize.c"
  scheme_register_static ((void * ) & empty_eq_hash_tree , sizeof (empty_eq_hash_tree ) ) ; 
  empty_eq_hash_tree = scheme_make_hash_tree (SCHEME_hashtr_eq ) ; 


  register_traversers () ; 
}






static void note_match (int actual , int expected , Optimize_Info * warn_info ) 
{
  char * __funcarg58 = NULLED_OUT ; 
  PREPARE_VAR_STACK_ONCE(1);
  BLOCK_SETUP_TOP((PUSH(warn_info, 0)));
# define XfOrM2_COUNT (1)
# define SETUP_XfOrM2(x) SETUP(XfOrM2_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 238 "./../src/optimize.c"
  if (! warn_info || (expected == - 1 ) )
    RET_NOTHING ; 

  if (actual != expected ) {
#   define XfOrM3_COUNT (0+XfOrM2_COUNT)
#   define SETUP_XfOrM3(x) SETUP_XfOrM2(x)
    (__funcarg58 = FUNCCALL(SETUP_XfOrM3(_), scheme_optimize_context_to_string (warn_info -> context ) )

    , FUNCCALL_AGAIN(scheme_log (warn_info -> logger , 3 , 0 , "warning%s: %d values produced when %d expected" , __funcarg58 , actual , expected ) )) 
    ; 
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 251 "./../src/optimize.c"
char * scheme_optimize_context_to_string (Scheme_Object * context ) 

{
  DECL_RET_SAVE (char * ) PREPARE_VAR_STACK(9);
# define XfOrM4_COUNT (0)
# define SETUP_XfOrM4(x) SETUP(XfOrM4_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 254 "./../src/optimize.c"
  if (context ) {
    Scheme_Object * linklet , * func ; 
    const char * ctx , * prefix , * mctx , * mprefix ; 
    char * all ; 
    int clen , plen , mclen , mplen , len ; 
    BLOCK_SETUP((PUSH(prefix, 0+XfOrM4_COUNT), PUSH(mctx, 1+XfOrM4_COUNT), PUSH(all, 2+XfOrM4_COUNT), PUSH(mprefix, 3+XfOrM4_COUNT), PUSH(linklet, 4+XfOrM4_COUNT), PUSH(ctx, 5+XfOrM4_COUNT), PUSH(func, 6+XfOrM4_COUNT)));
#   define XfOrM5_COUNT (7+XfOrM4_COUNT)
#   define SETUP_XfOrM5(x) SETUP(XfOrM5_COUNT)
    linklet = NULLED_OUT ; 
    func = NULLED_OUT ; 
    ctx = NULLED_OUT ; 
    prefix = NULLED_OUT ; 
    mctx = NULLED_OUT ; 
    mprefix = NULLED_OUT ; 
    all = NULLED_OUT ; 

# 260 "./../src/optimize.c"
    if ((Scheme_Type ) (((((intptr_t ) (context ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (context ) ) -> type ) ) == (Scheme_Type ) (scheme_pair_type ) )  {
      func = (((Scheme_Simple_Object * ) (context ) ) -> u . pair_val . car ) ; 
      linklet = (((Scheme_Simple_Object * ) (context ) ) -> u . pair_val . cdr ) ; 
    }

# 263 "./../src/optimize.c"
    else if ((Scheme_Type ) (((((intptr_t ) (context ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (context ) ) -> type ) ) == (Scheme_Type ) (scheme_linklet_type ) )  {
      func = scheme_false ; 
      linklet = context ; 
    }

# 266 "./../src/optimize.c"
    else {
      func = context ; 
      linklet = scheme_false ; 
    }

    if ((Scheme_Type ) (((((intptr_t ) (func ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (func ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) )  {
      Scheme_Object * name ; 
      BLOCK_SETUP((PUSH(name, 0+XfOrM5_COUNT)));
#     define XfOrM9_COUNT (1+XfOrM5_COUNT)
#     define SETUP_XfOrM9(x) SETUP(XfOrM9_COUNT)
      name = NULLED_OUT ; 

# 274 "./../src/optimize.c"
      name = ((Scheme_Lambda * ) func ) -> name ; 
      if (name ) {
#       define XfOrM11_COUNT (0+XfOrM9_COUNT)
#       define SETUP_XfOrM11(x) SETUP_XfOrM9(x)

# 276 "./../src/optimize.c"
        if ((Scheme_Type ) (((((intptr_t ) (name ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (name ) ) -> type ) ) == (Scheme_Type ) (scheme_vector_type ) )  {
          Scheme_Object * port ; 
          int print_width = 1024 ; 
          intptr_t plen ; 
          BLOCK_SETUP((PUSH(port, 0+XfOrM11_COUNT)));
#         define XfOrM13_COUNT (1+XfOrM11_COUNT)
#         define SETUP_XfOrM13(x) SETUP(XfOrM13_COUNT)
          port = NULLED_OUT ; 

# 281 "./../src/optimize.c"
          port = FUNCCALL(SETUP_XfOrM13(_), scheme_make_byte_string_output_port () ); 
          FUNCCALL_AGAIN(scheme_write_proc_context (port , print_width , (((Scheme_Vector * ) (name ) ) -> els ) [0 ] , (((Scheme_Vector * ) (name ) ) -> els ) [1 ] , (((Scheme_Vector * ) (name ) ) -> els ) [2 ] , (((Scheme_Vector * ) (name ) ) -> els ) [3 ] , (((Scheme_Vector * ) (name ) ) -> els ) [4 ] , (! ((((((Scheme_Vector * ) (name ) ) -> els ) [6 ] ) ) == (scheme_false ) ) ) ) )




          ; 

          ctx = FUNCCALL_AGAIN(scheme_get_sized_byte_string_output (port , & plen ) ); 
          prefix = " in: " ; 
        }

# 291 "./../src/optimize.c"
        else {
#         define XfOrM12_COUNT (0+XfOrM11_COUNT)
#         define SETUP_XfOrM12(x) SETUP_XfOrM11(x)

# 292 "./../src/optimize.c"
          ctx = FUNCCALL(SETUP_XfOrM12(_), scheme_get_proc_name (func , & len , 0 ) ); 
          prefix = " in: " ; 
        }
      }

# 295 "./../src/optimize.c"
      else {
        ctx = "" ; 
        prefix = "" ; 
      }
    }

# 299 "./../src/optimize.c"
    else {
      ctx = "" ; 
      prefix = "" ; 
    }

    if ((Scheme_Type ) (((((intptr_t ) (linklet ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (linklet ) ) -> type ) ) == (Scheme_Type ) (scheme_linklet_type ) )  {
#     define XfOrM7_COUNT (0+XfOrM5_COUNT)
#     define SETUP_XfOrM7(x) SETUP_XfOrM5(x)

# 305 "./../src/optimize.c"
      mctx = FUNCCALL(SETUP_XfOrM7(_), scheme_display_to_string (((Scheme_Linklet * ) linklet ) -> name , ((void * ) 0 ) ) ); 
      mprefix = " in module: " ; 
    }

# 307 "./../src/optimize.c"
    else {
      mctx = "" ; 
      mprefix = "" ; 
    }

    clen = strlen (ctx ) ; 
    plen = strlen (prefix ) ; 
    mclen = strlen (mctx ) ; 
    mplen = strlen (mprefix ) ; 

    if (! clen && ! mclen )
      RET_VALUE_START ("" ) RET_VALUE_END ; 

    all = FUNCCALL(SETUP_XfOrM5(_), GC_malloc_atomic (clen + plen + mclen + mplen + 1 ) ); 
    (memcpy (all , prefix , plen ) ) ; 
    (memcpy (all + plen , ctx , clen ) ) ; 
    (memcpy (all + plen + clen , mprefix , mplen ) ) ; 
    (memcpy (all + plen + clen + mplen , mctx , mclen ) ) ; 
    all [clen + plen + mclen + mplen ] = 0 ; 
    RET_VALUE_START (all ) RET_VALUE_END ; 
  }

# 327 "./../src/optimize.c"
  else 
  RET_VALUE_START ("" ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 331 "./../src/optimize.c"
char * scheme_optimize_info_context (Optimize_Info * info ) 
{
  /* No conversion */

# 333 "./../src/optimize.c"
  return scheme_optimize_context_to_string (info -> context ) ; 
}

Scheme_Logger * scheme_optimize_info_logger (Optimize_Info * info ) 
{
  /* No conversion */

# 338 "./../src/optimize.c"
  return info -> logger ; 
}





static void set_optimize_mode (Scheme_IR_Local * var ) 
{
  PREPARE_VAR_STACK_ONCE(1);
  BLOCK_SETUP_TOP((PUSH(var, 0)));
# define XfOrM19_COUNT (1)
# define SETUP_XfOrM19(x) SETUP(XfOrM19_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 347 "./../src/optimize.c"
  ((void ) sizeof ((((Scheme_Type ) (var -> so . type ) == (Scheme_Type ) (scheme_ir_local_type ) ) ) ? 1 : 0 ) , __extension__ ({
#   define XfOrM20_COUNT (0+XfOrM19_COUNT)
#   define SETUP_XfOrM20(x) SETUP_XfOrM19(x)

# 347 "./../src/optimize.c" 3
    if ((Scheme_Type ) (var -> so . type ) == (Scheme_Type ) (scheme_ir_local_type ) ) 

# 347 "./../src/optimize.c" 3
    ; 

# 347 "./../src/optimize.c" 3
    else FUNCCALL(SETUP_XfOrM20(_), __assert_fail ("((Scheme_Type)(var->so.type) == (Scheme_Type)(scheme_ir_local_type))" , "./../src/optimize.c" , 347 , __extension__ __PRETTY_FUNCTION__ ) ); 
  }
  ) ) 
# 347 "./../src/optimize.c"
  ; 
  (memset (& var -> optimize , 0 , sizeof (var -> optimize ) ) ) ; 
  var -> mode = 3 ; 
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 354 "./../src/optimize.c"
int scheme_is_functional_nonfailing_primitive (Scheme_Object * rator , int num_args , int expected_vals ) 



{
  /* No conversion */

# 359 "./../src/optimize.c"
  if (((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
  && ((scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & ((1 << 8 ) | (1 << 3 ) ) ) 
  || scheme_is_omitable_primitive (rator , num_args ) ) 
  && (num_args >= ((Scheme_Primitive_Proc * ) rator ) -> mina ) 
  && (num_args <= ((Scheme_Primitive_Proc * ) rator ) -> mu . maxa ) 
  && ((expected_vals < 0 ) 
  || ((expected_vals == 1 ) && ! ((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) & 8 ) ) 
  || (((scheme_values_proc ) == (rator ) ) 
  && (expected_vals == num_args ) ) ) ) {
    if ((scheme_values_proc ) == (rator ) ) 
      return 2 ; 
    return 1 ; 
  }

# 371 "./../src/optimize.c"
  else 
  return 0 ; 
}

int scheme_is_omitable_primitive (Scheme_Object * rator , int num_args ) 
{
  /* No conversion */

# 377 "./../src/optimize.c"
  if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & ((1 << 4 ) 
  | (1 << 5 ) 
  | (1 << 3 ) ) )
    return 1 ; 

  if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 6 ) )
    return (num_args == 0 ) ; 

  if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 7 ) ) 
    return ! (num_args & 0x1 ) ; 

  return 0 ; 
}

int scheme_is_functional_nonfailing_primitive (Scheme_Object * rator , int num_args , int expected_vals ) ; 

static Scheme_Object * get_defn_shape (Optimize_Info * info , Scheme_IR_Toplevel * var ) 
{
  Scheme_Object * v ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(v, 1), PUSH(var, 2)));
# define XfOrM24_COUNT (3)
# define SETUP_XfOrM24(x) SETUP(XfOrM24_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  v = NULLED_OUT ; 

# 397 "./../src/optimize.c"
  if (info -> top_level_consts && (var -> instance_pos == - 1 ) ) {
#   define XfOrM25_COUNT (0+XfOrM24_COUNT)
#   define SETUP_XfOrM25(x) SETUP_XfOrM24(x)

# 398 "./../src/optimize.c"
    v = FUNCCALL(SETUP_XfOrM25(_), scheme_hash_get (info -> top_level_consts , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (var -> variable_pos ) ) ) << 1 ) | 0x1 ) ) ) ); 
    if (v )

# 399 "./../src/optimize.c"
    RET_VALUE_START (v ) RET_VALUE_END ; 

    v = FUNCCALL(SETUP_XfOrM25(_), scheme_hash_get (info -> top_level_consts , scheme_false ) ); 
    if (v && FUNCCALL_EMPTY(scheme_hash_get ((Scheme_Hash_Table * ) v , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (var -> variable_pos ) ) ) << 1 ) | 0x1 ) ) ) ))
      RET_VALUE_START (scheme_default_prompt_tag ) RET_VALUE_END ; 
  }

  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 409 "./../src/optimize.c"
static Scheme_Object * get_struct_proc_shape (Scheme_Object * rator , Optimize_Info * info , int prop_ok ) 

{
  Scheme_Object * c ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(rator, 1), PUSH(c, 2)));
# define XfOrM26_COUNT (3)
# define SETUP_XfOrM26(x) SETUP(XfOrM26_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  c = NULLED_OUT ; 

# 414 "./../src/optimize.c"
  if (info && ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_toplevel_type ) ) ) {
#   define XfOrM27_COUNT (0+XfOrM26_COUNT)
#   define SETUP_XfOrM27(x) SETUP_XfOrM26(x)

# 415 "./../src/optimize.c"
    c = FUNCCALL(SETUP_XfOrM27(_), get_defn_shape (info , (Scheme_IR_Toplevel * ) rator ) ); 
    if (! c )
      c = FUNCCALL(SETUP_XfOrM27(_), get_import_shape (info , (Scheme_IR_Toplevel * ) rator ) ); 

    if (c && (((Scheme_Type ) (((((intptr_t ) (c ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (c ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) ) 
    || (prop_ok && ((Scheme_Type ) (((((intptr_t ) (c ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (c ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_prop_proc_shape_type ) ) ) ) )
      RET_VALUE_START (c ) RET_VALUE_END ; 
  }

  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 427 "./../src/optimize.c"
int scheme_is_struct_functional (Scheme_Object * rator , int num_args , Optimize_Info * info , int vals ) 

{
  Scheme_Object * c ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(1);
  BLOCK_SETUP_TOP((PUSH(c, 0)));
# define XfOrM28_COUNT (1)
# define SETUP_XfOrM28(x) SETUP(XfOrM28_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  c = NULLED_OUT ; 

# 432 "./../src/optimize.c"
  if ((vals == 1 ) || (vals == - 1 ) ) {
#   define XfOrM29_COUNT (0+XfOrM28_COUNT)
#   define SETUP_XfOrM29(x) SETUP_XfOrM28(x)

# 433 "./../src/optimize.c"
    c = FUNCCALL(SETUP_XfOrM29(_), get_struct_proc_shape (rator , info , 1 ) ); 
    if (c ) {
      if ((Scheme_Type ) (((((intptr_t ) (c ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (c ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) )  {
        int mode = (((Scheme_Struct_Proc_Shape * ) c ) -> mode & 0x7 ) ; 
        int field_count = (((Scheme_Struct_Proc_Shape * ) c ) -> mode >> 7 ) ; 
        if (((num_args == 1 ) && (mode == 2 ) ) 
        || ((num_args == field_count ) 
        && (mode == 1 ) 
        && (((Scheme_Struct_Proc_Shape * ) c ) -> mode & 0x20 ) ) ) {
          RET_VALUE_START (1 ) RET_VALUE_END ; 
        }
      }

# 444 "./../src/optimize.c"
      else if ((Scheme_Type ) (((((intptr_t ) (c ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (c ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_prop_proc_shape_type ) )  {
        int mode = (((Scheme_Small_Object * ) c ) -> u . int_val & 0x7 ) ; 
        if ((mode == 2 ) 
        && (num_args == 1 ) )
          RET_VALUE_START (1 ) RET_VALUE_END ; 
      }
    }
  }

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 456 "./../src/optimize.c"
static Scheme_Object * extract_specialized_proc (Scheme_Object * le , Scheme_Object * default_val ) 

{
  /* No conversion */

# 459 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_application2_type ) )  {
    Scheme_App2_Rec * app = (Scheme_App2_Rec * ) le ; 
    if ((app -> rator ) == (scheme_procedure_specialize_proc ) )  {
      if ((! (((intptr_t ) (app -> rand ) ) & 0x1 ) && ((((app -> rand ) -> type ) >= scheme_prim_type ) && (((app -> rand ) -> type ) <= scheme_proc_chaperone_type ) ) ) || (((Scheme_Type ) (((((intptr_t ) (app -> rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (app -> rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) )
        return app -> rand ; 
    }
  }

  return default_val ; 
}

int scheme_omittable_expr (Scheme_Object * o , int vals , int fuel , int flags , 
Optimize_Info * opt_info , Optimize_Info * warn_info ) 
# 487 "./../src/optimize.c"
{
  Scheme_Type vtype ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK(5);
  BLOCK_SETUP_TOP((PUSH(o, 0), PUSH(warn_info, 1), PUSH(opt_info, 2)));
# define XfOrM37_COUNT (3)
# define SETUP_XfOrM37(x) SETUP(XfOrM37_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 492 "./../src/optimize.c"
  try_again : 

  vtype = ((((intptr_t ) (o ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o ) ) -> type ) ; 

  if ((vtype > _scheme_ir_values_types_ ) 
  || ((vtype == scheme_ir_local_type ) 
  && ! (flags & 0x2 ) 
  && (! (flags & 0x4 ) 
  || ! ((Scheme_IR_Local * ) o ) -> mutated ) ) 
  || ((vtype == scheme_local_type ) 
  && ! (flags & 0x2 ) 
  && ! (((& ((Scheme_Local * ) (o ) ) -> iso ) -> so . keyex & ~ 0x2000 ) == 1 ) ) 
  || ((vtype == scheme_local_unbox_type ) 
  && ! (flags & (0x2 | 0x4 ) ) 
  && ! (((& ((Scheme_Local * ) (o ) ) -> iso ) -> so . keyex & ~ 0x2000 ) == 1 ) ) 
  || (vtype == scheme_lambda_type ) 
  || (vtype == scheme_ir_lambda_type ) 
  || (vtype == scheme_inline_variant_type ) 
  || (vtype == scheme_case_lambda_sequence_type ) 
  || (vtype == scheme_varref_form_type ) ) {
#   define XfOrM78_COUNT (0+XfOrM37_COUNT)
#   define SETUP_XfOrM78(x) SETUP_XfOrM37(x)
    FUNCCALL(SETUP_XfOrM78(_), note_match (1 , vals , warn_info ) )
# 512 "./../src/optimize.c"
    ; 
    RET_VALUE_START (((vals == 1 ) || (vals < 0 ) ) ) RET_VALUE_END ; 
  }

  if ((vtype == scheme_toplevel_type ) || (vtype == scheme_static_toplevel_type ) ) {
#   define XfOrM76_COUNT (0+XfOrM37_COUNT)
#   define SETUP_XfOrM76(x) SETUP_XfOrM37(x)
    FUNCCALL(SETUP_XfOrM76(_), note_match (1 , vals , warn_info ) )
# 517 "./../src/optimize.c"
    ; 
    if (! (flags & 0x2 ) && (flags & 0x1 ) && ((vals == 1 ) || (vals < 0 ) ) ) {
      int tl_flags = (& ((Scheme_Toplevel * ) (o ) ) -> iso ) -> so . keyex ; 
      if (tl_flags & 0x3 )
        RET_VALUE_START (1 ) RET_VALUE_END ; 
      else 
      RET_VALUE_START (0 ) RET_VALUE_END ; 
    }
  }

  if (vtype == scheme_ir_toplevel_type ) {
#   define XfOrM74_COUNT (0+XfOrM37_COUNT)
#   define SETUP_XfOrM74(x) SETUP_XfOrM37(x)
    FUNCCALL(SETUP_XfOrM74(_), note_match (1 , vals , warn_info ) )
# 528 "./../src/optimize.c"
    ; 
    if ((vals == 1 ) || (vals < 0 ) ) {
      if (! (flags & 0x2 ) 
      && (((& ((Scheme_IR_Toplevel * ) o ) -> iso ) -> so . keyex & 0x3 ) >= 1 ) )
        RET_VALUE_START (1 ) RET_VALUE_END ; 
      else if (((& ((Scheme_IR_Toplevel * ) o ) -> iso ) -> so . keyex & 0x3 ) >= 3 )
        RET_VALUE_START (1 ) RET_VALUE_END ; 
      else 
      RET_VALUE_START (0 ) RET_VALUE_END ; 
    }
  }


  if (! (flags & 0x10 ) ) {
    Scheme_Object * auto_e ; 
    int auto_e_depth ; 
    BLOCK_SETUP((PUSH(auto_e, 0+XfOrM37_COUNT)));
#   define XfOrM72_COUNT (1+XfOrM37_COUNT)
#   define SETUP_XfOrM72(x) SETUP(XfOrM72_COUNT)
    auto_e = NULLED_OUT ; 

# 544 "./../src/optimize.c"
    auto_e = FUNCCALL(SETUP_XfOrM72(_), scheme_is_simple_make_struct_type (o , vals , (((flags & 0x1 ) ? 0x1 : 0 ) | 0x2 | 0x4 ) , & auto_e_depth , ((void * ) 0 ) , ((void * ) 0 ) , opt_info , ((void * ) 0 ) , ((void * ) 0 ) , 0 , ((void * ) 0 ) , ((void * ) 0 ) , 5 ) )







    ; 
    if (auto_e ) {
#     define XfOrM73_COUNT (0+XfOrM72_COUNT)
#     define SETUP_XfOrM73(x) SETUP_XfOrM72(x)

# 554 "./../src/optimize.c"
      if (FUNCCALL(SETUP_XfOrM73(_), scheme_omittable_expr (auto_e , 1 , fuel - 1 , flags , opt_info , warn_info ) ))
        RET_VALUE_START (1 ) RET_VALUE_END ; 
    }
  }

  if (vtype == scheme_branch_type ) {
    Scheme_Branch_Rec * b ; 
    BLOCK_SETUP((PUSH(b, 0+XfOrM37_COUNT)));
#   define XfOrM71_COUNT (1+XfOrM37_COUNT)
#   define SETUP_XfOrM71(x) SETUP(XfOrM71_COUNT)
    b = NULLED_OUT ; 

# 561 "./../src/optimize.c"
    b = (Scheme_Branch_Rec * ) o ; 
    RET_VALUE_START ((FUNCCALL(SETUP_XfOrM71(_), scheme_omittable_expr (b -> test , 1 , fuel - 1 , flags , opt_info , warn_info ) )
    && FUNCCALL(SETUP_XfOrM71(_), scheme_omittable_expr (b -> tbranch , vals , fuel - 1 , flags , opt_info , warn_info ) )
    && FUNCCALL(SETUP_XfOrM71(_), scheme_omittable_expr (b -> fbranch , vals , fuel - 1 , flags , opt_info , warn_info ) )) ) 
# 562 "./../src/optimize.c"
    RET_VALUE_END 

    ; 
  }

  if (vtype == scheme_let_one_type ) {
    Scheme_Let_One * lo = (Scheme_Let_One * ) o ; 
    BLOCK_SETUP((PUSH(lo, 0+XfOrM37_COUNT)));
#   define XfOrM70_COUNT (1+XfOrM37_COUNT)
#   define SETUP_XfOrM70(x) SETUP(XfOrM70_COUNT)

# 569 "./../src/optimize.c"
    RET_VALUE_START ((FUNCCALL(SETUP_XfOrM70(_), scheme_omittable_expr (lo -> value , 1 , fuel - 1 , flags , opt_info , warn_info ) )
    && FUNCCALL(SETUP_XfOrM70(_), scheme_omittable_expr (lo -> body , vals , fuel - 1 , flags , opt_info , warn_info ) )) ) 
# 569 "./../src/optimize.c"
    RET_VALUE_END 
    ; 
  }

  if (vtype == scheme_let_void_type ) {
    Scheme_Let_Void * lv = (Scheme_Let_Void * ) o ; 
    BLOCK_SETUP((PUSH(lv, 0+XfOrM37_COUNT)));
#   define XfOrM66_COUNT (1+XfOrM37_COUNT)
#   define SETUP_XfOrM66(x) SETUP(XfOrM66_COUNT)

# 576 "./../src/optimize.c"
    ((void ) sizeof ((flags & 0x1 ) ? 1 : 0 ) , __extension__ ({
#     define XfOrM69_COUNT (0+XfOrM66_COUNT)
#     define SETUP_XfOrM69(x) SETUP_XfOrM66(x)

# 576 "./../src/optimize.c" 3
      if (flags & 0x1 )

# 576 "./../src/optimize.c" 3
      ; 

# 576 "./../src/optimize.c" 3
      else FUNCCALL(SETUP_XfOrM69(_), __assert_fail ("flags & 0x1" , "./../src/optimize.c" , 576 , __extension__ __PRETTY_FUNCTION__ ) ); 
    }
    ) ) 
# 576 "./../src/optimize.c"
    ; 
    if ((Scheme_Type ) (((((intptr_t ) (lv -> body ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (lv -> body ) ) -> type ) ) == (Scheme_Type ) (scheme_let_value_type ) )  {
      Scheme_Let_Value * lv2 = (Scheme_Let_Value * ) lv -> body ; 
      BLOCK_SETUP((PUSH(lv2, 0+XfOrM66_COUNT)));
#     define XfOrM67_COUNT (1+XfOrM66_COUNT)
#     define SETUP_XfOrM67(x) SETUP(XfOrM67_COUNT)

# 579 "./../src/optimize.c"
      if ((lv2 -> count == 1 ) 
      && (lv2 -> position == 0 ) 
      && FUNCCALL(SETUP_XfOrM67(_), scheme_omittable_expr (lv2 -> value , 1 , fuel - 1 , flags , opt_info , warn_info ) )) {
        o = lv2 -> body ; 
      }

# 583 "./../src/optimize.c"
      else 
      o = lv -> body ; 
    }

# 585 "./../src/optimize.c"
    else 
    o = lv -> body ; 
    goto try_again ; 
  }

  if (vtype == scheme_ir_let_header_type ) {

    Scheme_IR_Let_Header * lh = (Scheme_IR_Let_Header * ) o ; 
    int i ; 
    BLOCK_SETUP((PUSH(lh, 0+XfOrM37_COUNT)));
#   define XfOrM62_COUNT (1+XfOrM37_COUNT)
#   define SETUP_XfOrM62(x) SETUP(XfOrM62_COUNT)

# 594 "./../src/optimize.c"
    ((void ) sizeof ((! (flags & 0x1 ) ) ? 1 : 0 ) , __extension__ ({
#     define XfOrM65_COUNT (0+XfOrM62_COUNT)
#     define SETUP_XfOrM65(x) SETUP_XfOrM62(x)

# 594 "./../src/optimize.c" 3
      if (! (flags & 0x1 ) )

# 594 "./../src/optimize.c" 3
      ; 

# 594 "./../src/optimize.c" 3
      else FUNCCALL(SETUP_XfOrM65(_), __assert_fail ("!(flags & 0x1)" , "./../src/optimize.c" , 594 , __extension__ __PRETTY_FUNCTION__ ) ); 
    }
    ) ) 
# 594 "./../src/optimize.c"
    ; 
    o = lh -> body ; 
    for (i = 0 ; i < lh -> num_clauses ; i ++ ) {
      Scheme_IR_Let_Value * lv = (Scheme_IR_Let_Value * ) o ; 
      BLOCK_SETUP((PUSH(lv, 0+XfOrM62_COUNT)));
#     define XfOrM64_COUNT (1+XfOrM62_COUNT)
#     define SETUP_XfOrM64(x) SETUP(XfOrM64_COUNT)

# 598 "./../src/optimize.c"
      if (! FUNCCALL(SETUP_XfOrM64(_), scheme_omittable_expr (lv -> value , lv -> count , fuel - 1 , flags , opt_info , warn_info ) ))
        RET_VALUE_START (0 ) RET_VALUE_END ; 
      o = lv -> body ; 
    }
    goto try_again ; 
  }

  if (vtype == scheme_letrec_type ) {
#   define XfOrM60_COUNT (0+XfOrM37_COUNT)
#   define SETUP_XfOrM60(x) SETUP_XfOrM37(x)

# 606 "./../src/optimize.c"
    ((void ) sizeof ((flags & 0x1 ) ? 1 : 0 ) , __extension__ ({
#     define XfOrM61_COUNT (0+XfOrM60_COUNT)
#     define SETUP_XfOrM61(x) SETUP_XfOrM60(x)

# 606 "./../src/optimize.c" 3
      if (flags & 0x1 )

# 606 "./../src/optimize.c" 3
      ; 

# 606 "./../src/optimize.c" 3
      else FUNCCALL(SETUP_XfOrM61(_), __assert_fail ("flags & 0x1" , "./../src/optimize.c" , 606 , __extension__ __PRETTY_FUNCTION__ ) ); 
    }
    ) ) 
# 606 "./../src/optimize.c"
    ; 
    o = ((Scheme_Letrec * ) o ) -> body ; 
    goto try_again ; 
  }

  if (vtype == scheme_application_type ) {
    Scheme_App_Rec * app = (Scheme_App_Rec * ) o ; 
    BLOCK_SETUP((PUSH(app, 0+XfOrM37_COUNT)));
#   define XfOrM52_COUNT (1+XfOrM37_COUNT)
#   define SETUP_XfOrM52(x) SETUP(XfOrM52_COUNT)

# 614 "./../src/optimize.c"
    if ((app -> num_args >= 4 ) && (app -> num_args <= 11 ) 
    && ((scheme_make_struct_type_proc ) == (app -> args [0 ] ) ) ) {
#     define XfOrM59_COUNT (0+XfOrM52_COUNT)
#     define SETUP_XfOrM59(x) SETUP_XfOrM52(x)
      FUNCCALL(SETUP_XfOrM59(_), note_match (5 , vals , warn_info ) )
# 616 "./../src/optimize.c"
      ; 
    }

    if ((scheme_is_functional_nonfailing_primitive (app -> args [0 ] , app -> num_args , vals ) ) 
    || FUNCCALL(SETUP_XfOrM52(_), scheme_is_struct_functional (app -> args [0 ] , app -> num_args , opt_info , vals ) )
    || (((& (app ) -> iso ) -> so . keyex & (1 << 11 ) ) && ! (flags & 0x8 ) ) ) {
      int i ; 
#     define XfOrM56_COUNT (0+XfOrM52_COUNT)
#     define SETUP_XfOrM56(x) SETUP_XfOrM52(x)

# 623 "./../src/optimize.c"
      for (i = app -> num_args ; i -- ; ) {
#       define XfOrM58_COUNT (0+XfOrM56_COUNT)
#       define SETUP_XfOrM58(x) SETUP_XfOrM56(x)

# 624 "./../src/optimize.c"
        if (! FUNCCALL(SETUP_XfOrM58(_), scheme_omittable_expr (app -> args [i + 1 ] , 1 , fuel - 1 , flags , opt_info , warn_info ) ))
          RET_VALUE_START (0 ) RET_VALUE_END ; 
      }
      RET_VALUE_START (1 ) RET_VALUE_END ; 
    }

# 628 "./../src/optimize.c"
    else if ((Scheme_Type ) (((((intptr_t ) (app -> args [0 ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> args [0 ] ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) )  {
#     define XfOrM53_COUNT (0+XfOrM52_COUNT)
#     define SETUP_XfOrM53(x) SETUP_XfOrM52(x)

# 629 "./../src/optimize.c"
      if (! ((((Scheme_Prim_Proc_Header * ) app -> args [0 ] ) -> flags ) & 8 ) ) {
#       define XfOrM55_COUNT (0+XfOrM53_COUNT)
#       define SETUP_XfOrM55(x) SETUP_XfOrM53(x)
        FUNCCALL(SETUP_XfOrM55(_), note_match (1 , vals , warn_info ) )
# 630 "./../src/optimize.c"
        ; 
      }

# 631 "./../src/optimize.c"
      else if ((scheme_values_proc ) == (app -> args [0 ] ) )  {
#       define XfOrM54_COUNT (0+XfOrM53_COUNT)
#       define SETUP_XfOrM54(x) SETUP_XfOrM53(x)
        FUNCCALL(SETUP_XfOrM54(_), note_match (app -> num_args , vals , warn_info ) )
# 632 "./../src/optimize.c"
        ; 
      }
    }

    if (! ((scheme_make_struct_type_proc ) == (app -> args [0 ] ) ) )
      RET_VALUE_START (0 ) RET_VALUE_END ; 
  }

  if (vtype == scheme_application2_type ) {
    Scheme_App2_Rec * app = (Scheme_App2_Rec * ) o ; 
    BLOCK_SETUP((PUSH(app, 0+XfOrM37_COUNT)));
#   define XfOrM46_COUNT (1+XfOrM37_COUNT)
#   define SETUP_XfOrM46(x) SETUP(XfOrM46_COUNT)

# 642 "./../src/optimize.c"
    if ((scheme_is_functional_nonfailing_primitive (app -> rator , 1 , vals ) ) 
    || FUNCCALL(SETUP_XfOrM46(_), scheme_is_struct_functional (app -> rator , 1 , opt_info , vals ) )
    || (((& (app ) -> iso ) -> so . keyex & (1 << 11 ) ) && ! (flags & 0x8 ) ) ) {
#     define XfOrM51_COUNT (0+XfOrM46_COUNT)
#     define SETUP_XfOrM51(x) SETUP_XfOrM46(x)

# 645 "./../src/optimize.c"
      if (FUNCCALL(SETUP_XfOrM51(_), scheme_omittable_expr (app -> rand , 1 , fuel - 1 , flags , opt_info , warn_info ) ))
        RET_VALUE_START (1 ) RET_VALUE_END ; 
    }

# 647 "./../src/optimize.c"
    else if (((app -> rator ) == (scheme_make_vector_proc ) ) 
    && (vals == 1 || vals == - 1 ) 
    && ((((intptr_t ) (app -> rand ) ) & 0x1 ) 
    && ((((intptr_t ) (app -> rand ) ) >> 1 ) >= 0 ) ) 
    && ((((((intptr_t ) (app -> rand ) ) >> 1 ) ) >= - 1073741824 ) && (((((intptr_t ) (app -> rand ) ) >> 1 ) ) <= 1073741823 ) ) ) {
      RET_VALUE_START (1 ) RET_VALUE_END ; 
    }

# 653 "./../src/optimize.c"
    else if ((app -> rator ) == (scheme_procedure_specialize_proc ) )  {
#     define XfOrM49_COUNT (0+XfOrM46_COUNT)
#     define SETUP_XfOrM49(x) SETUP_XfOrM46(x)

# 654 "./../src/optimize.c"
      if ((vals == 1 || vals == - 1 ) && FUNCCALL(SETUP_XfOrM49(_), extract_specialized_proc (o , ((void * ) 0 ) ) ))
        RET_VALUE_START (1 ) RET_VALUE_END ; 
    }

# 656 "./../src/optimize.c"
    else if ((Scheme_Type ) (((((intptr_t ) (app -> rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) )  {
#     define XfOrM47_COUNT (0+XfOrM46_COUNT)
#     define SETUP_XfOrM47(x) SETUP_XfOrM46(x)

# 657 "./../src/optimize.c"
      if (! ((((Scheme_Prim_Proc_Header * ) app -> rator ) -> flags ) & 8 ) 
      || ((scheme_values_proc ) == (app -> rator ) ) ) {
#       define XfOrM48_COUNT (0+XfOrM47_COUNT)
#       define SETUP_XfOrM48(x) SETUP_XfOrM47(x)
        FUNCCALL(SETUP_XfOrM48(_), note_match (1 , vals , warn_info ) )
# 659 "./../src/optimize.c"
        ; 
      }
    }

    if (! ((scheme_make_struct_type_property_proc ) == (app -> rator ) ) )
      RET_VALUE_START (0 ) RET_VALUE_END ; 
  }

  if (vtype == scheme_application3_type ) {
    Scheme_App3_Rec * app = (Scheme_App3_Rec * ) o ; 
    BLOCK_SETUP((PUSH(app, 0+XfOrM37_COUNT)));
#   define XfOrM40_COUNT (1+XfOrM37_COUNT)
#   define SETUP_XfOrM40(x) SETUP(XfOrM40_COUNT)

# 669 "./../src/optimize.c"
    if ((scheme_is_functional_nonfailing_primitive (app -> rator , 2 , vals ) ) 
    || FUNCCALL(SETUP_XfOrM40(_), scheme_is_struct_functional (app -> rator , 2 , opt_info , vals ) )
    || (((& (app ) -> iso ) -> so . keyex & (1 << 11 ) ) && ! (flags & 0x8 ) ) ) {
#     define XfOrM45_COUNT (0+XfOrM40_COUNT)
#     define SETUP_XfOrM45(x) SETUP_XfOrM40(x)

# 672 "./../src/optimize.c"
      if (FUNCCALL(SETUP_XfOrM45(_), scheme_omittable_expr (app -> rand1 , 1 , fuel - 1 , flags , opt_info , warn_info ) )
      && FUNCCALL(SETUP_XfOrM45(_), scheme_omittable_expr (app -> rand2 , 1 , fuel - 1 , flags , opt_info , warn_info ) ))
        RET_VALUE_START (1 ) RET_VALUE_END ; 
    }

# 675 "./../src/optimize.c"
    else if (((app -> rator ) == (scheme_make_vector_proc ) ) 
    && (vals == 1 || vals == - 1 ) 
    && ((((intptr_t ) (app -> rand1 ) ) & 0x1 ) 
    && ((((intptr_t ) (app -> rand1 ) ) >> 1 ) >= 0 ) 
    && ((((((intptr_t ) (app -> rand1 ) ) >> 1 ) ) >= - 1073741824 ) && (((((intptr_t ) (app -> rand1 ) ) >> 1 ) ) <= 1073741823 ) ) ) 
    && FUNCCALL(SETUP_XfOrM40(_), scheme_omittable_expr (app -> rand2 , 1 , fuel - 1 , flags , opt_info , warn_info ) )) {
      RET_VALUE_START (1 ) RET_VALUE_END ; 
    }

# 682 "./../src/optimize.c"
    else if ((Scheme_Type ) (((((intptr_t ) (app -> rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) )  {
#     define XfOrM41_COUNT (0+XfOrM40_COUNT)
#     define SETUP_XfOrM41(x) SETUP_XfOrM40(x)

# 683 "./../src/optimize.c"
      if (! ((((Scheme_Prim_Proc_Header * ) app -> rator ) -> flags ) & 8 ) ) {
#       define XfOrM43_COUNT (0+XfOrM41_COUNT)
#       define SETUP_XfOrM43(x) SETUP_XfOrM41(x)
        FUNCCALL(SETUP_XfOrM43(_), note_match (1 , vals , warn_info ) )
# 684 "./../src/optimize.c"
        ; 
      }

# 685 "./../src/optimize.c"
      else if ((scheme_values_proc ) == (app -> rator ) )  {
#       define XfOrM42_COUNT (0+XfOrM41_COUNT)
#       define SETUP_XfOrM42(x) SETUP_XfOrM41(x)
        FUNCCALL(SETUP_XfOrM42(_), note_match (2 , vals , warn_info ) )
# 686 "./../src/optimize.c"
        ; 
      }
    }

    if (! ((scheme_make_struct_type_property_proc ) == (app -> rator ) ) )
      RET_VALUE_START (0 ) RET_VALUE_END ; 
  }


  if (vtype == scheme_set_bang_type ) {
    Scheme_Set_Bang * sb = (Scheme_Set_Bang * ) o ; 
    if (((Scheme_Type ) (scheme_local_type ) == (Scheme_Type ) (((((intptr_t ) (sb -> var ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (sb -> var ) ) -> type ) ) ) 
    && ((Scheme_Type ) (scheme_local_type ) == (Scheme_Type ) (((((intptr_t ) (sb -> val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (sb -> val ) ) -> type ) ) ) 
    && ((((Scheme_Local * ) (sb -> var ) ) -> position ) == (((Scheme_Local * ) (sb -> val ) ) -> position ) ) )
      RET_VALUE_START (1 ) RET_VALUE_END ; 
    else if (((Scheme_Type ) (scheme_ir_local_type ) == (Scheme_Type ) (((((intptr_t ) (sb -> var ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (sb -> var ) ) -> type ) ) ) 
    && ((sb -> var ) == (sb -> val ) ) 
    && ((((Scheme_IR_Local * ) sb -> var ) -> mode != 1 ) 
    || ! ((Scheme_IR_Local * ) sb -> var ) -> compile . keep_assignment ) )
      RET_VALUE_START (1 ) RET_VALUE_END ; 
  }


  if (! (flags & 0x10 ) ) {
#   define XfOrM38_COUNT (0+XfOrM37_COUNT)
#   define SETUP_XfOrM38(x) SETUP_XfOrM37(x)

# 710 "./../src/optimize.c"
    if (FUNCCALL_EMPTY(scheme_is_simple_make_struct_type_property (o , vals , (((flags & 0x1 ) ? 0x1 : 0 ) | 0x2 ) , ((void * ) 0 ) , opt_info , ((void * ) 0 ) , ((void * ) 0 ) , 0 , ((void * ) 0 ) , 5 ) ))






    RET_VALUE_START (1 ) RET_VALUE_END ; 
  }

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 723 "./../src/optimize.c"
static Scheme_Object * ensure_single_value (Scheme_Object * e , Optimize_Info * info ) 

{
  Scheme_App2_Rec * app2 ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(e, 0), PUSH(app2, 1)));
# define XfOrM79_COUNT (2)
# define SETUP_XfOrM79(x) SETUP(XfOrM79_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  app2 = NULLED_OUT ; 

# 727 "./../src/optimize.c"
  if (FUNCCALL(SETUP_XfOrM79(_), single_valued_expression (e , info , 5 ) ))
    RET_VALUE_START (e ) RET_VALUE_END ; 

  app2 = ((Scheme_App2_Rec * ) FUNCCALL(SETUP_XfOrM79(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_App2_Rec ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
  app2 -> iso . so . type = scheme_application2_type ; 
  app2 -> rator = scheme_values_proc ; 
  app2 -> rand = e ; 
  (& (app2 ) -> iso ) -> so . keyex |= ((1 << 12 ) | (1 << 13 ) ) ; 

  RET_VALUE_START ((Scheme_Object * ) app2 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 739 "./../src/optimize.c"
static Scheme_Object * ensure_single_value_noncm (Scheme_Object * e , Optimize_Info * info ) 



{
  Scheme_App2_Rec * app2 ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(e, 0), PUSH(app2, 1)));
# define XfOrM80_COUNT (2)
# define SETUP_XfOrM80(x) SETUP(XfOrM80_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  app2 = NULLED_OUT ; 

# 745 "./../src/optimize.c"
  if (FUNCCALL(SETUP_XfOrM80(_), single_valued_noncm_expression (e , info , 5 ) ))
    RET_VALUE_START (e ) RET_VALUE_END ; 

  app2 = ((Scheme_App2_Rec * ) FUNCCALL(SETUP_XfOrM80(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_App2_Rec ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
  app2 -> iso . so . type = scheme_application2_type ; 
  app2 -> rator = scheme_values_proc ; 
  app2 -> rand = e ; 
  (& (app2 ) -> iso ) -> so . keyex |= ((1 << 12 ) | (1 << 13 ) ) ; 

  RET_VALUE_START ((Scheme_Object * ) app2 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 757 "./../src/optimize.c"
static Scheme_Object * ensure_noncm (Scheme_Object * e , Optimize_Info * info ) 




{
  Scheme_Sequence * seq ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(e, 0), PUSH(seq, 1)));
# define XfOrM81_COUNT (2)
# define SETUP_XfOrM81(x) SETUP(XfOrM81_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  seq = NULLED_OUT ; 

# 765 "./../src/optimize.c"
  if (FUNCCALL(SETUP_XfOrM81(_), noncm_expression (e , info , 5 ) ))
    RET_VALUE_START (e ) RET_VALUE_END ; 

  seq = FUNCCALL(SETUP_XfOrM81(_), scheme_malloc_sequence (1 ) ); 
  seq -> so . type = scheme_begin0_sequence_type ; 
  seq -> count = 1 ; 
  seq -> array [0 ] = e ; 

  RET_VALUE_START ((Scheme_Object * ) seq ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 776 "./../src/optimize.c"
static Scheme_Object * do_make_discarding_sequence (Scheme_Object * e1 , Scheme_Object * e2 , 
Optimize_Info * info , 
int ignored , int rev ) 




{

# 179 "./../src/optimize.c"
  Scheme_Object * __funcarg61 = NULLED_OUT ; 
  Scheme_Object * __funcarg60 = NULLED_OUT ; 
  Scheme_Object * __funcarg59 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSH(e1, 0), PUSH(info, 1), PUSH(e2, 2)));
# define XfOrM82_COUNT (3)
# define SETUP_XfOrM82(x) SETUP(XfOrM82_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 784 "./../src/optimize.c"
  if (ignored )
    e2 = FUNCCALL(SETUP_XfOrM82(_), optimize_ignored (e2 , info , 1 , 0 , 5 ) ); 

  e2 = FUNCCALL(SETUP_XfOrM82(_), ensure_single_value_noncm (e2 , info ) ); 

  if (FUNCCALL(SETUP_XfOrM82(_), scheme_omittable_expr (e1 , 1 , 5 , 0 , info , ((void * ) 0 ) ) ))
    RET_VALUE_START (e2 ) RET_VALUE_END ; 

  e1 = (__funcarg61 = FUNCCALL(SETUP_XfOrM82(_), optimize_ignored (e1 , info , 1 , 0 , 5 ) ), FUNCCALL_AGAIN(ensure_single_value (__funcarg61 , info ) )) ; 

  if (ignored && FUNCCALL(SETUP_XfOrM82(_), scheme_omittable_expr (e2 , 1 , 5 , 0 , info , ((void * ) 0 ) ) ))
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_single_value_noncm (e1 , info ) )) RET_VALUE_EMPTY_END ; 


  if (rev && FUNCCALL(SETUP_XfOrM82(_), movable_expression (e2 , info , 0 , 1 , 1 , 0 , 50 ) ))
    rev = 0 ; 

  if (! rev && ((Scheme_Type ) (((((intptr_t ) (e1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e1 ) ) -> type ) ) == (Scheme_Type ) (scheme_sequence_type ) ) ) {
    Scheme_Sequence * seq = (Scheme_Sequence * ) e1 ; 

    if (((((intptr_t ) (seq -> array [seq -> count - 1 ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (seq -> array [seq -> count - 1 ] ) ) -> type ) > _scheme_ir_values_types_ ) {
      seq -> array [seq -> count - 1 ] = e2 ; 
      RET_VALUE_START (e1 ) RET_VALUE_END ; 
    }
  }

  RET_VALUE_START ((__funcarg59 = (__funcarg60 = FUNCCALL(SETUP_XfOrM82(_), scheme_make_pair ((rev ? e1 : e2 ) , scheme_null ) ), FUNCCALL_AGAIN(scheme_make_pair ((rev ? e2 : e1 ) , __funcarg60 ) )) 
  , FUNCCALL_EMPTY(scheme_make_sequence_compilation (__funcarg59 , rev ? - 1 : 1 , 0 ) )) ) 
# 810 "./../src/optimize.c"
  RET_VALUE_END 


  ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 816 "./../src/optimize.c"
static Scheme_Object * make_discarding_sequence (Scheme_Object * e1 , Scheme_Object * e2 , 
Optimize_Info * info ) 
{
  /* No conversion */

# 819 "./../src/optimize.c"
  return do_make_discarding_sequence (e1 , e2 , info , 0 , 0 ) ; 
}

static Scheme_Object * make_discarding_reverse_sequence (Scheme_Object * e1 , Scheme_Object * e2 , 
Optimize_Info * info ) 
{
  /* No conversion */

# 825 "./../src/optimize.c"
  return do_make_discarding_sequence (e1 , e2 , info , 0 , 1 ) ; 
}

static Scheme_Object * make_discarding_sequence_3 (Scheme_Object * e1 , Scheme_Object * e2 , Scheme_Object * e3 , 
Optimize_Info * info ) 
{

# 816 "./../src/optimize.c"
  Scheme_Object * __funcarg62 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(e1, 0), PUSH(info, 1)));
# define XfOrM87_COUNT (2)
# define SETUP_XfOrM87(x) SETUP(XfOrM87_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)






  RET_VALUE_START ((__funcarg62 = FUNCCALL(SETUP_XfOrM87(_), make_discarding_sequence (e2 , e3 , info ) ), FUNCCALL_EMPTY(make_discarding_sequence (e1 , __funcarg62 , info ) )) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 834 "./../src/optimize.c"
static Scheme_Object * make_discarding_app_sequence (Scheme_App_Rec * appr , int result_pos , Scheme_Object * result , 
Optimize_Info * info ) 





{
  int i ; 
  Scheme_Object * l = scheme_null ; 
  Scheme_Object * __funcarg64 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(4);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(appr, 1), PUSH(l, 2)));
# define XfOrM88_COUNT (3)
# define SETUP_XfOrM88(x) SETUP(XfOrM88_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 845 "./../src/optimize.c"
  result_pos = result_pos + 1 ; 
  if (result )
    l = FUNCCALL(SETUP_XfOrM88(_), scheme_make_pair (result , l ) ); 

  for (i = appr -> num_args ; i ; i -- ) {
    Scheme_Object * e ; 
    BLOCK_SETUP((PUSH(e, 0+XfOrM88_COUNT)));
#   define XfOrM94_COUNT (1+XfOrM88_COUNT)
#   define SETUP_XfOrM94(x) SETUP(XfOrM94_COUNT)
    e = NULLED_OUT ; 

# 851 "./../src/optimize.c"
    e = appr -> args [i ] ; 
    e = FUNCCALL(SETUP_XfOrM94(_), ensure_single_value (e , info ) ); 
    if (i == result_pos ) {
#     define XfOrM96_COUNT (0+XfOrM94_COUNT)
#     define SETUP_XfOrM96(x) SETUP_XfOrM94(x)

# 854 "./../src/optimize.c"
      if ((l ) == (scheme_null ) )  {
#       define XfOrM98_COUNT (0+XfOrM96_COUNT)
#       define SETUP_XfOrM98(x) SETUP_XfOrM96(x)

# 855 "./../src/optimize.c"
        e = FUNCCALL(SETUP_XfOrM98(_), ensure_single_value_noncm (e , info ) ); 
        l = FUNCCALL(SETUP_XfOrM98(_), scheme_make_pair (e , scheme_null ) ); 
      }

# 857 "./../src/optimize.c"
      else {
#       define XfOrM97_COUNT (0+XfOrM96_COUNT)
#       define SETUP_XfOrM97(x) SETUP_XfOrM96(x)

# 858 "./../src/optimize.c"
        l = (__funcarg64 = FUNCCALL(SETUP_XfOrM97(_), scheme_make_pair (e , l ) ), FUNCCALL_AGAIN(scheme_make_sequence_compilation (__funcarg64 , - 1 , 0 ) )) ; 
        l = FUNCCALL(SETUP_XfOrM97(_), scheme_make_pair (l , scheme_null ) ); 
      }
    }

# 861 "./../src/optimize.c"
    else {
#     define XfOrM95_COUNT (0+XfOrM94_COUNT)
#     define SETUP_XfOrM95(x) SETUP_XfOrM94(x)

# 862 "./../src/optimize.c"
      e = FUNCCALL(SETUP_XfOrM95(_), optimize_ignored (e , info , 1 , 1 , 5 ) ); 
      if (e )
        l = FUNCCALL(SETUP_XfOrM95(_), scheme_make_pair (e , l ) ); 
    }
  }

  if ((l ) == (scheme_null ) ) 
    RET_VALUE_START (scheme_void ) RET_VALUE_END ; 

  if (((((Scheme_Simple_Object * ) (l ) ) -> u . pair_val . cdr ) ) == (scheme_null ) ) 
    RET_VALUE_START ((((Scheme_Simple_Object * ) (l ) ) -> u . pair_val . car ) ) RET_VALUE_END ; 

  RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(scheme_make_sequence_compilation (l , 1 , 0 ) )) RET_VALUE_EMPTY_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 877 "./../src/optimize.c"
static Scheme_Object * optimize_ignored (Scheme_Object * e , Optimize_Info * info , 
int expected_vals , int maybe_omittable , 
int fuel ) 



{

# 776 "./../src/optimize.c"
  Scheme_Object * __funcarg65 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(6);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(e, 1)));
# define XfOrM99_COUNT (2)
# define SETUP_XfOrM99(x) SETUP(XfOrM99_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 884 "./../src/optimize.c"
  if (FUNCCALL(SETUP_XfOrM99(_), scheme_omittable_expr (e , expected_vals , 5 , 0 , info , ((void * ) 0 ) ) ))
    RET_VALUE_START (maybe_omittable ? ((void * ) 0 ) : scheme_false ) RET_VALUE_END ; 

  if (fuel ) {
#   define XfOrM100_COUNT (0+XfOrM99_COUNT)
#   define SETUP_XfOrM100(x) SETUP_XfOrM99(x)
    switch (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) {
#     define XfOrM101_COUNT (0+XfOrM100_COUNT)
#     define SETUP_XfOrM101(x) SETUP_XfOrM100(x)

# 891 "./../src/optimize.c"
      case scheme_application2_type : 
      {
        Scheme_App2_Rec * app = (Scheme_App2_Rec * ) e ; 
#       define XfOrM119_COUNT (0+XfOrM101_COUNT)
#       define SETUP_XfOrM119(x) SETUP_XfOrM101(x)

# 895 "./../src/optimize.c"
        if (! ((app -> rator ) == (scheme_values_proc ) ) )
          if (scheme_is_functional_nonfailing_primitive (app -> rator , 1 , expected_vals ) ) 
          RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(do_make_discarding_sequence (app -> rand , scheme_void , info , 1 , 0 ) )) RET_VALUE_EMPTY_END ; 


        if (((app -> rator ) == (scheme_make_vector_proc ) ) 
        && ((((intptr_t ) (app -> rand ) ) & 0x1 ) 
        && ((((intptr_t ) (app -> rand ) ) >> 1 ) >= 0 ) ) 
        && ((((((intptr_t ) (app -> rand ) ) >> 1 ) ) >= - 1073741824 ) && (((((intptr_t ) (app -> rand ) ) >> 1 ) ) <= 1073741823 ) ) )
          RET_VALUE_START ((maybe_omittable ? ((void * ) 0 ) : scheme_void ) ) RET_VALUE_END ; 
      }
      break ; 
      case scheme_application3_type : 
      {
        Scheme_App3_Rec * app = (Scheme_App3_Rec * ) e ; 
        BLOCK_SETUP((PUSH(app, 0+XfOrM101_COUNT)));
#       define XfOrM117_COUNT (1+XfOrM101_COUNT)
#       define SETUP_XfOrM117(x) SETUP(XfOrM117_COUNT)

# 911 "./../src/optimize.c"
        if (scheme_is_functional_nonfailing_primitive (app -> rator , 2 , expected_vals ) ) 
          RET_VALUE_START ((__funcarg65 = FUNCCALL(SETUP_XfOrM117(_), do_make_discarding_sequence (app -> rand2 , scheme_void , info , 1 , 0 ) )



        , FUNCCALL_EMPTY(do_make_discarding_sequence (app -> rand1 , __funcarg65 , info , 1 , 0 ) )) ) 
# 912 "./../src/optimize.c"
        RET_VALUE_END 





        ; 


        if (((app -> rator ) == (scheme_make_vector_proc ) ) 
        && ((((intptr_t ) (app -> rand1 ) ) & 0x1 ) 
        && ((((intptr_t ) (app -> rand1 ) ) >> 1 ) >= 0 ) ) 
        && ((((((intptr_t ) (app -> rand1 ) ) >> 1 ) ) >= - 1073741824 ) && (((((intptr_t ) (app -> rand1 ) ) >> 1 ) ) <= 1073741823 ) ) ) {
          Scheme_Object * val ; 
          BLOCK_SETUP((PUSH(val, 0+XfOrM117_COUNT)));
#         define XfOrM118_COUNT (1+XfOrM117_COUNT)
#         define SETUP_XfOrM118(x) SETUP(XfOrM118_COUNT)
          val = NULLED_OUT ; 

# 926 "./../src/optimize.c"
          val = FUNCCALL(SETUP_XfOrM118(_), ensure_single_value_noncm (app -> rand2 , info ) ); 
          RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_ignored (val , info , 1 , maybe_omittable , 5 ) )) RET_VALUE_EMPTY_END ; 
        }
      }
      break ; 
      case scheme_application_type : 
      {
        Scheme_App_Rec * app = (Scheme_App_Rec * ) e ; 
#       define XfOrM116_COUNT (0+XfOrM101_COUNT)
#       define SETUP_XfOrM116(x) SETUP_XfOrM101(x)

# 935 "./../src/optimize.c"
        if (scheme_is_functional_nonfailing_primitive (app -> args [0 ] , app -> num_args , expected_vals ) ) 
          RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_app_sequence (app , - 1 , ((void * ) 0 ) , info ) )) RET_VALUE_EMPTY_END ; 
      }
      break ; 
      case scheme_branch_type : 
      {
        Scheme_Branch_Rec * b = (Scheme_Branch_Rec * ) e ; 
        Scheme_Object * tb , * fb ; 
        BLOCK_SETUP((PUSH(b, 0+XfOrM101_COUNT), PUSH(tb, 1+XfOrM101_COUNT), PUSH(fb, 2+XfOrM101_COUNT)));
#       define XfOrM113_COUNT (3+XfOrM101_COUNT)
#       define SETUP_XfOrM113(x) SETUP(XfOrM113_COUNT)
        tb = NULLED_OUT ; 
        fb = NULLED_OUT ; 

# 944 "./../src/optimize.c"
        tb = FUNCCALL(SETUP_XfOrM113(_), optimize_ignored (b -> tbranch , info , expected_vals , 1 , fuel - 1 ) ); 
        fb = FUNCCALL_AGAIN(optimize_ignored (b -> fbranch , info , expected_vals , 1 , fuel - 1 ) ); 

        if (tb || fb ) {
          b -> tbranch = tb ? tb : scheme_false ; 
          b -> fbranch = fb ? fb : scheme_false ; 
          RET_VALUE_START ((Scheme_Object * ) b ) RET_VALUE_END ; 
        }

# 951 "./../src/optimize.c"
        else {
          Scheme_Object * val ; 
          BLOCK_SETUP((PUSH(val, 0+XfOrM113_COUNT)));
#         define XfOrM114_COUNT (1+XfOrM113_COUNT)
#         define SETUP_XfOrM114(x) SETUP(XfOrM114_COUNT)
          val = NULLED_OUT ; 

# 953 "./../src/optimize.c"
          val = FUNCCALL(SETUP_XfOrM114(_), ensure_single_value_noncm (b -> test , info ) ); 
          RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_ignored (val , info , 1 , maybe_omittable , 5 ) )) RET_VALUE_EMPTY_END ; 
        }
      }
      break ; 
      case scheme_sequence_type : 
      {
        Scheme_Sequence * seq = (Scheme_Sequence * ) e ; 
        Scheme_Object * last ; 
        BLOCK_SETUP((PUSH(last, 0+XfOrM101_COUNT), PUSH(seq, 1+XfOrM101_COUNT)));
#       define XfOrM109_COUNT (2+XfOrM101_COUNT)
#       define SETUP_XfOrM109(x) SETUP(XfOrM109_COUNT)
        last = NULLED_OUT ; 

# 963 "./../src/optimize.c"
        last = FUNCCALL(SETUP_XfOrM109(_), optimize_ignored (seq -> array [seq -> count - 1 ] , info , expected_vals , 1 , fuel - 1 ) ); 

        if (last ) {
          seq -> array [seq -> count - 1 ] = last ; 
          RET_VALUE_START ((Scheme_Object * ) seq ) RET_VALUE_END ; 
        }

# 968 "./../src/optimize.c"
        else if (seq -> count == 2 
        && (expected_vals == - 1 
        || FUNCCALL(SETUP_XfOrM109(_), single_valued_noncm_expression (seq -> array [0 ] , info , 5 ) )) ) {
          RET_VALUE_START (seq -> array [0 ] ) RET_VALUE_END ; 
        }

# 972 "./../src/optimize.c"
        else {
          seq -> array [seq -> count - 1 ] = scheme_false ; 
          RET_VALUE_START ((Scheme_Object * ) seq ) RET_VALUE_END ; 
        }
      }
      case scheme_begin0_sequence_type : 
      {
        Scheme_Sequence * seq = (Scheme_Sequence * ) e ; 
        Scheme_Object * first ; 
        BLOCK_SETUP((PUSH(seq, 0+XfOrM101_COUNT), PUSH(first, 1+XfOrM101_COUNT)));
#       define XfOrM105_COUNT (2+XfOrM101_COUNT)
#       define SETUP_XfOrM105(x) SETUP(XfOrM105_COUNT)
        first = NULLED_OUT ; 

# 982 "./../src/optimize.c"
        first = FUNCCALL(SETUP_XfOrM105(_), optimize_ignored (seq -> array [0 ] , info , expected_vals , 1 , fuel - 1 ) ); 

        if (first ) {
          seq -> array [0 ] = first ; 
          RET_VALUE_START ((Scheme_Object * ) seq ) RET_VALUE_END ; 
        }

# 987 "./../src/optimize.c"
        else if (seq -> count == 2 
        && (expected_vals == - 1 
        || FUNCCALL(SETUP_XfOrM105(_), single_valued_noncm_expression (seq -> array [1 ] , info , 5 ) )) ) {
          RET_VALUE_START (seq -> array [1 ] ) RET_VALUE_END ; 
        }

# 991 "./../src/optimize.c"
        else {
          seq -> array [0 ] = scheme_false ; 
          RET_VALUE_START ((Scheme_Object * ) seq ) RET_VALUE_END ; 
        }
      }
      break ; 
      case scheme_ir_let_header_type : 
      {
        Scheme_IR_Let_Header * head = (Scheme_IR_Let_Header * ) e ; 
        Scheme_IR_Let_Value * lv ; 
        Scheme_Object * body ; 
        int i ; 
        BLOCK_SETUP((PUSH(body, 0+XfOrM101_COUNT), PUSH(lv, 1+XfOrM101_COUNT), PUSH(head, 2+XfOrM101_COUNT)));
#       define XfOrM102_COUNT (3+XfOrM101_COUNT)
#       define SETUP_XfOrM102(x) SETUP(XfOrM102_COUNT)
        lv = NULLED_OUT ; 
        body = NULLED_OUT ; 

# 1004 "./../src/optimize.c"
        body = head -> body ; 
        if (0 == head -> num_clauses )
          lv = (Scheme_IR_Let_Value * ) body ; 
        for (i = head -> num_clauses ; i -- ; ) {
          lv = (Scheme_IR_Let_Value * ) body ; 
          body = lv -> body ; 
        }
        body = FUNCCALL(SETUP_XfOrM102(_), optimize_ignored (body , info , expected_vals , 0 , fuel - 1 ) ); 
        lv -> body = body ; 
        RET_VALUE_START ((Scheme_Object * ) head ) RET_VALUE_END ; 
      }
      break ; 
    }
  }

  RET_VALUE_START (e ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1022 "./../src/optimize.c"
static Scheme_Object * make_sequence_2 (Scheme_Object * a , Scheme_Object * b ) 
{
  Scheme_Object * __funcarg67 = NULLED_OUT ; 
  Scheme_Object * __funcarg66 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(1);
  BLOCK_SETUP_TOP((PUSH(a, 0)));
# define XfOrM120_COUNT (1)
# define SETUP_XfOrM120(x) SETUP(XfOrM120_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 1024 "./../src/optimize.c"
  RET_VALUE_START ((__funcarg66 = (__funcarg67 = FUNCCALL(SETUP_XfOrM120(_), scheme_make_pair (b , scheme_null ) ), FUNCCALL_AGAIN(scheme_make_pair (a , __funcarg67 ) )) , FUNCCALL_EMPTY(scheme_make_sequence_compilation (__funcarg66 , 1 , 0 ) )) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1027 "./../src/optimize.c"
static Scheme_Object * make_discarding_first_sequence (Scheme_Object * e1 , Scheme_Object * e2 , 
Optimize_Info * info ) 


{
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSH(e1, 0), PUSH(info, 1), PUSH(e2, 2)));
# define XfOrM121_COUNT (3)
# define SETUP_XfOrM121(x) SETUP(XfOrM121_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 1032 "./../src/optimize.c"
  e1 = FUNCCALL(SETUP_XfOrM121(_), optimize_ignored (e1 , info , 1 , 1 , 5 ) ); 
  if (! e1 )
    RET_VALUE_START (e2 ) RET_VALUE_END ; 
  e1 = FUNCCALL(SETUP_XfOrM121(_), ensure_single_value (e1 , info ) ); 
  RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_sequence_2 (e1 , e2 ) )) RET_VALUE_EMPTY_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1039 "./../src/optimize.c"
static Scheme_Object * make_application_2 (Scheme_Object * a , Scheme_Object * b , Optimize_Info * info ) 
{
  Scheme_Object * __funcarg69 = NULLED_OUT ; 
  Scheme_Object * __funcarg68 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(a, 0), PUSH(info, 1)));
# define XfOrM122_COUNT (2)
# define SETUP_XfOrM122(x) SETUP(XfOrM122_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 1041 "./../src/optimize.c"
  RET_VALUE_START ((__funcarg68 = (__funcarg69 = FUNCCALL(SETUP_XfOrM122(_), scheme_make_pair (b , scheme_null ) ), FUNCCALL_AGAIN(scheme_make_pair (a , __funcarg69 ) )) , FUNCCALL_EMPTY(scheme_make_application (__funcarg68 , info ) )) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1044 "./../src/optimize.c"
static Scheme_Object * make_application_3 (Scheme_Object * a , Scheme_Object * b , Scheme_Object * c , 
Optimize_Info * info ) 
{
  Scheme_Object * __funcarg72 = NULLED_OUT ; 
  Scheme_Object * __funcarg71 = NULLED_OUT ; 
  Scheme_Object * __funcarg70 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSH(b, 0), PUSH(a, 1), PUSH(info, 2)));
# define XfOrM123_COUNT (3)
# define SETUP_XfOrM123(x) SETUP(XfOrM123_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 1047 "./../src/optimize.c"
  RET_VALUE_START ((__funcarg70 = (__funcarg71 = (__funcarg72 = FUNCCALL(SETUP_XfOrM123(_), scheme_make_pair (c , scheme_null ) ), FUNCCALL_AGAIN(scheme_make_pair (b , __funcarg72 ) )) , FUNCCALL_AGAIN(scheme_make_pair (a , __funcarg71 ) )) , FUNCCALL_EMPTY(scheme_make_application (__funcarg70 , info ) )) ) RET_VALUE_END 
  ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1051 "./../src/optimize.c"
static Scheme_Object * replace_tail_inside (Scheme_Object * alt , Scheme_Object * inside , Scheme_Object * orig ) 


{
  /* No conversion */

# 1055 "./../src/optimize.c"
  if (inside ) {
    switch (((((intptr_t ) (inside ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (inside ) ) -> type ) ) {
      case scheme_sequence_type : 
      if (((Scheme_Sequence * ) inside ) -> count )
        ((Scheme_Sequence * ) inside ) -> array [((Scheme_Sequence * ) inside ) -> count - 1 ] = alt ; 
      else 
      scheme_signal_error ("internal error: strange inside replacement" ) ; 
      break ; 
      case scheme_ir_let_header_type : 
      ((Scheme_IR_Let_Header * ) inside ) -> body = alt ; 
      break ; 
      case scheme_ir_let_value_type : 
      ((Scheme_IR_Let_Value * ) inside ) -> body = alt ; 
      break ; 
      case scheme_with_cont_mark_type : 
      ((Scheme_With_Continuation_Mark * ) inside ) -> body = alt ; 
      break ; 
      default : 
      scheme_signal_error ("internal error: strange inside replacement" ) ; 
    }
    return orig ; 
  }
  return alt ; 
}

static void extract_tail_inside (Scheme_Object * * _t2 , Scheme_Object * * _inside , int for_immediate_body ) 


{
  /* No conversion */

# 1084 "./../src/optimize.c"
  while (1 ) {
    if ((Scheme_Type ) (((((intptr_t ) (* _t2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (* _t2 ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_let_header_type ) )  {
      Scheme_IR_Let_Header * head = (Scheme_IR_Let_Header * ) * _t2 ; 
      int i ; 
      * _inside = * _t2 ; 
      * _t2 = head -> body ; 
      for (i = head -> num_clauses ; i -- ; ) {
        * _inside = * _t2 ; 
        * _t2 = ((Scheme_IR_Let_Value * ) * _t2 ) -> body ; 
      }
    }

# 1094 "./../src/optimize.c"
    else if ((Scheme_Type ) (((((intptr_t ) (* _t2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (* _t2 ) ) -> type ) ) == (Scheme_Type ) (scheme_sequence_type ) )  {
      Scheme_Sequence * seq = (Scheme_Sequence * ) * _t2 ; 
      if (seq -> count ) {
        * _inside = * _t2 ; 
        * _t2 = seq -> array [seq -> count - 1 ] ; 
      }

# 1099 "./../src/optimize.c"
      else 
      break ; 
    }

# 1101 "./../src/optimize.c"
    else if (((Scheme_Type ) (((((intptr_t ) (* _t2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (* _t2 ) ) -> type ) ) == (Scheme_Type ) (scheme_with_cont_mark_type ) ) 
    && for_immediate_body ) {
      Scheme_With_Continuation_Mark * wcm = (Scheme_With_Continuation_Mark * ) * _t2 ; 
      * _inside = * _t2 ; 
      * _t2 = wcm -> body ; 
    }

# 1106 "./../src/optimize.c"
    else 
    break ; 
  }
}

Scheme_Object * scheme_optimize_extract_tail_inside (Scheme_Object * t2 ) 
{
  Scheme_Object * inside ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(t2, 0), PUSH(inside, 1)));
# define XfOrM142_COUNT (2)
# define SETUP_XfOrM142(x) SETUP(XfOrM142_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  inside = NULLED_OUT ; 
  FUNCCALL(SETUP_XfOrM142(_), extract_tail_inside (& t2 , & inside , 0 ) )
# 1114 "./../src/optimize.c"
  ; 
  RET_VALUE_START (t2 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

static int is_inspector_call (Scheme_Object * a ) 

{
  /* No conversion */

# 1125 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (a ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (a ) ) -> type ) ) == (Scheme_Type ) (scheme_application_type ) )  {
    Scheme_App_Rec * app = (Scheme_App_Rec * ) a ; 
    if (! app -> num_args 
    && (((app -> args [0 ] ) == (scheme_current_inspector_proc ) ) 
    || ((app -> args [0 ] ) == (scheme_make_inspector_proc ) ) ) )
      return 1 ; 
  }
  return 0 ; 
}

static int is_proc_spec_proc (Scheme_Object * p , int init_field_count ) 

{
  Scheme_Type vtype ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(1);
  BLOCK_SETUP_TOP((PUSH(p, 0)));
# define XfOrM145_COUNT (1)
# define SETUP_XfOrM145(x) SETUP(XfOrM145_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 1140 "./../src/optimize.c"
  if ((((intptr_t ) (p ) ) & 0x1 ) 
  && ((((intptr_t ) (p ) ) >> 1 ) >= 0 ) 
  && ((((intptr_t ) (p ) ) >> 1 ) < init_field_count ) )
    RET_VALUE_START (1 ) RET_VALUE_END ; 

  if (! (((intptr_t ) (p ) ) & 0x1 ) && ((((p ) -> type ) >= scheme_prim_type ) && (((p ) -> type ) <= scheme_proc_chaperone_type ) ) )  {
#   define XfOrM147_COUNT (0+XfOrM145_COUNT)
#   define SETUP_XfOrM147(x) SETUP_XfOrM145(x)

# 1146 "./../src/optimize.c"
    p = FUNCCALL(SETUP_XfOrM147(_), scheme_get_or_check_arity (p , - 1 ) ); 
    if (((intptr_t ) (p ) ) & 0x1 )  {
      RET_VALUE_START (((((intptr_t ) (p ) ) >> 1 ) >= 1 ) ) RET_VALUE_END ; 
    }

# 1149 "./../src/optimize.c"
    else if ((((Scheme_Type ) (((((intptr_t ) (p ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (p ) ) -> type ) ) == (Scheme_Type ) (scheme_structure_type ) ) || ((Scheme_Type ) (((((intptr_t ) (p ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (p ) ) -> type ) ) == (Scheme_Type ) (scheme_proc_struct_type ) ) ) 
    && (scheme_is_struct_instance (scheme_arity_at_least , p ) ) ) {
      p = ((Scheme_Structure * ) p ) -> slots [0 ] ; 
      if (((intptr_t ) (p ) ) & 0x1 ) 
        RET_VALUE_START (((((intptr_t ) (p ) ) >> 1 ) >= 1 ) ) RET_VALUE_END ; 
    }
    RET_VALUE_START (0 ) RET_VALUE_END ; 
  }

  vtype = ((((intptr_t ) (p ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (p ) ) -> type ) ; 

  if ((vtype == scheme_lambda_type ) || (vtype == scheme_ir_lambda_type ) ) {
    if (((Scheme_Lambda * ) p ) -> num_params >= 1 )
      RET_VALUE_START (1 ) RET_VALUE_END ; 
  }

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1168 "./../src/optimize.c"
static int is_local_ref (Scheme_Object * e , int p , int r , Scheme_IR_Local * * vars ) 



{
  /* No conversion */

# 1173 "./../src/optimize.c"
  if (! vars && ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_local_type ) ) ) {
    if (((((Scheme_Local * ) (e ) ) -> position ) >= p ) 
    && ((((Scheme_Local * ) (e ) ) -> position ) < (p + r ) ) )
      return 1 ; 
  }

# 1177 "./../src/optimize.c"
  else if (vars && ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) ) {
    int i ; 
    for (i = p ; i < p + r ; i ++ ) {
      if ((e ) == ((Scheme_Object * ) vars [i ] ) ) 
        return 1 ; 
    }
  }

  return 0 ; 
}

static int is_int_list (Scheme_Object * o , int up_to ) 

{
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK(3);
  BLOCK_SETUP_TOP((PUSH(o, 0)));
# define XfOrM155_COUNT (1)
# define SETUP_XfOrM155(x) SETUP(XfOrM155_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 1191 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (o ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o ) ) -> type ) ) == (Scheme_Type ) (scheme_pair_type ) )  {
    char * s , quick [8 ] ; 
    Scheme_Object * e ; 
    BLOCK_SETUP((PUSH(e, 0+XfOrM155_COUNT), PUSH(s, 1+XfOrM155_COUNT)));
#   define XfOrM156_COUNT (2+XfOrM155_COUNT)
#   define SETUP_XfOrM156(x) SETUP(XfOrM156_COUNT)
    s = NULLED_OUT ; 
    e = NULLED_OUT ; 

# 1194 "./../src/optimize.c"
    if (up_to <= 8 )
      s = quick ; 
    else 
    s = (char * ) FUNCCALL(SETUP_XfOrM156(_), GC_malloc_atomic (up_to ) ); 
    (memset (s , 0 , up_to ) ) ; 
    while (((Scheme_Type ) (((((intptr_t ) (o ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o ) ) -> type ) ) == (Scheme_Type ) (scheme_pair_type ) ) ) {
      e = (((Scheme_Simple_Object * ) (o ) ) -> u . pair_val . car ) ; 
      o = (((Scheme_Simple_Object * ) (o ) ) -> u . pair_val . cdr ) ; 
      if (! (((intptr_t ) (e ) ) & 0x1 ) 
      || ((((intptr_t ) (e ) ) >> 1 ) < 0 ) 
      || ((((intptr_t ) (e ) ) >> 1 ) > up_to ) 
      || s [(((intptr_t ) (e ) ) >> 1 ) ] )
        RET_VALUE_START (0 ) RET_VALUE_END ; 
      s [(((intptr_t ) (e ) ) >> 1 ) ] = 1 ; 
    }
  }

  RET_VALUE_START (((o ) == (scheme_null ) ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1214 "./../src/optimize.c"
static int ok_proc_creator_args (Scheme_Object * rator , Scheme_Object * rand1 , Scheme_Object * rand2 , Scheme_Object * rand3 , 
int delta2 , int field_count , Scheme_IR_Local * * vars ) 

{
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(5);
  BLOCK_SETUP_TOP((PUSH(rand1, 0), PUSH(rand3, 1), PUSH(rand2, 2), PUSH(rator, 3), PUSH(vars, 4)));
# define XfOrM159_COUNT (5)
# define SETUP_XfOrM159(x) SETUP(XfOrM159_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 1218 "./../src/optimize.c"
  if ((((rator ) == (scheme_make_struct_field_accessor_proc ) ) 
  && FUNCCALL(SETUP_XfOrM159(_), is_local_ref (rand1 , delta2 + 3 , 1 , vars ) )) 
  || (((rator ) == (scheme_make_struct_field_mutator_proc ) ) 
  && FUNCCALL(SETUP_XfOrM159(_), is_local_ref (rand1 , delta2 + 4 , 1 , vars ) )) ) {
    if ((((intptr_t ) (rand2 ) ) & 0x1 ) 
    && ((((intptr_t ) (rand2 ) ) >> 1 ) >= 0 ) 
    && ((((intptr_t ) (rand2 ) ) >> 1 ) < field_count ) 
    && (! rand3 || ((Scheme_Type ) (((((intptr_t ) (rand3 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rand3 ) ) -> type ) ) == (Scheme_Type ) (scheme_symbol_type ) ) ) ) {
      RET_VALUE_START (1 ) RET_VALUE_END ; 
    }
  }

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1233 "./../src/optimize.c"
static int is_values_with_accessors_and_mutators (Scheme_Object * e , int vals , int resolved , 
Simple_Struct_Type_Info * _stinfo , 
Scheme_IR_Local * * vars ) 


{
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK(4);
  BLOCK_SETUP_TOP((PUSH(_stinfo, 0), PUSH(vars, 1)));
# define XfOrM162_COUNT (2)
# define SETUP_XfOrM162(x) SETUP(XfOrM162_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 1239 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_application_type ) )  {
    Scheme_App_Rec * app = (Scheme_App_Rec * ) e ; 
    int delta = (resolved ? app -> num_args : 0 ) ; 
    BLOCK_SETUP((PUSH(app, 0+XfOrM162_COUNT)));
#   define XfOrM163_COUNT (1+XfOrM162_COUNT)
#   define SETUP_XfOrM163(x) SETUP(XfOrM163_COUNT)

# 1242 "./../src/optimize.c"
    if (((app -> args [0 ] ) == (scheme_values_proc ) ) 
    && (app -> num_args == vals ) 
    && (app -> num_args >= 3 ) 
    && FUNCCALL(SETUP_XfOrM163(_), is_local_ref (app -> args [1 ] , delta , 1 , vars ) )
    && FUNCCALL(SETUP_XfOrM163(_), is_local_ref (app -> args [2 ] , delta + 1 , 1 , vars ) )
    && FUNCCALL(SETUP_XfOrM163(_), is_local_ref (app -> args [3 ] , delta + 2 , 1 , vars ) )) {
      int i , num_gets = 0 , num_sets = 0 , normal_ops = 1 ; 
      int setter_fields = 0 , normal_sets = 1 ; 
      int prev_setter_pos = app -> num_args ; 
#     define XfOrM164_COUNT (0+XfOrM163_COUNT)
#     define SETUP_XfOrM164(x) SETUP_XfOrM163(x)

# 1251 "./../src/optimize.c"
      for (i = app -> num_args ; i > 3 ; i -- ) {
#       define XfOrM180_COUNT (0+XfOrM164_COUNT)
#       define SETUP_XfOrM180(x) SETUP_XfOrM164(x)

# 1252 "./../src/optimize.c"
        if (FUNCCALL(SETUP_XfOrM180(_), is_local_ref (app -> args [i ] , delta , 5 , vars ) )) {
          normal_ops = 0 ; 
        }

# 1254 "./../src/optimize.c"
        else if (((Scheme_Type ) (((((intptr_t ) (app -> args [i ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> args [i ] ) ) -> type ) ) == (Scheme_Type ) (scheme_application_type ) ) 
        && _stinfo -> normal_ops && ! _stinfo -> indexed_ops ) {
          Scheme_App_Rec * app3 = (Scheme_App_Rec * ) app -> args [i ] ; 
          int delta2 = delta + (resolved ? app3 -> num_args : 0 ) ; 
          BLOCK_SETUP((PUSH(app3, 0+XfOrM180_COUNT)));
#         define XfOrM184_COUNT (1+XfOrM180_COUNT)
#         define SETUP_XfOrM184(x) SETUP(XfOrM184_COUNT)

# 1258 "./../src/optimize.c"
          if (app3 -> num_args == 3 ) {
#           define XfOrM185_COUNT (0+XfOrM184_COUNT)
#           define SETUP_XfOrM185(x) SETUP_XfOrM184(x)

# 1259 "./../src/optimize.c"
            if (! FUNCCALL(SETUP_XfOrM185(_), ok_proc_creator_args (app3 -> args [0 ] , app3 -> args [1 ] , app3 -> args [2 ] , app3 -> args [3 ] , delta2 , _stinfo -> field_count , vars ) ))

            break ; 
            if ((app3 -> args [0 ] ) == (scheme_make_struct_field_mutator_proc ) )  {
              int pos = (((intptr_t ) (app3 -> args [2 ] ) ) >> 1 ) ; 
              if (num_gets ) {


                normal_ops = 0 ; 
              }
              if (normal_sets ) {
                if (pos >= prev_setter_pos ) {

                  normal_sets = 0 ; 
                  setter_fields = 0 ; 
                }

# 1274 "./../src/optimize.c"
                else if (pos < (31 - 7 ) ) {
                  setter_fields |= (1 << pos ) ; 
                  prev_setter_pos = pos ; 
                }
              }
              num_sets ++ ; 
            }

# 1280 "./../src/optimize.c"
            else {
              if ((((intptr_t ) (app3 -> args [2 ] ) ) >> 1 ) != (i - 4 ) ) {

                normal_ops = 0 ; 
              }
              num_gets ++ ; 
            }
          }

# 1287 "./../src/optimize.c"
          else 
          break ; 
        }

# 1289 "./../src/optimize.c"
        else if (((Scheme_Type ) (((((intptr_t ) (app -> args [i ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> args [i ] ) ) -> type ) ) == (Scheme_Type ) (scheme_application3_type ) ) 
        && _stinfo -> normal_ops && ! _stinfo -> indexed_ops ) {
          Scheme_App3_Rec * app3 = (Scheme_App3_Rec * ) app -> args [i ] ; 
          int delta2 = delta + (resolved ? 2 : 0 ) ; 
          BLOCK_SETUP((PUSH(app3, 0+XfOrM180_COUNT)));
#         define XfOrM181_COUNT (1+XfOrM180_COUNT)
#         define SETUP_XfOrM181(x) SETUP(XfOrM181_COUNT)

# 1293 "./../src/optimize.c"
          if (! FUNCCALL(SETUP_XfOrM181(_), ok_proc_creator_args (app3 -> rator , app3 -> rand1 , app3 -> rand2 , ((void * ) 0 ) , delta2 , _stinfo -> field_count , vars ) ))

          break ; 
          if ((app3 -> rator ) == (scheme_make_struct_field_mutator_proc ) )  {
            if (num_gets )

# 1297 "./../src/optimize.c"
            normal_ops = 0 ; 
            num_sets ++ ; 
          }

# 1299 "./../src/optimize.c"
          else {
            if ((((intptr_t ) (app3 -> rand2 ) ) >> 1 ) != (i - 4 ) )

# 1300 "./../src/optimize.c"
            normal_ops = 0 ; 
            num_gets ++ ; 
          }
        }

# 1303 "./../src/optimize.c"
        else 
        break ; 
      }
      if (i <= 3 ) {
        _stinfo -> normal_ops = normal_ops ; 
        _stinfo -> indexed_ops = 1 ; 
        _stinfo -> num_gets = num_gets ; 
        _stinfo -> num_sets = num_sets ; 
        _stinfo -> setter_fields = setter_fields ; 
        RET_VALUE_START (1 ) RET_VALUE_END ; 
      }
    }
  }

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1320 "./../src/optimize.c"
static Scheme_Object * skip_clears (Scheme_Object * body ) 


{
  /* No conversion */

# 1324 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (body ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (body ) ) -> type ) ) == (Scheme_Type ) (scheme_sequence_type ) )  {
    Scheme_Sequence * seq = (Scheme_Sequence * ) body ; 
    int i ; 
    for (i = seq -> count - 1 ; i -- ; ) {
      if (! ((Scheme_Type ) (((((intptr_t ) (seq -> array [i ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (seq -> array [i ] ) ) -> type ) ) == (Scheme_Type ) (scheme_local_type ) ) )
        break ; 
    }
    if (i < 0 )
      return seq -> array [seq -> count - 1 ] ; 
  }
  return body ; 
}

typedef int (* Ok_Value_Callback ) (void * data , Scheme_Object * v , int mode ) ; 






static int is_ok_value (Ok_Value_Callback ok_value , void * data , 
Scheme_Object * arg , 
Optimize_Info * info , 
Scheme_Hash_Table * top_level_table , 
Scheme_Object * * runstack , int rs_delta , 
Scheme_Linklet * enclosing_linklet ) 

{
  int pos ; 
  Scheme_Object * v ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(7);
  BLOCK_SETUP_TOP((PUSH(runstack, 0), PUSH(info, 1), PUSH(top_level_table, 2), PUSH(arg, 3), PUSH(data, 4), PUSH(v, 5), PUSH(enclosing_linklet, 6)));
# define XfOrM198_COUNT (7)
# define SETUP_XfOrM198(x) SETUP(XfOrM198_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  v = NULLED_OUT ; 

# 1355 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (arg ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (arg ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_toplevel_type ) )  {
#   define XfOrM206_COUNT (0+XfOrM198_COUNT)
#   define SETUP_XfOrM206(x) SETUP_XfOrM198(x)

# 1356 "./../src/optimize.c"
    if (info ) {
#     define XfOrM207_COUNT (0+XfOrM206_COUNT)
#     define SETUP_XfOrM207(x) SETUP_XfOrM206(x)

# 1358 "./../src/optimize.c"
      v = FUNCCALL(SETUP_XfOrM207(_), get_defn_shape (info , (Scheme_IR_Toplevel * ) arg ) ); 
      if (! v )
        v = FUNCCALL(SETUP_XfOrM207(_), get_import_shape (info , (Scheme_IR_Toplevel * ) arg ) ); 
      if (v )
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ok_value (data , v , 1 ) )) RET_VALUE_EMPTY_END ; 
    }
  }

# 1364 "./../src/optimize.c"
  else if (((Scheme_Type ) (((((intptr_t ) (arg ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (arg ) ) -> type ) ) == (Scheme_Type ) (scheme_toplevel_type ) ) 
  || ((Scheme_Type ) (((((intptr_t ) (arg ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (arg ) ) -> type ) ) == (Scheme_Type ) (scheme_static_toplevel_type ) ) ) {
#   define XfOrM199_COUNT (0+XfOrM198_COUNT)
#   define SETUP_XfOrM199(x) SETUP_XfOrM198(x)

# 1366 "./../src/optimize.c"
    pos = (((Scheme_Toplevel * ) (arg ) ) -> position ) ; 
    if (runstack ) {


      Scheme_Bucket * b ; 
      Scheme_Prefix * toplevels ; 
#     define XfOrM205_COUNT (0+XfOrM199_COUNT)
#     define SETUP_XfOrM205(x) SETUP_XfOrM199(x)
      b = NULLED_OUT ; 
      toplevels = NULLED_OUT ; 

# 1372 "./../src/optimize.c"
      if ((Scheme_Type ) (((((intptr_t ) (arg ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (arg ) ) -> type ) ) == (Scheme_Type ) (scheme_toplevel_type ) ) 
        toplevels = (Scheme_Prefix * ) runstack [(((Scheme_Toplevel * ) (arg ) ) -> u . depth ) - rs_delta ] ; 
      else 
      toplevels = (((Scheme_Toplevel * ) (arg ) ) -> u . prefix ) ; 
      b = (Scheme_Bucket * ) toplevels -> a [pos ] ; 
      if (b -> val && (((Scheme_Bucket_With_Flags * ) b ) -> flags & 2 ) )
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ok_value (data , b -> val , 5 ) )) RET_VALUE_EMPTY_END ; 
    }

# 1379 "./../src/optimize.c"
    else if (enclosing_linklet ) {
#     define XfOrM202_COUNT (0+XfOrM199_COUNT)
#     define SETUP_XfOrM202(x) SETUP_XfOrM199(x)
      if (pos > enclosing_linklet -> num_total_imports ) {
        Scheme_Object * name ; 
#       define XfOrM204_COUNT (0+XfOrM202_COUNT)
#       define SETUP_XfOrM204(x) SETUP_XfOrM202(x)
        name = NULLED_OUT ; 

# 1384 "./../src/optimize.c"
        pos -= (enclosing_linklet -> num_total_imports + 1 ) ; 
        name = (((Scheme_Vector * ) (enclosing_linklet -> defns ) ) -> els ) [pos ] ; 
        v = FUNCCALL(SETUP_XfOrM204(_), scheme_hash_get (enclosing_linklet -> constants , name ) ); 
        if (v )
          RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ok_value (data , v , 4 ) )) RET_VALUE_EMPTY_END ; 
      }

# 1389 "./../src/optimize.c"
      else if (pos >= 1 
      && (pos <= enclosing_linklet -> num_total_imports ) 
      && enclosing_linklet -> import_shapes ) {
#       define XfOrM203_COUNT (0+XfOrM202_COUNT)
#       define SETUP_XfOrM203(x) SETUP_XfOrM202(x)

# 1392 "./../src/optimize.c"
        pos -= 1 ; 
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ok_value (data , (((Scheme_Vector * ) (enclosing_linklet -> import_shapes ) ) -> els ) [pos ] , 2 ) )) RET_VALUE_EMPTY_END ; 
      }
    }

# 1395 "./../src/optimize.c"
    else if (top_level_table ) {
#     define XfOrM200_COUNT (0+XfOrM199_COUNT)
#     define SETUP_XfOrM200(x) SETUP_XfOrM199(x)
      v = FUNCCALL(SETUP_XfOrM200(_), scheme_hash_get (top_level_table , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (pos ) ) ) << 1 ) | 0x1 ) ) ) ); 
      if (v ) {
#       define XfOrM201_COUNT (0+XfOrM200_COUNT)
#       define SETUP_XfOrM201(x) SETUP_XfOrM200(x)

# 1400 "./../src/optimize.c"
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ok_value (data , v , 3 ) )) RET_VALUE_EMPTY_END ; 
      }
    }
  }

# 1403 "./../src/optimize.c"
  else if (((((intptr_t ) (arg ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (arg ) ) -> type ) > _scheme_ir_values_types_ )
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ok_value (data , arg , 5 ) )) RET_VALUE_EMPTY_END ; 

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1409 "./../src/optimize.c"
static int ok_constant_super_value (void * data , Scheme_Object * v , int mode ) 

{
  Scheme_Object * * _parent_identity = (Scheme_Object * * ) ((void * * ) data ) [0 ] ; 
  int * _nonfail_constr = (int * ) ((void * * ) data ) [1 ] ; 
  int * _prefab = (int * ) ((void * * ) data ) [2 ] ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(4);
  BLOCK_SETUP_TOP((PUSH(_nonfail_constr, 0), PUSH(v, 1), PUSH(_prefab, 2), PUSH(_parent_identity, 3)));
# define XfOrM208_COUNT (4)
# define SETUP_XfOrM208(x) SETUP(XfOrM208_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 1416 "./../src/optimize.c"
  if (mode == 1 ) {
    if ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) )  {
      int mode = (((Scheme_Struct_Proc_Shape * ) v ) -> mode & 0x7 ) ; 
      int field_count = (((Scheme_Struct_Proc_Shape * ) v ) -> mode >> 7 ) ; 
      if (mode == 0 ) {
        if (_parent_identity )
          * _parent_identity = ((Scheme_Struct_Proc_Shape * ) v ) -> identity ; 
        if (_nonfail_constr )
          * _nonfail_constr = ((Scheme_Struct_Proc_Shape * ) v ) -> mode & 0x20 ; 
        if (_prefab )
          * _prefab = ((Scheme_Struct_Proc_Shape * ) v ) -> mode & 0x40 ; 
        RET_VALUE_START (field_count + 1 ) RET_VALUE_END ; 
      }
    }
  }

# 1430 "./../src/optimize.c"
  else if (mode == 2 ) {
    intptr_t k ; 
#   define XfOrM218_COUNT (0+XfOrM208_COUNT)
#   define SETUP_XfOrM218(x) SETUP_XfOrM208(x)

# 1432 "./../src/optimize.c"
    if (FUNCCALL(SETUP_XfOrM218(_), scheme_decode_struct_shape (v , & k ) )) {
      if ((k & 0x7 ) == 0 ) {
        if (_nonfail_constr )
          * _nonfail_constr = k & 0x20 ; 
        if (_prefab )
          * _prefab = k & 0x40 ; 
        RET_VALUE_START ((k >> 7 ) + 1 ) RET_VALUE_END ; 
      }
    }
  }

# 1441 "./../src/optimize.c"
  else if (mode == 3 ) {
    int k = (((intptr_t ) (v ) ) >> 1 ) ; 
    if ((k >= 0 ) 
    && (k & 0x7 ) == 0 ) {
      if (_nonfail_constr )
        * _nonfail_constr = k & 0x20 ; 
      if (_prefab )
        * _prefab = k & 0x40 ; 
      RET_VALUE_START ((k >> 7 ) + 1 ) RET_VALUE_END ; 
    }
  }

# 1451 "./../src/optimize.c"
  else if (mode == 4 ) {
    if (((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_vector_type ) ) && ((((Scheme_Vector * ) (v ) ) -> size ) == 3 ) ) {
      if (_parent_identity )
        * _parent_identity = (((Scheme_Vector * ) (v ) ) -> els ) [2 ] ; 
      v = (((Scheme_Vector * ) (v ) ) -> els ) [1 ] ; 
      if (v && (((intptr_t ) (v ) ) & 0x1 ) ) {
        int mode = ((((intptr_t ) (v ) ) >> 1 ) & 0x7 ) ; 
        int field_count = ((((intptr_t ) (v ) ) >> 1 ) >> 7 ) ; 
        if (mode == 0 ) {
          if (_nonfail_constr )
            * _nonfail_constr = (((intptr_t ) (v ) ) >> 1 ) & 0x20 ; 
          if (_prefab )
            * _prefab = (((intptr_t ) (v ) ) >> 1 ) & 0x40 ; 
          RET_VALUE_START (field_count + 1 ) RET_VALUE_END ; 
        }
      }
    }
  }

# 1468 "./../src/optimize.c"
  else if (mode == 5 ) {
    if ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_type_type ) )  {
      Scheme_Struct_Type * st = (Scheme_Struct_Type * ) v ; 
      if (st -> num_slots == st -> num_islots ) {
        if (_nonfail_constr )
          * _nonfail_constr = (st -> more_flags & 0x1 ) ; 
        if (_prefab )
          * _prefab = ! ! st -> prefab_key ; 
        RET_VALUE_START (st -> num_slots + 1 ) RET_VALUE_END ; 
      }
    }
  }

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1484 "./../src/optimize.c"
static int is_constant_super (Scheme_Object * arg , 
Optimize_Info * info , 
Scheme_Hash_Table * top_level_table , 
Scheme_Object * * runstack , int rs_delta , 
Scheme_Linklet * enclosing_linklet , 
Scheme_Object * * _parent_identity , 
int * _nonfail_constr , 
int * _prefab ) 

{
  void * data [3 ] ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSHARRAY(data, 3, 0)));
# define XfOrM224_COUNT (3)
# define SETUP_XfOrM224(x) SETUP(XfOrM224_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  data[0] = NULLED_OUT ; 
  data[1] = NULLED_OUT ; 
  data[2] = NULLED_OUT ; 

# 1496 "./../src/optimize.c"
  data [0 ] = _parent_identity ; 
  data [1 ] = _nonfail_constr ; 
  data [2 ] = _prefab ; 

  RET_VALUE_START (FUNCCALL(SETUP_XfOrM224(_), is_ok_value (ok_constant_super_value , data , arg , info , top_level_table , runstack , rs_delta , enclosing_linklet ) )) RET_VALUE_END 




  ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1508 "./../src/optimize.c"
static int ok_constant_property_without_guard (void * data , Scheme_Object * v , int mode ) 
{
  intptr_t k = - 1 ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(1);
  BLOCK_SETUP_TOP((PUSH(v, 0)));
# define XfOrM225_COUNT (1)
# define SETUP_XfOrM225(x) SETUP(XfOrM225_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 1512 "./../src/optimize.c"
  if (mode == 1 ) {
    if ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_prop_proc_shape_type ) )  {
      k = ((Scheme_Small_Object * ) v ) -> u . int_val ; 
    }
  }

# 1516 "./../src/optimize.c"
  else if (mode == 2 ) {
#   define XfOrM231_COUNT (0+XfOrM225_COUNT)
#   define SETUP_XfOrM231(x) SETUP_XfOrM225(x)

# 1517 "./../src/optimize.c"
    if (! FUNCCALL(SETUP_XfOrM231(_), scheme_decode_struct_prop_shape (v , & k ) ))
      k = 0 ; 
  }

# 1519 "./../src/optimize.c"
  else if (mode == 3 ) {
    int k = (((intptr_t ) (v ) ) >> 1 ) ; 
    if (k < 0 )
      k = - (k + 1 ) ; 
    else 
    k = 0 ; 
  }

# 1525 "./../src/optimize.c"
  else if (mode == 4 ) {
    if (((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_vector_type ) ) && ((((Scheme_Vector * ) (v ) ) -> size ) == 4 ) ) {
      v = (((Scheme_Vector * ) (v ) ) -> els ) [1 ] ; 
      if (v && (((intptr_t ) (v ) ) & 0x1 ) )
        k = (((intptr_t ) (v ) ) >> 1 ) ; 
    }
  }

# 1531 "./../src/optimize.c"
  else if (mode == 5 ) {
    if ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_property_type ) )  {
      if (! ((Scheme_Struct_Property * ) v ) -> guard )
        RET_VALUE_START (1 ) RET_VALUE_END ; 
    }
  }

  RET_VALUE_START ((k == 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1541 "./../src/optimize.c"
static int is_struct_type_property_without_guard (Scheme_Object * arg , 
Optimize_Info * info , 
Scheme_Hash_Table * top_level_table , 
Scheme_Object * * runstack , int rs_delta , 
Scheme_Linklet * enclosing_linklet ) 

{
  /* No conversion */

# 1548 "./../src/optimize.c"
  return is_ok_value (ok_constant_property_without_guard , ((void * ) 0 ) , 
  arg , 
  info , 
  top_level_table , 
  runstack , rs_delta , 
  enclosing_linklet ) ; 
}

static int is_simple_property_list (Scheme_Object * a , int resolved , 
Optimize_Info * info , 
Scheme_Hash_Table * top_level_table , 
Scheme_Object * * runstack , int rs_delta , 
Scheme_Linklet * enclosing_linklet , 
int just_for_authentic , int * _authentic ) 

{
  Scheme_Object * arg ; 
  int i , count ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK(8);
  BLOCK_SETUP_TOP((PUSH(runstack, 0), PUSH(info, 1), PUSH(arg, 2), PUSH(top_level_table, 3), PUSH(a, 4), PUSH(_authentic, 5), PUSH(enclosing_linklet, 6)));
# define XfOrM235_COUNT (7)
# define SETUP_XfOrM235(x) SETUP(XfOrM235_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  arg = NULLED_OUT ; 

# 1567 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (a ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (a ) ) -> type ) ) == (Scheme_Type ) (scheme_application_type ) )  {
    if (! ((((Scheme_App_Rec * ) a ) -> args [0 ] ) == (scheme_list_proc ) ) )
      RET_VALUE_START (0 ) RET_VALUE_END ; 
    count = ((Scheme_App_Rec * ) a ) -> num_args ; 
  }

# 1571 "./../src/optimize.c"
  else if ((Scheme_Type ) (((((intptr_t ) (a ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (a ) ) -> type ) ) == (Scheme_Type ) (scheme_application2_type ) )  {
    if (! ((((Scheme_App2_Rec * ) a ) -> rator ) == (scheme_list_proc ) ) )
      RET_VALUE_START (0 ) RET_VALUE_END ; 
    count = 1 ; 
  }

# 1575 "./../src/optimize.c"
  else if ((Scheme_Type ) (((((intptr_t ) (a ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (a ) ) -> type ) ) == (Scheme_Type ) (scheme_application3_type ) )  {
    if (! ((((Scheme_App3_Rec * ) a ) -> rator ) == (scheme_list_proc ) ) )
      RET_VALUE_START (0 ) RET_VALUE_END ; 
    count = 2 ; 
  }

# 1579 "./../src/optimize.c"
  else 
  RET_VALUE_START (0 ) RET_VALUE_END ; 

  for (i = 0 ; i < count ; i ++ ) {
#   define XfOrM244_COUNT (0+XfOrM235_COUNT)
#   define SETUP_XfOrM244(x) SETUP_XfOrM235(x)

# 1583 "./../src/optimize.c"
    if ((Scheme_Type ) (((((intptr_t ) (a ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (a ) ) -> type ) ) == (Scheme_Type ) (scheme_application_type ) ) 
      arg = ((Scheme_App_Rec * ) a ) -> args [i + 1 ] ; 
    else if ((Scheme_Type ) (((((intptr_t ) (a ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (a ) ) -> type ) ) == (Scheme_Type ) (scheme_application2_type ) ) 
      arg = ((Scheme_App2_Rec * ) a ) -> rand ; 
    else {
      if (i == 0 )
        arg = ((Scheme_App3_Rec * ) a ) -> rand1 ; 
      else 
      arg = ((Scheme_App3_Rec * ) a ) -> rand2 ; 
    }

    if ((Scheme_Type ) (((((intptr_t ) (arg ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (arg ) ) -> type ) ) == (Scheme_Type ) (scheme_application3_type ) )  {
      Scheme_App3_Rec * a3 = (Scheme_App3_Rec * ) arg ; 
      BLOCK_SETUP((PUSH(a3, 0+XfOrM244_COUNT)));
#     define XfOrM246_COUNT (1+XfOrM244_COUNT)
#     define SETUP_XfOrM246(x) SETUP(XfOrM246_COUNT)

# 1597 "./../src/optimize.c"
      if (! ((a3 -> rator ) == (scheme_cons_proc ) ) ) {
        if (! just_for_authentic )
          RET_VALUE_START (0 ) RET_VALUE_END ; 
      }

# 1600 "./../src/optimize.c"
      else {
#       define XfOrM247_COUNT (0+XfOrM246_COUNT)
#       define SETUP_XfOrM247(x) SETUP_XfOrM246(x)

# 1601 "./../src/optimize.c"
        if (_authentic && ((a3 -> rand1 ) == (scheme_authentic_property ) ) )
          * _authentic = 1 ; 
        if (! just_for_authentic ) {
#         define XfOrM248_COUNT (0+XfOrM247_COUNT)
#         define SETUP_XfOrM248(x) SETUP_XfOrM247(x)

# 1604 "./../src/optimize.c"
          if (FUNCCALL(SETUP_XfOrM248(_), is_struct_type_property_without_guard (a3 -> rand1 , info , top_level_table , runstack , rs_delta , enclosing_linklet ) )) 



          {
#           define XfOrM249_COUNT (0+XfOrM248_COUNT)
#           define SETUP_XfOrM249(x) SETUP_XfOrM248(x)

# 1609 "./../src/optimize.c"
            if (! FUNCCALL(SETUP_XfOrM249(_), scheme_omittable_expr (a3 -> rand2 , 1 , 3 , (resolved ? 0x1 : 0 ) , ((void * ) 0 ) , ((void * ) 0 ) ) ))
              RET_VALUE_START (0 ) RET_VALUE_END ; 
          }

# 1611 "./../src/optimize.c"
          else 
          RET_VALUE_START (0 ) RET_VALUE_END ; 
        }
      }
    }

# 1615 "./../src/optimize.c"
    else {
      if (! just_for_authentic )
        RET_VALUE_START (0 ) RET_VALUE_END ; 
    }
  }

  RET_VALUE_START (1 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1624 "./../src/optimize.c"
Scheme_Object * scheme_is_simple_make_struct_type (Scheme_Object * e , int vals , int flags , 
GC_CAN_IGNORE int * _auto_e_depth , 
Simple_Struct_Type_Info * _stinfo , 
Scheme_Object * * _parent_identity , 
Optimize_Info * info , 
Scheme_Hash_Table * top_level_table , 
Scheme_Object * * runstack , int rs_delta , 
Scheme_Linklet * enclosing_linklet , 
Scheme_Object * * _name , 
int fuel ) 








{
  int resolved = (flags & 0x1 ) ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(12);
  BLOCK_SETUP_TOP((PUSH(runstack, 0), PUSH(info, 1), PUSH(e, 2), PUSH(_parent_identity, 3), PUSH(top_level_table, 4), PUSH(_stinfo, 5), PUSH(_name, 6), PUSH(enclosing_linklet, 7)));
# define XfOrM255_COUNT (8)
# define SETUP_XfOrM255(x) SETUP(XfOrM255_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 1645 "./../src/optimize.c"
  if (! fuel )

# 1645 "./../src/optimize.c"
  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

  if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_application_type ) )  {
#   define XfOrM269_COUNT (0+XfOrM255_COUNT)
#   define SETUP_XfOrM269(x) SETUP_XfOrM255(x)

# 1648 "./../src/optimize.c"
    if ((vals == 5 ) || (vals < 0 ) ) {
      Scheme_App_Rec * app = (Scheme_App_Rec * ) e ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM269_COUNT)));
#     define XfOrM270_COUNT (1+XfOrM269_COUNT)
#     define SETUP_XfOrM270(x) SETUP(XfOrM270_COUNT)

# 1651 "./../src/optimize.c"
      if ((app -> num_args >= 4 ) && (app -> num_args <= 11 ) 
      && ((scheme_make_struct_type_proc ) == (app -> args [0 ] ) ) ) {
        int super_count_plus_one , super_nonfail_constr = 1 , super_prefab = 1 ; 
#       define XfOrM271_COUNT (0+XfOrM270_COUNT)
#       define SETUP_XfOrM271(x) SETUP_XfOrM270(x)

# 1655 "./../src/optimize.c"
        if (_parent_identity )
          * _parent_identity = scheme_null ; 
        if (! (((app -> args [2 ] ) ) == (scheme_false ) ) )
          super_count_plus_one = FUNCCALL(SETUP_XfOrM271(_), is_constant_super (app -> args [2 ] , info , top_level_table , runstack , rs_delta + app -> num_args , enclosing_linklet , _parent_identity , & super_nonfail_constr , & super_prefab ) )




        ; 
        else 
        super_count_plus_one = 0 ; 

        if (((Scheme_Type ) (((((intptr_t ) (app -> args [1 ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> args [1 ] ) ) -> type ) ) == (Scheme_Type ) (scheme_symbol_type ) ) 
        && ((((app -> args [2 ] ) ) == (scheme_false ) ) 
        || super_count_plus_one ) 
        && (((intptr_t ) (app -> args [3 ] ) ) & 0x1 ) 
        && ((((intptr_t ) (app -> args [3 ] ) ) >> 1 ) >= 0 ) 
        && (((intptr_t ) (app -> args [4 ] ) ) & 0x1 ) 
        && ((((intptr_t ) (app -> args [4 ] ) ) >> 1 ) >= 0 ) 
        && ((app -> num_args < 5 ) 

        || (flags & 0x4 ) 
        || FUNCCALL(SETUP_XfOrM271(_), scheme_omittable_expr (app -> args [5 ] , 1 , 3 , (resolved ? 0x1 : 0 ) , ((void * ) 0 ) , ((void * ) 0 ) ) )) 
        && ((app -> num_args < 6 ) 

        || ((app -> args [6 ] ) == (scheme_null ) ) 





        || (! (flags & 0x2 ) 
        && FUNCCALL(SETUP_XfOrM271(_), scheme_omittable_expr (app -> args [6 ] , 1 , 4 , (resolved ? 0x1 : 0 ) , ((void * ) 0 ) , ((void * ) 0 ) ) )) 
        || ((flags & 0x2 ) 
        && FUNCCALL(SETUP_XfOrM271(_), is_simple_property_list (app -> args [6 ] , resolved , info , top_level_table , runstack , rs_delta , enclosing_linklet , 0 , ((void * ) 0 ) ) )) ) 





        && ((app -> num_args < 7 ) 

        || (((app -> args [7 ] ) ) == (scheme_false ) ) 
        || (super_prefab 
        && ((Scheme_Type ) (((((intptr_t ) (app -> args [7 ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> args [7 ] ) ) -> type ) ) == (Scheme_Type ) (scheme_symbol_type ) ) 
        && ! (strcmp ("prefab" , (((Scheme_Symbol * ) ((Scheme_Simple_Object * ) (app -> args [7 ] ) ) ) -> s ) ) ) 
        && ! ((& ((Scheme_Symbol * ) (app -> args [7 ] ) ) -> iso ) -> so . keyex & 0x3 ) ) 
        || FUNCCALL(SETUP_XfOrM271(_), is_inspector_call (app -> args [7 ] ) )) 
        && ((app -> num_args < 8 ) 

        || (((app -> args [8 ] ) ) == (scheme_false ) ) 
        || FUNCCALL(SETUP_XfOrM271(_), is_proc_spec_proc (app -> args [8 ] , (((intptr_t ) (app -> args [3 ] ) ) >> 1 ) ) )) 
        && ((app -> num_args < 9 ) 

        || FUNCCALL(SETUP_XfOrM271(_), is_int_list (app -> args [9 ] , (((intptr_t ) (app -> args [3 ] ) ) >> 1 ) ) )) 

        && ((app -> num_args < 10 ) 

        || (((app -> args [10 ] ) ) == (scheme_false ) ) 

        || ! (flags & 0x2 ) ) 
        && ((app -> num_args < 11 ) 

        || (((app -> args [11 ] ) ) == (scheme_false ) ) 
        || ((Scheme_Type ) (((((intptr_t ) (app -> args [11 ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> args [11 ] ) ) -> type ) ) == (Scheme_Type ) (scheme_symbol_type ) ) ) ) {
#         define XfOrM272_COUNT (0+XfOrM271_COUNT)
#         define SETUP_XfOrM272(x) SETUP_XfOrM271(x)

# 1720 "./../src/optimize.c"
          if (_auto_e_depth )
            * _auto_e_depth = (resolved ? app -> num_args : 0 ) ; 
          if (_name )
            * _name = app -> args [1 ] ; 
          if (_stinfo ) {
            int authentic = 0 ; 
            int super_count = (super_count_plus_one 
            ? (super_count_plus_one - 1 ) 
            : 0 ) ; 
#           define XfOrM273_COUNT (0+XfOrM272_COUNT)
#           define SETUP_XfOrM273(x) SETUP_XfOrM272(x)

# 1729 "./../src/optimize.c"
            _stinfo -> init_field_count = (((intptr_t ) (app -> args [3 ] ) ) >> 1 ) + super_count ; 
            _stinfo -> field_count = ((((intptr_t ) (app -> args [3 ] ) ) >> 1 ) 
            + (((intptr_t ) (app -> args [4 ] ) ) >> 1 ) 
            + super_count ) ; 
            _stinfo -> uses_super = (super_count_plus_one ? 1 : 0 ) ; 
            _stinfo -> super_field_count = (super_count_plus_one ? (super_count_plus_one - 1 ) : 0 ) ; 
            _stinfo -> normal_ops = 1 ; 
            _stinfo -> indexed_ops = 0 ; 
            _stinfo -> authentic = 0 ; 
            if ((app -> num_args > 6 ) 
            && FUNCCALL(SETUP_XfOrM273(_), is_simple_property_list (app -> args [6 ] , resolved , info , top_level_table , runstack , rs_delta , enclosing_linklet , 1 , & authentic ) ))





            _stinfo -> authentic = authentic ; 
            _stinfo -> nonfail_constructor = (super_nonfail_constr 
            && ((app -> num_args < 10 ) || (((app -> args [10 ] ) ) == (scheme_false ) ) ) ) ; 
            _stinfo -> prefab = ((app -> num_args > 7 ) 
            && ((Scheme_Type ) (((((intptr_t ) (app -> args [7 ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> args [7 ] ) ) -> type ) ) == (Scheme_Type ) (scheme_symbol_type ) ) ) ; 
            _stinfo -> num_gets = 1 ; 
            _stinfo -> num_sets = 1 ; 
          }
          RET_VALUE_START (((app -> num_args < 5 ) ? scheme_true : app -> args [5 ] ) ) RET_VALUE_END ; 
        }
      }
    }
  }

  if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_let_header_type ) )  {


    Scheme_IR_Let_Header * lh = (Scheme_IR_Let_Header * ) e ; 
#   define XfOrM263_COUNT (0+XfOrM255_COUNT)
#   define SETUP_XfOrM263(x) SETUP_XfOrM255(x)

# 1763 "./../src/optimize.c"
    if ((lh -> count == 5 ) && (lh -> num_clauses == 1 ) ) {
#     define XfOrM264_COUNT (0+XfOrM263_COUNT)
#     define SETUP_XfOrM264(x) SETUP_XfOrM263(x)

# 1764 "./../src/optimize.c"
      if ((Scheme_Type ) (((((intptr_t ) (lh -> body ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (lh -> body ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_let_value_type ) )  {
        Scheme_IR_Let_Value * lv = (Scheme_IR_Let_Value * ) lh -> body ; 
        BLOCK_SETUP((PUSH(lv, 0+XfOrM264_COUNT)));
#       define XfOrM265_COUNT (1+XfOrM264_COUNT)
#       define SETUP_XfOrM265(x) SETUP(XfOrM265_COUNT)

# 1766 "./../src/optimize.c"
        if ((Scheme_Type ) (((((intptr_t ) (lv -> value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (lv -> value ) ) -> type ) ) == (Scheme_Type ) (scheme_application_type ) )  {
          Scheme_Object * auto_e ; 
          Simple_Struct_Type_Info stinfo ; 
          BLOCK_SETUP((PUSH(auto_e, 0+XfOrM265_COUNT)));
#         define XfOrM266_COUNT (1+XfOrM265_COUNT)
#         define SETUP_XfOrM266(x) SETUP(XfOrM266_COUNT)
          auto_e = NULLED_OUT ; 

# 1769 "./../src/optimize.c"
          if (! _stinfo )

# 1769 "./../src/optimize.c"
          _stinfo = & stinfo ; 
          auto_e = FUNCCALL(SETUP_XfOrM266(_), scheme_is_simple_make_struct_type (lv -> value , 5 , flags , _auto_e_depth , _stinfo , _parent_identity , info , top_level_table , runstack , rs_delta , enclosing_linklet , _name , fuel - 1 ) )





          ; 
          if (auto_e ) {
#           define XfOrM267_COUNT (0+XfOrM266_COUNT)
#           define SETUP_XfOrM267(x) SETUP_XfOrM266(x)
            if (FUNCCALL(SETUP_XfOrM267(_), is_values_with_accessors_and_mutators (lv -> body , vals , resolved , _stinfo , lv -> vars ) )) {
              RET_VALUE_START (auto_e ) RET_VALUE_END ; 
            }
          }
        }
      }
    }
  }

  if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_let_void_type ) )  {

    Scheme_Let_Void * lvd = (Scheme_Let_Void * ) e ; 
    BLOCK_SETUP((PUSH(lvd, 0+XfOrM255_COUNT)));
#   define XfOrM256_COUNT (1+XfOrM255_COUNT)
#   define SETUP_XfOrM256(x) SETUP(XfOrM256_COUNT)

# 1792 "./../src/optimize.c"
    if (lvd -> count == 5 ) {
#     define XfOrM257_COUNT (0+XfOrM256_COUNT)
#     define SETUP_XfOrM257(x) SETUP_XfOrM256(x)

# 1793 "./../src/optimize.c"
      if ((Scheme_Type ) (((((intptr_t ) (lvd -> body ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (lvd -> body ) ) -> type ) ) == (Scheme_Type ) (scheme_let_value_type ) )  {
        Scheme_Let_Value * lv = (Scheme_Let_Value * ) lvd -> body ; 
        BLOCK_SETUP((PUSH(lv, 0+XfOrM257_COUNT)));
#       define XfOrM258_COUNT (1+XfOrM257_COUNT)
#       define SETUP_XfOrM258(x) SETUP(XfOrM258_COUNT)

# 1795 "./../src/optimize.c"
        if ((lv -> position == 0 ) && (lv -> count == 5 ) ) {
          Scheme_Object * e2 ; 
          BLOCK_SETUP((PUSH(e2, 0+XfOrM258_COUNT)));
#         define XfOrM259_COUNT (1+XfOrM258_COUNT)
#         define SETUP_XfOrM259(x) SETUP(XfOrM259_COUNT)
          e2 = NULLED_OUT ; 

# 1797 "./../src/optimize.c"
          e2 = FUNCCALL(SETUP_XfOrM259(_), skip_clears (lv -> value ) ); 
          if ((Scheme_Type ) (((((intptr_t ) (e2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e2 ) ) -> type ) ) == (Scheme_Type ) (scheme_application_type ) )  {
            Scheme_Object * auto_e ; 
            Simple_Struct_Type_Info stinfo ; 
            BLOCK_SETUP((PUSH(auto_e, 0+XfOrM259_COUNT)));
#           define XfOrM260_COUNT (1+XfOrM259_COUNT)
#           define SETUP_XfOrM260(x) SETUP(XfOrM260_COUNT)
            auto_e = NULLED_OUT ; 

# 1801 "./../src/optimize.c"
            if (! _stinfo )

# 1801 "./../src/optimize.c"
            _stinfo = & stinfo ; 
            auto_e = FUNCCALL(SETUP_XfOrM260(_), scheme_is_simple_make_struct_type (e2 , 5 , flags , _auto_e_depth , _stinfo , _parent_identity , info , top_level_table , runstack , rs_delta + lvd -> count , enclosing_linklet , _name , fuel - 1 ) )





            ; 
            if (auto_e ) {
#             define XfOrM261_COUNT (0+XfOrM260_COUNT)
#             define SETUP_XfOrM261(x) SETUP_XfOrM260(x)
              e2 = FUNCCALL(SETUP_XfOrM261(_), skip_clears (lv -> body ) ); 
              if (FUNCCALL(SETUP_XfOrM261(_), is_values_with_accessors_and_mutators (e2 , vals , resolved , _stinfo , ((void * ) 0 ) ) )) {
                if (_auto_e_depth )

# 1814 "./../src/optimize.c"
                * _auto_e_depth += lvd -> count ; 
                RET_VALUE_START (auto_e ) RET_VALUE_END ; 
              }
            }
          }
        }
      }
    }
  }

  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1827 "./../src/optimize.c"
int scheme_is_simple_make_struct_type_property (Scheme_Object * e , int vals , int flags , 
int * _has_guard , 
Optimize_Info * info , 
Scheme_Hash_Table * top_level_table , 
Scheme_Object * * runstack , int rs_delta , 
Scheme_Linklet * enclosing_linklet , 
int fuel ) 



{
  int resolved = (flags & 0x1 ) ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(1);
  BLOCK_SETUP_TOP((PUSH(_has_guard, 0)));
# define XfOrM274_COUNT (1)
# define SETUP_XfOrM274(x) SETUP(XfOrM274_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 1840 "./../src/optimize.c"
  if ((vals != 3 ) && (vals >= 0 ) )

# 1840 "./../src/optimize.c"
  RET_VALUE_START (0 ) RET_VALUE_END ; 

  if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_application2_type ) )  {
    Scheme_App2_Rec * app = (Scheme_App2_Rec * ) e ; 
    if ((app -> rator ) == (scheme_make_struct_type_property_proc ) )  {
      if ((Scheme_Type ) (((((intptr_t ) (app -> rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand ) ) -> type ) ) == (Scheme_Type ) (scheme_symbol_type ) )  {
        if (_has_guard )

# 1846 "./../src/optimize.c"
        * _has_guard = 0 ; 
        RET_VALUE_START (1 ) RET_VALUE_END ; 
      }
    }
  }

  if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_application3_type ) )  {
    Scheme_App3_Rec * app = (Scheme_App3_Rec * ) e ; 
#   define XfOrM275_COUNT (0+XfOrM274_COUNT)
#   define SETUP_XfOrM275(x) SETUP_XfOrM274(x)

# 1854 "./../src/optimize.c"
    if ((app -> rator ) == (scheme_make_struct_type_property_proc ) )  {
#     define XfOrM276_COUNT (0+XfOrM275_COUNT)
#     define SETUP_XfOrM276(x) SETUP_XfOrM275(x)

# 1855 "./../src/optimize.c"
      if (((Scheme_Type ) (((((intptr_t ) (app -> rand1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand1 ) ) -> type ) ) == (Scheme_Type ) (scheme_symbol_type ) ) 
      && (! (flags & 0x2 ) 
      || (((app -> rand2 ) ) == (scheme_false ) ) 
      || ((((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) 
      && (((Scheme_Lambda * ) app -> rand2 ) -> num_params == 2 ) ) ) 
      && (FUNCCALL(SETUP_XfOrM276(_), scheme_omittable_expr (app -> rator , 1 , 4 , (resolved ? 0x1 : 0 ) , ((void * ) 0 ) , ((void * ) 0 ) ) )) ) {
        if (_has_guard )

# 1861 "./../src/optimize.c"
        * _has_guard = 1 ; 
        RET_VALUE_START (1 ) RET_VALUE_END ; 
      }
    }
  }

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

intptr_t scheme_get_struct_proc_shape (int k , Simple_Struct_Type_Info * stinfo ) 
{
  /* No conversion */

# 1876 "./../src/optimize.c"
  switch (k ) {
    case 0 : 
    if (stinfo -> field_count == stinfo -> init_field_count )
      return (0 
    | (stinfo -> authentic ? 0x10 : 0 ) 
    | (stinfo -> nonfail_constructor ? 0x20 : 0 ) 
    | (stinfo -> prefab ? 0x40 : 0 ) 
    | (stinfo -> field_count << 7 ) ) ; 
    else 
    return 5 ; 
    break ; 
    case 1 : 
    return (1 
    | (stinfo -> init_field_count << 7 ) 
    | (stinfo -> nonfail_constructor ? 0x20 : 0 ) ) ; 
    break ; 
    case 2 : 
    return (2 
    | (stinfo -> authentic ? 0x10 : 0 ) ) ; 
    break ; 
    default : 
    if (stinfo && stinfo -> normal_ops && stinfo -> indexed_ops ) {
      if (k - 3 < stinfo -> num_gets ) {

        return (3 
        | (stinfo -> authentic ? 0x10 : 0 ) 
        | ((stinfo -> super_field_count + (k - 3 ) ) << 7 ) ) ; 
      }

# 1903 "./../src/optimize.c"
      else {
        int idx = (k - 3 - stinfo -> num_gets ) , setter_fields = stinfo -> setter_fields , pos = 0 ; 


        while ((idx > 0 ) || ! (setter_fields & 1 ) ) {
          if (setter_fields & 1 ) {
            idx -- ; 
          }
          setter_fields = setter_fields >> 1 ; 
          pos ++ ; 
          if (! setter_fields )

# 1913 "./../src/optimize.c"
          break ; 
        }

        if (! idx && (setter_fields & 1 ) )
          pos += stinfo -> super_field_count + 1 ; 
        else {

          pos = 0 ; 
        }

        return (4 
        | (stinfo -> authentic ? 0x10 : 0 ) 
        | (pos << 7 ) ) ; 
      }
    }
  }

  return 5 ; 
}

Scheme_Object * scheme_make_struct_proc_shape (intptr_t k , Scheme_Object * identity ) 
{
  Scheme_Object * ps ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(identity, 0), PUSH(ps, 1)));
# define XfOrM291_COUNT (2)
# define SETUP_XfOrM291(x) SETUP(XfOrM291_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  ps = NULLED_OUT ; 

# 1937 "./../src/optimize.c"
  ps = FUNCCALL(SETUP_XfOrM291(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_Simple_Object ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) ); 
  ps -> type = scheme_struct_proc_shape_type ; 
  ((Scheme_Struct_Proc_Shape * ) ps ) -> mode = k ; 
  ((Scheme_Struct_Proc_Shape * ) ps ) -> identity = identity ; 

  RET_VALUE_START (ps ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1945 "./../src/optimize.c"
intptr_t scheme_get_struct_property_proc_shape (int k , int has_guard ) 
{
  /* No conversion */

# 1947 "./../src/optimize.c"
  switch (k ) {
    case 0 : 
    if (has_guard )
      return 1 ; 
    else 
    return 0 ; 
    case 1 : 
    return 2 ; 
    case 2 : 
    default : 
    return 3 ; 
  }
}

Scheme_Object * scheme_make_struct_property_proc_shape (intptr_t k ) 
{
  Scheme_Object * ps ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(1);
  BLOCK_SETUP_TOP((PUSH(ps, 0)));
# define XfOrM294_COUNT (1)
# define SETUP_XfOrM294(x) SETUP(XfOrM294_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  ps = NULLED_OUT ; 

# 1965 "./../src/optimize.c"
  ps = ((Scheme_Object * ) FUNCCALL(SETUP_XfOrM294(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_Small_Object ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
  ps -> type = scheme_struct_prop_proc_shape_type ; 
  ((Scheme_Small_Object * ) ps ) -> u . int_val = k ; 

  RET_VALUE_START (ps ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 1972 "./../src/optimize.c"
__xform_nongcing__ static int is_struct_identity_subtype (Scheme_Object * sub , Scheme_Object * sup ) 
{
  /* No conversion */


  while (((Scheme_Type ) (((((intptr_t ) (sub ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (sub ) ) -> type ) ) == (Scheme_Type ) (scheme_pair_type ) ) ) {
    if ((sub ) == (sup ) ) 
      return 1 ; 
    sub = (((Scheme_Simple_Object * ) (sub ) ) -> u . pair_val . cdr ) ; 
  }
  return 0 ; 
}

static int single_valued_noncm_function (Scheme_Object * rator , int num_args , 
Optimize_Info * info , int s_v , int non_cm ) 
{
  /* No conversion */

# 1988 "./../src/optimize.c"
  int flags ; 

  if (! s_v && ! non_cm )
    return 1 ; 

  flags = get_rator_flags (rator , num_args , info ) ; 
  if (s_v && ! (flags & 32 ) )
    return 0 ; 
  if (non_cm && ! (flags & 4 ) )
    return 0 ; 

  return 1 ; 
}

static int do_single_valued_noncm_expression (Scheme_Object * expr , Optimize_Info * info , int fuel , int s_v , int non_cm ) 





{
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK(3);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(expr, 1)));
# define XfOrM299_COUNT (2)
# define SETUP_XfOrM299(x) SETUP(XfOrM299_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 2009 "./../src/optimize.c"
  if (! s_v && ! non_cm )
    RET_VALUE_START (1 ) RET_VALUE_END ; 

  while (fuel ) {
#   define XfOrM313_COUNT (0+XfOrM299_COUNT)
#   define SETUP_XfOrM313(x) SETUP_XfOrM299(x)

# 2013 "./../src/optimize.c"
    switch (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ) {
#     define XfOrM314_COUNT (0+XfOrM313_COUNT)
#     define SETUP_XfOrM314(x) SETUP_XfOrM313(x)

# 2014 "./../src/optimize.c"
      case scheme_ir_local_type : 
      case scheme_local_type : 
      case scheme_local_unbox_type : 
      case scheme_ir_toplevel_type : 
      RET_VALUE_START (1 ) RET_VALUE_END ; 
      break ; 
      case scheme_application_type : 
      {
        Scheme_App_Rec * app = (Scheme_App_Rec * ) expr ; 
#       define XfOrM325_COUNT (0+XfOrM314_COUNT)
#       define SETUP_XfOrM325(x) SETUP_XfOrM314(x)

# 2023 "./../src/optimize.c"
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(single_valued_noncm_function (app -> args [0 ] , app -> num_args , info , s_v , non_cm ) )) RET_VALUE_EMPTY_END ; 
      }
      break ; 
      case scheme_application2_type : 
      {
        Scheme_App2_Rec * app = (Scheme_App2_Rec * ) expr ; 
#       define XfOrM324_COUNT (0+XfOrM314_COUNT)
#       define SETUP_XfOrM324(x) SETUP_XfOrM314(x)

# 2029 "./../src/optimize.c"
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(single_valued_noncm_function (app -> rator , 1 , info , s_v , non_cm ) )) RET_VALUE_EMPTY_END ; 
      }
      break ; 
      case scheme_application3_type : 
      {
        Scheme_App3_Rec * app = (Scheme_App3_Rec * ) expr ; 
#       define XfOrM323_COUNT (0+XfOrM314_COUNT)
#       define SETUP_XfOrM323(x) SETUP_XfOrM314(x)

# 2035 "./../src/optimize.c"
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(single_valued_noncm_function (app -> rator , 2 , info , s_v , non_cm ) )) RET_VALUE_EMPTY_END ; 
      }
      break ; 
      case scheme_branch_type : 
      {
        Scheme_Branch_Rec * b = (Scheme_Branch_Rec * ) expr ; 
        BLOCK_SETUP((PUSH(b, 0+XfOrM314_COUNT)));
#       define XfOrM322_COUNT (1+XfOrM314_COUNT)
#       define SETUP_XfOrM322(x) SETUP(XfOrM322_COUNT)

# 2041 "./../src/optimize.c"
        RET_VALUE_START ((FUNCCALL(SETUP_XfOrM322(_), do_single_valued_noncm_expression (b -> tbranch , info , fuel - 1 , s_v , non_cm ) )
        && FUNCCALL(SETUP_XfOrM322(_), do_single_valued_noncm_expression (b -> fbranch , info , fuel - 1 , s_v , non_cm ) )) ) 
# 2041 "./../src/optimize.c"
        RET_VALUE_END 
        ; 
      }
      break ; 
      case scheme_ir_let_header_type : 
      {
        Scheme_IR_Let_Header * hl = (Scheme_IR_Let_Header * ) expr ; 
        expr = hl -> body ; 
      }
      break ; 
      case scheme_ir_let_value_type : 
      {
        Scheme_IR_Let_Value * lv = (Scheme_IR_Let_Value * ) expr ; 
        expr = lv -> body ; 
      }
      break ; 
      case scheme_sequence_type : 
      {
        Scheme_Sequence * seq = (Scheme_Sequence * ) expr ; 
        expr = seq -> array [seq -> count - 1 ] ; 
      }
      break ; 
      case scheme_begin0_sequence_type : 
      {
        Scheme_Sequence * seq = (Scheme_Sequence * ) expr ; 
        expr = seq -> array [0 ] ; 
      }
      break ; 
      case scheme_with_cont_mark_type : 
      {
        Scheme_With_Continuation_Mark * wcm = (Scheme_With_Continuation_Mark * ) expr ; 
#       define XfOrM315_COUNT (0+XfOrM314_COUNT)
#       define SETUP_XfOrM315(x) SETUP_XfOrM314(x)

# 2072 "./../src/optimize.c"
        if (non_cm ) {
#         define XfOrM317_COUNT (0+XfOrM315_COUNT)
#         define SETUP_XfOrM317(x) SETUP_XfOrM315(x)
          RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(scheme_omittable_expr (wcm -> body , s_v ? 1 : - 1 , 5 , 0 , ((void * ) 0 ) , ((void * ) 0 ) ) )) RET_VALUE_EMPTY_END ; 
        }

# 2076 "./../src/optimize.c"
        else {
          expr = wcm -> body ; 
        }
      }
      break ; 
      case scheme_ir_lambda_type : 
      case scheme_case_lambda_sequence_type : 
      case scheme_set_bang_type : 
      RET_VALUE_START (1 ) RET_VALUE_END ; 
      break ; 
      default : 
      if (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) > _scheme_ir_values_types_ )
        RET_VALUE_START (1 ) RET_VALUE_END ; 
      break ; 
    }
    fuel -- ; 
  }

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 2097 "./../src/optimize.c"
static int single_valued_noncm_expression (Scheme_Object * expr , Optimize_Info * info , int fuel ) 
{
  /* No conversion */

# 2099 "./../src/optimize.c"
  return do_single_valued_noncm_expression (expr , info , fuel , 1 , 1 ) ; 
}

static int single_valued_expression (Scheme_Object * expr , Optimize_Info * info , int fuel ) 
{
  /* No conversion */

# 2104 "./../src/optimize.c"
  return do_single_valued_noncm_expression (expr , info , fuel , 1 , 0 ) ; 
}

static int noncm_expression (Scheme_Object * expr , Optimize_Info * info , int fuel ) 
{
  /* No conversion */

# 2109 "./../src/optimize.c"
  return do_single_valued_noncm_expression (expr , info , fuel , 0 , 1 ) ; 
}

static int is_movable_prim (Scheme_Object * rator , int n , int cross_lambda , int cross_k , Optimize_Info * info ) 



{
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(1);
  BLOCK_SETUP_TOP((PUSH(rator, 0)));
# define XfOrM329_COUNT (1)
# define SETUP_XfOrM329(x) SETUP(XfOrM329_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 2117 "./../src/optimize.c"
  if (rator && ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) ) {
#   define XfOrM330_COUNT (0+XfOrM329_COUNT)
#   define SETUP_XfOrM330(x) SETUP_XfOrM329(x)

# 2118 "./../src/optimize.c"
    if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 8 ) ) {
#     define XfOrM331_COUNT (0+XfOrM330_COUNT)
#     define SETUP_XfOrM331(x) SETUP_XfOrM330(x)

      if (cross_lambda )

# 2122 "./../src/optimize.c"
      RET_VALUE_START (0 ) RET_VALUE_END ; 
      if (cross_k 
      && ! (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 15 ) ) 
      && (FUNCCALL(SETUP_XfOrM331(_), produces_local_type (rator , n ) )!= 2 ) ) {
        RET_VALUE_START (0 ) RET_VALUE_END ; 
      }
      RET_VALUE_START (- 1 ) RET_VALUE_END ; 
    }
  }

  if ((scheme_void_proc ) == (rator ) ) 
    RET_VALUE_START (- 1 ) RET_VALUE_END ; 

  if (! cross_lambda 
  && ! cross_k 


  && (((scheme_list_proc ) == (rator ) ) 
  || (((scheme_cons_proc ) == (rator ) ) && (n == 2 ) ) 
  || (((scheme_mcons_proc ) == (rator ) ) && (n == 2 ) ) 
  || (((scheme_unsafe_cons_list_proc ) == (rator ) ) && (n == 2 ) ) 
  || ((scheme_list_star_proc ) == (rator ) ) 
  || ((scheme_vector_proc ) == (rator ) ) 
  || ((scheme_vector_immutable_proc ) == (rator ) ) 
  || (((scheme_box_proc ) == (rator ) ) && (n == 1 ) ) 
  || (((scheme_box_immutable_proc ) == (rator ) ) && (n == 1 ) ) ) )
    RET_VALUE_START (1 ) RET_VALUE_END ; 

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 2153 "./../src/optimize.c"
static int movable_expression (Scheme_Object * expr , Optimize_Info * info , 
int cross_lambda , int cross_k , int cross_s , 
int check_space , int fuel ) 




{
  int can_move ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK(3);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(expr, 1)));
# define XfOrM333_COUNT (2)
# define SETUP_XfOrM333(x) SETUP(XfOrM333_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 2163 "./../src/optimize.c"
  if (fuel < 0 )

# 2163 "./../src/optimize.c"
  RET_VALUE_START (0 ) RET_VALUE_END ; 

  switch (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ) {
#   define XfOrM334_COUNT (0+XfOrM333_COUNT)
#   define SETUP_XfOrM334(x) SETUP_XfOrM333(x)

# 2166 "./../src/optimize.c"
    case scheme_toplevel_type : 
    case scheme_static_toplevel_type : 
    RET_VALUE_START ((((& ((Scheme_Toplevel * ) (expr ) ) -> iso ) -> so . keyex & 0x3 ) >= 2 ) ) RET_VALUE_END ; 
    case scheme_ir_local_type : 
    {

      if (! ((Scheme_IR_Local * ) expr ) -> mutated ) {
        if (check_space ) {
          if (((Scheme_IR_Local * ) expr ) -> val_type )
            RET_VALUE_START (1 ) RET_VALUE_END ; 


          RET_VALUE_START (0 ) RET_VALUE_END ; 
        }
        RET_VALUE_START (1 ) RET_VALUE_END ; 
      }
    }
    break ; 
    case scheme_application_type : 
    if (! cross_lambda 
    && ! cross_k 
    && ((& ((Scheme_App_Rec * ) expr ) -> iso ) -> so . keyex & (1 << 11 ) ) )
      can_move = - 1 ; 
    else 
    can_move = FUNCCALL(SETUP_XfOrM334(_), is_movable_prim (((Scheme_App_Rec * ) expr ) -> args [0 ] , ((Scheme_App_Rec * ) expr ) -> num_args , cross_lambda , cross_k , info ) )
    ; 
    if (can_move ) {
      int i ; 
#     define XfOrM338_COUNT (0+XfOrM334_COUNT)
#     define SETUP_XfOrM338(x) SETUP_XfOrM334(x)

# 2194 "./../src/optimize.c"
      for (i = ((Scheme_App_Rec * ) expr ) -> num_args ; i -- ; ) {
#       define XfOrM340_COUNT (0+XfOrM338_COUNT)
#       define SETUP_XfOrM340(x) SETUP_XfOrM338(x)

# 2195 "./../src/optimize.c"
        if (! FUNCCALL(SETUP_XfOrM340(_), movable_expression (((Scheme_App_Rec * ) expr ) -> args [i + 1 ] , info , cross_lambda , cross_k , cross_s , check_space || (cross_s && (can_move < 0 ) ) , fuel - 1 ) ))


        RET_VALUE_START (0 ) RET_VALUE_END ; 
      }
      RET_VALUE_START (1 ) RET_VALUE_END ; 
    }
    break ; 
    case scheme_application2_type : 
    if (! cross_lambda 
    && ! cross_k 
    && ((& ((Scheme_App2_Rec * ) expr ) -> iso ) -> so . keyex & (1 << 11 ) ) )
      can_move = - 1 ; 
    else 
    can_move = FUNCCALL(SETUP_XfOrM334(_), is_movable_prim (((Scheme_App2_Rec * ) expr ) -> rator , 1 , cross_lambda , cross_k , info ) ); 
    if (can_move ) {
#     define XfOrM337_COUNT (0+XfOrM334_COUNT)
#     define SETUP_XfOrM337(x) SETUP_XfOrM334(x)

# 2211 "./../src/optimize.c"
      if (FUNCCALL(SETUP_XfOrM337(_), movable_expression (((Scheme_App2_Rec * ) expr ) -> rand , info , cross_lambda , cross_k , cross_s , check_space || (cross_s && (can_move < 0 ) ) , fuel - 1 ) ))


      RET_VALUE_START (1 ) RET_VALUE_END ; 
    }
    break ; 
    case scheme_application3_type : 
    if (! cross_lambda 
    && ! cross_k 
    && ((& ((Scheme_App3_Rec * ) expr ) -> iso ) -> so . keyex & (1 << 11 ) ) )
      can_move = - 1 ; 
    else 
    can_move = FUNCCALL(SETUP_XfOrM334(_), is_movable_prim (((Scheme_App3_Rec * ) expr ) -> rator , 2 , cross_lambda , cross_k , info ) ); 
    if (can_move ) {
#     define XfOrM336_COUNT (0+XfOrM334_COUNT)
#     define SETUP_XfOrM336(x) SETUP_XfOrM334(x)

# 2225 "./../src/optimize.c"
      if (FUNCCALL(SETUP_XfOrM336(_), movable_expression (((Scheme_App3_Rec * ) expr ) -> rand1 , info , cross_lambda , cross_k , cross_s , check_space || (cross_s && (can_move < 0 ) ) , fuel - 1 ) )


      && FUNCCALL(SETUP_XfOrM336(_), movable_expression (((Scheme_App3_Rec * ) expr ) -> rand2 , info , cross_lambda , cross_k , cross_s , check_space || (cross_s && (can_move < 0 ) ) , fuel - 1 ) ))


      RET_VALUE_START (1 ) RET_VALUE_END ; 
    }
    break ; 
    case scheme_branch_type : 
    {
      Scheme_Branch_Rec * b = (Scheme_Branch_Rec * ) expr ; 
      BLOCK_SETUP((PUSH(b, 0+XfOrM334_COUNT)));
#     define XfOrM335_COUNT (1+XfOrM334_COUNT)
#     define SETUP_XfOrM335(x) SETUP(XfOrM335_COUNT)

# 2237 "./../src/optimize.c"
      if (FUNCCALL(SETUP_XfOrM335(_), movable_expression (b -> test , info , cross_lambda , cross_k , cross_s , check_space , fuel - 1 ) )

      && FUNCCALL(SETUP_XfOrM335(_), movable_expression (b -> tbranch , info , cross_lambda , cross_k , cross_s , check_space || cross_s , fuel - 1 ) )
      && FUNCCALL(SETUP_XfOrM335(_), movable_expression (b -> fbranch , info , cross_lambda , cross_k , cross_s , check_space || cross_s , fuel - 1 ) ))
        RET_VALUE_START (1 ) RET_VALUE_END ; 
    }
    break ; 
    case scheme_ir_lambda_type : 
    case scheme_case_lambda_sequence_type : 



    RET_VALUE_START (! cross_lambda && ! cross_k ) RET_VALUE_END ; 
    default : 
    if (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) > _scheme_ir_values_types_ )
      RET_VALUE_START (1 ) RET_VALUE_END ; 
  }

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 2258 "./../src/optimize.c"
int scheme_is_ir_lambda (Scheme_Object * o , int can_be_closed , int can_be_liftable ) 
{
  /* No conversion */

# 2260 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (o ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) )  {
    if (! can_be_closed || ! can_be_liftable ) {
      Scheme_Lambda * lam ; 
      lam = (Scheme_Lambda * ) o ; 

      if (! can_be_closed && ! lam -> closure_size )
        return 0 ; 

      if (! can_be_liftable && (lam -> closure_size == 1 ) && lambda_has_top_level (lam ) )
        return 0 ; 
    }
    return 1 ; 
  }

# 2272 "./../src/optimize.c"
  else 
  return 0 ; 
}

__xform_nongcing__ static int small_inline_number (Scheme_Object * o ) 
{
  /* No conversion */

# 2278 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (o ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o ) ) -> type ) ) == (Scheme_Type ) (scheme_bignum_type ) ) 
    return (((Scheme_Bignum * ) o ) -> len ) < 32 ; 
  else if (! (((intptr_t ) (o ) ) & 0x1 ) && ((((o ) -> type ) == scheme_complex_type ) ) ) 
    return (small_inline_number (scheme_complex_real_part (o ) ) 
  && small_inline_number (scheme_complex_imaginary_part (o ) ) ) ; 
  else if ((Scheme_Type ) (((((intptr_t ) (o ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o ) ) -> type ) ) == (Scheme_Type ) (scheme_rational_type ) ) 
    return (small_inline_number (scheme_rational_numerator (o ) ) 
  && small_inline_number (scheme_rational_denominator (o ) ) ) ; 
  else 
  return 1 ; 
}



int scheme_ir_duplicate_ok (Scheme_Object * fb , int cross_linklet ) 

{
  /* No conversion */

# 2295 "./../src/optimize.c"
  return ((((fb ) ) == (scheme_void ) ) 
  || ((fb ) == (scheme_true ) ) 
  || ((fb ) == (scheme_undefined ) ) 
  || (((fb ) ) == (scheme_false ) ) 
  || (((Scheme_Type ) (((((intptr_t ) (fb ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (fb ) ) -> type ) ) == (Scheme_Type ) (scheme_symbol_type ) ) 
  && (! cross_linklet || (! ((& ((Scheme_Symbol * ) (fb ) ) -> iso ) -> so . keyex & 0x3 ) 
  && ((((Scheme_Symbol * ) ((Scheme_Simple_Object * ) (fb ) ) ) -> len ) < 256 ) ) ) ) 
  || (((Scheme_Type ) (((((intptr_t ) (fb ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (fb ) ) -> type ) ) == (Scheme_Type ) (scheme_keyword_type ) ) 
  && (! cross_linklet || ((((Scheme_Symbol * ) ((Scheme_Simple_Object * ) (fb ) ) ) -> len ) < 256 ) ) ) 
  || (((fb ) ) == (scheme_eof ) ) 
  || (((intptr_t ) (fb ) ) & 0x1 ) 
  || ((fb ) == (scheme_null ) ) 
  || (((((((intptr_t ) (fb ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (fb ) ) -> type ) >= scheme_hash_tree_type ) && (((((intptr_t ) (fb ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (fb ) ) -> type ) <= scheme_hash_tree_indirection_type ) ) && ! ((Scheme_Hash_Tree * ) fb ) -> count ) 
  || (! cross_linklet && ((Scheme_Type ) (((((intptr_t ) (fb ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (fb ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_toplevel_type ) ) ) 
  || (! cross_linklet && ((Scheme_Type ) (((((intptr_t ) (fb ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (fb ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) ) 
  || ((Scheme_Type ) (((((intptr_t ) (fb ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (fb ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 


  || ((Scheme_Type ) (((((intptr_t ) (fb ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (fb ) ) -> type ) ) == (Scheme_Type ) (scheme_char_type ) ) 
  || (((Scheme_Type ) (((((intptr_t ) (fb ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (fb ) ) -> type ) ) == (Scheme_Type ) (scheme_char_string_type ) ) 
  && (! cross_linklet || ((((Scheme_Simple_Object * ) (fb ) ) -> u . char_str_val . tag_val ) < 256 ) ) ) 
  || (((Scheme_Type ) (((((intptr_t ) (fb ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (fb ) ) -> type ) ) == (Scheme_Type ) (scheme_byte_string_type ) ) 
  && (! cross_linklet || ((((Scheme_Simple_Object * ) (fb ) ) -> u . byte_str_val . tag_val ) < 256 ) ) ) 
  || ((Scheme_Type ) (((((intptr_t ) (fb ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (fb ) ) -> type ) ) == (Scheme_Type ) (scheme_regexp_type ) ) 
  || (((((intptr_t ) (fb ) ) & 0x1 ) || ((((fb ) -> type ) >= scheme_bignum_type ) && (((fb ) -> type ) <= scheme_complex_type ) ) ) 
  && (! cross_linklet || small_inline_number (fb ) ) ) 
  || ((Scheme_Type ) (((((intptr_t ) (fb ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (fb ) ) -> type ) ) == (Scheme_Type ) (scheme_ctype_type ) ) ) ; 
}





static Scheme_Object * finish_optimize_application (Scheme_App_Rec * app , Optimize_Info * info , int context ) ; 
static Scheme_Object * finish_optimize_application2 (Scheme_App2_Rec * app , Optimize_Info * info , int context ) ; 
static Scheme_Object * finish_optimize_application3 (Scheme_App3_Rec * app , Optimize_Info * info , int context ) ; 

static Scheme_Object * try_optimize_fold (Scheme_Object * f , Scheme_Object * args , Scheme_Object * o , Optimize_Info * info ) 

{
  Scheme_Object * __funcarg77 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(5);
  BLOCK_SETUP_TOP((PUSH(o, 0), PUSH(info, 1), PUSH(args, 2), PUSH(f, 3)));
# define XfOrM349_COUNT (4)
# define SETUP_XfOrM349(x) SETUP(XfOrM349_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 2335 "./../src/optimize.c"
  if (FUNCCALL(SETUP_XfOrM349(_), scheme_is_foldable_prim (f ) )) {
#   define XfOrM350_COUNT (0+XfOrM349_COUNT)
#   define SETUP_XfOrM350(x) SETUP_XfOrM349(x)

# 2337 "./../src/optimize.c"
    if (! args ) {
#     define XfOrM351_COUNT (0+XfOrM350_COUNT)
#     define SETUP_XfOrM351(x) SETUP_XfOrM350(x)

# 2338 "./../src/optimize.c"
      switch (((((intptr_t ) (o ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o ) ) -> type ) ) {
#       define XfOrM352_COUNT (0+XfOrM351_COUNT)
#       define SETUP_XfOrM352(x) SETUP_XfOrM351(x)

# 2339 "./../src/optimize.c"
        case scheme_application_type : 
        {
          Scheme_App_Rec * app = (Scheme_App_Rec * ) o ; 
          int i ; 
          BLOCK_SETUP((PUSH(app, 0+XfOrM352_COUNT)));
#         define XfOrM355_COUNT (1+XfOrM352_COUNT)
#         define SETUP_XfOrM355(x) SETUP(XfOrM355_COUNT)

# 2344 "./../src/optimize.c"
          args = scheme_null ; 
          for (i = app -> num_args ; i -- ; ) {
#           define XfOrM357_COUNT (0+XfOrM355_COUNT)
#           define SETUP_XfOrM357(x) SETUP_XfOrM355(x)

# 2346 "./../src/optimize.c"
            args = FUNCCALL(SETUP_XfOrM357(_), scheme_make_pair (app -> args [i + 1 ] , args ) ); 
          }
        }
        break ; 
        case scheme_application2_type : 
        {
          Scheme_App2_Rec * app = (Scheme_App2_Rec * ) o ; 
#         define XfOrM354_COUNT (0+XfOrM352_COUNT)
#         define SETUP_XfOrM354(x) SETUP_XfOrM352(x)

# 2353 "./../src/optimize.c"
          args = FUNCCALL(SETUP_XfOrM354(_), scheme_make_pair (app -> rand , scheme_null ) ); 
        }
        break ; 
        case scheme_application3_type : 
        default : 
        {
          Scheme_App3_Rec * app = (Scheme_App3_Rec * ) o ; 
          BLOCK_SETUP((PUSH(app, 0+XfOrM352_COUNT)));
#         define XfOrM353_COUNT (1+XfOrM352_COUNT)
#         define SETUP_XfOrM353(x) SETUP(XfOrM353_COUNT)

# 2360 "./../src/optimize.c"
          args = (__funcarg77 = FUNCCALL(SETUP_XfOrM353(_), scheme_make_pair (app -> rand2 , scheme_null ) ), FUNCCALL_AGAIN(scheme_make_pair (app -> rand1 , __funcarg77 ) )) 

          ; 
        }
        break ; 
      }
    }

    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(scheme_try_apply (f , args , info ) )) RET_VALUE_EMPTY_END ; 
  }

  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 2374 "./../src/optimize.c"
static int estimate_expr_size (Scheme_Object * expr , int sz , int fuel ) 
{
  Scheme_Type t ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK(4);
  BLOCK_SETUP_TOP((PUSH(expr, 0)));
# define XfOrM358_COUNT (1)
# define SETUP_XfOrM358(x) SETUP(XfOrM358_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 2378 "./../src/optimize.c"
  if (sz > 128 )
    RET_VALUE_START (sz ) RET_VALUE_END ; 
  if (fuel < 0 )
    RET_VALUE_START (sz + 128 ) RET_VALUE_END ; 

  t = ((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ; 

  switch (t ) {
#   define XfOrM359_COUNT (0+XfOrM358_COUNT)
#   define SETUP_XfOrM359(x) SETUP_XfOrM358(x)

# 2386 "./../src/optimize.c"
    case scheme_ir_local_type : 
    {
      sz += 1 ; 
      break ; 
    }
    case scheme_case_lambda_sequence_type : 
    {
      int max_sz = sz + 1 , a_sz ; 
      Scheme_Case_Lambda * cl = (Scheme_Case_Lambda * ) expr ; 
      int i ; 
      BLOCK_SETUP((PUSH(cl, 0+XfOrM359_COUNT)));
#     define XfOrM373_COUNT (1+XfOrM359_COUNT)
#     define SETUP_XfOrM373(x) SETUP(XfOrM373_COUNT)

# 2396 "./../src/optimize.c"
      for (i = cl -> count ; i -- ; ) {
#       define XfOrM375_COUNT (0+XfOrM373_COUNT)
#       define SETUP_XfOrM375(x) SETUP_XfOrM373(x)

# 2397 "./../src/optimize.c"
        a_sz = FUNCCALL(SETUP_XfOrM375(_), estimate_expr_size (cl -> array [i ] , sz , fuel ) ); 
        if (a_sz > max_sz )

# 2398 "./../src/optimize.c"
        max_sz = a_sz ; 
      }
      sz = max_sz ; 
    }
    break ; 
    case scheme_application2_type : 
    {
      Scheme_App2_Rec * app = (Scheme_App2_Rec * ) expr ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM359_COUNT)));
#     define XfOrM372_COUNT (1+XfOrM359_COUNT)
#     define SETUP_XfOrM372(x) SETUP(XfOrM372_COUNT)

# 2407 "./../src/optimize.c"
      sz = FUNCCALL(SETUP_XfOrM372(_), estimate_expr_size (app -> rator , sz , fuel - 1 ) ); 
      sz = FUNCCALL(SETUP_XfOrM372(_), estimate_expr_size (app -> rand , sz , fuel - 1 ) ); 
      sz ++ ; 

      break ; 
    }
    case scheme_application_type : 
    {
      Scheme_App_Rec * app = (Scheme_App_Rec * ) expr ; 
      int i ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM359_COUNT)));
#     define XfOrM369_COUNT (1+XfOrM359_COUNT)
#     define SETUP_XfOrM369(x) SETUP(XfOrM369_COUNT)

# 2418 "./../src/optimize.c"
      for (i = app -> num_args + 1 ; i -- ; ) {
#       define XfOrM371_COUNT (0+XfOrM369_COUNT)
#       define SETUP_XfOrM371(x) SETUP_XfOrM369(x)

# 2419 "./../src/optimize.c"
        sz = FUNCCALL(SETUP_XfOrM371(_), estimate_expr_size (app -> args [i ] , sz , fuel - 1 ) ); 
      }
      sz ++ ; 

      break ; 
    }
    case scheme_application3_type : 
    {
      Scheme_App3_Rec * app = (Scheme_App3_Rec * ) expr ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM359_COUNT)));
#     define XfOrM368_COUNT (1+XfOrM359_COUNT)
#     define SETUP_XfOrM368(x) SETUP(XfOrM368_COUNT)

# 2429 "./../src/optimize.c"
      sz = FUNCCALL(SETUP_XfOrM368(_), estimate_expr_size (app -> rator , sz , fuel - 1 ) ); 
      sz = FUNCCALL(SETUP_XfOrM368(_), estimate_expr_size (app -> rand1 , sz , fuel - 1 ) ); 
      sz = FUNCCALL_AGAIN(estimate_expr_size (app -> rand2 , sz , fuel - 1 ) ); 
      sz ++ ; 

      break ; 
    }
    case scheme_ir_let_header_type : 
    {
      Scheme_IR_Let_Header * head = (Scheme_IR_Let_Header * ) expr ; 
      Scheme_Object * body ; 
      Scheme_IR_Let_Value * lv ; 
      int i ; 
      BLOCK_SETUP((PUSH(body, 0+XfOrM359_COUNT), PUSH(lv, 1+XfOrM359_COUNT), PUSH(head, 2+XfOrM359_COUNT)));
#     define XfOrM365_COUNT (3+XfOrM359_COUNT)
#     define SETUP_XfOrM365(x) SETUP(XfOrM365_COUNT)
      body = NULLED_OUT ; 
      lv = NULLED_OUT ; 

# 2443 "./../src/optimize.c"
      body = head -> body ; 
      for (i = head -> num_clauses ; i -- ; ) {
#       define XfOrM367_COUNT (0+XfOrM365_COUNT)
#       define SETUP_XfOrM367(x) SETUP_XfOrM365(x)

# 2445 "./../src/optimize.c"
        lv = (Scheme_IR_Let_Value * ) body ; 
        sz = FUNCCALL(SETUP_XfOrM367(_), estimate_expr_size (lv -> value , sz , fuel - 1 ) ); 
        body = lv -> body ; 
        sz ++ ; 
      }
      sz = FUNCCALL(SETUP_XfOrM365(_), estimate_expr_size (body , sz , fuel - 1 ) ); 
      break ; 
    }
    case scheme_sequence_type : 
    case scheme_begin0_sequence_type : 
    {
      Scheme_Sequence * seq = (Scheme_Sequence * ) expr ; 
      int i ; 
      BLOCK_SETUP((PUSH(seq, 0+XfOrM359_COUNT)));
#     define XfOrM362_COUNT (1+XfOrM359_COUNT)
#     define SETUP_XfOrM362(x) SETUP(XfOrM362_COUNT)

# 2459 "./../src/optimize.c"
      for (i = seq -> count ; i -- ; ) {
#       define XfOrM364_COUNT (0+XfOrM362_COUNT)
#       define SETUP_XfOrM364(x) SETUP_XfOrM362(x)

# 2460 "./../src/optimize.c"
        sz = FUNCCALL(SETUP_XfOrM364(_), estimate_expr_size (seq -> array [i ] , sz , fuel - 1 ) ); 
      }

      break ; 
    }
    case scheme_branch_type : 
    {
      Scheme_Branch_Rec * b = (Scheme_Branch_Rec * ) expr ; 
      BLOCK_SETUP((PUSH(b, 0+XfOrM359_COUNT)));
#     define XfOrM361_COUNT (1+XfOrM359_COUNT)
#     define SETUP_XfOrM361(x) SETUP(XfOrM361_COUNT)

# 2469 "./../src/optimize.c"
      sz = FUNCCALL(SETUP_XfOrM361(_), estimate_expr_size (b -> test , sz , fuel - 1 ) ); 
      sz = FUNCCALL(SETUP_XfOrM361(_), estimate_expr_size (b -> tbranch , sz , fuel - 1 ) ); 
      sz = FUNCCALL_AGAIN(estimate_expr_size (b -> fbranch , sz , fuel - 1 ) ); 
      break ; 
    }
    case scheme_ir_lambda_type : 
    {
#     define XfOrM360_COUNT (0+XfOrM359_COUNT)
#     define SETUP_XfOrM360(x) SETUP_XfOrM359(x)

# 2476 "./../src/optimize.c"
      sz = FUNCCALL_EMPTY(estimate_expr_size (((Scheme_Lambda * ) expr ) -> body , sz , fuel - 1 ) ); 
      sz ++ ; 
      break ; 
    }
    case scheme_ir_toplevel_type : 

    default : 
    sz += 1 ; 
    break ; 
  }

  RET_VALUE_START (sz ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 2490 "./../src/optimize.c"
static Scheme_Object * estimate_closure_size (Scheme_Object * e ) 
{
  Scheme_Object * wbl ; 
  int sz ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(wbl, 0), PUSH(e, 1)));
# define XfOrM377_COUNT (2)
# define SETUP_XfOrM377(x) SETUP(XfOrM377_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  wbl = NULLED_OUT ; 

# 2494 "./../src/optimize.c"
  sz = FUNCCALL(SETUP_XfOrM377(_), estimate_expr_size (e , 0 , 32 ) ); 

  wbl = ((Scheme_Object * ) FUNCCALL(SETUP_XfOrM377(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_Simple_Object ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
  wbl -> type = scheme_will_be_lambda_type ; 
  (((Scheme_Simple_Object * ) (wbl ) ) -> u . ptr_int_val . pint ) = sz ; 
  (((Scheme_Simple_Object * ) (wbl ) ) -> u . ptr_int_val . ptr ) = e ; 

  RET_VALUE_START (wbl ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 2504 "./../src/optimize.c"
static Scheme_Object * no_potential_size (Scheme_Object * v ) 
{
  /* No conversion */

# 2506 "./../src/optimize.c"
  if (v && ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_will_be_lambda_type ) ) )
    return ((void * ) 0 ) ; 
  else 
  return v ; 
}

static Scheme_Object * apply_inlined (Scheme_Lambda * lam , Optimize_Info * info , 
int argc , Scheme_App_Rec * app , Scheme_App2_Rec * app2 , Scheme_App3_Rec * app3 , 
int context , Scheme_Object * orig , Scheme_Object * le_prev , 
int single_use ) 

{
  Scheme_IR_Let_Header * lh ; 
  Scheme_IR_Let_Value * lv , * prev = ((void * ) 0 ) ; 
  Scheme_Object * val ; 
  int i , expected ; 
  Optimize_Info * sub_info ; 
  Scheme_IR_Local * * vars ; 
  Scheme_Object * p = lam -> body ; 
  PREPARE_VAR_STACK(15);
  BLOCK_SETUP_TOP((PUSH(le_prev, 0), PUSH(info, 1), PUSH(app3, 2), PUSH(app2, 3), PUSH(vars, 4), PUSH(app, 5), PUSH(orig, 6), PUSH(lh, 7), PUSH(prev, 8), PUSH(sub_info, 9), PUSH(p, 10), PUSH(lam, 11), PUSH(val, 12), PUSH(lv, 13)));
# define XfOrM379_COUNT (14)
# define SETUP_XfOrM379(x) SETUP(XfOrM379_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  lh = NULLED_OUT ; 
  lv = NULLED_OUT ; 
  val = NULLED_OUT ; 
  sub_info = NULLED_OUT ; 
  vars = NULLED_OUT ; 

# 2526 "./../src/optimize.c"
  expected = lam -> num_params ; 

  if (! expected ) {
#   define XfOrM388_COUNT (0+XfOrM379_COUNT)
#   define SETUP_XfOrM388(x) SETUP_XfOrM379(x)

# 2530 "./../src/optimize.c"
    sub_info = FUNCCALL(SETUP_XfOrM388(_), optimize_info_add_frame (info , 0 ) ); 
    if (! single_use || lam -> ir_info -> is_dup )
      sub_info -> inline_fuel >>= 1 ; 
    p = FUNCCALL(SETUP_XfOrM388(_), optimize_expr (p , sub_info , context ) ); 
    info -> single_result = sub_info -> single_result ; 
    info -> preserves_marks = sub_info -> preserves_marks ; 
    FUNCCALL(SETUP_XfOrM388(_), optimize_info_done (sub_info , ((void * ) 0 ) ) ); 
    FUNCCALL_AGAIN(merge_types (sub_info , info , ((void * ) 0 ) ) ); 

    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(replace_tail_inside (p , le_prev , orig ) )) RET_VALUE_EMPTY_END ; 
  }

  lh = ((Scheme_IR_Let_Header * ) FUNCCALL(SETUP_XfOrM379(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_IR_Let_Header ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
  lh -> iso . so . type = scheme_ir_let_header_type ; 
  lh -> count = expected ; 
  lh -> num_clauses = expected ; 

  for (i = 0 ; i < expected ; i ++ ) {
#   define XfOrM384_COUNT (0+XfOrM379_COUNT)
#   define SETUP_XfOrM384(x) SETUP_XfOrM379(x)

# 2548 "./../src/optimize.c"
    lv = ((Scheme_IR_Let_Value * ) FUNCCALL(SETUP_XfOrM384(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_IR_Let_Value ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
    lv -> iso . so . type = scheme_ir_let_value_type ; 
    lv -> count = 1 ; 

    vars = ((Scheme_IR_Local * * ) FUNCCALL(SETUP_XfOrM384(_), GC_malloc (sizeof (Scheme_IR_Local * ) * (1 ) ) )) ; 
    vars [0 ] = lam -> ir_info -> vars [i ] ; 
    lv -> vars = vars ; 

    if ((i == expected - 1 ) 
    && ((& (lam ) -> iso ) -> so . keyex & 1 ) ) {
      int j ; 
      Scheme_Object * l = scheme_null ; 
      BLOCK_SETUP((PUSH(l, 0+XfOrM384_COUNT)));
#     define XfOrM385_COUNT (1+XfOrM384_COUNT)
#     define SETUP_XfOrM385(x) SETUP(XfOrM385_COUNT)

# 2561 "./../src/optimize.c"
      for (j = argc ; j -- > i ; ) {
#       define XfOrM387_COUNT (0+XfOrM385_COUNT)
#       define SETUP_XfOrM387(x) SETUP_XfOrM385(x)

# 2562 "./../src/optimize.c"
        if (app )
          val = app -> args [j + 1 ] ; 
        else if (app3 )
          val = (j ? app3 -> rand2 : app3 -> rand1 ) ; 
        else if (app2 )
          val = app2 -> rand ; 
        else 
        val = scheme_false ; 

        l = FUNCCALL(SETUP_XfOrM387(_), scheme_make_pair (val , l ) ); 
      }
      l = FUNCCALL(SETUP_XfOrM385(_), scheme_make_pair (scheme_list_proc , l ) ); 
      val = FUNCCALL(SETUP_XfOrM385(_), scheme_make_application (l , info ) ); 
    }

# 2575 "./../src/optimize.c"
    else if (app )
      val = app -> args [i + 1 ] ; 
    else if (app3 )
      val = (i ? app3 -> rand2 : app3 -> rand1 ) ; 
    else 
    val = app2 -> rand ; 

    lv -> value = val ; 

    if (prev )
      prev -> body = (Scheme_Object * ) lv ; 
    else 
    lh -> body = (Scheme_Object * ) lv ; 
    prev = lv ; 
  }

  if (prev )
    prev -> body = p ; 
  else 
  lh -> body = p ; 

  sub_info = FUNCCALL(SETUP_XfOrM379(_), optimize_info_add_frame (info , 0 ) ); 
  if (! single_use || lam -> ir_info -> is_dup )
    sub_info -> inline_fuel >>= 1 ; 

  p = FUNCCALL(SETUP_XfOrM379(_), optimize_lets ((Scheme_Object * ) lh , sub_info , context ) ); 

  info -> single_result = sub_info -> single_result ; 
  info -> preserves_marks = sub_info -> preserves_marks ; 
  FUNCCALL(SETUP_XfOrM379(_), optimize_info_done (sub_info , ((void * ) 0 ) ) ); 
  FUNCCALL_AGAIN(merge_types (sub_info , info , ((void * ) 0 ) ) ); 

  RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(replace_tail_inside (p , le_prev , orig ) )) RET_VALUE_EMPTY_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 2610 "./../src/optimize.c"
int scheme_check_leaf_rator (Scheme_Object * le ) 
{
  /* No conversion */

# 2612 "./../src/optimize.c"
  if (le && ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) ) {
    int opt ; 
    opt = ((Scheme_Prim_Proc_Header * ) le ) -> flags & (1 | 2 ) ; 
    if (opt >= 2 )
      return 1 ; 
  }
  return 0 ; 
}

static int get_rator_flags (Scheme_Object * rator , int num_args , Optimize_Info * info ) 
{
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(1);
  BLOCK_SETUP_TOP((PUSH(rator, 0)));
# define XfOrM391_COUNT (1)
# define SETUP_XfOrM391(x) SETUP(XfOrM391_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 2623 "./../src/optimize.c"
  rator = FUNCCALL(SETUP_XfOrM391(_), lookup_constant_proc (info , rator , num_args ) ); 
  if (! rator ) {
    RET_VALUE_START (0 ) RET_VALUE_END ; 
  }

# 2626 "./../src/optimize.c"
  else if ((rator ) == (scheme_true ) )  {

    RET_VALUE_START ((4 | 32 ) ) RET_VALUE_END ; 
  }

# 2629 "./../src/optimize.c"
  else if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) )  {
    RET_VALUE_START ((4 | 32 ) ) RET_VALUE_END ; 
  }

# 2631 "./../src/optimize.c"
  else if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_prop_proc_shape_type ) )  {
    switch (((Scheme_Small_Object * ) rator ) -> u . int_val ) {
      case 2 : 
      RET_VALUE_START ((4 | 32 ) ) RET_VALUE_END ; 
      case 3 : 
      if (num_args == 1 )
        RET_VALUE_START ((4 | 32 ) ) RET_VALUE_END ; 
    }
  }

# 2639 "./../src/optimize.c"
  else if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) )  {
    int opt ; 

    if (((rator ) == (scheme_values_proc ) ) && num_args == 1 ) {
      RET_VALUE_START ((4 | 32 ) ) RET_VALUE_END ; 
    }
    if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 16 ) ) {
      RET_VALUE_START ((4 | 32 ) ) RET_VALUE_END ; 
    }
    opt = ((Scheme_Prim_Proc_Header * ) rator ) -> flags & (1 | 2 ) ; 
    if (opt >= 1 ) {
      RET_VALUE_START ((4 | 32 ) ) RET_VALUE_END ; 
    }
  }

# 2652 "./../src/optimize.c"
  else if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) )  {
    Scheme_Lambda * lam = (Scheme_Lambda * ) rator ; 
    RET_VALUE_START ((& (lam ) -> iso ) -> so . keyex ) RET_VALUE_END ; 
  }
  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 2659 "./../src/optimize.c"
int check_single_use (Scheme_Object * var ) 
{
  /* No conversion */

# 2661 "./../src/optimize.c"
  Scheme_IR_Local * v = ((Scheme_IR_Local * ) var ) ; 

  return ((v -> use_count == 1 ) 


  && ! v -> optimize_outside_binding 


  && ! v -> optimize_unready ) ; 
}

int check_potential_size (Scheme_Object * var ) 
{
  /* No conversion */

# 2674 "./../src/optimize.c"
  Scheme_Object * n ; 

  n = ((Scheme_IR_Local * ) var ) -> optimize . known_val ; 
  if (n && ((Scheme_Type ) (((((intptr_t ) (n ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (n ) ) -> type ) ) == (Scheme_Type ) (scheme_will_be_lambda_type ) ) ) {
    return (((Scheme_Simple_Object * ) (n ) ) -> u . ptr_int_val . pint ) ; 
  }

  return 0 ; 
}

Scheme_Object * do_lookup_constant_proc (Optimize_Info * info , Scheme_Object * le , 
int argc , int for_inline , int for_props , 
int * _single_use , Scheme_Object * * _single_use_var ) 








{
  Scheme_Object * prev = ((void * ) 0 ) ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(8);
  BLOCK_SETUP_TOP((PUSH(_single_use, 0), PUSH(info, 1), PUSH(prev, 2), PUSH(le, 3), PUSH(_single_use_var, 4)));
# define XfOrM405_COUNT (5)
# define SETUP_XfOrM405(x) SETUP(XfOrM405_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 2698 "./../src/optimize.c"
  * _single_use = 0 ; 


  if (! for_inline )
    FUNCCALL(SETUP_XfOrM405(_), extract_tail_inside (& le , & prev , 0 ) ); 

  le = FUNCCALL(SETUP_XfOrM405(_), extract_specialized_proc (le , le ) ); 

  if (((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) )  {

    * _single_use = 1 ; 
  }

  if ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) )  {
    int tmp ; 
#   define XfOrM431_COUNT (0+XfOrM405_COUNT)
#   define SETUP_XfOrM431(x) SETUP_XfOrM405(x)

# 2713 "./../src/optimize.c"
    tmp = FUNCCALL(SETUP_XfOrM431(_), check_single_use (le ) ); 
    * _single_use = tmp ; 
    if (tmp )
      * _single_use_var = le ; 
    if (((Scheme_IR_Local * ) le ) -> mode != 3 )  {

      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
    }
    le = ((Scheme_IR_Local * ) le ) -> optimize . known_val ; 
    if (! le )
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
  }

  if ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_toplevel_type ) )  {
    Scheme_Object * inl ; 
    BLOCK_SETUP((PUSH(inl, 0+XfOrM405_COUNT)));
#   define XfOrM428_COUNT (1+XfOrM405_COUNT)
#   define SETUP_XfOrM428(x) SETUP(XfOrM428_COUNT)
    inl = NULLED_OUT ; 

# 2728 "./../src/optimize.c"
    * _single_use = 0 ; 
    do {
#     define XfOrM430_COUNT (0+XfOrM428_COUNT)
#     define SETUP_XfOrM430(x) SETUP_XfOrM428(x)

# 2730 "./../src/optimize.c"
      inl = FUNCCALL(SETUP_XfOrM430(_), get_import_inline (info , (Scheme_IR_Toplevel * ) le , argc , for_props ) ); 
      if ((argc < 0 ) && ((inl ) == (scheme_stack_dump_key ) ) )
        RET_VALUE_START (inl ) RET_VALUE_END ; 
      if (! inl )

# 2733 "./../src/optimize.c"
      inl = FUNCCALL(SETUP_XfOrM430(_), get_defn_shape (info , (Scheme_IR_Toplevel * ) le ) ); 
      if (inl )

# 2734 "./../src/optimize.c"
      le = inl ; 
    }

# 2735 "./../src/optimize.c"
    while (inl && ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_toplevel_type ) ) ) ; 
  }

  if ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_will_be_lambda_type ) )  {
    if (for_inline )
      RET_VALUE_START (le ) RET_VALUE_END ; 
    else 
    le = (((Scheme_Simple_Object * ) (le ) ) -> u . ptr_int_val . ptr ) ; 
  }

  if (! for_inline && ((Scheme_Type ) (scheme_noninline_proc_type ) == (Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) ) ) {
    le = (((Scheme_Small_Object * ) (le ) ) -> u . ptr_val ) ; 
  }

  if ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) )  {
    int ok_arity ; 
    switch (((Scheme_Struct_Proc_Shape * ) le ) -> mode & 0x7 ) {
      case 1 : 
      ok_arity = (argc == (((Scheme_Struct_Proc_Shape * ) le ) -> mode >> 7 ) ) ; 
      break ; 
      case 2 : 
      ok_arity = (argc == 1 ) ; 
      break ; 
      case 3 : 
      ok_arity = (argc == 1 ) ; 
      break ; 
      case 4 : 
      ok_arity = (argc == 2 ) ; 
      break ; 
      default : 
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
    }
    if (ok_arity || (argc == - 1 ) ) {
      RET_VALUE_START (for_inline ? ((void * ) 0 ) : le ) RET_VALUE_END ; 
    }

# 2769 "./../src/optimize.c"
    else if (for_props )
      RET_VALUE_START (le ) RET_VALUE_END ; 
    else 
    RET_VALUE_START (scheme_true ) RET_VALUE_END ; 
  }

  if ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_prop_proc_shape_type ) )  {
    int ok_arity ; 
    switch (((Scheme_Small_Object * ) le ) -> u . int_val ) {
      case 2 : 
      ok_arity = (argc == 1 ) ; 
      break ; 
      case 3 : 
      ok_arity = (argc == 1 ) || (argc == 2 ) ; 
      break ; 
      default : 
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
    }
    if (ok_arity || (argc == - 1 ) ) {
      RET_VALUE_START (for_inline ? ((void * ) 0 ) : le ) RET_VALUE_END ; 
    }

# 2789 "./../src/optimize.c"
    else if (for_props )
      RET_VALUE_START (le ) RET_VALUE_END ; 
    else 
    RET_VALUE_START (scheme_true ) RET_VALUE_END ; 
  }

  if ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) )  {
    Scheme_Case_Lambda * cl = (Scheme_Case_Lambda * ) le ; 
    Scheme_Object * cp ; 
    int i , count ; 
    BLOCK_SETUP((PUSH(cp, 0+XfOrM405_COUNT), PUSH(cl, 1+XfOrM405_COUNT)));
#   define XfOrM410_COUNT (2+XfOrM405_COUNT)
#   define SETUP_XfOrM410(x) SETUP(XfOrM410_COUNT)
    cp = NULLED_OUT ; 

# 2800 "./../src/optimize.c"
    if ((argc == - 1 ) || for_props )
      RET_VALUE_START (le ) RET_VALUE_END ; 

    count = cl -> count ; 
    for (i = 0 ; i < count ; i ++ ) {
#     define XfOrM416_COUNT (0+XfOrM410_COUNT)
#     define SETUP_XfOrM416(x) SETUP_XfOrM410(x)

# 2805 "./../src/optimize.c"
      cp = cl -> array [i ] ; 
      if ((Scheme_Type ) (((((intptr_t ) (cp ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (cp ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) )  {
        Scheme_Lambda * lam = (Scheme_Lambda * ) cp ; 
        if ((lam -> num_params == argc ) 
        || (((& (lam ) -> iso ) -> so . keyex & 1 ) 
        && (argc + 1 >= lam -> num_params ) ) ) {
          RET_VALUE_START (cp ) RET_VALUE_END ; 
        }
      }

# 2813 "./../src/optimize.c"
      else {
#       define XfOrM417_COUNT (0+XfOrM416_COUNT)
#       define SETUP_XfOrM417(x) SETUP_XfOrM416(x)
        FUNCCALL(SETUP_XfOrM417(_), scheme_signal_error ("internal error: strange case-lambda" ) )
# 2814 "./../src/optimize.c"
        ; 
      }
    }
    if (i >= count ) {
      RET_VALUE_START (scheme_true ) RET_VALUE_END ; 
    }
  }

  if ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) )  {
    Scheme_Lambda * lam = (Scheme_Lambda * ) le ; 

    if ((argc == - 1 ) || for_props )
      RET_VALUE_START (le ) RET_VALUE_END ; 

    if ((lam -> num_params == argc ) 
    || (((& (lam ) -> iso ) -> so . keyex & 1 ) 
    && (argc + 1 >= lam -> num_params ) ) ) {
      RET_VALUE_START (le ) RET_VALUE_END ; 
    }

# 2832 "./../src/optimize.c"
    else {
      RET_VALUE_START (scheme_true ) RET_VALUE_END ; 
    }
  }

  if (! (((intptr_t ) (le ) ) & 0x1 ) && ((((le ) -> type ) >= scheme_prim_type ) && (((le ) -> type ) <= scheme_proc_chaperone_type ) ) )  {
    Scheme_Object * a [1 ] ; 
    BLOCK_SETUP((PUSHARRAY(a, 1, 0+XfOrM405_COUNT)));
#   define XfOrM406_COUNT (3+XfOrM405_COUNT)
#   define SETUP_XfOrM406(x) SETUP(XfOrM406_COUNT)
    a[0] = NULLED_OUT ; 

# 2840 "./../src/optimize.c"
    if ((argc == - 1 ) || for_props )
      RET_VALUE_START (le ) RET_VALUE_END ; 

    a [0 ] = le ; 
    if (FUNCCALL(SETUP_XfOrM406(_), scheme_check_proc_arity (((void * ) 0 ) , argc , 0 , 1 , a ) ))
      RET_VALUE_START (le ) RET_VALUE_END ; 
    else 
    RET_VALUE_START (scheme_true ) RET_VALUE_END ; 
  }

  if (for_props 
  && le 
  && (((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_lambda_type ) ) 
  || ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) )
    RET_VALUE_START (le ) RET_VALUE_END ; 

  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 2859 "./../src/optimize.c"
Scheme_Object * lookup_constant_proc (Optimize_Info * info , Scheme_Object * le , int argc ) 
{
  int single_use = 0 ; 
  Scheme_Object * single_use_var ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(1);
  BLOCK_SETUP_TOP((PUSH(single_use_var, 0)));
# define XfOrM434_COUNT (1)
# define SETUP_XfOrM434(x) SETUP(XfOrM434_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  single_use_var = NULLED_OUT ; 

# 2863 "./../src/optimize.c"
  RET_VALUE_START (FUNCCALL(SETUP_XfOrM434(_), do_lookup_constant_proc (info , le , argc , 0 , 0 , & single_use , & single_use_var ) )) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}



Scheme_Object * optimize_for_inline (Optimize_Info * info , Scheme_Object * le , int argc , 
Scheme_App_Rec * app , Scheme_App2_Rec * app2 , Scheme_App3_Rec * app3 , 
int context , int optimized_rator ) 




{
  int single_use = 0 , psize = 0 ; 
  Scheme_Object * prev = ((void * ) 0 ) , * orig_le = le , * le2 , * single_use_var = ((void * ) 0 ) ; 
  int already_opt = optimized_rator ; 

# 251 "./../src/optimize.c"
  char * __funcarg83 = NULLED_OUT ; 
  char * __funcarg82 = NULLED_OUT ; 

# 251 "./../src/optimize.c"
  char * __funcarg81 = NULLED_OUT ; 
  char * __funcarg80 = NULLED_OUT ; 

# 251 "./../src/optimize.c"
  char * __funcarg79 = NULLED_OUT ; 
  char * __funcarg78 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(14);
  BLOCK_SETUP_TOP((PUSH(__funcarg78, 0), PUSH(info, 1), PUSH(le, 2), PUSH(app3, 3), PUSH(app2, 4), PUSH(__funcarg82, 5), PUSH(single_use_var, 6), PUSH(le2, 7), PUSH(prev, 8), PUSH(app, 9), PUSH(__funcarg80, 10), PUSH(orig_le, 11)));
# define XfOrM435_COUNT (12)
# define SETUP_XfOrM435(x) SETUP(XfOrM435_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  le2 = NULLED_OUT ; 

# 2884 "./../src/optimize.c"
  if ((info -> inline_fuel < 0 ) && info -> has_nonleaf )
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 



  if (already_opt )
    FUNCCALL(SETUP_XfOrM435(_), extract_tail_inside (& le , & prev , 0 ) ); 

  le = FUNCCALL(SETUP_XfOrM435(_), extract_specialized_proc (le , le ) ); 

  if (! already_opt 
  && (((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) ) {


    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
  }

  le2 = le ; 
  le = FUNCCALL(SETUP_XfOrM435(_), do_lookup_constant_proc (info , le , argc , 1 , 0 , & single_use , & single_use_var ) ); 

  if (! le ) {
    info -> has_nonleaf = 1 ; 
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
  }

  if ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_will_be_lambda_type ) )  {
    psize = (((Scheme_Simple_Object * ) (le ) ) -> u . ptr_int_val . pint ) ; 
    ; 


    if (psize <= (info -> inline_fuel * (argc + 2 ) ) )
      info -> psize += psize ; 
    info -> has_nonleaf = 1 ; 
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
  }

  if ((le ) == (scheme_true ) )  {

    int len ; 
    const char * pname = ((void * ) 0 ) , * context ; 
    BLOCK_SETUP((PUSH(pname, 0+XfOrM435_COUNT), PUSH(context, 1+XfOrM435_COUNT)));
#   define XfOrM442_COUNT (2+XfOrM435_COUNT)
#   define SETUP_XfOrM442(x) SETUP(XfOrM442_COUNT)
    context = NULLED_OUT ; 

# 2924 "./../src/optimize.c"
    info -> escapes = 1 ; 
    le2 = FUNCCALL(SETUP_XfOrM442(_), do_lookup_constant_proc (info , le2 , argc , 1 , 1 , & single_use , & single_use_var ) ); 
    if (! ((Scheme_Type ) (((((intptr_t ) (le2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le2 ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) ) 
    && ! ((Scheme_Type ) (((((intptr_t ) (le2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le2 ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_prop_proc_shape_type ) ) ) {
#     define XfOrM443_COUNT (0+XfOrM442_COUNT)
#     define SETUP_XfOrM443(x) SETUP_XfOrM442(x)

# 2928 "./../src/optimize.c"
      pname = FUNCCALL(SETUP_XfOrM443(_), scheme_get_proc_name (le2 , & len , 0 ) ); 
    }
    context = FUNCCALL(SETUP_XfOrM442(_), scheme_optimize_context_to_string (info -> context ) ); 
    FUNCCALL(SETUP_XfOrM442(_), scheme_log (info -> logger , 3 , 0 , "warning%s: optimizer detects procedure incorrectly applied to %d arguments%s%s" , context , argc , pname ? ": " : "" , pname ? pname : "" ) )






    ; 
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
  }

  if (((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) && (info -> inline_fuel >= 0 ) ) {
    Scheme_Lambda * lam = (Scheme_Lambda * ) le ; 
    int sz , threshold , is_leaf = 0 ; 
    BLOCK_SETUP((PUSH(lam, 0+XfOrM435_COUNT)));
#   define XfOrM436_COUNT (1+XfOrM435_COUNT)
#   define SETUP_XfOrM436(x) SETUP(XfOrM436_COUNT)

# 2946 "./../src/optimize.c"
    sz = FUNCCALL(SETUP_XfOrM436(_), lambda_body_size_plus_info (lam , 1 , info , & is_leaf ) ); 
    if (is_leaf ) {

      sz >>= 2 ; 
    }
    threshold = info -> inline_fuel * (2 + argc ) ; 


    if ((sz >= 0 ) && (single_use || (sz <= threshold ) ) ) {
      Optimize_Info * sub_info ; 
      BLOCK_SETUP((PUSH(sub_info, 0+XfOrM436_COUNT)));
#     define XfOrM438_COUNT (1+XfOrM436_COUNT)
#     define SETUP_XfOrM438(x) SETUP(XfOrM438_COUNT)
      sub_info = NULLED_OUT ; 

# 2956 "./../src/optimize.c"
      sub_info = info ; 


      le = FUNCCALL(SETUP_XfOrM438(_), optimize_clone (single_use , (Scheme_Object * ) lam , sub_info , empty_eq_hash_tree , 0 ) ); 

      if (le ) {
#       define XfOrM440_COUNT (0+XfOrM438_COUNT)
#       define SETUP_XfOrM440(x) SETUP_XfOrM438(x)

# 2963 "./../src/optimize.c"
        ; 
        if (FUNCCALL(SETUP_XfOrM440(_), scheme_log_level_p (info -> logger , 5 ) ))
          (__funcarg82 = FUNCCALL(SETUP_XfOrM440(_), scheme_write_to_string (lam -> name ? lam -> name : scheme_false , ((void * ) 0 ) ) )



        , __funcarg83 = FUNCCALL(SETUP_XfOrM440(_), scheme_optimize_context_to_string (info -> context ) ), FUNCCALL_AGAIN(scheme_log (info -> logger , 5 , 0 , "inlining %s size: %d threshold: %d#<separator>%s" , __funcarg82 , sz , threshold , __funcarg83 ) )) 


        ; 
        if (single_use_var )
          ((Scheme_IR_Local * ) single_use_var ) -> optimize_used = 0 ; 
        le = FUNCCALL(SETUP_XfOrM440(_), apply_inlined ((Scheme_Lambda * ) le , sub_info , argc , app , app2 , app3 , context , orig_le , prev , single_use ) )
        ; 
        RET_VALUE_START (le ) RET_VALUE_END ; 
      }

# 2978 "./../src/optimize.c"
      else {
#       define XfOrM439_COUNT (0+XfOrM438_COUNT)
#       define SETUP_XfOrM439(x) SETUP_XfOrM438(x)

# 2979 "./../src/optimize.c"
        ; 
        if (FUNCCALL(SETUP_XfOrM439(_), scheme_log_level_p (info -> logger , 5 ) ))
          (__funcarg80 = FUNCCALL(SETUP_XfOrM439(_), scheme_write_to_string (lam -> name ? lam -> name : scheme_false , ((void * ) 0 ) ) )



        , __funcarg81 = FUNCCALL(SETUP_XfOrM439(_), scheme_optimize_context_to_string (info -> context ) ), FUNCCALL_AGAIN(scheme_log (info -> logger , 5 , 0 , "no-inlining %s size: %d threshold: %d#<separator>%s" , __funcarg80 , sz , threshold , __funcarg81 ) )) 


        ; 
      }
    }

# 2990 "./../src/optimize.c"
    else {
#     define XfOrM437_COUNT (0+XfOrM436_COUNT)
#     define SETUP_XfOrM437(x) SETUP_XfOrM436(x)
      ; 
      if (FUNCCALL(SETUP_XfOrM437(_), scheme_log_level_p (info -> logger , 5 ) ))
        (__funcarg78 = FUNCCALL(SETUP_XfOrM437(_), scheme_write_to_string (lam -> name ? lam -> name : scheme_false , ((void * ) 0 ) ) )



      , __funcarg79 = FUNCCALL(SETUP_XfOrM437(_), scheme_optimize_context_to_string (info -> context ) ), FUNCCALL_AGAIN(scheme_log (info -> logger , 5 , 0 , "out-of-fuel %s size: %d threshold: %d#<separator>%s" , __funcarg78 , sz , threshold , __funcarg79 ) )) 


      ; 
    }
  }

  if (! FUNCCALL(SETUP_XfOrM435(_), scheme_check_leaf_rator (le ) ))
    info -> has_nonleaf = 1 ; 

  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 3012 "./../src/optimize.c"
static int is_local_type_expression (Scheme_Object * expr , Optimize_Info * info ) 

{

# 135 "./../src/optimize.c"
  Scheme_Object * __funcarg84 = NULLED_OUT ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(0);
# define XfOrM447_COUNT (0)
# define SETUP_XfOrM447(x) SETUP(XfOrM447_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 3015 "./../src/optimize.c"
  RET_VALUE_START ((__funcarg84 = FUNCCALL_EMPTY(expr_implies_predicate (expr , info ) ), (scheme_predicate_to_local_type (__funcarg84 ) ) ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 3018 "./../src/optimize.c"
static void register_local_argument_types (Scheme_App_Rec * app , Scheme_App2_Rec * app2 , Scheme_App3_Rec * app3 , 
Optimize_Info * info ) 



{
  Scheme_Object * rator , * rand , * le ; 
  int n , i , nth_app ; 
  PREPARE_VAR_STACK(10);
  BLOCK_SETUP_TOP((PUSH(rand, 0), PUSH(info, 1), PUSH(app, 2), PUSH(le, 3), PUSH(app3, 4), PUSH(app2, 5)));
# define XfOrM448_COUNT (6)
# define SETUP_XfOrM448(x) SETUP(XfOrM448_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  rator = NULLED_OUT ; 
  rand = NULLED_OUT ; 
  le = NULLED_OUT ; 

# 3027 "./../src/optimize.c"
  if (app ) {
    rator = app -> args [0 ] ; 
    n = app -> num_args ; 
    nth_app = (& (app ) -> iso ) -> so . keyex & 7 ; 
  }

# 3031 "./../src/optimize.c"
  else if (app2 ) {
    rator = app2 -> rator ; 
    n = 1 ; 
    nth_app = (& (app2 ) -> iso ) -> so . keyex & 7 ; 
  }

# 3035 "./../src/optimize.c"
  else {
    rator = app3 -> rator ; 
    n = 2 ; 
    nth_app = (& (app3 ) -> iso ) -> so . keyex & 7 ; 
  }

  if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) )  {
#   define XfOrM449_COUNT (0+XfOrM448_COUNT)
#   define SETUP_XfOrM449(x) SETUP_XfOrM448(x)

# 3042 "./../src/optimize.c"
    le = FUNCCALL(SETUP_XfOrM449(_), optimize_info_lookup (rator ) ); 
    if (le && ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_will_be_lambda_type ) ) )
      le = (((Scheme_Simple_Object * ) (le ) ) -> u . ptr_int_val . ptr ) ; 

    if (le && ((Scheme_Type ) (((((intptr_t ) (le ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (le ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) ) {
      Scheme_Lambda * lam = (Scheme_Lambda * ) le ; 
      BLOCK_SETUP((PUSH(lam, 0+XfOrM449_COUNT)));
#     define XfOrM450_COUNT (1+XfOrM449_COUNT)
#     define SETUP_XfOrM450(x) SETUP(XfOrM450_COUNT)

# 3048 "./../src/optimize.c"
      if ((lam -> num_params == n ) 
      && ! ((& (lam ) -> iso ) -> so . keyex & 1 ) ) {
        Scheme_Object * pred ; 
        BLOCK_SETUP((PUSH(pred, 0+XfOrM450_COUNT)));
#       define XfOrM451_COUNT (1+XfOrM450_COUNT)
#       define SETUP_XfOrM451(x) SETUP(XfOrM451_COUNT)
        pred = NULLED_OUT ; 

# 3052 "./../src/optimize.c"
        if (! lam -> ir_info -> arg_types ) {
          Scheme_Object * * arg_types ; 
          short * contributors ; 
          BLOCK_SETUP((PUSH(contributors, 0+XfOrM451_COUNT), PUSH(arg_types, 1+XfOrM451_COUNT)));
#         define XfOrM472_COUNT (2+XfOrM451_COUNT)
#         define SETUP_XfOrM472(x) SETUP(XfOrM472_COUNT)
          arg_types = NULLED_OUT ; 
          contributors = NULLED_OUT ; 

# 3055 "./../src/optimize.c"
          arg_types = ((Scheme_Object * * ) FUNCCALL(SETUP_XfOrM472(_), GC_malloc (sizeof (Scheme_Object * ) * (n ) ) )) ; 
          lam -> ir_info -> arg_types = arg_types ; 
          contributors = ((short * ) FUNCCALL(SETUP_XfOrM472(_), GC_malloc_atomic (sizeof (short ) * (n ) ) )) ; 
          (memset (contributors , 0 , sizeof (short ) * n ) ) ; 
          lam -> ir_info -> arg_type_contributors = contributors ; 
        }

        for (i = 0 ; i < n ; i ++ ) {
#         define XfOrM462_COUNT (0+XfOrM451_COUNT)
#         define SETUP_XfOrM462(x) SETUP_XfOrM451(x)

# 3063 "./../src/optimize.c"
          if (app )
            rand = app -> args [i + 1 ] ; 
          else if (app2 )
            rand = app2 -> rand ; 
          else {
            if (! i )
              rand = app3 -> rand1 ; 
            else 
            rand = app3 -> rand2 ; 
          }

          if (lam -> ir_info -> arg_types [i ] 
          || ! lam -> ir_info -> arg_type_contributors [i ] ) {
            int widen_to_top = 0 ; 
#           define XfOrM463_COUNT (0+XfOrM462_COUNT)
#           define SETUP_XfOrM463(x) SETUP_XfOrM462(x)

# 3078 "./../src/optimize.c"
            pred = FUNCCALL(SETUP_XfOrM463(_), expr_implies_predicate (rand , info ) ); 

            if (pred ) {
#             define XfOrM467_COUNT (0+XfOrM463_COUNT)
#             define SETUP_XfOrM467(x) SETUP_XfOrM463(x)

# 3081 "./../src/optimize.c"
              if (! lam -> ir_info -> arg_type_contributors [i ] ) {
                lam -> ir_info -> arg_types [i ] = pred ; 
                if (nth_app )
                  lam -> ir_info -> arg_type_contributors [i ] |= (1 << (nth_app - 1 ) ) ; 
              }

# 3085 "./../src/optimize.c"
              else if (predicate_implies (pred , lam -> ir_info -> arg_types [i ] ) )  {

                if (nth_app )
                  lam -> ir_info -> arg_type_contributors [i ] |= (1 << (nth_app - 1 ) ) ; 
              }

# 3089 "./../src/optimize.c"
              else if (predicate_implies (lam -> ir_info -> arg_types [i ] , pred ) )  {

                lam -> ir_info -> arg_types [i ] = pred ; 
                if (nth_app )
                  lam -> ir_info -> arg_type_contributors [i ] |= (1 << (nth_app - 1 ) ) ; 
              }

# 3094 "./../src/optimize.c"
              else 
              widen_to_top = 1 ; 
            }

# 3096 "./../src/optimize.c"
            else 
            widen_to_top = 1 ; 

            if (widen_to_top ) {
              if (nth_app ) {
              }



              else {




                lam -> ir_info -> arg_type_contributors [i ] |= (1 << (7 - 1 ) ) ; 
                lam -> ir_info -> arg_types [i ] = ((void * ) 0 ) ; 
              }
            }
          }
        }
      }
    }
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 3121 "./../src/optimize.c"
static void reset_rator (Scheme_Object * app , Scheme_Object * a ) 
{
  /* No conversion */

# 3123 "./../src/optimize.c"
  switch (((((intptr_t ) (app ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app ) ) -> type ) ) {
    case scheme_application_type : 
    ((Scheme_App_Rec * ) app ) -> args [0 ] = a ; 
    break ; 
    case scheme_application2_type : 
    ((Scheme_App2_Rec * ) app ) -> rator = a ; 
    break ; 
    case scheme_application3_type : 
    ((Scheme_App3_Rec * ) app ) -> rator = a ; 
    break ; 
  }
}

static void set_application_omittable (Scheme_Object * app , Scheme_Object * a ) 
{
  /* No conversion */

# 3138 "./../src/optimize.c"
  switch (((((intptr_t ) (app ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app ) ) -> type ) ) {
    case scheme_application_type : 
    (& ((Scheme_App_Rec * ) app ) -> iso ) -> so . keyex |= (1 << 11 ) ; 
    break ; 
    case scheme_application2_type : 
    (& ((Scheme_App2_Rec * ) app ) -> iso ) -> so . keyex |= (1 << 11 ) ; 
    break ; 
    case scheme_application3_type : 
    (& ((Scheme_App3_Rec * ) app ) -> iso ) -> so . keyex |= (1 << 11 ) ; 
    break ; 
  }
}

static Scheme_Object * check_app_let_rator (Scheme_Object * app , Scheme_Object * rator , Optimize_Info * info , 
int argc , int context ) 




{
  Scheme_Object * orig_rator = rator , * inside = ((void * ) 0 ) ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(5);
  BLOCK_SETUP_TOP((PUSH(orig_rator, 0), PUSH(info, 1), PUSH(app, 2), PUSH(rator, 3), PUSH(inside, 4)));
# define XfOrM480_COUNT (5)
# define SETUP_XfOrM480(x) SETUP(XfOrM480_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  FUNCCALL(SETUP_XfOrM480(_), extract_tail_inside (& rator , & inside , 0 ) )
# 3160 "./../src/optimize.c"
  ; 

  if (! inside )
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 


  if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) )  {
    Scheme_IR_Local * var = ((Scheme_IR_Local * ) rator ) ; 
    if (var -> non_app_count < 7 )
      -- var -> non_app_count ; 
  }
  FUNCCALL(SETUP_XfOrM480(_), reset_rator (app , rator ) )
  ; 
  orig_rator = FUNCCALL(SETUP_XfOrM480(_), replace_tail_inside (app , inside , orig_rator ) ); 

  RET_VALUE_START (FUNCCALL(SETUP_XfOrM480(_), optimize_expr (orig_rator , info , context ) )) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 3178 "./../src/optimize.c"
__xform_nongcing__ static int is_primitive_allocating (Scheme_Object * rator , int argc ) 
{
  /* No conversion */

# 3180 "./../src/optimize.c"
  if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & ((1 << 5 ) 
  | (1 << 6 ) 
  | (1 << 7 ) ) )
    return scheme_is_omitable_primitive (rator , argc ) ; 

  return 0 ; 
}

__xform_nongcing__ static int is_nonmutating_nondependant_primitive (Scheme_Object * rator , int argc ) 



{
  /* No conversion */

# 3193 "./../src/optimize.c"
  if (((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
  && (((scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 4 ) ) 
  || is_primitive_allocating (rator , argc ) ) 
  && ! (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & ((1 << 3 ) ) ) 
  && ! ((((scheme_values_proc ) == (rator ) ) && (argc != 1 ) ) ) ) 
  && (argc >= ((Scheme_Primitive_Proc * ) rator ) -> mina ) 
  && (argc <= ((Scheme_Primitive_Proc * ) rator ) -> mu . maxa ) )
    return 1 ; 

  return 0 ; 
}

__xform_nongcing__ static int is_noncapturing_primitive (Scheme_Object * rator , int n ) 
{
  /* No conversion */

# 3207 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) )  {
    int opt , t ; 
    opt = ((Scheme_Prim_Proc_Header * ) rator ) -> flags & (1 | 2 ) ; 
    if (opt >= 2 )
      return 1 ; 
    if (opt >= 1 ) {
      if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 16 ) ) {

        return 1 ; 
      }
    }
    t = (((Scheme_Primitive_Proc * ) rator ) -> pp . flags & (32 | 64 | 128 | 256 ) ) ; 
    if (! n && (t == 64 ) )
      return 1 ; 
    if ((rator ) == (scheme_values_proc ) ) 
      return 1 ; 
  }

  return 0 ; 
}

__xform_nongcing__ static int is_nonsaving_primitive (Scheme_Object * rator , int n ) 
{
  /* No conversion */

# 3230 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) )  {
    int opt ; 
    opt = ((Scheme_Prim_Proc_Header * ) rator ) -> flags & (1 | 2 ) ; 
    if (opt >= 2 )
      return 1 ; 
    if ((rator ) == (scheme_values_proc ) ) 
      return 1 ; 
  }

  return 0 ; 
}

__xform_nongcing__ static int is_always_escaping_primitive (Scheme_Object * rator ) 
{
  /* No conversion */

# 3244 "./../src/optimize.c"
  if (((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
  && (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 16 ) ) ) {
    return 1 ; 
  }
  return 0 ; 
}



__xform_nongcing__ static int wants_local_type_arguments (Scheme_Object * rator , int argpos ) 
{
  /* No conversion */

# 3255 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) )  {
    int flags ; 
    flags = scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] ; 

    if (argpos == 0 ) {
      if (flags & (1 << 9 ) )
        return 1 ; 
      if (flags & (1 << 12 ) )
        return 3 ; 
    }

# 3264 "./../src/optimize.c"
    else if (argpos == 1 ) {
      if (flags & (1 << 10 ) )
        return 1 ; 
      if (flags & (1 << 13 ) )
        return 3 ; 
    }

# 3269 "./../src/optimize.c"
    else if (argpos == 2 ) {
      if (flags & (1 << 11 ) )
        return 1 ; 
      if (flags & (1 << 14 ) )
        return 3 ; 
    }
  }

  return 0 ; 
}

static int produces_local_type (Scheme_Object * rator , int argc ) 
{
  /* No conversion */

# 3282 "./../src/optimize.c"
  if (((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
  && (argc >= ((Scheme_Primitive_Proc * ) rator ) -> mina ) 
  && (argc <= ((Scheme_Primitive_Proc * ) rator ) -> mu . maxa ) ) {
    int flags ; 
    flags = scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] ; 
    return ((flags & (0x3 << 26 ) ) >> 26 ) ; 
  }

  return 0 ; 
}

static Scheme_Object * local_type_to_predicate (int t ) 
{
  /* No conversion */

# 3295 "./../src/optimize.c"
  switch (t ) {
    case 1 : 
    return scheme_flonum_p_proc ; 
    case 2 : 
    return scheme_fixnum_p_proc ; 
    case 3 : 
    return scheme_extflonum_p_proc ; 
  }
  return ((void * ) 0 ) ; 
}

int scheme_predicate_to_local_type (Scheme_Object * pred ) 
{
  /* No conversion */

# 3308 "./../src/optimize.c"
  if (! pred )
    return 0 ; 
  if ((scheme_flonum_p_proc ) == (pred ) ) 
    return 1 ; 
  if ((scheme_fixnum_p_proc ) == (pred ) ) 
    return 2 ; 
  if ((scheme_extflonum_p_proc ) == (pred ) ) 
    return 3 ; 
  return 0 ; 
}

int scheme_expr_produces_local_type (Scheme_Object * expr , int * _involves_k_cross ) 
{

# 136 "./../src/optimize.c"
  Scheme_Object * __funcarg85 = NULLED_OUT ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(0);
# define XfOrM502_COUNT (0)
# define SETUP_XfOrM502(x) SETUP(XfOrM502_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 3321 "./../src/optimize.c"
  if (_involves_k_cross )

# 3321 "./../src/optimize.c"
  * _involves_k_cross = 0 ; 
  RET_VALUE_START ((__funcarg85 = FUNCCALL_EMPTY(do_expr_implies_predicate (expr , ((void * ) 0 ) , _involves_k_cross , 10 , empty_eq_hash_tree ) ), (scheme_predicate_to_local_type (__funcarg85 ) ) ) ) RET_VALUE_END 
  ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 3326 "./../src/optimize.c"
static Scheme_Object * rator_implies_predicate (Scheme_Object * rator , Optimize_Info * info , int argc ) 
{

# 3280 "./../src/optimize.c"
  int __funcarg86 ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(3);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(rator, 1)));
# define XfOrM503_COUNT (2)
# define SETUP_XfOrM503(x) SETUP(XfOrM503_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 3328 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) )  {
#   define XfOrM510_COUNT (0+XfOrM503_COUNT)
#   define SETUP_XfOrM510(x) SETUP_XfOrM503(x)

# 3329 "./../src/optimize.c"
    if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 18 ) )
      RET_VALUE_START (scheme_real_p_proc ) RET_VALUE_END ; 
    else if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 19 ) )
      RET_VALUE_START (scheme_number_p_proc ) RET_VALUE_END ; 
    else if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 25 ) )
      RET_VALUE_START (scheme_boolean_p_proc ) RET_VALUE_END ; 
    else if ((rator ) == (scheme_cons_proc ) ) 
      RET_VALUE_START (scheme_pair_p_proc ) RET_VALUE_END ; 
    else if ((rator ) == (scheme_unsafe_cons_list_proc ) ) 
      RET_VALUE_START (scheme_list_pair_p_proc ) RET_VALUE_END ; 
    else if ((rator ) == (scheme_mcons_proc ) ) 
      RET_VALUE_START (scheme_mpair_p_proc ) RET_VALUE_END ; 
    else if ((rator ) == (scheme_list_proc ) )  {
      if (argc >= 1 )
        RET_VALUE_START (scheme_list_pair_p_proc ) RET_VALUE_END ; 
      else 
      RET_VALUE_START (scheme_null_p_proc ) RET_VALUE_END ; 
    }

# 3346 "./../src/optimize.c"
    else if ((rator ) == (scheme_list_star_proc ) )  {
      if (argc > 2 )
        RET_VALUE_START (scheme_pair_p_proc ) RET_VALUE_END ; 
    }

# 3349 "./../src/optimize.c"
    else if ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "vector->list" ) ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "map" ) ) ) ) {
      RET_VALUE_START (scheme_list_p_proc ) RET_VALUE_END ; 
    }

# 3352 "./../src/optimize.c"
    else if (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "string-ref" ) ) )  {
      RET_VALUE_START (scheme_char_p_proc ) RET_VALUE_END ; 
    }

# 3354 "./../src/optimize.c"
    else if ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "string-append" ) ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "string-append-immutable" ) ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "string->immutable-string" ) ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "symbol->string" ) ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "symbol->immutable-string" ) ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "keyword->string" ) ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "keyword->immutable-string" ) ) ) ) {
      RET_VALUE_START (scheme_string_p_proc ) RET_VALUE_END ; 
    }

# 3362 "./../src/optimize.c"
    else if ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "bytes-append" ) ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "bytes->immutable-bytes" ) ) ) ) {
      RET_VALUE_START (scheme_byte_string_p_proc ) RET_VALUE_END ; 
    }

# 3365 "./../src/optimize.c"
    else if (((rator ) == (scheme_vector_proc ) ) 
    || ((rator ) == (scheme_vector_immutable_proc ) ) 
    || ((rator ) == (scheme_make_vector_proc ) ) 
    || ((rator ) == (scheme_list_to_vector_proc ) ) 
    || ((rator ) == (scheme_struct_to_vector_proc ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "vector->immutable-vector" ) ) ) )
      RET_VALUE_START (scheme_vector_p_proc ) RET_VALUE_END ; 
    else if (((rator ) == (scheme_box_proc ) ) 
    || ((rator ) == (scheme_box_immutable_proc ) ) )
      RET_VALUE_START (scheme_box_p_proc ) RET_VALUE_END ; 
    else if ((rator ) == (scheme_void_proc ) ) 
      RET_VALUE_START (scheme_void_p_proc ) RET_VALUE_END ; 
    else if ((rator ) == (scheme_procedure_specialize_proc ) ) 
      RET_VALUE_START (scheme_procedure_p_proc ) RET_VALUE_END ; 
    else if ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "vector-set!" ) ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "string-set!" ) ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "bytes-set!" ) ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "set-box!" ) ) ) )
      RET_VALUE_START (scheme_void_p_proc ) RET_VALUE_END ; 
    else if ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "string->symbol" ) ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "gensym" ) ) ) )
      RET_VALUE_START (scheme_symbol_p_proc ) RET_VALUE_END ; 
    else if (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "string->keyword" ) ) ) 
      RET_VALUE_START (scheme_keyword_p_proc ) RET_VALUE_END ; 

    {
      Scheme_Object * p ; 
      BLOCK_SETUP((PUSH(p, 0+XfOrM510_COUNT)));
#     define XfOrM511_COUNT (1+XfOrM510_COUNT)
#     define SETUP_XfOrM511(x) SETUP(XfOrM511_COUNT)
      p = NULLED_OUT ; 

# 3392 "./../src/optimize.c"
      p = (__funcarg86 = FUNCCALL(SETUP_XfOrM511(_), produces_local_type (rator , argc ) ), FUNCCALL_AGAIN(local_type_to_predicate (__funcarg86 ) )) ; 
      if (p )
        RET_VALUE_START (p ) RET_VALUE_END ; 
    }
  }

  {
    Scheme_Object * shape ; 
    BLOCK_SETUP((PUSH(shape, 0+XfOrM503_COUNT)));
#   define XfOrM504_COUNT (1+XfOrM503_COUNT)
#   define SETUP_XfOrM504(x) SETUP(XfOrM504_COUNT)
    shape = NULLED_OUT ; 

# 3400 "./../src/optimize.c"
    shape = FUNCCALL(SETUP_XfOrM504(_), get_struct_proc_shape (rator , info , 1 ) ); 
    if (shape ) {
      if ((Scheme_Type ) (((((intptr_t ) (shape ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (shape ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) )  {
        if ((((Scheme_Struct_Proc_Shape * ) shape ) -> mode & 0x7 ) == 2 )  {
          RET_VALUE_START (scheme_boolean_p_proc ) RET_VALUE_END ; 
        }
      }

# 3406 "./../src/optimize.c"
      else if ((Scheme_Type ) (((((intptr_t ) (shape ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (shape ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_prop_proc_shape_type ) )  {
        if (((Scheme_Small_Object * ) shape ) -> u . int_val == 2 ) {
          RET_VALUE_START (scheme_boolean_p_proc ) RET_VALUE_END ; 
        }
      }
    }
  }

  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 3417 "./../src/optimize.c"
static Scheme_Object * do_expr_implies_predicate (Scheme_Object * expr , Optimize_Info * info , 
int * _involves_k_cross , int fuel , 
Scheme_Hash_Tree * ignore_vars ) 




{
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(7);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(expr, 1), PUSH(ignore_vars, 2), PUSH(_involves_k_cross, 3)));
# define XfOrM518_COUNT (4)
# define SETUP_XfOrM518(x) SETUP(XfOrM518_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 3425 "./../src/optimize.c"
  if (fuel <= 0 )
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

  switch (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ) {
#   define XfOrM520_COUNT (0+XfOrM518_COUNT)
#   define SETUP_XfOrM520(x) SETUP_XfOrM518(x)

# 3429 "./../src/optimize.c"
    case scheme_ir_local_type : 
    {
#     define XfOrM552_COUNT (0+XfOrM520_COUNT)
#     define SETUP_XfOrM552(x) SETUP_XfOrM520(x)

# 3431 "./../src/optimize.c"
      if (scheme_eq_hash_tree_get (ignore_vars , expr ) ) 
        RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

      if (! ((Scheme_IR_Local * ) expr ) -> mutated ) {
        Scheme_Object * p ; 
        BLOCK_SETUP((PUSH(p, 0+XfOrM552_COUNT)));
#       define XfOrM553_COUNT (1+XfOrM552_COUNT)
#       define SETUP_XfOrM553(x) SETUP(XfOrM553_COUNT)
        p = NULLED_OUT ; 

# 3437 "./../src/optimize.c"
        if (info ) {
#         define XfOrM555_COUNT (0+XfOrM553_COUNT)
#         define SETUP_XfOrM555(x) SETUP_XfOrM553(x)

# 3438 "./../src/optimize.c"
          p = FUNCCALL(SETUP_XfOrM555(_), optimize_get_predicate (info , expr , 0 ) ); 
          if (p )
            RET_VALUE_START (p ) RET_VALUE_END ; 
        }

        p = FUNCCALL(SETUP_XfOrM553(_), local_type_to_predicate (((Scheme_IR_Local * ) expr ) -> val_type ) ); 
        if (p ) {
          if (_involves_k_cross 
          && ((Scheme_IR_Local * ) expr ) -> escapes_after_k_tick )
            * _involves_k_cross = 1 ; 
          RET_VALUE_START (p ) RET_VALUE_END ; 
        }

        if ((((Scheme_IR_Local * ) expr ) -> mode == 3 ) 
        && ((Scheme_IR_Local * ) expr ) -> optimize . known_val )
          RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(do_expr_implies_predicate (((Scheme_IR_Local * ) expr ) -> optimize . known_val , info , _involves_k_cross , fuel - 1 , ignore_vars ) )) RET_VALUE_EMPTY_END 
        ; 
      }
    }
    break ; 
    case scheme_application2_type : 
    {
      Scheme_App2_Rec * app = (Scheme_App2_Rec * ) expr ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM520_COUNT)));
#     define XfOrM549_COUNT (1+XfOrM520_COUNT)
#     define SETUP_XfOrM549(x) SETUP(XfOrM549_COUNT)

# 3462 "./../src/optimize.c"
      if (((Scheme_Type ) (((((intptr_t ) (app -> rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
      && scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) app -> rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 23 ) ) {
        Scheme_Object * p ; 
        BLOCK_SETUP((PUSH(p, 0+XfOrM549_COUNT)));
#       define XfOrM551_COUNT (1+XfOrM549_COUNT)
#       define SETUP_XfOrM551(x) SETUP(XfOrM551_COUNT)
        p = NULLED_OUT ; 

# 3465 "./../src/optimize.c"
        p = FUNCCALL(SETUP_XfOrM551(_), do_expr_implies_predicate (app -> rand , info , ((void * ) 0 ) , fuel - 1 , ignore_vars ) ); 
        if (p && (predicate_implies (p , scheme_real_p_proc ) ) )
          RET_VALUE_START (scheme_real_p_proc ) RET_VALUE_END ; 
      }

      if (((app -> rator ) == (scheme_cdr_proc ) ) 
      || ((app -> rator ) == (scheme_unsafe_cdr_proc ) ) ) {
        Scheme_Object * p ; 
        BLOCK_SETUP((PUSH(p, 0+XfOrM549_COUNT)));
#       define XfOrM550_COUNT (1+XfOrM549_COUNT)
#       define SETUP_XfOrM550(x) SETUP(XfOrM550_COUNT)
        p = NULLED_OUT ; 

# 3473 "./../src/optimize.c"
        p = FUNCCALL(SETUP_XfOrM550(_), do_expr_implies_predicate (app -> rand , info , ((void * ) 0 ) , fuel - 1 , ignore_vars ) ); 
        if (predicate_implies (p , scheme_list_p_proc ) ) 
          RET_VALUE_START (scheme_list_p_proc ) RET_VALUE_END ; 
      }

      RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(rator_implies_predicate (app -> rator , info , 1 ) )) RET_VALUE_EMPTY_END ; 
    }
    break ; 
    case scheme_application3_type : 
    {
      Scheme_App3_Rec * app = (Scheme_App3_Rec * ) expr ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM520_COUNT)));
#     define XfOrM541_COUNT (1+XfOrM520_COUNT)
#     define SETUP_XfOrM541(x) SETUP(XfOrM541_COUNT)

# 3484 "./../src/optimize.c"
      if (((Scheme_Type ) (((((intptr_t ) (app -> rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
      && (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) app -> rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 1 ) ) 
      && (! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "bitwise-and" ) ) ) ) {




        if (((((intptr_t ) (app -> rand1 ) ) & 0x1 ) 
        && ((((intptr_t ) (app -> rand1 ) ) >> 1 ) >= 0 ) 
        && ((((((intptr_t ) (app -> rand1 ) ) >> 1 ) ) >= - 1073741824 ) && (((((intptr_t ) (app -> rand1 ) ) >> 1 ) ) <= 1073741823 ) ) ) 
        || ((((intptr_t ) (app -> rand2 ) ) & 0x1 ) 
        && ((((intptr_t ) (app -> rand2 ) ) >> 1 ) >= 0 ) 
        && ((((((intptr_t ) (app -> rand2 ) ) >> 1 ) ) >= - 1073741824 ) && (((((intptr_t ) (app -> rand2 ) ) >> 1 ) ) <= 1073741823 ) ) ) ) {
          RET_VALUE_START (scheme_fixnum_p_proc ) RET_VALUE_END ; 
        }
      }

      if (((Scheme_Type ) (((((intptr_t ) (app -> rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
      && scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) app -> rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 23 ) ) {
        Scheme_Object * p ; 
        BLOCK_SETUP((PUSH(p, 0+XfOrM541_COUNT)));
#       define XfOrM544_COUNT (1+XfOrM541_COUNT)
#       define SETUP_XfOrM544(x) SETUP(XfOrM544_COUNT)
        p = NULLED_OUT ; 

# 3504 "./../src/optimize.c"
        p = FUNCCALL(SETUP_XfOrM544(_), do_expr_implies_predicate (app -> rand1 , info , ((void * ) 0 ) , fuel - 1 , ignore_vars ) ); 
        if (p && (predicate_implies (p , scheme_real_p_proc ) ) ) {
#         define XfOrM545_COUNT (0+XfOrM544_COUNT)
#         define SETUP_XfOrM545(x) SETUP_XfOrM544(x)

# 3506 "./../src/optimize.c"
          p = FUNCCALL(SETUP_XfOrM545(_), do_expr_implies_predicate (app -> rand2 , info , ((void * ) 0 ) , fuel - 1 , ignore_vars ) ); 
          if (p && (predicate_implies (p , scheme_real_p_proc ) ) ) {
            RET_VALUE_START (scheme_real_p_proc ) RET_VALUE_END ; 
          }
        }
      }

      if ((app -> rator ) == (scheme_cons_proc ) )  {
        Scheme_Object * p ; 
        BLOCK_SETUP((PUSH(p, 0+XfOrM541_COUNT)));
#       define XfOrM543_COUNT (1+XfOrM541_COUNT)
#       define SETUP_XfOrM543(x) SETUP(XfOrM543_COUNT)
        p = NULLED_OUT ; 

# 3515 "./../src/optimize.c"
        p = FUNCCALL(SETUP_XfOrM543(_), do_expr_implies_predicate (app -> rand2 , info , ((void * ) 0 ) , fuel - 1 , ignore_vars ) ); 
        if (((p ) == (scheme_list_pair_p_proc ) ) 
        || ((p ) == (scheme_list_p_proc ) ) 
        || ((p ) == (scheme_null_p_proc ) ) )
          RET_VALUE_START (scheme_list_pair_p_proc ) RET_VALUE_END ; 
      }

      if (((Scheme_Type ) (((((intptr_t ) (app -> rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
      && (! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "append" ) ) ) ) {
        Scheme_Object * p ; 
        BLOCK_SETUP((PUSH(p, 0+XfOrM541_COUNT)));
#       define XfOrM542_COUNT (1+XfOrM541_COUNT)
#       define SETUP_XfOrM542(x) SETUP(XfOrM542_COUNT)
        p = NULLED_OUT ; 

# 3525 "./../src/optimize.c"
        p = FUNCCALL(SETUP_XfOrM542(_), do_expr_implies_predicate (app -> rand2 , info , ((void * ) 0 ) , fuel - 1 , ignore_vars ) ); 
        if ((p ) == (scheme_list_pair_p_proc ) ) 
          RET_VALUE_START (scheme_list_pair_p_proc ) RET_VALUE_END ; 
        if (((p ) == (scheme_list_p_proc ) ) 
        || ((p ) == (scheme_null_p_proc ) ) )
          RET_VALUE_START (scheme_list_p_proc ) RET_VALUE_END ; 
      }

      RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(rator_implies_predicate (app -> rator , info , 2 ) )) RET_VALUE_EMPTY_END ; 
    }
    break ; 
    case scheme_application_type : 
    {
      Scheme_App_Rec * app = (Scheme_App_Rec * ) expr ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM520_COUNT)));
#     define XfOrM536_COUNT (1+XfOrM520_COUNT)
#     define SETUP_XfOrM536(x) SETUP(XfOrM536_COUNT)

# 3540 "./../src/optimize.c"
      if (((Scheme_Type ) (((((intptr_t ) (app -> args [0 ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> args [0 ] ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
      && scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) app -> args [0 ] ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 23 ) ) {
        Scheme_Object * p ; 
        int i ; 
        BLOCK_SETUP((PUSH(p, 0+XfOrM536_COUNT)));
#       define XfOrM538_COUNT (1+XfOrM536_COUNT)
#       define SETUP_XfOrM538(x) SETUP(XfOrM538_COUNT)
        p = NULLED_OUT ; 

# 3544 "./../src/optimize.c"
        for (i = 0 ; i < app -> num_args ; i ++ ) {
#         define XfOrM540_COUNT (0+XfOrM538_COUNT)
#         define SETUP_XfOrM540(x) SETUP_XfOrM538(x)

# 3545 "./../src/optimize.c"
          p = FUNCCALL(SETUP_XfOrM540(_), do_expr_implies_predicate (app -> args [i + 1 ] , info , ((void * ) 0 ) , fuel - 1 , ignore_vars ) ); 
          if (! p || ! (predicate_implies (p , scheme_real_p_proc ) ) )
            break ; 
        }
        if (i >= app -> num_args )
          RET_VALUE_START (scheme_real_p_proc ) RET_VALUE_END ; 
      }

      if (((Scheme_Type ) (((((intptr_t ) (app -> args [0 ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> args [0 ] ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
      && (! (strcmp (((Scheme_Primitive_Proc * ) app -> args [0 ] ) -> name , "append" ) ) ) ) {
        Scheme_Object * p ; 
        BLOCK_SETUP((PUSH(p, 0+XfOrM536_COUNT)));
#       define XfOrM537_COUNT (1+XfOrM536_COUNT)
#       define SETUP_XfOrM537(x) SETUP(XfOrM537_COUNT)
        p = NULLED_OUT ; 

# 3556 "./../src/optimize.c"
        p = FUNCCALL(SETUP_XfOrM537(_), do_expr_implies_predicate (app -> args [app -> num_args ] , info , ((void * ) 0 ) , fuel - 1 , ignore_vars ) ); 
        if ((p ) == (scheme_list_pair_p_proc ) ) 
          RET_VALUE_START (scheme_list_pair_p_proc ) RET_VALUE_END ; 
        if (((p ) == (scheme_list_p_proc ) ) 
        || ((p ) == (scheme_null_p_proc ) ) )
          RET_VALUE_START (scheme_list_p_proc ) RET_VALUE_END ; 
      }

      RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(rator_implies_predicate (app -> args [0 ] , info , app -> num_args ) )) RET_VALUE_EMPTY_END ; 
    }
    break ; 
    case scheme_ir_lambda_type : 
    RET_VALUE_START (scheme_procedure_p_proc ) RET_VALUE_END ; 
    break ; 
    case scheme_case_lambda_sequence_type : 
    RET_VALUE_START (scheme_procedure_p_proc ) RET_VALUE_END ; 
    break ; 
    case scheme_branch_type : 
    {
      Scheme_Object * l , * r ; 
      Scheme_Branch_Rec * b = (Scheme_Branch_Rec * ) expr ; 
      BLOCK_SETUP((PUSH(b, 0+XfOrM520_COUNT), PUSH(r, 1+XfOrM520_COUNT), PUSH(l, 2+XfOrM520_COUNT)));
#     define XfOrM534_COUNT (3+XfOrM520_COUNT)
#     define SETUP_XfOrM534(x) SETUP(XfOrM534_COUNT)
      l = NULLED_OUT ; 
      r = NULLED_OUT ; 

# 3577 "./../src/optimize.c"
      l = FUNCCALL(SETUP_XfOrM534(_), do_expr_implies_predicate (b -> tbranch , info , _involves_k_cross , fuel - 1 , ignore_vars ) ); 
      if (l ) {
#       define XfOrM535_COUNT (0+XfOrM534_COUNT)
#       define SETUP_XfOrM535(x) SETUP_XfOrM534(x)

# 3579 "./../src/optimize.c"
        r = FUNCCALL(SETUP_XfOrM535(_), do_expr_implies_predicate (b -> fbranch , info , _involves_k_cross , fuel - 1 , ignore_vars ) ); 
        if (predicate_implies (l , r ) ) 
          RET_VALUE_START (r ) RET_VALUE_END ; 
        else if (predicate_implies (r , l ) ) 
          RET_VALUE_START (l ) RET_VALUE_END ; 
        else 
        RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
      }
    }
    break ; 
    case scheme_sequence_type : 
    {
      Scheme_Sequence * seq = (Scheme_Sequence * ) expr ; 
#     define XfOrM533_COUNT (0+XfOrM520_COUNT)
#     define SETUP_XfOrM533(x) SETUP_XfOrM520(x)

# 3593 "./../src/optimize.c"
      RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(do_expr_implies_predicate (seq -> array [seq -> count - 1 ] , info , _involves_k_cross , fuel - 1 , ignore_vars ) )) RET_VALUE_EMPTY_END ; 
    }
    case scheme_with_cont_mark_type : 
    {
      Scheme_With_Continuation_Mark * wcm = (Scheme_With_Continuation_Mark * ) expr ; 
#     define XfOrM532_COUNT (0+XfOrM520_COUNT)
#     define SETUP_XfOrM532(x) SETUP_XfOrM520(x)

# 3599 "./../src/optimize.c"
      RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(do_expr_implies_predicate (wcm -> body , info , _involves_k_cross , fuel - 1 , ignore_vars ) )) RET_VALUE_EMPTY_END ; 
    }
    case scheme_ir_let_header_type : 
    {
      Scheme_IR_Let_Header * lh = (Scheme_IR_Let_Header * ) expr ; 
      Scheme_IR_Let_Value * irlv ; 
      int i , j ; 
      BLOCK_SETUP((PUSH(irlv, 0+XfOrM520_COUNT), PUSH(lh, 1+XfOrM520_COUNT)));
#     define XfOrM525_COUNT (2+XfOrM520_COUNT)
#     define SETUP_XfOrM525(x) SETUP(XfOrM525_COUNT)
      irlv = NULLED_OUT ; 

# 3606 "./../src/optimize.c"
      expr = lh -> body ; 
      for (i = 0 ; i < lh -> num_clauses ; i ++ ) {
#       define XfOrM529_COUNT (0+XfOrM525_COUNT)
#       define SETUP_XfOrM529(x) SETUP_XfOrM525(x)

# 3608 "./../src/optimize.c"
        irlv = (Scheme_IR_Let_Value * ) expr ; 
        for (j = 0 ; j < irlv -> count ; j ++ ) {
#         define XfOrM531_COUNT (0+XfOrM529_COUNT)
#         define SETUP_XfOrM531(x) SETUP_XfOrM529(x)

# 3610 "./../src/optimize.c"
          ignore_vars = FUNCCALL(SETUP_XfOrM531(_), scheme_hash_tree_set (ignore_vars , (Scheme_Object * ) irlv -> vars [j ] , scheme_true ) )
          ; 
        }
        expr = irlv -> body ; 
      }
      RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(do_expr_implies_predicate (expr , info , _involves_k_cross , fuel - 1 , ignore_vars ) )) RET_VALUE_EMPTY_END ; 
    }
    break ; 
    case scheme_begin0_sequence_type : 
    {
      Scheme_Sequence * seq = (Scheme_Sequence * ) expr ; 
#     define XfOrM524_COUNT (0+XfOrM520_COUNT)
#     define SETUP_XfOrM524(x) SETUP_XfOrM520(x)

# 3622 "./../src/optimize.c"
      RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(do_expr_implies_predicate (seq -> array [0 ] , info , _involves_k_cross , fuel - 1 , ignore_vars ) )) RET_VALUE_EMPTY_END ; 
    }
    case scheme_vector_type : 
    RET_VALUE_START (scheme_vector_p_proc ) RET_VALUE_END ; 
    break ; 
    case scheme_box_type : 
    RET_VALUE_START (scheme_box_p_proc ) RET_VALUE_END ; 
    break ; 
    default : 
    if ((((intptr_t ) (expr ) ) & 0x1 ) || ((((expr ) -> type ) >= scheme_bignum_type ) && (((expr ) -> type ) <= scheme_complex_type ) ) )  {
      if (((Scheme_Type ) (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ) == (Scheme_Type ) (scheme_float_type ) ) || ((Scheme_Type ) (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ) == (Scheme_Type ) (scheme_double_type ) ) ) 
        RET_VALUE_START (scheme_flonum_p_proc ) RET_VALUE_END ; 
      if ((Scheme_Type ) (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ) == (Scheme_Type ) (scheme_long_double_type ) ) 
        RET_VALUE_START (scheme_extflonum_p_proc ) RET_VALUE_END ; 
      if ((((intptr_t ) (expr ) ) & 0x1 ) 
      && ((((((intptr_t ) (expr ) ) >> 1 ) ) >= - 1073741824 ) && (((((intptr_t ) (expr ) ) >> 1 ) ) <= 1073741823 ) ) )
        RET_VALUE_START (scheme_fixnum_p_proc ) RET_VALUE_END ; 
      if ((((intptr_t ) (expr ) ) & 0x1 ) || ((((expr ) -> type ) >= scheme_bignum_type ) && (((expr ) -> type ) < scheme_complex_type ) ) ) 
        RET_VALUE_START (scheme_real_p_proc ) RET_VALUE_END ; 
      RET_VALUE_START (scheme_number_p_proc ) RET_VALUE_END ; 
    }

    if ((expr ) == (scheme_null ) ) 
      RET_VALUE_START (scheme_null_p_proc ) RET_VALUE_END ; 
    if ((Scheme_Type ) (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ) == (Scheme_Type ) (scheme_pair_type ) )  {
#     define XfOrM522_COUNT (0+XfOrM520_COUNT)
#     define SETUP_XfOrM522(x) SETUP_XfOrM520(x)

# 3647 "./../src/optimize.c"
      if (scheme_is_list (expr ) ) 
        RET_VALUE_START (scheme_list_pair_p_proc ) RET_VALUE_END ; 
      RET_VALUE_START (scheme_pair_p_proc ) RET_VALUE_END ; 
    }
    if ((Scheme_Type ) (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ) == (Scheme_Type ) (scheme_mutable_pair_type ) ) 
      RET_VALUE_START (scheme_mpair_p_proc ) RET_VALUE_END ; 
    if ((Scheme_Type ) (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ) == (Scheme_Type ) (scheme_char_string_type ) ) 
      RET_VALUE_START (scheme_string_p_proc ) RET_VALUE_END ; 
    if ((Scheme_Type ) (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ) == (Scheme_Type ) (scheme_byte_string_type ) ) 
      RET_VALUE_START (scheme_byte_string_p_proc ) RET_VALUE_END ; 
    if (((expr ) ) == (scheme_void ) ) 
      RET_VALUE_START (scheme_void_p_proc ) RET_VALUE_END ; 
    if (((expr ) ) == (scheme_eof ) ) 
      RET_VALUE_START (scheme_eof_object_p_proc ) RET_VALUE_END ; 
    if ((Scheme_Type ) (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ) == (Scheme_Type ) (scheme_keyword_type ) ) 
      RET_VALUE_START (scheme_keyword_p_proc ) RET_VALUE_END ; 
    if ((Scheme_Type ) (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ) == (Scheme_Type ) (scheme_symbol_type ) ) 
      RET_VALUE_START (scheme_symbol_p_proc ) RET_VALUE_END ; 
    if ((Scheme_Type ) (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ) == (Scheme_Type ) (scheme_char_type ) )  {
      if ((((Scheme_Small_Object * ) (expr ) ) -> u . char_val ) < 256 )
        RET_VALUE_START (scheme_interned_char_p_proc ) RET_VALUE_END ; 
      RET_VALUE_START (scheme_char_p_proc ) RET_VALUE_END ; 
    }
    if ((expr ) == (scheme_true ) ) 
      RET_VALUE_START (scheme_true_object_p_proc ) RET_VALUE_END ; 
    if (((expr ) ) == (scheme_false ) ) 
      RET_VALUE_START (scheme_not_proc ) RET_VALUE_END ; 
    if (! (((intptr_t ) (expr ) ) & 0x1 ) && ((((expr ) -> type ) >= scheme_prim_type ) && (((expr ) -> type ) <= scheme_proc_chaperone_type ) ) ) 
      RET_VALUE_START (scheme_procedure_p_proc ) RET_VALUE_END ; 
    if ((Scheme_Type ) (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ) == (Scheme_Type ) (scheme_long_double_type ) ) 
      RET_VALUE_START (scheme_extflonum_p_proc ) RET_VALUE_END ; 
  }


  if (info 
  && FUNCCALL_EMPTY(lookup_constant_proc (info , expr , - 1 ) )) {
    RET_VALUE_START (scheme_procedure_p_proc ) RET_VALUE_END ; 
  }

  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 3689 "./../src/optimize.c"
static Scheme_Object * expr_implies_predicate (Scheme_Object * expr , Optimize_Info * info ) 
{
  /* No conversion */

# 3691 "./../src/optimize.c"
  return do_expr_implies_predicate (expr , info , ((void * ) 0 ) , 5 , empty_eq_hash_tree ) ; 
}

static Scheme_Object * finish_optimize_app (Scheme_Object * o , Optimize_Info * info , int context ) 
{
  /* No conversion */

# 3696 "./../src/optimize.c"
  switch (((((intptr_t ) (o ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o ) ) -> type ) ) {
    case scheme_application_type : 
    return finish_optimize_application ((Scheme_App_Rec * ) o , info , context ) ; 
    case scheme_application2_type : 
    return finish_optimize_application2 ((Scheme_App2_Rec * ) o , info , context ) ; 
    case scheme_application3_type : 
    return finish_optimize_application3 ((Scheme_App3_Rec * ) o , info , context ) ; 
    default : 
    return o ; 
  }
}

static Scheme_Object * direct_apply (Scheme_Object * expr , Scheme_Object * rator , Scheme_Object * last_rand , Optimize_Info * info ) 

{
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(4);
  BLOCK_SETUP_TOP((PUSH(expr, 0), PUSH(info, 1), PUSH(last_rand, 2)));
# define XfOrM559_COUNT (3)
# define SETUP_XfOrM559(x) SETUP(XfOrM559_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 3711 "./../src/optimize.c"
  if ((rator ) == (scheme_apply_proc ) )  {
#   define XfOrM560_COUNT (0+XfOrM559_COUNT)
#   define SETUP_XfOrM560(x) SETUP_XfOrM559(x)

# 3712 "./../src/optimize.c"
    switch (((((intptr_t ) (last_rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (last_rand ) ) -> type ) ) {
#     define XfOrM568_COUNT (0+XfOrM560_COUNT)
#     define SETUP_XfOrM568(x) SETUP_XfOrM560(x)

# 3713 "./../src/optimize.c"
      case scheme_application_type : 
      rator = ((Scheme_App_Rec * ) last_rand ) -> args [0 ] ; 
      break ; 
      case scheme_application2_type : 
      rator = ((Scheme_App2_Rec * ) last_rand ) -> rator ; 
      break ; 
      case scheme_application3_type : 
      rator = ((Scheme_App3_Rec * ) last_rand ) -> rator ; 
      break ; 
      case scheme_pair_type : 
      if (scheme_is_list (last_rand ) ) 
        rator = scheme_list_proc ; 
      else 
      rator = ((void * ) 0 ) ; 
      break ; 
      case scheme_null_type : 
      rator = scheme_list_proc ; 
      break ; 
      default : 
      rator = ((void * ) 0 ) ; 
      break ; 
    }

    if (rator && ((rator ) == (scheme_list_proc ) ) ) {


      Scheme_Object * l = scheme_null ; 
      int i ; 
      BLOCK_SETUP((PUSH(l, 0+XfOrM560_COUNT)));
#     define XfOrM561_COUNT (1+XfOrM560_COUNT)
#     define SETUP_XfOrM561(x) SETUP(XfOrM561_COUNT)

# 3742 "./../src/optimize.c"
      switch (((((intptr_t ) (last_rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (last_rand ) ) -> type ) ) {
#       define XfOrM565_COUNT (0+XfOrM561_COUNT)
#       define SETUP_XfOrM565(x) SETUP_XfOrM561(x)

# 3743 "./../src/optimize.c"
        case scheme_application_type : 
        for (i = ((Scheme_App_Rec * ) last_rand ) -> num_args ; i -- ; ) {
#         define XfOrM567_COUNT (0+XfOrM565_COUNT)
#         define SETUP_XfOrM567(x) SETUP_XfOrM565(x)

# 3745 "./../src/optimize.c"
          l = FUNCCALL(SETUP_XfOrM567(_), scheme_make_pair (((Scheme_App_Rec * ) last_rand ) -> args [i + 1 ] , l ) ); 
        }
        break ; 
        case scheme_application2_type : 
        l = FUNCCALL(SETUP_XfOrM565(_), scheme_make_pair (((Scheme_App2_Rec * ) last_rand ) -> rand , l ) ); 
        break ; 
        case scheme_application3_type : 
        l = FUNCCALL(SETUP_XfOrM565(_), scheme_make_pair (((Scheme_App3_Rec * ) last_rand ) -> rand2 , l ) ); 
        l = FUNCCALL(SETUP_XfOrM565(_), scheme_make_pair (((Scheme_App3_Rec * ) last_rand ) -> rand1 , l ) ); 
        break ; 
        case scheme_pair_type : 
        l = last_rand ; 
        break ; 
        case scheme_null_type : 
        l = scheme_null ; 
        break ; 
      }

      switch (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ) {
#       define XfOrM562_COUNT (0+XfOrM561_COUNT)
#       define SETUP_XfOrM562(x) SETUP_XfOrM561(x)

# 3764 "./../src/optimize.c"
        case scheme_application_type : 
        for (i = ((Scheme_App_Rec * ) expr ) -> num_args - 1 ; i -- ; ) {
#         define XfOrM564_COUNT (0+XfOrM562_COUNT)
#         define SETUP_XfOrM564(x) SETUP_XfOrM562(x)

# 3766 "./../src/optimize.c"
          l = FUNCCALL(SETUP_XfOrM564(_), scheme_make_pair (((Scheme_App_Rec * ) expr ) -> args [i + 1 ] , l ) ); 
        }
        break ; 
        default : 
        case scheme_application3_type : 
        l = FUNCCALL(SETUP_XfOrM562(_), scheme_make_pair (((Scheme_App3_Rec * ) expr ) -> rand1 , l ) ); 
        break ; 
      }

      RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(scheme_make_application (l , info ) )) RET_VALUE_EMPTY_END ; 
    }
  }

  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 3782 "./../src/optimize.c"
static Scheme_Object * call_with_immed_mark (Scheme_Object * rator , 
Scheme_Object * rand1 , 
Scheme_Object * rand2 , 
Scheme_Object * rand3 , 
Optimize_Info * info ) 


{
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(5);
  BLOCK_SETUP_TOP((PUSH(rand1, 0), PUSH(rand3, 1), PUSH(rand2, 2)));
# define XfOrM569_COUNT (3)
# define SETUP_XfOrM569(x) SETUP(XfOrM569_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 3790 "./../src/optimize.c"
  if (((rator ) == (scheme_call_with_immed_mark_proc ) ) 
  && ((Scheme_Type ) (((((intptr_t ) (rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) 
  && (((Scheme_Lambda * ) rand2 ) -> num_params == 1 ) 
  && ! ((& (((Scheme_Lambda * ) rand2 ) ) -> iso ) -> so . keyex & 1 ) ) {
    Scheme_With_Continuation_Mark * wcm ; 
    Scheme_Object * e ; 
    BLOCK_SETUP((PUSH(wcm, 0+XfOrM569_COUNT), PUSH(e, 1+XfOrM569_COUNT)));
#   define XfOrM570_COUNT (2+XfOrM569_COUNT)
#   define SETUP_XfOrM570(x) SETUP(XfOrM570_COUNT)
    wcm = NULLED_OUT ; 
    e = NULLED_OUT ; 

# 3797 "./../src/optimize.c"
    wcm = ((Scheme_With_Continuation_Mark * ) FUNCCALL(SETUP_XfOrM570(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_With_Continuation_Mark ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
    wcm -> so . type = scheme_with_immed_mark_type ; 

    wcm -> key = rand1 ; 
    wcm -> val = (rand3 ? rand3 : scheme_false ) ; 

    e = (Scheme_Object * ) ((Scheme_Lambda * ) rand2 ) -> ir_info -> vars [0 ] ; 
    e = FUNCCALL(SETUP_XfOrM570(_), scheme_make_mutable_pair (e , ((Scheme_Lambda * ) rand2 ) -> body ) ); 
    wcm -> body = e ; 

    RET_VALUE_START ((Scheme_Object * ) wcm ) RET_VALUE_END ; 
  }

  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 3813 "./../src/optimize.c"
static Scheme_Object * optimize_application (Scheme_Object * o , Optimize_Info * info , int context ) 
{
  Scheme_Object * le ; 
  Scheme_App_Rec * app ; 
  int i , n , rator_apply_escapes = 0 , sub_context = 0 ; 
  Optimize_Info_Sequence info_seq ; 
  Scheme_Object * __funcarg88 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(6);
  BLOCK_SETUP_TOP((PUSH(o, 0), PUSH(info, 1), PUSH(app, 2), PUSH(le, 3)));
# define XfOrM571_COUNT (4)
# define SETUP_XfOrM571(x) SETUP(XfOrM571_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  le = NULLED_OUT ; 
  app = NULLED_OUT ; 

# 3820 "./../src/optimize.c"
  app = (Scheme_App_Rec * ) o ; 


  le = FUNCCALL(SETUP_XfOrM571(_), direct_apply ((Scheme_Object * ) app , app -> args [0 ] , app -> args [app -> num_args ] , info ) ); 
  if (le )
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_expr (le , info , context ) )) RET_VALUE_EMPTY_END ; 

  if (app -> num_args == 3 ) {
#   define XfOrM592_COUNT (0+XfOrM571_COUNT)
#   define SETUP_XfOrM592(x) SETUP_XfOrM571(x)

# 3828 "./../src/optimize.c"
    le = FUNCCALL(SETUP_XfOrM592(_), call_with_immed_mark (app -> args [0 ] , app -> args [1 ] , app -> args [2 ] , app -> args [3 ] , info ) ); 
    if (le )
      RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_expr (le , info , context ) )) RET_VALUE_EMPTY_END ; 
  }

  le = FUNCCALL(SETUP_XfOrM571(_), check_app_let_rator (o , app -> args [0 ] , info , app -> num_args , context ) ); 
  if (le )
    RET_VALUE_START (le ) RET_VALUE_END ; 

  n = app -> num_args + 1 ; 
  FUNCCALL(SETUP_XfOrM571(_), optimize_info_seq_init (info , & info_seq ) )
  ; 

  for (i = 0 ; i < n ; i ++ ) {
#   define XfOrM583_COUNT (0+XfOrM571_COUNT)
#   define SETUP_XfOrM583(x) SETUP_XfOrM571(x)

# 3842 "./../src/optimize.c"
    if (! i ) {
#     define XfOrM591_COUNT (0+XfOrM583_COUNT)
#     define SETUP_XfOrM591(x) SETUP_XfOrM583(x)

# 3843 "./../src/optimize.c"
      le = FUNCCALL(SETUP_XfOrM591(_), optimize_for_inline (info , app -> args [i ] , n - 1 , app , ((void * ) 0 ) , ((void * ) 0 ) , context , 0 ) ); 
      if (le )
        RET_VALUE_START (le ) RET_VALUE_END ; 
    }

    sub_context = 0x4 ; 
    if (i > 0 ) {
      int ty ; 
#     define XfOrM590_COUNT (0+XfOrM583_COUNT)
#     define SETUP_XfOrM590(x) SETUP_XfOrM583(x)

# 3851 "./../src/optimize.c"
      ty = (wants_local_type_arguments (app -> args [0 ] , i - 1 ) ) ; 
      if (ty )
        sub_context |= (ty << 4 ) ; 
    }
    FUNCCALL(SETUP_XfOrM583(_), optimize_info_seq_step (info , & info_seq ) )
    ; 
    le = FUNCCALL(SETUP_XfOrM583(_), optimize_expr (app -> args [i ] , info , sub_context ) ); 
    app -> args [i ] = le ; 
    if (info -> escapes ) {
      int j ; 
      Scheme_Object * e , * l ; 
      BLOCK_SETUP((PUSH(e, 0+XfOrM583_COUNT), PUSH(l, 1+XfOrM583_COUNT)));
#     define XfOrM585_COUNT (2+XfOrM583_COUNT)
#     define SETUP_XfOrM585(x) SETUP(XfOrM585_COUNT)
      e = NULLED_OUT ; 
      l = NULLED_OUT ; 
      FUNCCALL(SETUP_XfOrM585(_), optimize_info_seq_done (info , & info_seq ) )
# 3862 "./../src/optimize.c"
      ; 

      l = FUNCCALL_AGAIN(scheme_make_pair (app -> args [i ] , scheme_null ) ); 

      for (j = i - 1 ; j >= 0 ; j -- ) {
#       define XfOrM588_COUNT (0+XfOrM585_COUNT)
#       define SETUP_XfOrM588(x) SETUP_XfOrM585(x)

# 3867 "./../src/optimize.c"
        e = app -> args [j ] ; 
        e = FUNCCALL(SETUP_XfOrM588(_), optimize_ignored (e , info , 1 , 1 , 5 ) ); 
        if (e ) {
#         define XfOrM589_COUNT (0+XfOrM588_COUNT)
#         define SETUP_XfOrM589(x) SETUP_XfOrM588(x)

# 3870 "./../src/optimize.c"
          e = FUNCCALL(SETUP_XfOrM589(_), ensure_single_value (e , info ) ); 
          l = FUNCCALL(SETUP_XfOrM589(_), scheme_make_pair (e , l ) ); 
        }
      }
      RET_VALUE_START ((__funcarg88 = FUNCCALL(SETUP_XfOrM585(_), scheme_make_sequence_compilation (l , 1 , 0 ) ), FUNCCALL_EMPTY(ensure_noncm (__funcarg88 , info ) )) ) RET_VALUE_END ; 
    }

    if (! i ) {
#     define XfOrM584_COUNT (0+XfOrM583_COUNT)
#     define SETUP_XfOrM584(x) SETUP_XfOrM583(x)

# 3879 "./../src/optimize.c"
      le = FUNCCALL(SETUP_XfOrM584(_), optimize_for_inline (info , app -> args [i ] , n - 1 , app , ((void * ) 0 ) , ((void * ) 0 ) , context , 1 ) ); 
      if (le )
        RET_VALUE_START (le ) RET_VALUE_END ; 
      if (((app -> args [0 ] ) == (scheme_values_proc ) ) 
      || ((app -> args [0 ] ) == (scheme_apply_proc ) ) )
        info -> maybe_values_argument = 1 ; 
      rator_apply_escapes = info -> escapes ; 
    }
  }
  FUNCCALL(SETUP_XfOrM571(_), optimize_info_seq_done (info , & info_seq ) )
  ; 


  le = FUNCCALL(SETUP_XfOrM571(_), direct_apply ((Scheme_Object * ) app , app -> args [0 ] , app -> args [app -> num_args ] , info ) ); 
  if (le )

# 3893 "./../src/optimize.c"
  RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(finish_optimize_app (le , info , context ) )) RET_VALUE_EMPTY_END ; 



  if ((app -> num_args == 3 ) 
  && ((scheme_hash_ref_proc ) == (app -> args [0 ] ) ) 
  && ((((((intptr_t ) (app -> args [1 ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> args [1 ] ) ) -> type ) >= scheme_hash_tree_type ) && (((((intptr_t ) (app -> args [1 ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> args [1 ] ) ) -> type ) <= scheme_hash_tree_indirection_type ) ) 
  && ((Scheme_Type ) (scheme_ir_lambda_type ) == (Scheme_Type ) (((((intptr_t ) (app -> args [3 ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> args [3 ] ) ) -> type ) ) ) 
  && (((Scheme_Lambda * ) (app -> args [3 ] ) ) -> num_params == 0 ) 
  && (((((intptr_t ) (((Scheme_Lambda * ) app -> args [3 ] ) -> body ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (((Scheme_Lambda * ) app -> args [3 ] ) -> body ) ) -> type ) > _scheme_ir_values_types_ ) 
  && ! (! (((intptr_t ) (((Scheme_Lambda * ) app -> args [3 ] ) -> body ) ) & 0x1 ) && ((((((Scheme_Lambda * ) app -> args [3 ] ) -> body ) -> type ) >= scheme_prim_type ) && (((((Scheme_Lambda * ) app -> args [3 ] ) -> body ) -> type ) <= scheme_proc_chaperone_type ) ) ) ) {
    app -> args [3 ] = ((Scheme_Lambda * ) app -> args [3 ] ) -> body ; 
  }

  if (rator_apply_escapes ) {
    info -> escapes = 1 ; 
    (& (app ) -> iso ) -> so . keyex |= ((1 << 12 ) | (1 << 13 ) ) ; 
  }

  RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(finish_optimize_application (app , info , context ) )) RET_VALUE_EMPTY_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 3915 "./../src/optimize.c"
static int appn_flags (Scheme_Object * rator , Optimize_Info * info ) 

{
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(1);
  BLOCK_SETUP_TOP((PUSH(rator, 0)));
# define XfOrM593_COUNT (1)
# define SETUP_XfOrM593(x) SETUP(XfOrM593_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 3918 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_toplevel_type ) )  {
#   define XfOrM595_COUNT (0+XfOrM593_COUNT)
#   define SETUP_XfOrM595(x) SETUP_XfOrM593(x)

# 3919 "./../src/optimize.c"
    rator = FUNCCALL(SETUP_XfOrM595(_), get_defn_shape (info , (Scheme_IR_Toplevel * ) rator ) ); 
    rator = FUNCCALL(SETUP_XfOrM595(_), no_potential_size (rator ) ); 
    if (! rator )

# 3921 "./../src/optimize.c"
    RET_VALUE_START (0 ) RET_VALUE_END ; 
    if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_proc_shape_type ) )  {
      RET_VALUE_START ((1 << 13 ) ) RET_VALUE_END ; 
    }

# 3924 "./../src/optimize.c"
    else if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) )  {
      int ps = ((Scheme_Struct_Proc_Shape * ) rator ) -> mode & 0x7 ; 
      if ((ps == 2 ) 
      || (ps == 3 ) 
      || (ps == 4 ) 
      || (ps == 1 ) )
        RET_VALUE_START (((1 << 12 ) | (1 << 13 ) ) ) RET_VALUE_END ; 
      RET_VALUE_START (0 ) RET_VALUE_END ; 
    }
  }

  if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) )  {
    int opt = ((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) & (1 | 2 ) ) ; 
    if (opt >= 2 )
      RET_VALUE_START (((1 << 12 ) | (1 << 13 ) ) ) RET_VALUE_END ; 
    RET_VALUE_START (0 ) RET_VALUE_END ; 
  }

  if ((((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) 
  || ((Scheme_Type ) (scheme_noninline_proc_type ) == (Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) ) )
    RET_VALUE_START ((1 << 13 ) ) RET_VALUE_END ; 

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 3951 "./../src/optimize.c"
static int check_known_variant (Optimize_Info * info , Scheme_Object * app , 
Scheme_Object * rator , Scheme_Object * rand , 
const char * who , Scheme_Object * expect_pred , 
Scheme_Object * unsafe , int unsafe_mode , 
Scheme_Object * implies_pred ) 
# 3965 "./../src/optimize.c"
{
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK(9);
  BLOCK_SETUP_TOP((PUSH(expect_pred, 0), PUSH(info, 1), PUSH(rand, 2), PUSH(who, 3), PUSH(unsafe, 4), PUSH(implies_pred, 5), PUSH(app, 6), PUSH(rator, 7)));
# define XfOrM598_COUNT (8)
# define SETUP_XfOrM598(x) SETUP(XfOrM598_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)







  ((void ) sizeof ((((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) ) ? 1 : 0 ) , __extension__ ({
#   define XfOrM605_COUNT (0+XfOrM598_COUNT)
#   define SETUP_XfOrM605(x) SETUP_XfOrM598(x)

# 3981 "./../src/optimize.c" 3
    if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 

# 3981 "./../src/optimize.c" 3
    ; 

# 3981 "./../src/optimize.c" 3
    else FUNCCALL(SETUP_XfOrM605(_), __assert_fail ("((Scheme_Type)(((((intptr_t)(rator)) & 0x1)?(Scheme_Type)scheme_integer_type:((Scheme_Object *)(rator))->type)) == (Scheme_Type)(scheme_prim_type))" , "./../src/optimize.c" , 3981 , __extension__ __PRETTY_FUNCTION__ ) ); 
  }
  ) ) 
# 3981 "./../src/optimize.c"
  ; 
  if (! who || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , who ) ) ) ) {
    Scheme_Object * pred ; 
    BLOCK_SETUP((PUSH(pred, 0+XfOrM598_COUNT)));
#   define XfOrM599_COUNT (1+XfOrM598_COUNT)
#   define SETUP_XfOrM599(x) SETUP(XfOrM599_COUNT)
    pred = NULLED_OUT ; 

# 3985 "./../src/optimize.c"
    if (unsafe_mode )
      pred = expect_pred ; 
    else 
    pred = FUNCCALL(SETUP_XfOrM599(_), expr_implies_predicate (rand , info ) ); 

    if (pred ) {
#     define XfOrM601_COUNT (0+XfOrM599_COUNT)
#     define SETUP_XfOrM601(x) SETUP_XfOrM599(x)

# 3991 "./../src/optimize.c"
      if (predicate_implies (pred , expect_pred ) )  {
#       define XfOrM603_COUNT (0+XfOrM601_COUNT)
#       define SETUP_XfOrM603(x) SETUP_XfOrM601(x)

# 3992 "./../src/optimize.c"
        if (unsafe ) {
#         define XfOrM604_COUNT (0+XfOrM603_COUNT)
#         define SETUP_XfOrM604(x) SETUP_XfOrM603(x)

# 3993 "./../src/optimize.c"
          if ((unsafe ) == (scheme_true ) ) 
            FUNCCALL(SETUP_XfOrM604(_), set_application_omittable (app , unsafe ) ); 
          else FUNCCALL(SETUP_XfOrM604(_), reset_rator (app , unsafe ) )
          ; 
        }
        RET_VALUE_START (1 ) RET_VALUE_END ; 
      }

# 3999 "./../src/optimize.c"
      else if (predicate_implies_not (pred , implies_pred ) )  {
        info -> escapes = 1 ; 
      }
    }

# 4002 "./../src/optimize.c"
    else {
#     define XfOrM600_COUNT (0+XfOrM599_COUNT)
#     define SETUP_XfOrM600(x) SETUP_XfOrM599(x)

# 4003 "./../src/optimize.c"
      if ((Scheme_Type ) (((((intptr_t ) (rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rand ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) 
        FUNCCALL_EMPTY(add_type (info , rand , implies_pred ) ); 
    }
  }

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 4011 "./../src/optimize.c"
static void check_known (Optimize_Info * info , Scheme_Object * app , 
Scheme_Object * rator , Scheme_Object * rand , 
const char * who , Scheme_Object * expect_pred , 
Scheme_Object * unsafe , int unsafe_mode ) 

{
  /* No conversion */

# 4017 "./../src/optimize.c"
  (void ) check_known_variant (info , app , rator , rand , who , expect_pred , unsafe , unsafe_mode , expect_pred ) ; 
}

static void check_known_rator (Optimize_Info * info , Scheme_Object * rator ) 

{
  Scheme_Object * pred ; 
  PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(pred, 1), PUSH(rator, 2)));
# define XfOrM607_COUNT (3)
# define SETUP_XfOrM607(x) SETUP(XfOrM607_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  pred = NULLED_OUT ; 

# 4025 "./../src/optimize.c"
  pred = FUNCCALL(SETUP_XfOrM607(_), expr_implies_predicate (rator , info ) ); 
  if (pred ) {
#   define XfOrM609_COUNT (0+XfOrM607_COUNT)
#   define SETUP_XfOrM609(x) SETUP_XfOrM607(x)

# 4027 "./../src/optimize.c"
    if (predicate_implies_not (pred , scheme_procedure_p_proc ) ) 
      info -> escapes = 1 ; 
  }

# 4029 "./../src/optimize.c"
  else {
#   define XfOrM608_COUNT (0+XfOrM607_COUNT)
#   define SETUP_XfOrM608(x) SETUP_XfOrM607(x)

# 4030 "./../src/optimize.c"
    if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) 
      FUNCCALL_EMPTY(add_type (info , rator , scheme_procedure_p_proc ) ); 
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 4035 "./../src/optimize.c"
static void check_known_both_try (Optimize_Info * info , Scheme_Object * app , 
Scheme_Object * rator , Scheme_Object * rand1 , Scheme_Object * rand2 , 
const char * who , Scheme_Object * expect_pred , 
Scheme_Object * unsafe , int unsafe_mode ) 


{
  PREPARE_VAR_STACK(10);
  BLOCK_SETUP_TOP((PUSH(rand1, 0), PUSH(expect_pred, 1), PUSH(info, 2), PUSH(rand2, 3), PUSH(who, 4), PUSH(unsafe, 5), PUSH(app, 6), PUSH(rator, 7)));
# define XfOrM610_COUNT (8)
# define SETUP_XfOrM610(x) SETUP(XfOrM610_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 4042 "./../src/optimize.c"
  ((void ) sizeof ((((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) ) ? 1 : 0 ) , __extension__ ({
#   define XfOrM616_COUNT (0+XfOrM610_COUNT)
#   define SETUP_XfOrM616(x) SETUP_XfOrM610(x)

# 4042 "./../src/optimize.c" 3
    if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 

# 4042 "./../src/optimize.c" 3
    ; 

# 4042 "./../src/optimize.c" 3
    else FUNCCALL(SETUP_XfOrM616(_), __assert_fail ("((Scheme_Type)(((((intptr_t)(rator)) & 0x1)?(Scheme_Type)scheme_integer_type:((Scheme_Object *)(rator))->type)) == (Scheme_Type)(scheme_prim_type))" , "./../src/optimize.c" , 4042 , __extension__ __PRETTY_FUNCTION__ ) ); 
  }
  ) ) 
# 4042 "./../src/optimize.c"
  ; 
  if (! who || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , who ) ) ) ) {
    Scheme_Object * pred1 , * pred2 ; 
    BLOCK_SETUP((PUSH(pred2, 0+XfOrM610_COUNT), PUSH(pred1, 1+XfOrM610_COUNT)));
#   define XfOrM611_COUNT (2+XfOrM610_COUNT)
#   define SETUP_XfOrM611(x) SETUP(XfOrM611_COUNT)
    pred1 = NULLED_OUT ; 
    pred2 = NULLED_OUT ; 

# 4046 "./../src/optimize.c"
    if (unsafe_mode ) {
#     define XfOrM615_COUNT (0+XfOrM611_COUNT)
#     define SETUP_XfOrM615(x) SETUP_XfOrM611(x)
      FUNCCALL(SETUP_XfOrM615(_), reset_rator (app , unsafe ) )
# 4047 "./../src/optimize.c"
      ; 
    }

# 4048 "./../src/optimize.c"
    else {
#     define XfOrM612_COUNT (0+XfOrM611_COUNT)
#     define SETUP_XfOrM612(x) SETUP_XfOrM611(x)

# 4049 "./../src/optimize.c"
      pred1 = FUNCCALL(SETUP_XfOrM612(_), expr_implies_predicate (rand1 , info ) ); 
      if (pred1 && (predicate_implies (pred1 , expect_pred ) ) ) {
#       define XfOrM613_COUNT (0+XfOrM612_COUNT)
#       define SETUP_XfOrM613(x) SETUP_XfOrM612(x)

# 4051 "./../src/optimize.c"
        pred2 = FUNCCALL(SETUP_XfOrM613(_), expr_implies_predicate (rand2 , info ) ); 
        if (pred2 && (predicate_implies (pred2 , expect_pred ) ) ) {
#         define XfOrM614_COUNT (0+XfOrM613_COUNT)
#         define SETUP_XfOrM614(x) SETUP_XfOrM613(x)
          FUNCCALL_EMPTY(reset_rator (app , unsafe ) )
# 4053 "./../src/optimize.c"
          ; 
        }
      }
    }
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 4060 "./../src/optimize.c"
static void check_known_both_variant (Optimize_Info * info , Scheme_Object * app , 
Scheme_Object * rator , Scheme_Object * rand1 , Scheme_Object * rand2 , 
const char * who , Scheme_Object * expect_pred , 
Scheme_Object * unsafe , int unsafe_mode , 
Scheme_Object * implies_pred ) 
{
  PREPARE_VAR_STACK_ONCE(9);
  BLOCK_SETUP_TOP((PUSH(rand1, 0), PUSH(expect_pred, 1), PUSH(rand2, 2), PUSH(info, 3), PUSH(who, 4), PUSH(unsafe, 5), PUSH(implies_pred, 6), PUSH(app, 7), PUSH(rator, 8)));
# define XfOrM617_COUNT (9)
# define SETUP_XfOrM617(x) SETUP(XfOrM617_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 4066 "./../src/optimize.c"
  ((void ) sizeof ((((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) ) ? 1 : 0 ) , __extension__ ({
#   define XfOrM619_COUNT (0+XfOrM617_COUNT)
#   define SETUP_XfOrM619(x) SETUP_XfOrM617(x)

# 4066 "./../src/optimize.c" 3
    if ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 

# 4066 "./../src/optimize.c" 3
    ; 

# 4066 "./../src/optimize.c" 3
    else FUNCCALL(SETUP_XfOrM619(_), __assert_fail ("((Scheme_Type)(((((intptr_t)(rator)) & 0x1)?(Scheme_Type)scheme_integer_type:((Scheme_Object *)(rator))->type)) == (Scheme_Type)(scheme_prim_type))" , "./../src/optimize.c" , 4066 , __extension__ __PRETTY_FUNCTION__ ) ); 
  }
  ) ) 
# 4066 "./../src/optimize.c"
  ; 
  if (! who || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , who ) ) ) ) {
    int ok1 ; 
#   define XfOrM618_COUNT (0+XfOrM617_COUNT)
#   define SETUP_XfOrM618(x) SETUP_XfOrM617(x)

# 4069 "./../src/optimize.c"
    ok1 = FUNCCALL(SETUP_XfOrM618(_), check_known_variant (info , app , rator , rand1 , who , expect_pred , ((void * ) 0 ) , unsafe_mode , implies_pred ) ); 
    FUNCCALL_EMPTY(check_known_variant (info , app , rator , rand2 , who , expect_pred , (ok1 ? unsafe : ((void * ) 0 ) ) , unsafe_mode , implies_pred ) ); 
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 4074 "./../src/optimize.c"
static void check_known_both (Optimize_Info * info , Scheme_Object * app , 
Scheme_Object * rator , Scheme_Object * rand1 , Scheme_Object * rand2 , 
const char * who , Scheme_Object * expect_pred , 
Scheme_Object * unsafe , int unsafe_mode ) 
{
  /* No conversion */

# 4079 "./../src/optimize.c"
  check_known_both_variant (info , app , rator , rand1 , rand2 , who , expect_pred , unsafe , unsafe_mode , expect_pred ) ; 
}


static void check_known_all (Optimize_Info * info , Scheme_Object * _app , int skip_head , int skip_tail , 
const char * who , Scheme_Object * expect_pred , 
Scheme_Object * unsafe , int unsafe_mode ) 
{
  Scheme_App_Rec * app = (Scheme_App_Rec * ) _app ; 
  PREPARE_VAR_STACK_ONCE(6);
  BLOCK_SETUP_TOP((PUSH(_app, 0), PUSH(expect_pred, 1), PUSH(info, 2), PUSH(app, 3), PUSH(unsafe, 4), PUSH(who, 5)));
# define XfOrM621_COUNT (6)
# define SETUP_XfOrM621(x) SETUP(XfOrM621_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 4088 "./../src/optimize.c"
  if (((Scheme_Type ) (((((intptr_t ) (app -> args [0 ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> args [0 ] ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) && (! who || (! (strcmp (((Scheme_Primitive_Proc * ) app -> args [0 ] ) -> name , who ) ) ) ) ) {
    int ok_so_far = 1 , i ; 
#   define XfOrM622_COUNT (0+XfOrM621_COUNT)
#   define SETUP_XfOrM622(x) SETUP_XfOrM621(x)

# 4091 "./../src/optimize.c"
    for (i = skip_head ; i < app -> num_args - skip_tail ; i ++ ) {
#     define XfOrM625_COUNT (0+XfOrM622_COUNT)
#     define SETUP_XfOrM625(x) SETUP_XfOrM622(x)

# 4092 "./../src/optimize.c"
      if (! FUNCCALL(SETUP_XfOrM625(_), check_known_variant (info , _app , app -> args [0 ] , app -> args [i + 1 ] , who , expect_pred , ((void * ) 0 ) , unsafe_mode , expect_pred ) ))

      ok_so_far = 0 ; 
    }

    if (ok_so_far && unsafe ) {
#     define XfOrM623_COUNT (0+XfOrM622_COUNT)
#     define SETUP_XfOrM623(x) SETUP_XfOrM622(x)

# 4098 "./../src/optimize.c"
      if ((unsafe ) == (scheme_true ) ) 
        FUNCCALL(SETUP_XfOrM623(_), set_application_omittable (_app , unsafe ) ); 
      else FUNCCALL_EMPTY(reset_rator (_app , unsafe ) )
      ; 
    }
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 4106 "./../src/optimize.c"
static Scheme_Object * finish_optimize_any_application (Scheme_Object * app , Scheme_Object * rator , int argc , 
Optimize_Info * info , int context ) 
{
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(4);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(app, 1), PUSH(rator, 2)));
# define XfOrM626_COUNT (3)
# define SETUP_XfOrM626(x) SETUP(XfOrM626_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  FUNCCALL(SETUP_XfOrM626(_), check_known_rator (info , rator ) )
# 4109 "./../src/optimize.c"
  ; 

  if ((context & 0x1 ) && ! info -> escapes ) {
    Scheme_Object * pred ; 
    BLOCK_SETUP((PUSH(pred, 0+XfOrM626_COUNT)));
#   define XfOrM628_COUNT (1+XfOrM626_COUNT)
#   define SETUP_XfOrM628(x) SETUP(XfOrM628_COUNT)
    pred = NULLED_OUT ; 

# 4113 "./../src/optimize.c"
    pred = FUNCCALL(SETUP_XfOrM628(_), rator_implies_predicate (rator , info , argc ) ); 
    if (pred && (predicate_implies_not (pred , scheme_not_proc ) ) )
      RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_sequence (app , scheme_true , info ) )) RET_VALUE_EMPTY_END ; 
    else if (pred && (predicate_implies (pred , scheme_not_proc ) ) )
      RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_sequence (app , scheme_false , info ) )) RET_VALUE_EMPTY_END ; 
  }

  if ((rator ) == (scheme_void_proc ) ) 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_sequence (app , scheme_void , info ) )) RET_VALUE_EMPTY_END ; 

  if (is_always_escaping_primitive (rator ) )  {
    info -> escapes = 1 ; 
  }

  RET_VALUE_START (app ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 4130 "./../src/optimize.c"
static void increment_clock_counts_for_application (GC_CAN_IGNORE int * _vclock , 
GC_CAN_IGNORE int * _aclock , 
GC_CAN_IGNORE int * _kclock , 
GC_CAN_IGNORE int * _sclock , 
Scheme_Object * rator , 
int argc ) 
{
  /* No conversion */

# 4137 "./../src/optimize.c"
  if (! is_nonmutating_nondependant_primitive (rator , argc ) )
    * _vclock += 1 ; 
  else if (is_primitive_allocating (rator , argc ) )
    * _aclock += 1 ; 

  if (! is_noncapturing_primitive (rator , argc ) )
    * _kclock += 1 ; 

  if (! is_nonsaving_primitive (rator , argc ) )
    * _sclock += 1 ; 
}

static void increment_clocks_for_application (Optimize_Info * info , 
Scheme_Object * rator , 
int argc ) 
{
  int v , a , k , s ; 
  PREPARE_VAR_STACK_ONCE(1);
  BLOCK_SETUP_TOP((PUSH(info, 0)));
# define XfOrM630_COUNT (1)
# define SETUP_XfOrM630(x) SETUP(XfOrM630_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 4155 "./../src/optimize.c"
  v = info -> vclock ; 
  a = info -> aclock ; 
  k = info -> kclock ; 
  s = info -> sclock ; 
  FUNCCALL(SETUP_XfOrM630(_), increment_clock_counts_for_application (& v , & a , & k , & s , rator , argc ) )
  ; 

  info -> vclock = v ; 
  info -> aclock = a ; 
  info -> kclock = k ; 
  info -> sclock = s ; 
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 4168 "./../src/optimize.c"
static Scheme_Object * finish_optimize_application (Scheme_App_Rec * app , Optimize_Info * info , int context ) 
{
  Scheme_Object * le ; 
  Scheme_Object * rator = app -> args [0 ] ; 
  int all_vals = 1 , i , flags , rator_flags ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(8);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(app, 1), PUSH(le, 2), PUSH(rator, 3)));
# define XfOrM631_COUNT (4)
# define SETUP_XfOrM631(x) SETUP(XfOrM631_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  le = NULLED_OUT ; 

# 4174 "./../src/optimize.c"
  for (i = app -> num_args ; i -- ; ) {
    if (((((intptr_t ) (app -> args [i + 1 ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> args [i + 1 ] ) ) -> type ) < _scheme_ir_values_types_ )
      all_vals = 0 ; 
  }

  info -> size += 1 ; 
  info -> preserves_marks = 1 ; 
  info -> single_result = 1 ; 

  if (all_vals ) {
#   define XfOrM636_COUNT (0+XfOrM631_COUNT)
#   define SETUP_XfOrM636(x) SETUP_XfOrM631(x)

# 4184 "./../src/optimize.c"
    le = FUNCCALL(SETUP_XfOrM636(_), try_optimize_fold (rator , ((void * ) 0 ) , (Scheme_Object * ) app , info ) ); 
    if (le )
      RET_VALUE_START (le ) RET_VALUE_END ; 
  }

  if (! app -> num_args && ((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) ) {
#   define XfOrM635_COUNT (0+XfOrM631_COUNT)
#   define SETUP_XfOrM635(x) SETUP_XfOrM631(x)

# 4190 "./../src/optimize.c"
    if ((rator ) == (scheme_list_proc ) ) 
      RET_VALUE_START (scheme_null ) RET_VALUE_END ; 
    if ((rator ) == (scheme_append_proc ) ) 
      RET_VALUE_START (scheme_null ) RET_VALUE_END ; 
    if ((rator ) == (scheme_hasheq_proc ) ) 
      RET_VALUE_START ((Scheme_Object * ) (scheme_make_hash_tree (0 ) ) ) RET_VALUE_END ; 
    if ((rator ) == (scheme_hash_proc ) ) 
      RET_VALUE_START ((Scheme_Object * ) (scheme_make_hash_tree (1 ) ) ) RET_VALUE_END ; 
    if ((rator ) == (scheme_hasheqv_proc ) ) 
      RET_VALUE_START ((Scheme_Object * ) (scheme_make_hash_tree (2 ) ) ) RET_VALUE_END ; 
  }

  if (((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
  && (app -> num_args >= ((Scheme_Primitive_Proc * ) rator ) -> mina ) 
  && (app -> num_args <= ((Scheme_Primitive_Proc * ) rator ) -> mu . maxa ) ) {
    Scheme_Object * app_o = (Scheme_Object * ) app ; 
    Scheme_Object * rand1 = ((void * ) 0 ) , * rand2 = ((void * ) 0 ) , * rand3 = ((void * ) 0 ) ; 
    BLOCK_SETUP((PUSH(rand1, 0+XfOrM631_COUNT), PUSH(rand3, 1+XfOrM631_COUNT), PUSH(rand2, 2+XfOrM631_COUNT), PUSH(app_o, 3+XfOrM631_COUNT)));
#   define XfOrM633_COUNT (4+XfOrM631_COUNT)
#   define SETUP_XfOrM633(x) SETUP(XfOrM633_COUNT)

# 4208 "./../src/optimize.c"
    if (app -> num_args >= 1 )
      rand1 = app -> args [1 ] ; 

    if (app -> num_args >= 2 )
      rand2 = app -> args [2 ] ; 

    if (app -> num_args >= 3 )
      rand3 = app -> args [3 ] ; 

    if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 24 ) ) {
#     define XfOrM634_COUNT (0+XfOrM633_COUNT)
#     define SETUP_XfOrM634(x) SETUP_XfOrM633(x)
      FUNCCALL(SETUP_XfOrM634(_), check_known (info , app_o , rator , rand1 , "vector-set!" , scheme_vector_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) )
# 4218 "./../src/optimize.c"
      ; 
      FUNCCALL(SETUP_XfOrM634(_), check_known (info , app_o , rator , rand2 , "vector-set!" , scheme_fixnum_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "vector*-set!" , scheme_vector_p_proc , (info -> unsafe_mode ? scheme_unsafe_vector_star_set_proc : ((void * ) 0 ) ) , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand2 , "vector*-set!" , scheme_fixnum_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "procedure-arity-includes?" , scheme_procedure_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "map" , scheme_procedure_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "for-each" , scheme_procedure_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "andmap" , scheme_procedure_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "ormap" , scheme_procedure_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_all (info , app_o , 1 , 0 , "map" , scheme_list_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_all (info , app_o , 1 , 0 , "for-each" , scheme_list_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_all (info , app_o , 1 , 0 , "andmap" , scheme_list_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_all (info , app_o , 1 , 0 , "ormap" , scheme_list_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "string-set!" , scheme_string_p_proc , (info -> unsafe_mode ? scheme_unsafe_string_set_proc : ((void * ) 0 ) ) , info -> unsafe_mode ) )

      ; 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand2 , "string-set!" , scheme_fixnum_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand3 , "string-set!" , scheme_char_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "bytes-set!" , scheme_byte_string_p_proc , (info -> unsafe_mode ? scheme_unsafe_bytes_set_proc : ((void * ) 0 ) ) , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand2 , "bytes-set!" , scheme_fixnum_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand3 , "bytes-set!" , scheme_fixnum_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_all (info , app_o , 0 , 0 , "string-append" , scheme_string_p_proc , scheme_true , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known_all (info , app_o , 0 , 0 , "bytes-append" , scheme_byte_string_p_proc , scheme_true , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_all (info , app_o , 0 , 1 , "append" , scheme_list_p_proc , scheme_true , info -> unsafe_mode ) )
      ; 
    }

    if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 20 ) )
      FUNCCALL(SETUP_XfOrM633(_), check_known_all (info , app_o , 0 , 0 , ((void * ) 0 ) , scheme_real_p_proc , (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 22 ) ) ? scheme_true : ((void * ) 0 ) , info -> unsafe_mode ) )

    ; 
    if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 21 ) )
      FUNCCALL(SETUP_XfOrM633(_), check_known_all (info , app_o , 0 , 0 , ((void * ) 0 ) , scheme_number_p_proc , (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 22 ) ) ? scheme_true : ((void * ) 0 ) , info -> unsafe_mode ) )

    ; 


    rator = app -> args [0 ] ; 
  }
  FUNCCALL(SETUP_XfOrM631(_), increment_clocks_for_application (info , rator , app -> num_args ) )
  ; 

  rator_flags = FUNCCALL(SETUP_XfOrM631(_), get_rator_flags (rator , app -> num_args , info ) ); 
  info -> preserves_marks = ! ! (rator_flags & 4 ) ; 
  info -> single_result = ! ! (rator_flags & 32 ) ; 
  if ((rator_flags & (64 | 128 ) ) == 64 ) {
    info -> preserves_marks = - info -> preserves_marks ; 
    info -> single_result = - info -> single_result ; 
  }
  FUNCCALL(SETUP_XfOrM631(_), register_local_argument_types (app , ((void * ) 0 ) , ((void * ) 0 ) , info ) )
  ; 

  flags = FUNCCALL(SETUP_XfOrM631(_), appn_flags (app -> args [0 ] , info ) ); 
  (& (app ) -> iso ) -> so . keyex |= flags ; 

  RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(finish_optimize_any_application ((Scheme_Object * ) app , app -> args [0 ] , app -> num_args , info , context ) )) RET_VALUE_EMPTY_END 
  ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 4282 "./../src/optimize.c"
static Scheme_Object * try_reduce_predicate (Scheme_Object * rator , Scheme_Object * rand , 
Optimize_Info * info ) 


{
  Scheme_Object * pred ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(4);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(pred, 1), PUSH(rator, 2), PUSH(rand, 3)));
# define XfOrM639_COUNT (4)
# define SETUP_XfOrM639(x) SETUP(XfOrM639_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  pred = NULLED_OUT ; 

# 4289 "./../src/optimize.c"
  if (! (relevant_predicate (rator ) ) )
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

  pred = FUNCCALL(SETUP_XfOrM639(_), expr_implies_predicate (rand , info ) ); 

  if (! pred )
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

  if (predicate_implies (pred , rator ) ) 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_sequence (rand , scheme_true , info ) )) RET_VALUE_EMPTY_END ; 
  else if (predicate_implies_not (pred , rator ) ) 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_sequence (rand , scheme_false , info ) )) RET_VALUE_EMPTY_END ; 

  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 4305 "./../src/optimize.c"
static Scheme_Object * check_ignored_call_cc (Scheme_Object * rator , Scheme_Object * rand , 
Optimize_Info * info , int context ) 

{
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(6);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(rator, 1), PUSH(rand, 2)));
# define XfOrM640_COUNT (3)
# define SETUP_XfOrM640(x) SETUP(XfOrM640_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 4309 "./../src/optimize.c"
  if (((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
  && ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "call-with-current-continuation" ) ) ) 
  || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "call-with-composable-continuation" ) ) ) 
  || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "call-with-escape-continuation" ) ) ) ) ) {
    Scheme_Object * proc ; 
    BLOCK_SETUP((PUSH(proc, 0+XfOrM640_COUNT)));
#   define XfOrM641_COUNT (1+XfOrM640_COUNT)
#   define SETUP_XfOrM641(x) SETUP(XfOrM641_COUNT)
    proc = NULLED_OUT ; 

# 4315 "./../src/optimize.c"
    proc = FUNCCALL(SETUP_XfOrM641(_), lookup_constant_proc (info , rand , 1 ) ); 

    if (proc && ((Scheme_Type ) (((((intptr_t ) (proc ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (proc ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) ) {
      Scheme_Lambda * lam = (Scheme_Lambda * ) proc ; 
#     define XfOrM642_COUNT (0+XfOrM641_COUNT)
#     define SETUP_XfOrM642(x) SETUP_XfOrM641(x)

# 4319 "./../src/optimize.c"
      if (lam -> num_params == 1 ) {
        Scheme_IR_Lambda_Info * cl = lam -> ir_info ; 
#       define XfOrM643_COUNT (0+XfOrM642_COUNT)
#       define SETUP_XfOrM643(x) SETUP_XfOrM642(x)

# 4321 "./../src/optimize.c"
        if (! cl -> vars [0 ] -> use_count ) {
          Scheme_Object * expr ; 
          BLOCK_SETUP((PUSH(expr, 0+XfOrM643_COUNT)));
#         define XfOrM644_COUNT (1+XfOrM643_COUNT)
#         define SETUP_XfOrM644(x) SETUP(XfOrM644_COUNT)
          expr = NULLED_OUT ; 

# 4323 "./../src/optimize.c"
          info -> vclock ++ ; 
          expr = FUNCCALL(SETUP_XfOrM644(_), make_application_2 (rand , scheme_void , info ) ); 
          if (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "call-with-escape-continuation" ) ) )  {
            Scheme_Sequence * seq ; 
            BLOCK_SETUP((PUSH(seq, 0+XfOrM644_COUNT)));
#           define XfOrM645_COUNT (1+XfOrM644_COUNT)
#           define SETUP_XfOrM645(x) SETUP(XfOrM645_COUNT)
            seq = NULLED_OUT ; 

# 4328 "./../src/optimize.c"
            seq = FUNCCALL(SETUP_XfOrM645(_), scheme_malloc_sequence (1 ) ); 
            seq -> so . type = scheme_begin0_sequence_type ; 
            seq -> count = 1 ; 
            seq -> array [0 ] = expr ; 

            expr = (Scheme_Object * ) seq ; 
          }
          RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_expr (expr , info , context ) )) RET_VALUE_EMPTY_END ; 
        }
      }
    }
  }
  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 4343 "./../src/optimize.c"
static Scheme_Object * make_optimize_prim_application2 (Scheme_Object * prim , Scheme_Object * rand , 
Optimize_Info * info , int context ) 

{
  Scheme_Object * alt ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(alt, 1)));
# define XfOrM646_COUNT (2)
# define SETUP_XfOrM646(x) SETUP(XfOrM646_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  alt = NULLED_OUT ; 

# 4348 "./../src/optimize.c"
  alt = FUNCCALL(SETUP_XfOrM646(_), make_application_2 (prim , rand , info ) ); 

  if ((Scheme_Type ) (((((intptr_t ) (alt ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (alt ) ) -> type ) ) == (Scheme_Type ) (scheme_application2_type ) )  {
#   define XfOrM647_COUNT (0+XfOrM646_COUNT)
#   define SETUP_XfOrM647(x) SETUP_XfOrM646(x)

# 4351 "./../src/optimize.c"
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(finish_optimize_application2 ((Scheme_App2_Rec * ) alt , info , context ) )) RET_VALUE_EMPTY_END ; 
  }

# 4352 "./../src/optimize.c"
  else 
  RET_VALUE_START (alt ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 4357 "./../src/optimize.c"
static Scheme_Object * optimize_application2 (Scheme_Object * o , Optimize_Info * info , int context ) 
{
  Scheme_App2_Rec * app ; 
  Scheme_Object * le ; 
  int rator_apply_escapes , sub_context , ty ; 
  Optimize_Info_Sequence info_seq ; 

# 1027 "./../src/optimize.c"
  Scheme_Object * __funcarg89 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(app, 1), PUSH(le, 2)));
# define XfOrM648_COUNT (3)
# define SETUP_XfOrM648(x) SETUP(XfOrM648_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  app = NULLED_OUT ; 
  le = NULLED_OUT ; 

# 4364 "./../src/optimize.c"
  app = (Scheme_App2_Rec * ) o ; 

  le = FUNCCALL(SETUP_XfOrM648(_), check_app_let_rator (o , app -> rator , info , 1 , context ) ); 
  if (le )
    RET_VALUE_START (le ) RET_VALUE_END ; 

  le = FUNCCALL(SETUP_XfOrM648(_), check_ignored_call_cc (app -> rator , app -> rand , info , context ) ); 
  if (le )
    RET_VALUE_START (le ) RET_VALUE_END ; 

  le = FUNCCALL(SETUP_XfOrM648(_), optimize_for_inline (info , app -> rator , 1 , ((void * ) 0 ) , app , ((void * ) 0 ) , context , 0 ) ); 
  if (le )
    RET_VALUE_START (le ) RET_VALUE_END ; 
  FUNCCALL(SETUP_XfOrM648(_), optimize_info_seq_init (info , & info_seq ) )
  ; 

  sub_context = 0x4 ; 

  le = FUNCCALL(SETUP_XfOrM648(_), optimize_expr (app -> rator , info , sub_context ) ); 
  app -> rator = le ; 
  if (info -> escapes ) {
#   define XfOrM654_COUNT (0+XfOrM648_COUNT)
#   define SETUP_XfOrM654(x) SETUP_XfOrM648(x)
    FUNCCALL(SETUP_XfOrM654(_), optimize_info_seq_done (info , & info_seq ) )
# 4385 "./../src/optimize.c"
    ; 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_noncm (app -> rator , info ) )) RET_VALUE_EMPTY_END ; 
  }

  {
#   define XfOrM653_COUNT (0+XfOrM648_COUNT)
#   define SETUP_XfOrM653(x) SETUP_XfOrM648(x)

# 4391 "./../src/optimize.c"
    le = FUNCCALL(SETUP_XfOrM653(_), optimize_for_inline (info , app -> rator , 1 , ((void * ) 0 ) , app , ((void * ) 0 ) , context , 1 ) ); 
    if (le )
      RET_VALUE_START (le ) RET_VALUE_END ; 
    rator_apply_escapes = info -> escapes ; 
  }

  if ((scheme_not_proc ) == (app -> rator ) )  {
    sub_context |= 0x1 ; 
  }

# 4399 "./../src/optimize.c"
  else {
#   define XfOrM651_COUNT (0+XfOrM648_COUNT)
#   define SETUP_XfOrM651(x) SETUP_XfOrM648(x)

# 4400 "./../src/optimize.c"
    ty = (wants_local_type_arguments (app -> rator , 0 ) ) ; 
    if (ty )
      sub_context |= (ty << 4 ) ; 
  }
  FUNCCALL(SETUP_XfOrM648(_), optimize_info_seq_step (info , & info_seq ) )
  ; 

  le = FUNCCALL(SETUP_XfOrM648(_), optimize_expr (app -> rand , info , sub_context ) ); 
  app -> rand = le ; 
  FUNCCALL(SETUP_XfOrM648(_), optimize_info_seq_done (info , & info_seq ) ); 
  if (info -> escapes ) {
#   define XfOrM650_COUNT (0+XfOrM648_COUNT)
#   define SETUP_XfOrM650(x) SETUP_XfOrM648(x)

# 4411 "./../src/optimize.c"
    info -> size += 1 ; 
    RET_VALUE_START ((__funcarg89 = FUNCCALL(SETUP_XfOrM650(_), make_discarding_first_sequence (app -> rator , app -> rand , info ) ), FUNCCALL_EMPTY(ensure_noncm (__funcarg89 , info ) )) ) RET_VALUE_END ; 
  }

  if (rator_apply_escapes ) {
    info -> escapes = 1 ; 
    (& (app ) -> iso ) -> so . keyex |= ((1 << 12 ) | (1 << 13 ) ) ; 
  }

  RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(finish_optimize_application2 (app , info , context ) )) RET_VALUE_EMPTY_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 4423 "./../src/optimize.c"
static Scheme_Object * finish_optimize_application2 (Scheme_App2_Rec * app , Optimize_Info * info , int context ) 
{
  int flags , rator_flags ; 
  Scheme_Object * rator = app -> rator ; 
  Scheme_Object * rand , * inside = ((void * ) 0 ) , * alt ; 
  Scheme_Object * __funcarg94 = NULLED_OUT ; 
  Scheme_Object * __funcarg93 = NULLED_OUT ; 

# 816 "./../src/optimize.c"
  Scheme_Object * __funcarg92 = NULLED_OUT ; 

# 816 "./../src/optimize.c"
  Scheme_Object * __funcarg91 = NULLED_OUT ; 

# 1933 "./../src/optimize.c"
  Scheme_Object * __funcarg90 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(9);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(app, 1), PUSH(alt, 2), PUSH(rator, 3), PUSH(rand, 4), PUSH(inside, 5)));
# define XfOrM655_COUNT (6)
# define SETUP_XfOrM655(x) SETUP(XfOrM655_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  rand = NULLED_OUT ; 
  alt = NULLED_OUT ; 

# 4429 "./../src/optimize.c"
  info -> size += 1 ; 
  info -> preserves_marks = 1 ; 
  info -> single_result = 1 ; 


  if (((((intptr_t ) (app -> rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand ) ) -> type ) > _scheme_ir_values_types_ ) {
    Scheme_Object * le ; 
    BLOCK_SETUP((PUSH(le, 0+XfOrM655_COUNT)));
#   define XfOrM707_COUNT (1+XfOrM655_COUNT)
#   define SETUP_XfOrM707(x) SETUP(XfOrM707_COUNT)
    le = NULLED_OUT ; 

# 4436 "./../src/optimize.c"
    le = FUNCCALL(SETUP_XfOrM707(_), try_optimize_fold (rator , ((void * ) 0 ) , (Scheme_Object * ) app , info ) ); 
    if (le )
      RET_VALUE_START (le ) RET_VALUE_END ; 
  }

  rand = app -> rand ; 
  FUNCCALL(SETUP_XfOrM655(_), extract_tail_inside (& rand , & inside , 0 ) )


  ; 

  if (((((intptr_t ) (rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rand ) ) -> type ) > _scheme_ir_values_types_ ) {
    Scheme_Object * le ; 
    BLOCK_SETUP((PUSH(le, 0+XfOrM655_COUNT)));
#   define XfOrM706_COUNT (1+XfOrM655_COUNT)
#   define SETUP_XfOrM706(x) SETUP(XfOrM706_COUNT)
    le = NULLED_OUT ; 

# 4449 "./../src/optimize.c"
    le = (__funcarg94 = FUNCCALL(SETUP_XfOrM706(_), scheme_make_pair (rand , scheme_null ) ), FUNCCALL_AGAIN(try_optimize_fold (rator , __funcarg94 , ((void * ) 0 ) , info ) )) ; 
    if (le )
      RET_VALUE_START (FUNCCALL(SETUP_XfOrM706(_), replace_tail_inside (le , inside , app -> rand ) )) RET_VALUE_END ; 
  }

  if (((scheme_values_proc ) == (rator ) ) 
  || ((scheme_list_star_proc ) == (rator ) ) 
  || (((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) && (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "append" ) ) ) ) ) {
#   define XfOrM704_COUNT (0+XfOrM655_COUNT)
#   define SETUP_XfOrM704(x) SETUP_XfOrM655(x)

# 4457 "./../src/optimize.c"
    (& (app ) -> iso ) -> so . keyex |= ((1 << 12 ) | (1 << 13 ) ) ; 
    if ((context & 0x4 ) 
    || FUNCCALL(SETUP_XfOrM704(_), scheme_omittable_expr (rand , 1 , - 1 , 0 , info , info ) )
    || FUNCCALL(SETUP_XfOrM704(_), single_valued_noncm_expression (rand , info , 5 ) )) {
#     define XfOrM705_COUNT (0+XfOrM704_COUNT)
#     define SETUP_XfOrM705(x) SETUP_XfOrM704(x)

# 4461 "./../src/optimize.c"
      RET_VALUE_START (FUNCCALL(SETUP_XfOrM705(_), replace_tail_inside (rand , inside , app -> rand ) )) RET_VALUE_END ; 
    }
    app -> rator = scheme_values_proc ; 
    rator = scheme_values_proc ; 
  }

  if (((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
  && (1 >= ((Scheme_Primitive_Proc * ) rator ) -> mina ) 
  && (1 <= ((Scheme_Primitive_Proc * ) rator ) -> mu . maxa ) ) {
#   define XfOrM664_COUNT (0+XfOrM655_COUNT)
#   define SETUP_XfOrM664(x) SETUP_XfOrM655(x)

# 4471 "./../src/optimize.c"
    switch (((((intptr_t ) (rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rand ) ) -> type ) ) {
#     define XfOrM681_COUNT (0+XfOrM664_COUNT)
#     define SETUP_XfOrM681(x) SETUP_XfOrM664(x)

# 4472 "./../src/optimize.c"
      case scheme_application2_type : 
      {
        Scheme_App2_Rec * app2 = (Scheme_App2_Rec * ) rand ; 
        BLOCK_SETUP((PUSH(app2, 0+XfOrM681_COUNT)));
#       define XfOrM697_COUNT (1+XfOrM681_COUNT)
#       define SETUP_XfOrM697(x) SETUP(XfOrM697_COUNT)

# 4475 "./../src/optimize.c"
        if ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "car" ) ) ) 
        || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "unsafe-car" ) ) ) ) {
#         define XfOrM702_COUNT (0+XfOrM697_COUNT)
#         define SETUP_XfOrM702(x) SETUP_XfOrM697(x)

# 4477 "./../src/optimize.c"
          if ((scheme_list_proc ) == (app2 -> rator ) )  {
#           define XfOrM703_COUNT (0+XfOrM702_COUNT)
#           define SETUP_XfOrM703(x) SETUP_XfOrM702(x)

# 4479 "./../src/optimize.c"
            alt = FUNCCALL(SETUP_XfOrM703(_), ensure_single_value_noncm (app2 -> rand , info ) ); 
            RET_VALUE_START (FUNCCALL(SETUP_XfOrM703(_), replace_tail_inside (alt , inside , app -> rand ) )) RET_VALUE_END ; 
          }
        }

# 4482 "./../src/optimize.c"
        else if ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "cdr" ) ) ) 
        || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "unsafe-cdr" ) ) ) ) {
#         define XfOrM700_COUNT (0+XfOrM697_COUNT)
#         define SETUP_XfOrM700(x) SETUP_XfOrM697(x)

# 4484 "./../src/optimize.c"
          if ((scheme_list_proc ) == (app2 -> rator ) )  {
#           define XfOrM701_COUNT (0+XfOrM700_COUNT)
#           define SETUP_XfOrM701(x) SETUP_XfOrM700(x)

# 4486 "./../src/optimize.c"
            alt = FUNCCALL(SETUP_XfOrM701(_), make_discarding_sequence (app2 -> rand , scheme_null , info ) ); 
            RET_VALUE_START (FUNCCALL(SETUP_XfOrM701(_), replace_tail_inside (alt , inside , app -> rand ) )) RET_VALUE_END ; 
          }
        }

# 4489 "./../src/optimize.c"
        else if ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "unbox" ) ) ) 
        || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "unsafe-unbox" ) ) ) 
        || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "unsafe-unbox*" ) ) ) ) {
#         define XfOrM698_COUNT (0+XfOrM697_COUNT)
#         define SETUP_XfOrM698(x) SETUP_XfOrM697(x)

# 4492 "./../src/optimize.c"
          if ((scheme_box_proc ) == (app2 -> rator ) )  {
#           define XfOrM699_COUNT (0+XfOrM698_COUNT)
#           define SETUP_XfOrM699(x) SETUP_XfOrM698(x)

# 4494 "./../src/optimize.c"
            alt = FUNCCALL(SETUP_XfOrM699(_), ensure_single_value_noncm (app2 -> rand , info ) ); 
            RET_VALUE_START (FUNCCALL(SETUP_XfOrM699(_), replace_tail_inside (alt , inside , app -> rand ) )) RET_VALUE_END ; 
          }
        }
        break ; 
      }
      case scheme_application3_type : 
      {
        Scheme_App3_Rec * app3 = (Scheme_App3_Rec * ) rand ; 
        BLOCK_SETUP((PUSH(app3, 0+XfOrM681_COUNT)));
#       define XfOrM689_COUNT (1+XfOrM681_COUNT)
#       define SETUP_XfOrM689(x) SETUP(XfOrM689_COUNT)

# 4503 "./../src/optimize.c"
        if ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "car" ) ) ) 
        || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "unsafe-car" ) ) ) ) {
#         define XfOrM695_COUNT (0+XfOrM689_COUNT)
#         define SETUP_XfOrM695(x) SETUP_XfOrM689(x)

# 4505 "./../src/optimize.c"
          if (((scheme_cons_proc ) == (app3 -> rator ) ) 
          || ((scheme_unsafe_cons_list_proc ) == (app3 -> rator ) ) 
          || ((scheme_list_proc ) == (app3 -> rator ) ) 
          || ((scheme_list_star_proc ) == (app3 -> rator ) ) ) {
#           define XfOrM696_COUNT (0+XfOrM695_COUNT)
#           define SETUP_XfOrM696(x) SETUP_XfOrM695(x)

# 4510 "./../src/optimize.c"
            alt = FUNCCALL(SETUP_XfOrM696(_), make_discarding_reverse_sequence (app3 -> rand2 , app3 -> rand1 , info ) ); 
            RET_VALUE_START (FUNCCALL(SETUP_XfOrM696(_), replace_tail_inside (alt , inside , app -> rand ) )) RET_VALUE_END ; 
          }
        }

# 4513 "./../src/optimize.c"
        else if ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "cdr" ) ) ) 
        || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "unsafe-cdr" ) ) ) ) {
#         define XfOrM692_COUNT (0+XfOrM689_COUNT)
#         define SETUP_XfOrM692(x) SETUP_XfOrM689(x)

# 4515 "./../src/optimize.c"
          if (((scheme_cons_proc ) == (app3 -> rator ) ) 
          || ((scheme_unsafe_cons_list_proc ) == (app3 -> rator ) ) 
          || ((scheme_list_star_proc ) == (app3 -> rator ) ) ) {
#           define XfOrM694_COUNT (0+XfOrM692_COUNT)
#           define SETUP_XfOrM694(x) SETUP_XfOrM692(x)

# 4519 "./../src/optimize.c"
            alt = FUNCCALL(SETUP_XfOrM694(_), make_discarding_sequence (app3 -> rand1 , app3 -> rand2 , info ) ); 
            RET_VALUE_START (FUNCCALL(SETUP_XfOrM694(_), replace_tail_inside (alt , inside , app -> rand ) )) RET_VALUE_END ; 
          }

# 4521 "./../src/optimize.c"
          else if ((scheme_list_proc ) == (app3 -> rator ) )  {
#           define XfOrM693_COUNT (0+XfOrM692_COUNT)
#           define SETUP_XfOrM693(x) SETUP_XfOrM692(x)

# 4523 "./../src/optimize.c"
            alt = FUNCCALL(SETUP_XfOrM693(_), make_application_2 (scheme_list_proc , app3 -> rand2 , info ) ); 
            (& (((Scheme_App_Rec * ) alt ) ) -> iso ) -> so . keyex |= ((1 << 12 ) | (1 << 13 ) ) ; 
            alt = FUNCCALL(SETUP_XfOrM693(_), make_discarding_sequence (app3 -> rand1 , alt , info ) ); 
            RET_VALUE_START (FUNCCALL(SETUP_XfOrM693(_), replace_tail_inside (alt , inside , app -> rand ) )) RET_VALUE_END ; 
          }
        }

# 4528 "./../src/optimize.c"
        else if (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "cadr" ) ) )  {
#         define XfOrM690_COUNT (0+XfOrM689_COUNT)
#         define SETUP_XfOrM690(x) SETUP_XfOrM689(x)

# 4529 "./../src/optimize.c"
          if ((scheme_list_proc ) == (app3 -> rator ) )  {
#           define XfOrM691_COUNT (0+XfOrM690_COUNT)
#           define SETUP_XfOrM691(x) SETUP_XfOrM690(x)

# 4531 "./../src/optimize.c"
            alt = FUNCCALL(SETUP_XfOrM691(_), make_discarding_sequence (app3 -> rand1 , app3 -> rand2 , info ) ); 
            RET_VALUE_START (FUNCCALL(SETUP_XfOrM691(_), replace_tail_inside (alt , inside , app -> rand ) )) RET_VALUE_END ; 
          }
        }
        break ; 
      }
      case scheme_application_type : 
      {
        Scheme_App_Rec * appr = (Scheme_App_Rec * ) rand ; 
        Scheme_Object * r = appr -> args [0 ] ; 
        BLOCK_SETUP((PUSH(r, 0+XfOrM681_COUNT), PUSH(appr, 1+XfOrM681_COUNT)));
#       define XfOrM682_COUNT (2+XfOrM681_COUNT)
#       define SETUP_XfOrM682(x) SETUP(XfOrM682_COUNT)

# 4541 "./../src/optimize.c"
        if ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "car" ) ) ) 
        || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "unsafe-car" ) ) ) ) {
#         define XfOrM687_COUNT (0+XfOrM682_COUNT)
#         define SETUP_XfOrM687(x) SETUP_XfOrM682(x)

# 4543 "./../src/optimize.c"
          if ((appr -> args > 0 ) 
          && (((scheme_list_proc ) == (r ) ) 
          || ((scheme_list_star_proc ) == (r ) ) ) ) {
#           define XfOrM688_COUNT (0+XfOrM687_COUNT)
#           define SETUP_XfOrM688(x) SETUP_XfOrM687(x)

# 4547 "./../src/optimize.c"
            alt = FUNCCALL(SETUP_XfOrM688(_), make_discarding_app_sequence (appr , 0 , ((void * ) 0 ) , info ) ); 
            RET_VALUE_START (FUNCCALL(SETUP_XfOrM688(_), replace_tail_inside (alt , inside , app -> rand ) )) RET_VALUE_END ; 
          }
        }

# 4550 "./../src/optimize.c"
        else if ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "cdr" ) ) ) 
        || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "unsafe-cdr" ) ) ) ) {
#         define XfOrM683_COUNT (0+XfOrM682_COUNT)
#         define SETUP_XfOrM683(x) SETUP_XfOrM682(x)

# 4553 "./../src/optimize.c"
          if ((appr -> args > 0 ) 
          && (((scheme_list_proc ) == (r ) ) 
          || ((scheme_list_star_proc ) == (r ) ) ) ) {
            Scheme_Object * al = scheme_null ; 
            int k ; 
            BLOCK_SETUP((PUSH(al, 0+XfOrM683_COUNT)));
#           define XfOrM684_COUNT (1+XfOrM683_COUNT)
#           define SETUP_XfOrM684(x) SETUP(XfOrM684_COUNT)

# 4558 "./../src/optimize.c"
            for (k = appr -> num_args ; k > 1 ; k -- ) {
#             define XfOrM686_COUNT (0+XfOrM684_COUNT)
#             define SETUP_XfOrM686(x) SETUP_XfOrM684(x)

# 4559 "./../src/optimize.c"
              al = FUNCCALL(SETUP_XfOrM686(_), scheme_make_pair (appr -> args [k ] , al ) ); 
            }
            al = FUNCCALL(SETUP_XfOrM684(_), scheme_make_pair (r , al ) ); 
            alt = FUNCCALL(SETUP_XfOrM684(_), scheme_make_application (al , info ) ); 
            (& (((Scheme_App_Rec * ) alt ) ) -> iso ) -> so . keyex |= ((1 << 12 ) | (1 << 13 ) ) ; 
            alt = FUNCCALL(SETUP_XfOrM684(_), make_discarding_sequence (appr -> args [1 ] , alt , info ) ); 
            RET_VALUE_START (FUNCCALL(SETUP_XfOrM684(_), replace_tail_inside (alt , inside , app -> rand ) )) RET_VALUE_END ; 
          }
        }
        break ; 
      }
    }

    if ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "length" ) ) ) 
    && (scheme_is_list (rand ) ) ) {
#     define XfOrM680_COUNT (0+XfOrM664_COUNT)
#     define SETUP_XfOrM680(x) SETUP_XfOrM664(x)

# 4574 "./../src/optimize.c"
      alt = ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) ((scheme_list_length (rand ) ) ) ) ) << 1 ) | 0x1 ) ) ; 
      RET_VALUE_START (FUNCCALL(SETUP_XfOrM680(_), replace_tail_inside (alt , inside , app -> rand ) )) RET_VALUE_END ; 
    }

    alt = FUNCCALL(SETUP_XfOrM664(_), try_reduce_predicate (rator , rand , info ) ); 
    if (alt )
      RET_VALUE_START (FUNCCALL(SETUP_XfOrM664(_), replace_tail_inside (alt , inside , app -> rand ) )) RET_VALUE_END ; 

    if ((scheme_struct_type_p_proc ) == (rator ) )  {
      Scheme_Object * c ; 
      BLOCK_SETUP((PUSH(c, 0+XfOrM664_COUNT)));
#     define XfOrM678_COUNT (1+XfOrM664_COUNT)
#     define SETUP_XfOrM678(x) SETUP(XfOrM678_COUNT)
      c = NULLED_OUT ; 

# 4584 "./../src/optimize.c"
      c = FUNCCALL(SETUP_XfOrM678(_), get_struct_proc_shape (rand , info , 0 ) ); 
      if (c && ((((Scheme_Struct_Proc_Shape * ) c ) -> mode & 0x7 ) 
      == 0 ) ) {
#       define XfOrM679_COUNT (0+XfOrM678_COUNT)
#       define SETUP_XfOrM679(x) SETUP_XfOrM678(x)

# 4587 "./../src/optimize.c"
        RET_VALUE_START (FUNCCALL(SETUP_XfOrM679(_), replace_tail_inside (scheme_true , inside , app -> rand ) )) RET_VALUE_END ; 
      }
    }

    if (((scheme_varref_const_p_proc ) == (rator ) ) 
    && ((Scheme_Type ) (((((intptr_t ) (rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rand ) ) -> type ) ) == (Scheme_Type ) (scheme_varref_form_type ) ) ) {
      Scheme_Object * var = (((Scheme_Simple_Object * ) (rand ) ) -> u . two_ptr_val . ptr1 ) ; 
      BLOCK_SETUP((PUSH(var, 0+XfOrM664_COUNT)));
#     define XfOrM673_COUNT (1+XfOrM664_COUNT)
#     define SETUP_XfOrM673(x) SETUP(XfOrM673_COUNT)

# 4594 "./../src/optimize.c"
      if ((var ) == (scheme_true ) )  {
#       define XfOrM677_COUNT (0+XfOrM673_COUNT)
#       define SETUP_XfOrM677(x) SETUP_XfOrM673(x)

# 4595 "./../src/optimize.c"
        RET_VALUE_START (FUNCCALL(SETUP_XfOrM677(_), replace_tail_inside (scheme_true , inside , app -> rand ) )) RET_VALUE_END ; 
      }

# 4596 "./../src/optimize.c"
      else if ((var ) == (scheme_false ) )  {
#       define XfOrM676_COUNT (0+XfOrM673_COUNT)
#       define SETUP_XfOrM676(x) SETUP_XfOrM673(x)

# 4597 "./../src/optimize.c"
        RET_VALUE_START (FUNCCALL(SETUP_XfOrM676(_), replace_tail_inside (scheme_false , inside , app -> rand ) )) RET_VALUE_END ; 
      }

# 4598 "./../src/optimize.c"
      else {
#       define XfOrM674_COUNT (0+XfOrM673_COUNT)
#       define SETUP_XfOrM674(x) SETUP_XfOrM673(x)

# 4599 "./../src/optimize.c"
        if (var && FUNCCALL(SETUP_XfOrM674(_), ir_propagate_ok (var , info , 1 , ((void * ) 0 ) ) )) {
#         define XfOrM675_COUNT (0+XfOrM674_COUNT)
#         define SETUP_XfOrM675(x) SETUP_XfOrM674(x)

# 4601 "./../src/optimize.c"
          RET_VALUE_START (FUNCCALL(SETUP_XfOrM675(_), replace_tail_inside (scheme_true , inside , app -> rand ) )) RET_VALUE_END ; 
        }
      }
    }



    if (((scheme_varref_unsafe_p_proc ) == (rator ) ) 
    && ((Scheme_Type ) (((((intptr_t ) (rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rand ) ) -> type ) ) == (Scheme_Type ) (scheme_varref_form_type ) ) ) {
      Scheme_Object * result = (info -> unsafe_mode ? scheme_true : scheme_false ) ; 
#     define XfOrM672_COUNT (0+XfOrM664_COUNT)
#     define SETUP_XfOrM672(x) SETUP_XfOrM664(x)

# 4611 "./../src/optimize.c"
      RET_VALUE_START (FUNCCALL(SETUP_XfOrM672(_), replace_tail_inside (result , inside , app -> rand ) )) RET_VALUE_END ; 
    }

    if (((Scheme_Type ) (((((intptr_t ) (rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
    && (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 25 ) ) 
    && ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "zero?" ) ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "positive?" ) ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "negative?" ) ) ) ) ) {
      Scheme_Object * pred ; 
      Scheme_App3_Rec * new ; 
      BLOCK_SETUP((PUSH(pred, 0+XfOrM664_COUNT), PUSH(new, 1+XfOrM664_COUNT)));
#     define XfOrM670_COUNT (2+XfOrM664_COUNT)
#     define SETUP_XfOrM670(x) SETUP(XfOrM670_COUNT)
      pred = NULLED_OUT ; 
      new = NULLED_OUT ; 

# 4622 "./../src/optimize.c"
      pred = FUNCCALL(SETUP_XfOrM670(_), expr_implies_predicate (rand , info ) ); 
      if (pred && ((pred ) == (scheme_fixnum_p_proc ) ) ) {
        Scheme_Object * cmp ; 
#       define XfOrM671_COUNT (0+XfOrM670_COUNT)
#       define SETUP_XfOrM671(x) SETUP_XfOrM670(x)
        cmp = NULLED_OUT ; 

# 4625 "./../src/optimize.c"
        if (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "positive?" ) ) ) 
          cmp = scheme_unsafe_fx_gt_proc ; 
        else if (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "negative?" ) ) ) 
          cmp = scheme_unsafe_fx_lt_proc ; 
        else 
        cmp = scheme_unsafe_fx_eq_proc ; 
        new = (Scheme_App3_Rec * ) FUNCCALL(SETUP_XfOrM671(_), make_application_3 (cmp , app -> rand , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (0 ) ) ) << 1 ) | 0x1 ) ) , info ) ); 
        (& (new ) -> iso ) -> so . keyex |= ((1 << 12 ) | (1 << 13 ) ) ; 
        RET_VALUE_START (FUNCCALL(SETUP_XfOrM671(_), finish_optimize_application3 (new , info , context ) )) RET_VALUE_END ; 
      }
    }

    if (((rator ) == (scheme_system_type_proc ) ) 
    && ((Scheme_Type ) (((((intptr_t ) (rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rand ) ) -> type ) ) == (Scheme_Type ) (scheme_symbol_type ) ) 
    && ! ((& ((Scheme_Symbol * ) (rand ) ) -> iso ) -> so . keyex & 0x3 ) 
    && ! (strcmp ((((Scheme_Symbol * ) ((Scheme_Simple_Object * ) (rand ) ) ) -> s ) , "vm" ) ) ) {
#     define XfOrM669_COUNT (0+XfOrM664_COUNT)
#     define SETUP_XfOrM669(x) SETUP_XfOrM664(x)
      RET_VALUE_START ((__funcarg93 = FUNCCALL(SETUP_XfOrM669(_), scheme_intern_symbol ("racket" ) ), FUNCCALL_AGAIN(replace_tail_inside (__funcarg93 , inside , app -> rand ) )) ) RET_VALUE_END ; 
    }

    {

      Scheme_Object * app_o = (Scheme_Object * ) app ; 
      BLOCK_SETUP((PUSH(app_o, 0+XfOrM664_COUNT)));
#     define XfOrM665_COUNT (1+XfOrM664_COUNT)
#     define SETUP_XfOrM665(x) SETUP(XfOrM665_COUNT)

# 4650 "./../src/optimize.c"
      if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 24 ) ) {
#       define XfOrM667_COUNT (0+XfOrM665_COUNT)
#       define SETUP_XfOrM667(x) SETUP_XfOrM665(x)
        FUNCCALL(SETUP_XfOrM667(_), check_known_variant (info , app_o , rator , rand , "bitwise-not" , scheme_fixnum_p_proc , scheme_unsafe_fxnot_proc , 0 , scheme_real_p_proc ) )
# 4651 "./../src/optimize.c"
        ; 
        FUNCCALL(SETUP_XfOrM667(_), check_known_variant (info , app_o , rator , rand , "fxnot" , scheme_fixnum_p_proc , scheme_unsafe_fxnot_proc , info -> unsafe_mode , scheme_real_p_proc ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "car" , scheme_pair_p_proc , scheme_unsafe_car_proc , info -> unsafe_mode ) )
        ; 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "unsafe-car" , scheme_pair_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "cdr" , scheme_pair_p_proc , scheme_unsafe_cdr_proc , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "unsafe-cdr" , scheme_pair_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "mcar" , scheme_mpair_p_proc , scheme_unsafe_mcar_proc , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "unsafe-mcar" , scheme_mpair_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "mcdr" , scheme_mpair_p_proc , scheme_unsafe_mcdr_proc , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "unsafe-mcdr" , scheme_mpair_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "string-length" , scheme_string_p_proc , scheme_unsafe_string_length_proc , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "bytes-length" , scheme_byte_string_p_proc , scheme_unsafe_byte_string_length_proc , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "unbox" , scheme_box_p_proc , scheme_unsafe_unbox_proc , info -> unsafe_mode ) )
        ; 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "unbox*" , scheme_box_p_proc , (info -> unsafe_mode ? scheme_unsafe_unbox_star_proc : ((void * ) 0 ) ) , info -> unsafe_mode ) )
        ; 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "unsafe-unbox" , scheme_box_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "unsafe-unbox*" , scheme_box_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "vector-length" , scheme_vector_p_proc , scheme_unsafe_vector_length_proc , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "vector*-length" , scheme_vector_p_proc , (info -> unsafe_mode ? scheme_unsafe_vector_star_length_proc : ((void * ) 0 ) ) , info -> unsafe_mode ) )
        ; 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "length" , scheme_list_p_proc , scheme_true , info -> unsafe_mode ) )
        ; 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "string-append" , scheme_string_p_proc , scheme_true , info -> unsafe_mode ) )
        ; 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "string-append-immutable" , scheme_string_p_proc , scheme_true , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "bytes-append" , scheme_byte_string_p_proc , scheme_true , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "string->immutable-string" , scheme_string_p_proc , scheme_true , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "bytes->immutable-bytes" , scheme_byte_string_p_proc , scheme_true , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "string->symbol" , scheme_string_p_proc , scheme_true , info -> unsafe_mode ) )
        ; 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "symbol->string" , scheme_symbol_p_proc , scheme_true , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "symbol->string-immutable" , scheme_symbol_p_proc , scheme_true , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "string->keyword" , scheme_string_p_proc , scheme_true , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "keyword->string" , scheme_keyword_p_proc , scheme_true , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "char->integer" , scheme_char_p_proc , scheme_unsafe_char_to_integer_proc , info -> unsafe_mode ) )
        ; 

        if ((! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "real->double-flonum" ) ) ) 
        || (! (strcmp (((Scheme_Primitive_Proc * ) rator ) -> name , "exact->inexact" ) ) ) ) {
          Scheme_Object * pred ; 
          BLOCK_SETUP((PUSH(pred, 0+XfOrM667_COUNT)));
#         define XfOrM668_COUNT (1+XfOrM667_COUNT)
#         define SETUP_XfOrM668(x) SETUP(XfOrM668_COUNT)
          pred = NULLED_OUT ; 

# 4693 "./../src/optimize.c"
          pred = FUNCCALL(SETUP_XfOrM668(_), expr_implies_predicate (rand , info ) ); 
          if (predicate_implies (pred , scheme_flonum_p_proc ) ) 
            RET_VALUE_START (FUNCCALL(SETUP_XfOrM668(_), replace_tail_inside (rand , inside , rand ) )) RET_VALUE_END ; 
          else if (predicate_implies (pred , scheme_fixnum_p_proc ) ) 
            FUNCCALL(SETUP_XfOrM668(_), reset_rator (app_o , scheme_unsafe_fx_to_fl_proc ) ); 
        }
      }

      if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 20 ) )
        FUNCCALL(SETUP_XfOrM665(_), check_known (info , app_o , rator , rand , ((void * ) 0 ) , scheme_real_p_proc , (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 22 ) ) ? scheme_true : ((void * ) 0 ) , info -> unsafe_mode ) )

      ; 
      if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 21 ) )
        FUNCCALL(SETUP_XfOrM665(_), check_known (info , app_o , rator , rand , ((void * ) 0 ) , scheme_number_p_proc , (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 22 ) ) ? scheme_true : ((void * ) 0 ) , info -> unsafe_mode ) )

      ; 

      if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 24 ) ) {
#       define XfOrM666_COUNT (0+XfOrM665_COUNT)
#       define SETUP_XfOrM666(x) SETUP_XfOrM665(x)
        FUNCCALL(SETUP_XfOrM666(_), check_known (info , app_o , rator , rand , "caar" , scheme_pair_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) )
# 4712 "./../src/optimize.c"
        ; 
        FUNCCALL(SETUP_XfOrM666(_), check_known (info , app_o , rator , rand , "cadr" , scheme_pair_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "cdar" , scheme_pair_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "cddr" , scheme_pair_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "caddr" , scheme_pair_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) )
        ; 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "cdddr" , scheme_pair_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "cadddr" , scheme_pair_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "cddddr" , scheme_pair_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "list->vector" , scheme_list_p_proc , scheme_true , info -> unsafe_mode ) )
        ; 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "vector->list" , scheme_vector_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "vector->values" , scheme_vector_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "vector->immutable-vector" , scheme_vector_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
        FUNCCALL_AGAIN(check_known (info , app_o , rator , rand , "make-vector" , scheme_fixnum_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      }


      rator = app -> rator ; 
    }
  }


  alt = FUNCCALL(SETUP_XfOrM655(_), get_struct_proc_shape (rator , info , 0 ) ); 
  if (alt ) {
    int mode = (((Scheme_Struct_Proc_Shape * ) alt ) -> mode & 0x7 ) ; 
#   define XfOrM657_COUNT (0+XfOrM655_COUNT)
#   define SETUP_XfOrM657(x) SETUP_XfOrM655(x)

# 4739 "./../src/optimize.c"
    if ((mode == 2 ) 
    || (mode == 3 ) ) {
      Scheme_Object * pred ; 
      int unsafe = 0 ; 
      BLOCK_SETUP((PUSH(pred, 0+XfOrM657_COUNT)));
#     define XfOrM658_COUNT (1+XfOrM657_COUNT)
#     define SETUP_XfOrM658(x) SETUP(XfOrM658_COUNT)
      pred = NULLED_OUT ; 

# 4744 "./../src/optimize.c"
      if (info -> unsafe_mode && (mode == 3 ) ) {
        pred = ((void * ) 0 ) ; 
        unsafe = 1 ; 
      }

# 4747 "./../src/optimize.c"
      else 
      pred = FUNCCALL(SETUP_XfOrM658(_), expr_implies_predicate (rand , info ) ); 

      if (unsafe 
      || (pred 
      && ((Scheme_Type ) (((((intptr_t ) (pred ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pred ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) ) 
      && (is_struct_identity_subtype (((Scheme_Struct_Proc_Shape * ) pred ) -> identity , 
      ((Scheme_Struct_Proc_Shape * ) alt ) -> identity ) ) ) ) {
#       define XfOrM660_COUNT (0+XfOrM658_COUNT)
#       define SETUP_XfOrM660(x) SETUP_XfOrM658(x)

# 4755 "./../src/optimize.c"
        if (mode == 2 ) {
#         define XfOrM662_COUNT (0+XfOrM660_COUNT)
#         define SETUP_XfOrM662(x) SETUP_XfOrM660(x)

# 4757 "./../src/optimize.c"
          RET_VALUE_START ((__funcarg92 = FUNCCALL(SETUP_XfOrM662(_), make_discarding_sequence (rand , scheme_true , info ) ), FUNCCALL_AGAIN(replace_tail_inside (__funcarg92 , inside , app -> rand ) )) ) RET_VALUE_END 

          ; 
        }

# 4760 "./../src/optimize.c"
        else {

          Scheme_App3_Rec * new ; 
          BLOCK_SETUP((PUSH(new, 0+XfOrM660_COUNT)));
#         define XfOrM661_COUNT (1+XfOrM660_COUNT)
#         define SETUP_XfOrM661(x) SETUP(XfOrM661_COUNT)
          new = NULLED_OUT ; 

# 4763 "./../src/optimize.c"
          new = (Scheme_App3_Rec * ) FUNCCALL(SETUP_XfOrM661(_), make_application_3 (((((Scheme_Struct_Proc_Shape * ) alt ) -> mode & 0x10 ) ? scheme_unsafe_struct_star_ref_proc : scheme_unsafe_struct_ref_proc ) , app -> rand , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (((Scheme_Struct_Proc_Shape * ) alt ) -> mode >> 7 ) ) ) << 1 ) | 0x1 ) ) , info ) )




          ; 
          (& (new ) -> iso ) -> so . keyex |= ((1 << 12 ) | (1 << 13 ) ) ; 
          RET_VALUE_START (FUNCCALL(SETUP_XfOrM661(_), finish_optimize_application3 (new , info , context ) )) RET_VALUE_END ; 
        }
      }

# 4772 "./../src/optimize.c"
      else if ((mode == 2 ) && pred && (predicate_implies_not (pred , alt ) ) ) {
#       define XfOrM659_COUNT (0+XfOrM658_COUNT)
#       define SETUP_XfOrM659(x) SETUP_XfOrM658(x)

# 4774 "./../src/optimize.c"
        RET_VALUE_START ((__funcarg91 = FUNCCALL(SETUP_XfOrM659(_), make_discarding_sequence (rand , scheme_false , info ) ), FUNCCALL_AGAIN(replace_tail_inside (__funcarg91 , inside , app -> rand ) )) ) RET_VALUE_END 

        ; 
      }


      if ((mode == 3 ) 
      && ! ((((Scheme_Struct_Proc_Shape * ) alt ) -> identity ) == (scheme_null ) ) 
      && ((Scheme_Type ) (((((intptr_t ) (rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (rand ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) )
        (__funcarg90 = FUNCCALL(SETUP_XfOrM658(_), scheme_make_struct_proc_shape (2 , ((Scheme_Struct_Proc_Shape * ) alt ) -> identity ) ), FUNCCALL_AGAIN(add_type (info , rand , __funcarg90 ) )) 
      ; 
    }
  }
  FUNCCALL(SETUP_XfOrM655(_), increment_clocks_for_application (info , rator , 1 ) )
  ; 

  rator_flags = FUNCCALL(SETUP_XfOrM655(_), get_rator_flags (rator , 1 , info ) ); 
  info -> preserves_marks = ! ! (rator_flags & 4 ) ; 
  info -> single_result = ! ! (rator_flags & 32 ) ; 
  if ((rator_flags & (64 | 128 ) ) == 64 ) {
    info -> preserves_marks = - info -> preserves_marks ; 
    info -> single_result = - info -> single_result ; 
  }
  FUNCCALL(SETUP_XfOrM655(_), register_local_argument_types (((void * ) 0 ) , app , ((void * ) 0 ) , info ) )
  ; 

  flags = FUNCCALL(SETUP_XfOrM655(_), appn_flags (rator , info ) ); 
  (& (app ) -> iso ) -> so . keyex |= flags ; 

  RET_VALUE_START (FUNCCALL(SETUP_XfOrM655(_), finish_optimize_any_application ((Scheme_Object * ) app , rator , 1 , info , context ) )) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 4806 "./../src/optimize.c"
static Scheme_Object * optimize_application3 (Scheme_Object * o , Optimize_Info * info , int context ) 
{
  Scheme_App3_Rec * app ; 
  Scheme_Object * le ; 
  int rator_apply_escapes , sub_context , ty , flags ; 
  Optimize_Info_Sequence info_seq ; 

# 251 "./../src/optimize.c"
  char * __funcarg97 = NULLED_OUT ; 

# 1027 "./../src/optimize.c"
  Scheme_Object * __funcarg96 = NULLED_OUT ; 

# 1027 "./../src/optimize.c"
  Scheme_Object * __funcarg95 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(4);
  BLOCK_SETUP_TOP((PUSH(o, 0), PUSH(info, 1), PUSH(app, 2), PUSH(le, 3)));
# define XfOrM708_COUNT (4)
# define SETUP_XfOrM708(x) SETUP(XfOrM708_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  app = NULLED_OUT ; 
  le = NULLED_OUT ; 

# 4813 "./../src/optimize.c"
  app = (Scheme_App3_Rec * ) o ; 

  if (((app -> rator ) == (scheme_check_not_undefined_proc ) ) 
  && ((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_symbol_type ) ) ) {
#   define XfOrM714_COUNT (0+XfOrM708_COUNT)
#   define SETUP_XfOrM714(x) SETUP_XfOrM708(x)

# 4817 "./../src/optimize.c"
    if (FUNCCALL(SETUP_XfOrM714(_), scheme_log_level_p (info -> logger , 5 ) ))
      (__funcarg97 = FUNCCALL(SETUP_XfOrM714(_), scheme_optimize_context_to_string (info -> context ) )



    , FUNCCALL_AGAIN(scheme_log (info -> logger , 5 , 0 , "warning%s: use-before-definition check inserted on variable: %S" , __funcarg97 , app -> rand2 ) )) 
    ; 
  }


  le = FUNCCALL(SETUP_XfOrM708(_), direct_apply ((Scheme_Object * ) app , app -> rator , app -> rand2 , info ) ); 
  if (le )
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_expr (le , info , context ) )) RET_VALUE_EMPTY_END ; 

  le = FUNCCALL(SETUP_XfOrM708(_), call_with_immed_mark (app -> rator , app -> rand1 , app -> rand2 , ((void * ) 0 ) , info ) ); 
  if (le )
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_expr (le , info , context ) )) RET_VALUE_EMPTY_END ; 

  le = FUNCCALL(SETUP_XfOrM708(_), check_app_let_rator (o , app -> rator , info , 2 , context ) ); 
  if (le )
    RET_VALUE_START (le ) RET_VALUE_END ; 

  le = FUNCCALL(SETUP_XfOrM708(_), optimize_for_inline (info , app -> rator , 2 , ((void * ) 0 ) , ((void * ) 0 ) , app , context , 0 ) ); 
  if (le )
    RET_VALUE_START (le ) RET_VALUE_END ; 
  FUNCCALL(SETUP_XfOrM708(_), optimize_info_seq_init (info , & info_seq ) )
  ; 

  sub_context = 0x4 ; 

  le = FUNCCALL(SETUP_XfOrM708(_), optimize_expr (app -> rator , info , sub_context ) ); 
  app -> rator = le ; 
  if (info -> escapes ) {
#   define XfOrM713_COUNT (0+XfOrM708_COUNT)
#   define SETUP_XfOrM713(x) SETUP_XfOrM708(x)
    FUNCCALL(SETUP_XfOrM713(_), optimize_info_seq_done (info , & info_seq ) )
# 4850 "./../src/optimize.c"
    ; 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_noncm (app -> rator , info ) )) RET_VALUE_EMPTY_END ; 
  }

  {
#   define XfOrM712_COUNT (0+XfOrM708_COUNT)
#   define SETUP_XfOrM712(x) SETUP_XfOrM708(x)

# 4856 "./../src/optimize.c"
    le = FUNCCALL(SETUP_XfOrM712(_), optimize_for_inline (info , app -> rator , 2 , ((void * ) 0 ) , ((void * ) 0 ) , app , context , 1 ) ); 
    if (le )
      RET_VALUE_START (le ) RET_VALUE_END ; 
    rator_apply_escapes = info -> escapes ; 
  }

  if (((app -> rator ) == (scheme_values_proc ) ) 
  || ((app -> rator ) == (scheme_apply_proc ) ) )
    info -> maybe_values_argument = 1 ; 



  ty = (wants_local_type_arguments (app -> rator , 0 ) ) ; 
  if (ty )
    sub_context |= (ty << 4 ) ; 
  FUNCCALL(SETUP_XfOrM708(_), optimize_info_seq_step (info , & info_seq ) )
  ; 

  le = FUNCCALL_AGAIN(optimize_expr (app -> rand1 , info , sub_context ) ); 
  app -> rand1 = le ; 
  if (info -> escapes ) {
#   define XfOrM711_COUNT (0+XfOrM708_COUNT)
#   define SETUP_XfOrM711(x) SETUP_XfOrM708(x)

# 4877 "./../src/optimize.c"
    info -> size += 1 ; 
    RET_VALUE_START ((__funcarg96 = FUNCCALL(SETUP_XfOrM711(_), make_discarding_first_sequence (app -> rator , app -> rand1 , info ) ), FUNCCALL_EMPTY(ensure_noncm (__funcarg96 , info ) )) ) RET_VALUE_END ; 
  }



  ty = (wants_local_type_arguments (app -> rator , 1 ) ) ; 
  if (ty )
    sub_context |= (ty << 4 ) ; 
  else 
  sub_context &= ~ (0x3 << 4 ) ; 
  FUNCCALL(SETUP_XfOrM708(_), optimize_info_seq_step (info , & info_seq ) )
  ; 

  le = FUNCCALL_AGAIN(optimize_expr (app -> rand2 , info , sub_context ) ); 
  app -> rand2 = le ; 
  FUNCCALL(SETUP_XfOrM708(_), optimize_info_seq_done (info , & info_seq ) ); 
  if (info -> escapes ) {
#   define XfOrM710_COUNT (0+XfOrM708_COUNT)
#   define SETUP_XfOrM710(x) SETUP_XfOrM708(x)

# 4895 "./../src/optimize.c"
    info -> size += 1 ; 
    le = (__funcarg95 = FUNCCALL(SETUP_XfOrM710(_), make_discarding_first_sequence (app -> rand1 , app -> rand2 , info ) )

    , FUNCCALL_AGAIN(make_discarding_first_sequence (app -> rator , __funcarg95 , info ) )) 
    ; 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_noncm (le , info ) )) RET_VALUE_EMPTY_END ; 
  }


  le = FUNCCALL(SETUP_XfOrM708(_), direct_apply ((Scheme_Object * ) app , app -> rator , app -> rand2 , info ) ); 
  if (le )

# 4905 "./../src/optimize.c"
  RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(finish_optimize_app (le , info , context ) )) RET_VALUE_EMPTY_END ; 

  flags = FUNCCALL(SETUP_XfOrM708(_), appn_flags (app -> rator , info ) ); 
  (& (app ) -> iso ) -> so . keyex |= flags ; 

  if (rator_apply_escapes ) {
    info -> escapes = 1 ; 
    (& (app ) -> iso ) -> so . keyex |= ((1 << 12 ) | (1 << 13 ) ) ; 
  }

  RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(finish_optimize_application3 (app , info , context ) )) RET_VALUE_EMPTY_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 4918 "./../src/optimize.c"
static Scheme_Object * finish_optimize_application3 (Scheme_App3_Rec * app , Optimize_Info * info , int context ) 
{
  int flags , rator_flags ; 
  Scheme_Object * le ; 
  int all_vals = 1 ; 
  Scheme_Object * __funcarg99 = NULLED_OUT ; 

# 1933 "./../src/optimize.c"
  Scheme_Object * __funcarg98 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(7);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(app, 1), PUSH(le, 2)));
# define XfOrM715_COUNT (3)
# define SETUP_XfOrM715(x) SETUP(XfOrM715_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  le = NULLED_OUT ; 

# 4924 "./../src/optimize.c"
  info -> size += 1 ; 
  info -> preserves_marks = 1 ; 
  info -> single_result = 1 ; 

  if (((((intptr_t ) (app -> rand1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand1 ) ) -> type ) < _scheme_ir_values_types_ )
    all_vals = 0 ; 
  if (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) < _scheme_ir_values_types_ )
    all_vals = 0 ; 


  if (all_vals ) {
#   define XfOrM759_COUNT (0+XfOrM715_COUNT)
#   define SETUP_XfOrM759(x) SETUP_XfOrM715(x)

# 4935 "./../src/optimize.c"
    le = FUNCCALL(SETUP_XfOrM759(_), try_optimize_fold (app -> rator , ((void * ) 0 ) , (Scheme_Object * ) app , info ) ); 
    if (le )
      RET_VALUE_START (le ) RET_VALUE_END ; 
  }


  if ((app -> rator ) == (scheme_call_with_values_proc ) )  {
#   define XfOrM756_COUNT (0+XfOrM715_COUNT)
#   define SETUP_XfOrM756(x) SETUP_XfOrM715(x)

# 4942 "./../src/optimize.c"
    if ((Scheme_Type ) (((((intptr_t ) (app -> rand1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand1 ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) )  {
      Scheme_Lambda * lam = (Scheme_Lambda * ) app -> rand1 ; 
#     define XfOrM757_COUNT (0+XfOrM756_COUNT)
#     define SETUP_XfOrM757(x) SETUP_XfOrM756(x)

# 4945 "./../src/optimize.c"
      if (! lam -> num_params ) {
#       define XfOrM758_COUNT (0+XfOrM757_COUNT)
#       define SETUP_XfOrM758(x) SETUP_XfOrM757(x)

# 4947 "./../src/optimize.c"
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_apply_values (app -> rand2 , lam -> body , info , (((& (lam ) -> iso ) -> so . keyex & 32 ) ? ((((& (lam ) -> iso ) -> so . keyex & (64 | 128 ) ) == 64 ) ? - 1 : 1 ) : 0 ) , context ) )) RET_VALUE_EMPTY_END 





        ; 
      }
    }
  }

  if ((scheme_procedure_arity_includes_proc ) == (app -> rator ) )  {
#   define XfOrM753_COUNT (0+XfOrM715_COUNT)
#   define SETUP_XfOrM753(x) SETUP_XfOrM715(x)

# 4959 "./../src/optimize.c"
    if ((((intptr_t ) (app -> rand2 ) ) & 0x1 ) && (((intptr_t ) (app -> rand2 ) ) >> 1 ) >= 0 ) {
      Scheme_Object * proc ; 
      BLOCK_SETUP((PUSH(proc, 0+XfOrM753_COUNT)));
#     define XfOrM754_COUNT (1+XfOrM753_COUNT)
#     define SETUP_XfOrM754(x) SETUP(XfOrM754_COUNT)
      proc = NULLED_OUT ; 

# 4962 "./../src/optimize.c"
      proc = FUNCCALL(SETUP_XfOrM754(_), lookup_constant_proc (info , app -> rand1 , (((intptr_t ) (app -> rand2 ) ) >> 1 ) ) ); 
      if (proc ) {
#       define XfOrM755_COUNT (0+XfOrM754_COUNT)
#       define SETUP_XfOrM755(x) SETUP_XfOrM754(x)

# 4964 "./../src/optimize.c"
        info -> preserves_marks = 1 ; 
        info -> single_result = 1 ; 
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_sequence (app -> rand1 , ((proc ) == (scheme_true ) ) ? scheme_false : scheme_true , info ) )) RET_VALUE_EMPTY_END 

        ; 
      }
    }
  }

  if (((app -> rator ) == (scheme_equal_proc ) ) 
  || ((app -> rator ) == (scheme_eqv_proc ) ) 
  || ((app -> rator ) == (scheme_eq_proc ) ) ) {
#   define XfOrM745_COUNT (0+XfOrM715_COUNT)
#   define SETUP_XfOrM745(x) SETUP_XfOrM715(x)

# 4976 "./../src/optimize.c"
    if (FUNCCALL(SETUP_XfOrM745(_), equivalent_exprs (app -> rand1 , app -> rand2 , ((void * ) 0 ) , ((void * ) 0 ) , 0 ) )) {
#     define XfOrM752_COUNT (0+XfOrM745_COUNT)
#     define SETUP_XfOrM752(x) SETUP_XfOrM745(x)

# 4977 "./../src/optimize.c"
      RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_sequence_3 (app -> rand1 , app -> rand2 , scheme_true , info ) )) RET_VALUE_EMPTY_END ; 
    }
    {
      Scheme_Object * pred1 , * pred2 , * pred_new = ((void * ) 0 ) ; 
      int rel1 = 0 , rel2 = 0 , rel_max , eq_type = 0 ; 
      BLOCK_SETUP((PUSH(pred2, 0+XfOrM745_COUNT), PUSH(pred1, 1+XfOrM745_COUNT), PUSH(pred_new, 2+XfOrM745_COUNT)));
#     define XfOrM746_COUNT (3+XfOrM745_COUNT)
#     define SETUP_XfOrM746(x) SETUP(XfOrM746_COUNT)
      pred1 = NULLED_OUT ; 
      pred2 = NULLED_OUT ; 

# 4983 "./../src/optimize.c"
      pred1 = FUNCCALL(SETUP_XfOrM746(_), expr_implies_predicate (app -> rand1 , info ) ); 
      pred2 = FUNCCALL_AGAIN(expr_implies_predicate (app -> rand2 , info ) ); 
      rel1 = (relevant_predicate (pred1 ) ) ; 
      rel2 = (relevant_predicate (pred2 ) ) ; 
      if ((pred1 && pred2 ) 
      && ((predicate_implies_not (pred1 , pred2 ) ) 
      || (predicate_implies_not (pred2 , pred1 ) ) ) ) {
#       define XfOrM751_COUNT (0+XfOrM746_COUNT)
#       define SETUP_XfOrM751(x) SETUP_XfOrM746(x)

# 4990 "./../src/optimize.c"
        info -> preserves_marks = 1 ; 
        info -> single_result = 1 ; 
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_sequence_3 (app -> rand1 , app -> rand2 , scheme_false , info ) )) RET_VALUE_EMPTY_END ; 
      }


      if (rel1 >= 4 ) {
        Scheme_Object * new_app ; 
        BLOCK_SETUP((PUSH(new_app, 0+XfOrM746_COUNT)));
#       define XfOrM750_COUNT (1+XfOrM746_COUNT)
#       define SETUP_XfOrM750(x) SETUP(XfOrM750_COUNT)
        new_app = NULLED_OUT ; 

# 4998 "./../src/optimize.c"
        new_app = FUNCCALL(SETUP_XfOrM750(_), make_optimize_prim_application2 (pred1 , app -> rand2 , info , context ) ); 
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_sequence (app -> rand1 , new_app , info ) )) RET_VALUE_EMPTY_END ; 
      }
      if (rel2 >= 4 ) {
        Scheme_Object * new_app ; 
        BLOCK_SETUP((PUSH(new_app, 0+XfOrM746_COUNT)));
#       define XfOrM749_COUNT (1+XfOrM746_COUNT)
#       define SETUP_XfOrM749(x) SETUP(XfOrM749_COUNT)
        new_app = NULLED_OUT ; 

# 5003 "./../src/optimize.c"
        new_app = FUNCCALL(SETUP_XfOrM749(_), make_optimize_prim_application2 (pred2 , app -> rand1 , info , context ) ); 
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_reverse_sequence (app -> rand2 , new_app , info ) )) RET_VALUE_EMPTY_END ; 
      }



      if ((app -> rator ) == (scheme_eqv_proc ) ) 
        eq_type = 2 ; 
      if ((app -> rator ) == (scheme_eq_proc ) ) 
        eq_type = 3 ; 

      rel_max = (rel1 >= rel2 ) ? rel1 : rel2 ; 
      if (rel_max >= 3 && eq_type < 3 )
        pred_new = scheme_eq_proc ; 
      else if (rel_max >= 2 && eq_type < 2 )
        pred_new = scheme_eqv_proc ; 

      if (pred_new ) {
#       define XfOrM747_COUNT (0+XfOrM746_COUNT)
#       define SETUP_XfOrM747(x) SETUP_XfOrM746(x)

# 5021 "./../src/optimize.c"
        app -> rator = pred_new ; 
        (& (app ) -> iso ) -> so . keyex |= ((1 << 12 ) | (1 << 13 ) ) ; 


        if (all_vals ) {
#         define XfOrM748_COUNT (0+XfOrM747_COUNT)
#         define SETUP_XfOrM748(x) SETUP_XfOrM747(x)

# 5026 "./../src/optimize.c"
          le = FUNCCALL(SETUP_XfOrM748(_), try_optimize_fold (app -> rator , ((void * ) 0 ) , (Scheme_Object * ) app , info ) ); 
          if (le )
            RET_VALUE_START (le ) RET_VALUE_END ; 
        }
      }
    }
  }


  if (((Scheme_Type ) (((((intptr_t ) (app -> rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
  && (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) app -> rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & ((1 << 8 ) | (1 << 3 ) ) ) ) {
    int z1 , z2 ; 
#   define XfOrM734_COUNT (0+XfOrM715_COUNT)
#   define SETUP_XfOrM734(x) SETUP_XfOrM715(x)

# 5039 "./../src/optimize.c"
    z1 = ((app -> rand1 ) == (((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (0 ) ) ) << 1 ) | 0x1 ) ) ) ) ; 
    z2 = ((app -> rand2 ) == (((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (0 ) ) ) << 1 ) | 0x1 ) ) ) ) ; 
    if (! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "unsafe-fx+" ) ) )  {
#     define XfOrM744_COUNT (0+XfOrM734_COUNT)
#     define SETUP_XfOrM744(x) SETUP_XfOrM734(x)

# 5042 "./../src/optimize.c"
      if (z1 )
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_single_value_noncm (app -> rand2 , info ) )) RET_VALUE_EMPTY_END ; 
      else if (z2 )
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_single_value_noncm (app -> rand1 , info ) )) RET_VALUE_EMPTY_END ; 
    }

# 5046 "./../src/optimize.c"
    else if (! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "unsafe-fx-" ) ) )  {
#     define XfOrM743_COUNT (0+XfOrM734_COUNT)
#     define SETUP_XfOrM743(x) SETUP_XfOrM734(x)

# 5047 "./../src/optimize.c"
      if (z2 )
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_single_value_noncm (app -> rand1 , info ) )) RET_VALUE_EMPTY_END ; 
    }

# 5049 "./../src/optimize.c"
    else if (! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "unsafe-fx*" ) ) )  {
#     define XfOrM741_COUNT (0+XfOrM734_COUNT)
#     define SETUP_XfOrM741(x) SETUP_XfOrM734(x)

# 5050 "./../src/optimize.c"
      if (z1 || z2 ) {
#       define XfOrM742_COUNT (0+XfOrM741_COUNT)
#       define SETUP_XfOrM742(x) SETUP_XfOrM741(x)

# 5051 "./../src/optimize.c"
        if (z1 && z2 )
          RET_VALUE_START (((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (0 ) ) ) << 1 ) | 0x1 ) ) ) RET_VALUE_END ; 
        else if (z2 )
          RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_sequence (app -> rand1 , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (0 ) ) ) << 1 ) | 0x1 ) ) , info ) )) RET_VALUE_EMPTY_END ; 
        else 
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_sequence (app -> rand2 , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (0 ) ) ) << 1 ) | 0x1 ) ) , info ) )) RET_VALUE_EMPTY_END ; 
      }
      if ((app -> rand1 ) == (((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (1 ) ) ) << 1 ) | 0x1 ) ) ) ) 
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_single_value_noncm (app -> rand2 , info ) )) RET_VALUE_EMPTY_END ; 
      if ((app -> rand2 ) == (((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (1 ) ) ) << 1 ) | 0x1 ) ) ) ) 
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_single_value_noncm (app -> rand1 , info ) )) RET_VALUE_EMPTY_END ; 
    }

# 5062 "./../src/optimize.c"
    else if (! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "unsafe-fxquotient" ) ) )  {
#     define XfOrM740_COUNT (0+XfOrM734_COUNT)
#     define SETUP_XfOrM740(x) SETUP_XfOrM734(x)

# 5063 "./../src/optimize.c"
      if (z1 )
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_sequence (app -> rand2 , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (0 ) ) ) << 1 ) | 0x1 ) ) , info ) )) RET_VALUE_EMPTY_END ; 
      if ((app -> rand2 ) == (((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (1 ) ) ) << 1 ) | 0x1 ) ) ) ) 
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_single_value_noncm (app -> rand1 , info ) )) RET_VALUE_EMPTY_END ; 
    }

# 5067 "./../src/optimize.c"
    else if ((! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "unsafe-fxremainder" ) ) ) 
    || (! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "unsafe-fxmodulo" ) ) ) ) {
#     define XfOrM739_COUNT (0+XfOrM734_COUNT)
#     define SETUP_XfOrM739(x) SETUP_XfOrM734(x)

# 5069 "./../src/optimize.c"
      if (z1 )
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_sequence (app -> rand2 , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (0 ) ) ) << 1 ) | 0x1 ) ) , info ) )) RET_VALUE_EMPTY_END ; 
      if ((app -> rand2 ) == (((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (1 ) ) ) << 1 ) | 0x1 ) ) ) ) 
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_sequence (app -> rand1 , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (0 ) ) ) << 1 ) | 0x1 ) ) , info ) )) RET_VALUE_EMPTY_END ; 
    }

    z1 = ((((Scheme_Type ) (((((intptr_t ) (app -> rand1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand1 ) ) -> type ) ) == (Scheme_Type ) (scheme_float_type ) ) || ((Scheme_Type ) (((((intptr_t ) (app -> rand1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand1 ) ) -> type ) ) == (Scheme_Type ) (scheme_double_type ) ) ) && ((((Scheme_Type ) (((((intptr_t ) (app -> rand1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand1 ) ) -> type ) ) == (Scheme_Type ) (scheme_double_type ) ) ? (((Scheme_Double * ) (app -> rand1 ) ) -> double_val ) : (((Scheme_Float * ) (app -> rand1 ) ) -> float_val ) ) == 0.0 ) ) ; 
    z2 = ((((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_float_type ) ) || ((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_double_type ) ) ) && ((((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_double_type ) ) ? (((Scheme_Double * ) (app -> rand2 ) ) -> double_val ) : (((Scheme_Float * ) (app -> rand2 ) ) -> float_val ) ) == 0.0 ) ) ; 

    if (! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "unsafe-fl+" ) ) )  {
#     define XfOrM738_COUNT (0+XfOrM734_COUNT)
#     define SETUP_XfOrM738(x) SETUP_XfOrM734(x)

# 5079 "./../src/optimize.c"
      if (z1 )
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_single_value_noncm (app -> rand2 , info ) )) RET_VALUE_EMPTY_END ; 
      else if (z2 )
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_single_value_noncm (app -> rand1 , info ) )) RET_VALUE_EMPTY_END ; 
    }

# 5083 "./../src/optimize.c"
    else if (! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "unsafe-fl-" ) ) )  {
#     define XfOrM737_COUNT (0+XfOrM734_COUNT)
#     define SETUP_XfOrM737(x) SETUP_XfOrM734(x)

# 5084 "./../src/optimize.c"
      if (z2 )
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_single_value_noncm (app -> rand1 , info ) )) RET_VALUE_EMPTY_END ; 
    }

# 5086 "./../src/optimize.c"
    else if (! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "unsafe-fl*" ) ) )  {
#     define XfOrM736_COUNT (0+XfOrM734_COUNT)
#     define SETUP_XfOrM736(x) SETUP_XfOrM734(x)

# 5087 "./../src/optimize.c"
      if ((((Scheme_Type ) (((((intptr_t ) (app -> rand1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand1 ) ) -> type ) ) == (Scheme_Type ) (scheme_float_type ) ) || ((Scheme_Type ) (((((intptr_t ) (app -> rand1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand1 ) ) -> type ) ) == (Scheme_Type ) (scheme_double_type ) ) ) && ((((Scheme_Type ) (((((intptr_t ) (app -> rand1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand1 ) ) -> type ) ) == (Scheme_Type ) (scheme_double_type ) ) ? (((Scheme_Double * ) (app -> rand1 ) ) -> double_val ) : (((Scheme_Float * ) (app -> rand1 ) ) -> float_val ) ) == 1.0 ) )
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_single_value_noncm (app -> rand2 , info ) )) RET_VALUE_EMPTY_END ; 
      if ((((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_float_type ) ) || ((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_double_type ) ) ) && ((((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_double_type ) ) ? (((Scheme_Double * ) (app -> rand2 ) ) -> double_val ) : (((Scheme_Float * ) (app -> rand2 ) ) -> float_val ) ) == 1.0 ) )
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_single_value_noncm (app -> rand1 , info ) )) RET_VALUE_EMPTY_END ; 
    }

# 5091 "./../src/optimize.c"
    else if (! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "unsafe-fl/" ) ) )  {
#     define XfOrM735_COUNT (0+XfOrM734_COUNT)
#     define SETUP_XfOrM735(x) SETUP_XfOrM734(x)

# 5092 "./../src/optimize.c"
      if ((((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_float_type ) ) || ((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_double_type ) ) ) && ((((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_double_type ) ) ? (((Scheme_Double * ) (app -> rand2 ) ) -> double_val ) : (((Scheme_Float * ) (app -> rand2 ) ) -> float_val ) ) == 1.0 ) )
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_single_value_noncm (app -> rand1 , info ) )) RET_VALUE_EMPTY_END ; 
    }
  }

# 5120 "./../src/optimize.c"
  else if (((Scheme_Type ) (((((intptr_t ) (app -> rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
  && (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) app -> rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 1 ) ) ) {
#   define XfOrM725_COUNT (0+XfOrM715_COUNT)
#   define SETUP_XfOrM725(x) SETUP_XfOrM715(x)

# 5122 "./../src/optimize.c"
    if (! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "arithmetic-shift" ) ) )  {
#     define XfOrM732_COUNT (0+XfOrM725_COUNT)
#     define SETUP_XfOrM732(x) SETUP_XfOrM725(x)

# 5123 "./../src/optimize.c"
      if ((((intptr_t ) (app -> rand2 ) ) & 0x1 ) && ((((intptr_t ) (app -> rand2 ) ) >> 1 ) <= 0 ) 
      && (FUNCCALL(SETUP_XfOrM732(_), is_local_type_expression (app -> rand1 , info ) )== 2 ) ) {
        app -> rator = scheme_unsafe_fxrshift_proc ; 
        app -> rand2 = ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (- ((((intptr_t ) (app -> rand2 ) ) >> 1 ) ) ) ) ) << 1 ) | 0x1 ) ) ; 
      }
    }

# 5128 "./../src/optimize.c"
    else if (! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "string=?" ) ) )  {
#     define XfOrM730_COUNT (0+XfOrM725_COUNT)
#     define SETUP_XfOrM730(x) SETUP_XfOrM725(x)

# 5129 "./../src/optimize.c"
      if (((Scheme_Type ) (((((intptr_t ) (app -> rand1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand1 ) ) -> type ) ) == (Scheme_Type ) (scheme_char_string_type ) ) 
      && ((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_char_string_type ) ) ) {
#       define XfOrM731_COUNT (0+XfOrM730_COUNT)
#       define SETUP_XfOrM731(x) SETUP_XfOrM730(x)

# 5131 "./../src/optimize.c"
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(scheme_string_eq_2 (app -> rand1 , app -> rand2 ) )) RET_VALUE_EMPTY_END ; 
      }
    }

# 5133 "./../src/optimize.c"
    else if (! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "bytes=?" ) ) )  {
#     define XfOrM728_COUNT (0+XfOrM725_COUNT)
#     define SETUP_XfOrM728(x) SETUP_XfOrM725(x)

# 5134 "./../src/optimize.c"
      if (((Scheme_Type ) (((((intptr_t ) (app -> rand1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand1 ) ) -> type ) ) == (Scheme_Type ) (scheme_byte_string_type ) ) 
      && ((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_byte_string_type ) ) ) {
#       define XfOrM729_COUNT (0+XfOrM728_COUNT)
#       define SETUP_XfOrM729(x) SETUP_XfOrM728(x)

# 5136 "./../src/optimize.c"
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(scheme_byte_string_eq_2 (app -> rand1 , app -> rand2 ) )) RET_VALUE_EMPTY_END ; 
      }
    }

# 5138 "./../src/optimize.c"
    else if (! (strcmp (((Scheme_Primitive_Proc * ) app -> rator ) -> name , "char=?" ) ) )  {
      if (((Scheme_Type ) (((((intptr_t ) (app -> rand1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand1 ) ) -> type ) ) == (Scheme_Type ) (scheme_char_type ) ) 
      && ((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_char_type ) ) ) {
        RET_VALUE_START (((((Scheme_Small_Object * ) (app -> rand1 ) ) -> u . char_val ) == (((Scheme_Small_Object * ) (app -> rand2 ) ) -> u . char_val ) ) ? scheme_true : scheme_false ) RET_VALUE_END ; 
      }
    }
  }

  if (((Scheme_Type ) (((((intptr_t ) (app -> rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
  && (2 >= ((Scheme_Primitive_Proc * ) app -> rator ) -> mina ) 
  && (2 <= ((Scheme_Primitive_Proc * ) app -> rator ) -> mu . maxa ) ) {
    Scheme_Object * app_o = (Scheme_Object * ) app , * rator = app -> rator , * rand1 = app -> rand1 , * rand2 = app -> rand2 ; 
    BLOCK_SETUP((PUSH(rand1, 0+XfOrM715_COUNT), PUSH(app_o, 1+XfOrM715_COUNT), PUSH(rand2, 2+XfOrM715_COUNT), PUSH(rator, 3+XfOrM715_COUNT)));
#   define XfOrM722_COUNT (4+XfOrM715_COUNT)
#   define SETUP_XfOrM722(x) SETUP(XfOrM722_COUNT)

# 5151 "./../src/optimize.c"
    if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 24 ) ) {
#     define XfOrM724_COUNT (0+XfOrM722_COUNT)
#     define SETUP_XfOrM724(x) SETUP_XfOrM722(x)
      FUNCCALL(SETUP_XfOrM724(_), check_known_both_variant (info , app_o , rator , rand1 , rand2 , "bitwise-and" , scheme_fixnum_p_proc , scheme_unsafe_fxand_proc , 0 , scheme_real_p_proc ) )
# 5153 "./../src/optimize.c"
      ; 
      FUNCCALL(SETUP_XfOrM724(_), check_known_both_variant (info , app_o , rator , rand1 , rand2 , "bitwise-ior" , scheme_fixnum_p_proc , scheme_unsafe_fxior_proc , 0 , scheme_real_p_proc ) )
      ; 
      FUNCCALL_AGAIN(check_known_both_variant (info , app_o , rator , rand1 , rand2 , "bitwise-xor" , scheme_fixnum_p_proc , scheme_unsafe_fxxor_proc , 0 , scheme_real_p_proc ) )
      ; 
      FUNCCALL_AGAIN(check_known_both_variant (info , app_o , rator , rand1 , rand2 , "fxand" , scheme_fixnum_p_proc , scheme_unsafe_fxand_proc , info -> unsafe_mode , scheme_real_p_proc ) )

      ; 
      FUNCCALL_AGAIN(check_known_both_variant (info , app_o , rator , rand1 , rand2 , "fxior" , scheme_fixnum_p_proc , scheme_unsafe_fxior_proc , info -> unsafe_mode , scheme_real_p_proc ) )
      ; 
      FUNCCALL_AGAIN(check_known_both_variant (info , app_o , rator , rand1 , rand2 , "fxxor" , scheme_fixnum_p_proc , scheme_unsafe_fxxor_proc , info -> unsafe_mode , scheme_real_p_proc ) )
      ; 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "=" , scheme_fixnum_p_proc , scheme_unsafe_fx_eq_proc , 0 ) )
      ; 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "<" , scheme_fixnum_p_proc , scheme_unsafe_fx_lt_proc , 0 ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , ">" , scheme_fixnum_p_proc , scheme_unsafe_fx_gt_proc , 0 ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "<=" , scheme_fixnum_p_proc , scheme_unsafe_fx_lt_eq_proc , 0 ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , ">=" , scheme_fixnum_p_proc , scheme_unsafe_fx_gt_eq_proc , 0 ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "min" , scheme_fixnum_p_proc , scheme_unsafe_fx_min_proc , 0 ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "max" , scheme_fixnum_p_proc , scheme_unsafe_fx_max_proc , 0 ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "fx=" , scheme_fixnum_p_proc , scheme_unsafe_fx_eq_proc , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "fx<" , scheme_fixnum_p_proc , scheme_unsafe_fx_lt_proc , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "fx>" , scheme_fixnum_p_proc , scheme_unsafe_fx_gt_proc , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "fx<=" , scheme_fixnum_p_proc , scheme_unsafe_fx_lt_eq_proc , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "fx>=" , scheme_fixnum_p_proc , scheme_unsafe_fx_gt_eq_proc , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "fxmin" , scheme_fixnum_p_proc , scheme_unsafe_fx_min_proc , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "fxmax" , scheme_fixnum_p_proc , scheme_unsafe_fx_max_proc , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "fx+" , scheme_fixnum_p_proc , scheme_unsafe_fx_plus_proc , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "fx-" , scheme_fixnum_p_proc , scheme_unsafe_fx_minus_proc , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "fx*" , scheme_fixnum_p_proc , scheme_unsafe_fx_times_proc , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "char=?" , scheme_char_p_proc , scheme_unsafe_char_eq_proc , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "char<?" , scheme_char_p_proc , scheme_unsafe_char_lt_proc , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "char>?" , scheme_char_p_proc , scheme_unsafe_char_gt_proc , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "char<=?" , scheme_char_p_proc , scheme_unsafe_char_lt_eq_proc , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known_both_try (info , app_o , rator , rand1 , rand2 , "char>=?" , scheme_char_p_proc , scheme_unsafe_char_gt_eq_proc , info -> unsafe_mode ) ); 

      rator = app -> rator ; 
      FUNCCALL(SETUP_XfOrM724(_), check_known_both (info , app_o , rator , rand1 , rand2 , "string-append" , scheme_string_p_proc , scheme_true , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known_both (info , app_o , rator , rand1 , rand2 , "bytes-append" , scheme_byte_string_p_proc , scheme_true , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "string-ref" , scheme_string_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand2 , "string-ref" , scheme_fixnum_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "bytes-ref" , scheme_byte_string_p_proc , (info -> unsafe_mode ? scheme_unsafe_bytes_ref_proc : ((void * ) 0 ) ) , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand2 , "bytes-ref" , scheme_fixnum_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "append" , scheme_list_p_proc , scheme_true , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "list-ref" , scheme_pair_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand2 , "list-ref" , scheme_fixnum_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
    }

    if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 20 ) )
      FUNCCALL(SETUP_XfOrM722(_), check_known_both (info , app_o , rator , rand1 , rand2 , ((void * ) 0 ) , scheme_real_p_proc , (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 22 ) ) ? scheme_true : ((void * ) 0 ) , info -> unsafe_mode ) )

    ; 
    if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 21 ) )
      FUNCCALL(SETUP_XfOrM722(_), check_known_both (info , app_o , rator , rand1 , rand2 , ((void * ) 0 ) , scheme_number_p_proc , (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 22 ) ) ? scheme_true : ((void * ) 0 ) , info -> unsafe_mode ) )

    ; 

    if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) rator ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 24 ) ) {
#     define XfOrM723_COUNT (0+XfOrM722_COUNT)
#     define SETUP_XfOrM723(x) SETUP_XfOrM722(x)
      FUNCCALL(SETUP_XfOrM723(_), check_known (info , app_o , rator , rand1 , "vector-ref" , scheme_vector_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) )
# 5217 "./../src/optimize.c"
      ; 
      FUNCCALL(SETUP_XfOrM723(_), check_known (info , app_o , rator , rand2 , "vector-ref" , scheme_fixnum_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "vector*-ref" , scheme_vector_p_proc , (info -> unsafe_mode ? scheme_unsafe_vector_star_ref_proc : ((void * ) 0 ) ) , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand2 , "vector*-ref" , scheme_fixnum_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "make-vector" , scheme_fixnum_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "set-box!" , scheme_box_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "set-box*!" , scheme_box_p_proc , (info -> unsafe_mode ? scheme_unsafe_set_box_star_proc : ((void * ) 0 ) ) , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "unsafe-set-box!" , scheme_box_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "unsafe-set-box*!" , scheme_box_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "procedure-closure-contents-eq?" , scheme_procedure_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand2 , "procedure-closure-contents-eq?" , scheme_procedure_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "procedure-arity-includes?" , scheme_procedure_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "map" , scheme_procedure_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) )
      ; 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "for-each" , scheme_procedure_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "andmap" , scheme_procedure_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand1 , "ormap" , scheme_procedure_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand2 , "map" , scheme_list_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand2 , "for-each" , scheme_list_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand2 , "andmap" , scheme_list_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
      FUNCCALL_AGAIN(check_known (info , app_o , rator , rand2 , "ormap" , scheme_list_p_proc , ((void * ) 0 ) , info -> unsafe_mode ) ); 
    }
  }



  {
    Scheme_Object * alt ; 
    BLOCK_SETUP((PUSH(alt, 0+XfOrM715_COUNT)));
#   define XfOrM717_COUNT (1+XfOrM715_COUNT)
#   define SETUP_XfOrM717(x) SETUP(XfOrM717_COUNT)
    alt = NULLED_OUT ; 

# 5249 "./../src/optimize.c"
    alt = FUNCCALL(SETUP_XfOrM717(_), get_struct_proc_shape (app -> rator , info , 0 ) ); 
    if (alt ) {
      int mode = (((Scheme_Struct_Proc_Shape * ) alt ) -> mode & 0x7 ) ; 
#     define XfOrM718_COUNT (0+XfOrM717_COUNT)
#     define SETUP_XfOrM718(x) SETUP_XfOrM717(x)

# 5253 "./../src/optimize.c"
      if (mode == 4 ) {
        Scheme_Object * pred ; 
        int unsafe = 0 ; 
        BLOCK_SETUP((PUSH(pred, 0+XfOrM718_COUNT)));
#       define XfOrM719_COUNT (1+XfOrM718_COUNT)
#       define SETUP_XfOrM719(x) SETUP(XfOrM719_COUNT)
        pred = NULLED_OUT ; 

# 5257 "./../src/optimize.c"
        if (info -> unsafe_mode ) {
          pred = ((void * ) 0 ) ; 
          unsafe = 1 ; 
        }

# 5260 "./../src/optimize.c"
        else 
        pred = FUNCCALL(SETUP_XfOrM719(_), expr_implies_predicate (app -> rand1 , info ) ); 

        if ((unsafe 
        || (pred 
        && ((Scheme_Type ) (((((intptr_t ) (pred ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pred ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) ) 
        && (is_struct_identity_subtype (((Scheme_Struct_Proc_Shape * ) pred ) -> identity , 
        ((Scheme_Struct_Proc_Shape * ) alt ) -> identity ) ) ) ) 

        && ((((Scheme_Struct_Proc_Shape * ) alt ) -> mode >> 7 ) != 0 ) ) {

          Scheme_Object * l ; 
          Scheme_App_Rec * new_app ; 
          int pos = (((Scheme_Struct_Proc_Shape * ) alt ) -> mode >> 7 ) - 1 ; 
          BLOCK_SETUP((PUSH(new_app, 0+XfOrM719_COUNT), PUSH(l, 1+XfOrM719_COUNT)));
#         define XfOrM720_COUNT (2+XfOrM719_COUNT)
#         define SETUP_XfOrM720(x) SETUP(XfOrM720_COUNT)
          l = NULLED_OUT ; 
          new_app = NULLED_OUT ; 

# 5274 "./../src/optimize.c"
          l = (__funcarg99 = FUNCCALL(SETUP_XfOrM720(_), scheme_make_pair (app -> rand2 , scheme_null ) ), FUNCCALL_AGAIN(scheme_make_pair (((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (pos ) ) ) << 1 ) | 0x1 ) ) , __funcarg99 ) )) 

          ; 
          l = FUNCCALL_AGAIN(scheme_make_pair (app -> rand1 , l ) ); 
          l = FUNCCALL_AGAIN(scheme_make_pair (((((Scheme_Struct_Proc_Shape * ) alt ) -> mode & 0x10 ) ? scheme_unsafe_struct_star_set_proc : scheme_unsafe_struct_set_proc ) , l ) )


          ; 
          new_app = (Scheme_App_Rec * ) FUNCCALL(SETUP_XfOrM720(_), scheme_make_application (l , info ) ); 
          (& (new_app ) -> iso ) -> so . keyex |= ((1 << 12 ) | (1 << 13 ) ) ; 
          RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(finish_optimize_application (new_app , info , context ) )) RET_VALUE_EMPTY_END ; 
        }


        if (! ((((Scheme_Struct_Proc_Shape * ) alt ) -> identity ) == (scheme_null ) ) 
        && ((Scheme_Type ) (((((intptr_t ) (app -> rand1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand1 ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) )
          (__funcarg98 = FUNCCALL(SETUP_XfOrM719(_), scheme_make_struct_proc_shape (2 , ((Scheme_Struct_Proc_Shape * ) alt ) -> identity ) ), FUNCCALL_AGAIN(add_type (info , app -> rand1 , __funcarg98 ) )) 
        ; 
      }
    }
  }
  FUNCCALL(SETUP_XfOrM715(_), increment_clocks_for_application (info , app -> rator , 2 ) )
  ; 

  rator_flags = FUNCCALL(SETUP_XfOrM715(_), get_rator_flags (app -> rator , 2 , info ) ); 
  info -> preserves_marks = ! ! (rator_flags & 4 ) ; 
  info -> single_result = ! ! (rator_flags & 32 ) ; 
  if ((rator_flags & (64 | 128 ) ) == 64 ) {
    info -> preserves_marks = - info -> preserves_marks ; 
    info -> single_result = - info -> single_result ; 
  }
  FUNCCALL(SETUP_XfOrM715(_), register_local_argument_types (((void * ) 0 ) , ((void * ) 0 ) , app , info ) )
  ; 

  flags = FUNCCALL(SETUP_XfOrM715(_), appn_flags (app -> rator , info ) ); 
  (& (app ) -> iso ) -> so . keyex |= flags ; 

  RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(finish_optimize_any_application ((Scheme_Object * ) app , app -> rator , 2 , info , context ) )) RET_VALUE_EMPTY_END 
  ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

Scheme_Object * optimize_apply_values (Scheme_Object * f , Scheme_Object * e , 
Optimize_Info * info , 
int e_single_result , 
int context ) 

{
  Scheme_Object * o_f ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(7);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(f, 1), PUSH(o_f, 2), PUSH(e, 3)));
# define XfOrM760_COUNT (4)
# define SETUP_XfOrM760(x) SETUP(XfOrM760_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  o_f = NULLED_OUT ; 

# 5327 "./../src/optimize.c"
  info -> preserves_marks = 0 ; 
  info -> single_result = 0 ; 

  o_f = FUNCCALL(SETUP_XfOrM760(_), lookup_constant_proc (info , f , (e_single_result > 0 ) ? 1 : - 1 ) ); 
  if (o_f ) {
    if ((Scheme_Type ) (((((intptr_t ) (o_f ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o_f ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) )  {
      Scheme_Lambda * lam = (Scheme_Lambda * ) o_f ; 
      int flags = (& (lam ) -> iso ) -> so . keyex ; 
      info -> preserves_marks = ! ! (flags & 4 ) ; 
      info -> single_result = ! ! (flags & 32 ) ; 
      if ((flags & (64 | 128 ) ) == 64 ) {
        info -> preserves_marks = - info -> preserves_marks ; 
        info -> single_result = - info -> single_result ; 
      }
    }
  }

  if (o_f && (e_single_result > 0 ) ) {

    Scheme_App2_Rec * app2 ; 
    Scheme_Object * e_cloned , * f_cloned ; 
    BLOCK_SETUP((PUSH(app2, 0+XfOrM760_COUNT), PUSH(e_cloned, 1+XfOrM760_COUNT), PUSH(f_cloned, 2+XfOrM760_COUNT)));
#   define XfOrM762_COUNT (3+XfOrM760_COUNT)
#   define SETUP_XfOrM762(x) SETUP(XfOrM762_COUNT)
    app2 = NULLED_OUT ; 
    e_cloned = NULLED_OUT ; 
    f_cloned = NULLED_OUT ; 

# 5349 "./../src/optimize.c"
    app2 = ((Scheme_App2_Rec * ) FUNCCALL(SETUP_XfOrM762(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_App2_Rec ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
    app2 -> iso . so . type = scheme_application2_type ; 



    e_cloned = FUNCCALL(SETUP_XfOrM762(_), optimize_clone (1 , e , info , empty_eq_hash_tree , 0 ) ); 
    if (e_cloned ) {
#     define XfOrM763_COUNT (0+XfOrM762_COUNT)
#     define SETUP_XfOrM763(x) SETUP_XfOrM762(x)

# 5356 "./../src/optimize.c"
      if ((Scheme_Type ) (((((intptr_t ) (f ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (f ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) 
        f_cloned = FUNCCALL(SETUP_XfOrM763(_), optimize_clone (1 , f , info , empty_eq_hash_tree , 0 ) ); 
      else {

        f_cloned = f ; 
      }

      if (f_cloned ) {
#       define XfOrM764_COUNT (0+XfOrM763_COUNT)
#       define SETUP_XfOrM764(x) SETUP_XfOrM763(x)

# 5364 "./../src/optimize.c"
        app2 -> rator = f_cloned ; 
        app2 -> rand = e_cloned ; 
        info -> inline_fuel >>= 1 ; 
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_application2 ((Scheme_Object * ) app2 , info , context ) )) RET_VALUE_EMPTY_END ; 
      }
    }

    app2 -> rator = f ; 
    app2 -> rand = e ; 
    RET_VALUE_START ((Scheme_Object * ) app2 ) RET_VALUE_END ; 
  }

  {
    Scheme_Object * av ; 
    BLOCK_SETUP((PUSH(av, 0+XfOrM760_COUNT)));
#   define XfOrM761_COUNT (1+XfOrM760_COUNT)
#   define SETUP_XfOrM761(x) SETUP(XfOrM761_COUNT)
    av = NULLED_OUT ; 

# 5378 "./../src/optimize.c"
    av = ((Scheme_Object * ) FUNCCALL(SETUP_XfOrM761(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_Simple_Object ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
    av -> type = scheme_apply_values_type ; 
    (((Scheme_Simple_Object * ) (av ) ) -> u . two_ptr_val . ptr1 ) = f ; 
    (((Scheme_Simple_Object * ) (av ) ) -> u . two_ptr_val . ptr2 ) = e ; 
    RET_VALUE_START (av ) RET_VALUE_END ; 
  }
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

static Scheme_Object * optimize_sequence (Scheme_Object * o , Optimize_Info * info , int context , int sub_opt ) ; 

static Scheme_Object * flatten_sequence (Scheme_Object * o , Optimize_Info * info , int context ) 
{
  Scheme_Sequence * s = (Scheme_Sequence * ) o , * s2 , * s3 ; 
  Scheme_Object * o3 ; 
  int i , j , k , count , extra = 0 , split = 0 , b0 , new_count ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(5);
  BLOCK_SETUP_TOP((PUSH(s3, 0), PUSH(info, 1), PUSH(s2, 2), PUSH(s, 3), PUSH(o3, 4)));
# define XfOrM769_COUNT (5)
# define SETUP_XfOrM769(x) SETUP(XfOrM769_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  s2 = NULLED_OUT ; 
  s3 = NULLED_OUT ; 
  o3 = NULLED_OUT ; 

# 5398 "./../src/optimize.c"
  if (! info -> flatten_fuel )
    RET_VALUE_START (o ) RET_VALUE_END ; 

  b0 = ((Scheme_Type ) (((((intptr_t ) (o ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o ) ) -> type ) ) == (Scheme_Type ) (scheme_begin0_sequence_type ) ) ; 
  count = s -> count ; 


  for (i = 0 ; i < count ; i ++ ) {
    o3 = s -> array [i ] ; 
    if ((((Scheme_Type ) (((((intptr_t ) (o3 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o3 ) ) -> type ) ) == (Scheme_Type ) (scheme_sequence_type ) ) && ! (! i && b0 ) ) 
    || (((Scheme_Type ) (((((intptr_t ) (o3 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o3 ) ) -> type ) ) == (Scheme_Type ) (scheme_begin0_sequence_type ) ) && ! (i == count - 1 && ! b0 ) ) ) {
      s3 = (Scheme_Sequence * ) o3 ; 
      extra += s3 -> count ; 
      split ++ ; 
    }
  }

  if (! split )
    RET_VALUE_START (o ) RET_VALUE_END ; 

  info -> flatten_fuel -- ; 
  info -> size -= split ; 

  new_count = s -> count + extra - split ; 
  if (new_count > 0 ) {
#   define XfOrM781_COUNT (0+XfOrM769_COUNT)
#   define SETUP_XfOrM781(x) SETUP_XfOrM769(x)

# 5423 "./../src/optimize.c"
    s2 = FUNCCALL(SETUP_XfOrM781(_), scheme_malloc_sequence (new_count ) ); 
    s2 -> so . type = s -> so . type ; 
    s2 -> count = new_count ; 
  }

# 5426 "./../src/optimize.c"
  else 
  s2 = ((void * ) 0 ) ; 
  k = 0 ; 


  for (i = 0 ; i < count ; i ++ ) {
    o3 = s -> array [i ] ; 
    if ((((Scheme_Type ) (((((intptr_t ) (o3 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o3 ) ) -> type ) ) == (Scheme_Type ) (scheme_sequence_type ) ) && ! (! i && b0 ) ) 
    || (((Scheme_Type ) (((((intptr_t ) (o3 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o3 ) ) -> type ) ) == (Scheme_Type ) (scheme_begin0_sequence_type ) ) && ! (i == count - 1 && ! b0 ) ) ) {
      s3 = (Scheme_Sequence * ) o3 ; 
      for (j = 0 ; j < s3 -> count ; j ++ ) {
        s2 -> array [k ++ ] = s3 -> array [j ] ; 
      }
    }

# 5439 "./../src/optimize.c"
    else {
      s2 -> array [k ++ ] = o3 ; 
    }
  }

  ((void ) sizeof ((k == new_count ) ? 1 : 0 ) , __extension__ ({
#   define XfOrM770_COUNT (0+XfOrM769_COUNT)
#   define SETUP_XfOrM770(x) SETUP_XfOrM769(x)

# 5444 "./../src/optimize.c" 3
    if (k == new_count )

# 5444 "./../src/optimize.c" 3
    ; 

# 5444 "./../src/optimize.c" 3
    else FUNCCALL(SETUP_XfOrM770(_), __assert_fail ("k == new_count" , "./../src/optimize.c" , 5444 , __extension__ __PRETTY_FUNCTION__ ) ); 
  }
  ) ) 
# 5444 "./../src/optimize.c"
  ; 

  if (s2 -> count == 1 )
    RET_VALUE_START (s2 -> array [0 ] ) RET_VALUE_END ; 

  if ((Scheme_Type ) (((((intptr_t ) (s2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (s2 ) ) -> type ) ) == (Scheme_Type ) (scheme_sequence_type ) ) 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_sequence ((Scheme_Object * ) s2 , info , context , 0 ) )) RET_VALUE_EMPTY_END ; 
  else 
  RET_VALUE_START ((Scheme_Object * ) s2 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 5455 "./../src/optimize.c"
static Scheme_Object * optimize_sequence (Scheme_Object * o , Optimize_Info * info , int context , int sub_opt ) 
{
  Scheme_Sequence * s = (Scheme_Sequence * ) o ; 
  Scheme_Object * le ; 
  int i , count , prev_size ; 
  int drop = 0 , preserves_marks = 0 , single_result = 0 ; 
  Optimize_Info_Sequence info_seq ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(4);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(le, 1), PUSH(s, 2)));
# define XfOrM786_COUNT (3)
# define SETUP_XfOrM786(x) SETUP(XfOrM786_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  le = NULLED_OUT ; 

# 5467 "./../src/optimize.c"
  if (sub_opt )
    FUNCCALL(SETUP_XfOrM786(_), optimize_info_seq_init (info , & info_seq ) ); 
  else (
  memset (& info_seq , 0 , sizeof (info_seq ) ) ) ; 

  count = s -> count ; 
  for (i = 0 ; i < count ; i ++ ) {
#   define XfOrM803_COUNT (0+XfOrM786_COUNT)
#   define SETUP_XfOrM803(x) SETUP_XfOrM786(x)

# 5474 "./../src/optimize.c"
    prev_size = info -> size ; 

    if (sub_opt ) {
#     define XfOrM812_COUNT (0+XfOrM803_COUNT)
#     define SETUP_XfOrM812(x) SETUP_XfOrM803(x)
      FUNCCALL(SETUP_XfOrM812(_), optimize_info_seq_step (info , & info_seq ) )
# 5477 "./../src/optimize.c"
      ; 
      le = FUNCCALL(SETUP_XfOrM812(_), optimize_expr (s -> array [i ] , info , ((i + 1 == count ) ? (context & (~ ((0x3 << 4 ) | 0x2 | 0x4 ) ) ) : 0 ) ) )


      ; 
    }

# 5482 "./../src/optimize.c"
    else 
    le = s -> array [i ] ; 

    if (i + 1 == count ) {
      single_result = info -> single_result ; 
      preserves_marks = info -> preserves_marks ; 
      s -> array [i ] = le ; 
    }

# 5489 "./../src/optimize.c"
    else {
#     define XfOrM804_COUNT (0+XfOrM803_COUNT)
#     define SETUP_XfOrM804(x) SETUP_XfOrM803(x)

# 5490 "./../src/optimize.c"
      if (! sub_opt || ! info -> escapes ) {
#       define XfOrM808_COUNT (0+XfOrM804_COUNT)
#       define SETUP_XfOrM808(x) SETUP_XfOrM804(x)

# 5492 "./../src/optimize.c"
        le = FUNCCALL(SETUP_XfOrM808(_), optimize_ignored (le , info , - 1 , 1 , 5 ) ); 
        if (! le ) {
          drop ++ ; 
          info -> size = prev_size ; 
          s -> array [i ] = ((void * ) 0 ) ; 
        }

# 5497 "./../src/optimize.c"
        else {
          s -> array [i ] = le ; 
        }
      }

# 5500 "./../src/optimize.c"
      else {
        int j ; 

        single_result = info -> single_result ; 
        preserves_marks = info -> preserves_marks ; 

        s -> array [count - 1 ] = le ; 
        for (j = i ; j < count - 1 ; j ++ ) {
          drop ++ ; 
          s -> array [j ] = ((void * ) 0 ) ; 
        }
        break ; 
      }
    }
  }

  if (sub_opt )
    FUNCCALL(SETUP_XfOrM786(_), optimize_info_seq_done (info , & info_seq ) ); 

  info -> preserves_marks = preserves_marks ; 
  info -> single_result = single_result ; 

  if (drop + 1 == s -> count ) {
#   define XfOrM792_COUNT (0+XfOrM786_COUNT)
#   define SETUP_XfOrM792(x) SETUP_XfOrM786(x)

# 5523 "./../src/optimize.c"
    le = s -> array [drop ] ; 
    if (info -> escapes )
      le = FUNCCALL(SETUP_XfOrM792(_), ensure_noncm (le , info ) ); 
    RET_VALUE_START (le ) RET_VALUE_END ; 
  }

  if (drop ) {
    Scheme_Sequence * s2 ; 
    int j = 0 ; 
    BLOCK_SETUP((PUSH(s2, 0+XfOrM786_COUNT)));
#   define XfOrM787_COUNT (1+XfOrM786_COUNT)
#   define SETUP_XfOrM787(x) SETUP(XfOrM787_COUNT)
    s2 = NULLED_OUT ; 

# 5533 "./../src/optimize.c"
    s2 = FUNCCALL(SETUP_XfOrM787(_), scheme_malloc_sequence (s -> count - drop ) ); 
    s2 -> so . type = s -> so . type ; 
    s2 -> count = s -> count - drop ; 

    for (i = 0 ; i < s -> count ; i ++ ) {
      if (s -> array [i ] ) {
        s2 -> array [j ++ ] = s -> array [i ] ; 
      }
    }

    s = s2 ; 
  }

  RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(flatten_sequence ((Scheme_Object * ) s , info , context ) )) RET_VALUE_EMPTY_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

static Scheme_Object * collapse_local (Scheme_Object * var , Optimize_Info * info , int context ) 

{
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(1);
# define XfOrM813_COUNT (0)
# define SETUP_XfOrM813(x) SETUP(XfOrM813_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 5556 "./../src/optimize.c"
  if (! ((Scheme_IR_Local * ) var ) -> mutated ) {
    Scheme_Object * pred ; 
    BLOCK_SETUP((PUSH(pred, 0+XfOrM813_COUNT)));
#   define XfOrM814_COUNT (1+XfOrM813_COUNT)
#   define SETUP_XfOrM814(x) SETUP(XfOrM814_COUNT)
    pred = NULLED_OUT ; 

# 5559 "./../src/optimize.c"
    pred = FUNCCALL(SETUP_XfOrM814(_), expr_implies_predicate (var , info ) ); 
    if (pred ) {
#     define XfOrM815_COUNT (0+XfOrM814_COUNT)
#     define SETUP_XfOrM815(x) SETUP_XfOrM814(x)

# 5561 "./../src/optimize.c"
      if (predicate_implies (pred , scheme_not_proc ) ) 
        RET_VALUE_START (scheme_false ) RET_VALUE_END ; 

      if (context & 0x1 ) {
#       define XfOrM816_COUNT (0+XfOrM815_COUNT)
#       define SETUP_XfOrM816(x) SETUP_XfOrM815(x)

# 5565 "./../src/optimize.c"
        if (predicate_implies_not (pred , scheme_not_proc ) ) 
          RET_VALUE_START (scheme_true ) RET_VALUE_END ; 
      }

      if ((pred ) == (scheme_true_object_p_proc ) ) 
        RET_VALUE_START (scheme_true ) RET_VALUE_END ; 
      if ((pred ) == (scheme_null_p_proc ) ) 
        RET_VALUE_START (scheme_null ) RET_VALUE_END ; 
      if ((pred ) == (scheme_void_p_proc ) ) 
        RET_VALUE_START (scheme_void ) RET_VALUE_END ; 
      if ((pred ) == (scheme_eof_object_p_proc ) ) 
        RET_VALUE_START (scheme_eof ) RET_VALUE_END ; 
    }
  }
  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}



static Scheme_Object * equivalent_exprs (Scheme_Object * a , Scheme_Object * b , 
Optimize_Info * a_info , Optimize_Info * b_info , int context ) 
{
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(4);
  BLOCK_SETUP_TOP((PUSH(b, 0), PUSH(a_info, 1), PUSH(a, 2)));
# define XfOrM817_COUNT (3)
# define SETUP_XfOrM817(x) SETUP(XfOrM817_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 5591 "./../src/optimize.c"
  if ((a ) == (b ) ) 
    RET_VALUE_START (a ) RET_VALUE_END ; 

  if (((Scheme_Type ) (((((intptr_t ) (a ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (a ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_toplevel_type ) ) 
  && ((Scheme_Type ) (((((intptr_t ) (b ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (b ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_toplevel_type ) ) 
  && ((((Scheme_IR_Toplevel * ) a ) -> instance_pos ) == (((Scheme_IR_Toplevel * ) b ) -> instance_pos ) ) 
  && ((((Scheme_IR_Toplevel * ) a ) -> variable_pos ) == (((Scheme_IR_Toplevel * ) b ) -> variable_pos ) ) )
    RET_VALUE_START (a ) RET_VALUE_END ; 

  if (b_info 
  && ((Scheme_Type ) (((((intptr_t ) (a ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (a ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) 
  && (((((intptr_t ) (b ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (b ) ) -> type ) > _scheme_ir_values_types_ ) ) {
    Scheme_Object * n ; 
    BLOCK_SETUP((PUSH(n, 0+XfOrM817_COUNT)));
#   define XfOrM819_COUNT (1+XfOrM817_COUNT)
#   define SETUP_XfOrM819(x) SETUP(XfOrM819_COUNT)
    n = NULLED_OUT ; 

# 5604 "./../src/optimize.c"
    n = FUNCCALL(SETUP_XfOrM819(_), collapse_local (a , b_info , context ) ); 
    if (n && ((n ) == (b ) ) )
      RET_VALUE_START (a ) RET_VALUE_END ; 
  }

  if (a_info 
  && ((Scheme_Type ) (((((intptr_t ) (b ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (b ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) 
  && (((((intptr_t ) (a ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (a ) ) -> type ) > _scheme_ir_values_types_ ) ) {
    Scheme_Object * n ; 
    BLOCK_SETUP((PUSH(n, 0+XfOrM817_COUNT)));
#   define XfOrM818_COUNT (1+XfOrM817_COUNT)
#   define SETUP_XfOrM818(x) SETUP(XfOrM818_COUNT)
    n = NULLED_OUT ; 

# 5613 "./../src/optimize.c"
    n = FUNCCALL(SETUP_XfOrM818(_), collapse_local (b , a_info , context ) ); 
    if (n && ((n ) == (a ) ) )
      RET_VALUE_START (b ) RET_VALUE_END ; 
  }

  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 5621 "./../src/optimize.c"
static void add_type (Optimize_Info * info , Scheme_Object * var , Scheme_Object * pred ) 


{
  Scheme_Hash_Tree * new_types = info -> types ; 
  Scheme_Object * old_pred ; 
  PREPARE_VAR_STACK_ONCE(5);
  BLOCK_SETUP_TOP((PUSH(new_types, 0), PUSH(info, 1), PUSH(pred, 2), PUSH(var, 3), PUSH(old_pred, 4)));
# define XfOrM820_COUNT (5)
# define SETUP_XfOrM820(x) SETUP(XfOrM820_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  old_pred = NULLED_OUT ; 

# 5628 "./../src/optimize.c"
  if (((Scheme_IR_Local * ) var ) -> mutated )
    RET_NOTHING ; 



  if (((Scheme_IR_Local * ) var ) -> val_type )
    RET_NOTHING ; 
  old_pred = FUNCCALL(SETUP_XfOrM820(_), optimize_get_predicate (info , var , 1 ) ); 
  if (old_pred && (predicate_implies (old_pred , pred ) ) )
    RET_NOTHING ; 


  if (old_pred ) {
    if ((((old_pred ) == (scheme_list_p_proc ) ) 
    && (((pred ) == (scheme_pair_p_proc ) ) ) ) 
    || (((old_pred ) == (scheme_pair_p_proc ) ) 
    && (((pred ) == (scheme_list_p_proc ) ) ) ) ) {
      pred = scheme_list_pair_p_proc ; 
    }
  }

  if (! new_types )
    new_types = (scheme_make_hash_tree (SCHEME_hashtr_eq ) ) ; 
  new_types = FUNCCALL(SETUP_XfOrM820(_), scheme_hash_tree_set (new_types , var , pred ) ); 
  info -> types = new_types ; 
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 5655 "./../src/optimize.c"
static void add_type_no (Optimize_Info * info , Scheme_Object * var , Scheme_Object * pred ) 

{
  Scheme_Object * old_pred ; 
  PREPARE_VAR_STACK_ONCE(4);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(pred, 1), PUSH(var, 2), PUSH(old_pred, 3)));
# define XfOrM823_COUNT (4)
# define SETUP_XfOrM823(x) SETUP(XfOrM823_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  old_pred = NULLED_OUT ; 

# 5660 "./../src/optimize.c"
  if (((Scheme_IR_Local * ) var ) -> mutated )
    RET_NOTHING ; 

  old_pred = FUNCCALL(SETUP_XfOrM823(_), optimize_get_predicate (info , var , 1 ) ); 

  if (old_pred && ((old_pred ) == (scheme_list_p_proc ) ) ) {
#   define XfOrM825_COUNT (0+XfOrM823_COUNT)
#   define SETUP_XfOrM825(x) SETUP_XfOrM823(x)

# 5667 "./../src/optimize.c"
    if ((pred ) == (scheme_null_p_proc ) ) 
      FUNCCALL(SETUP_XfOrM825(_), add_type (info , var , scheme_list_pair_p_proc ) ); 



    if (((pred ) == (scheme_pair_p_proc ) ) 
    || ((pred ) == (scheme_list_pair_p_proc ) ) )
      FUNCCALL(SETUP_XfOrM825(_), add_type (info , var , scheme_null_p_proc ) ); 
  }

  if (old_pred && ((old_pred ) == (scheme_boolean_p_proc ) ) ) {
#   define XfOrM824_COUNT (0+XfOrM823_COUNT)
#   define SETUP_XfOrM824(x) SETUP_XfOrM823(x)

# 5679 "./../src/optimize.c"
    if ((pred ) == (scheme_not_proc ) ) 
      FUNCCALL(SETUP_XfOrM824(_), add_type (info , var , scheme_true_object_p_proc ) ); 


    if ((pred ) == (scheme_true_object_p_proc ) ) 
      FUNCCALL_EMPTY(add_type (info , var , scheme_not_proc ) ); 
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 5690 "./../src/optimize.c"
static void merge_types (Optimize_Info * src_info , Optimize_Info * info , Scheme_Hash_Tree * skip_vars ) 
{
  Scheme_Hash_Tree * types = src_info -> types ; 
  Scheme_Object * var , * pred ; 
  intptr_t i ; 
  PREPARE_VAR_STACK_ONCE(5);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(pred, 1), PUSH(var, 2), PUSH(skip_vars, 3), PUSH(types, 4)));
# define XfOrM826_COUNT (5)
# define SETUP_XfOrM826(x) SETUP(XfOrM826_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  var = NULLED_OUT ; 
  pred = NULLED_OUT ; 

# 5696 "./../src/optimize.c"
  if (! types )
    RET_NOTHING ; 

  if (skip_vars ) {
#   define XfOrM830_COUNT (0+XfOrM826_COUNT)
#   define SETUP_XfOrM830(x) SETUP_XfOrM826(x)

# 5701 "./../src/optimize.c"
    i = (scheme_hash_tree_next (skip_vars , - 1 ) ) ; 
    while (i != - 1 ) {
#     define XfOrM832_COUNT (0+XfOrM830_COUNT)
#     define SETUP_XfOrM832(x) SETUP_XfOrM830(x)
      (
# 5703 "./../src/optimize.c"
      scheme_hash_tree_index (skip_vars , i , & var , ((void * ) 0 ) ) ) ; 
      types = FUNCCALL(SETUP_XfOrM832(_), scheme_hash_tree_set (types , var , ((void * ) 0 ) ) ); 
      i = (scheme_hash_tree_next (skip_vars , i ) ) ; 
    }
  }

  if (! info -> types || (types -> count > info -> types -> count ) ) {

    Scheme_Hash_Tree * old_types = info -> types ; 
    info -> types = types ; 
    if (! old_types )
      RET_NOTHING ; 
    types = old_types ; 
  }

  i = (scheme_hash_tree_next (types , - 1 ) ) ; 
  while (i != - 1 ) {
#   define XfOrM828_COUNT (0+XfOrM826_COUNT)
#   define SETUP_XfOrM828(x) SETUP_XfOrM826(x)
    (
# 5720 "./../src/optimize.c"
    scheme_hash_tree_index (types , i , & var , & pred ) ) ; 
    FUNCCALL(SETUP_XfOrM828(_), add_type (info , var , pred ) ); 
    i = (scheme_hash_tree_next (types , i ) ) ; 
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 5726 "./../src/optimize.c"
static void merge_branchs_types (Optimize_Info * t_info , Optimize_Info * f_info , 
Optimize_Info * base_info ) 



{
  Scheme_Hash_Tree * t_types = t_info -> types , * f_types = f_info -> types ; 
  Scheme_Object * var , * t_pred , * f_pred ; 
  intptr_t i ; 
  PREPARE_VAR_STACK_ONCE(6);
  BLOCK_SETUP_TOP((PUSH(t_pred, 0), PUSH(f_types, 1), PUSH(var, 2), PUSH(f_pred, 3), PUSH(base_info, 4), PUSH(t_types, 5)));
# define XfOrM833_COUNT (6)
# define SETUP_XfOrM833(x) SETUP(XfOrM833_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  var = NULLED_OUT ; 
  t_pred = NULLED_OUT ; 
  f_pred = NULLED_OUT ; 

# 5736 "./../src/optimize.c"
  if (! t_types || ! f_types )
    RET_NOTHING ; 

  if (f_types -> count > t_types -> count ) {
    Scheme_Hash_Tree * swap = f_types ; 
    f_types = t_types ; 
    t_types = swap ; 
  }

  i = (scheme_hash_tree_next (f_types , - 1 ) ) ; 
  while (i != - 1 ) {
#   define XfOrM839_COUNT (0+XfOrM833_COUNT)
#   define SETUP_XfOrM839(x) SETUP_XfOrM833(x)
    (
# 5747 "./../src/optimize.c"
    scheme_hash_tree_index (f_types , i , & var , & f_pred ) ) ; 
    t_pred = (scheme_eq_hash_tree_get (t_types , var ) ) ; 
    if (t_pred ) {
#     define XfOrM840_COUNT (0+XfOrM839_COUNT)
#     define SETUP_XfOrM840(x) SETUP_XfOrM839(x)

# 5750 "./../src/optimize.c"
      if (predicate_implies (f_pred , t_pred ) ) 
        FUNCCALL(SETUP_XfOrM840(_), add_type (base_info , var , t_pred ) ); 
      else if (predicate_implies (t_pred , f_pred ) ) 
        FUNCCALL(SETUP_XfOrM840(_), add_type (base_info , var , f_pred ) ); 
      else {
#       define XfOrM841_COUNT (0+XfOrM840_COUNT)
#       define SETUP_XfOrM841(x) SETUP_XfOrM840(x)

# 5756 "./../src/optimize.c"
        if ((((t_pred ) == (scheme_null_p_proc ) ) 
        && (((f_pred ) == (scheme_list_pair_p_proc ) ) ) ) 
        || (((t_pred ) == (scheme_list_pair_p_proc ) ) 
        && (((f_pred ) == (scheme_null_p_proc ) ) ) ) ) {
#         define XfOrM843_COUNT (0+XfOrM841_COUNT)
#         define SETUP_XfOrM843(x) SETUP_XfOrM841(x)
          FUNCCALL(SETUP_XfOrM843(_), add_type (base_info , var , scheme_list_p_proc ) )
# 5760 "./../src/optimize.c"
          ; 
        }

        if ((((t_pred ) == (scheme_not_proc ) ) 
        && (((f_pred ) == (scheme_true_object_p_proc ) ) ) ) 
        || (((t_pred ) == (scheme_true_object_p_proc ) ) 
        && (((f_pred ) == (scheme_not_proc ) ) ) ) ) {
#         define XfOrM842_COUNT (0+XfOrM841_COUNT)
#         define SETUP_XfOrM842(x) SETUP_XfOrM841(x)
          FUNCCALL(SETUP_XfOrM842(_), add_type (base_info , var , scheme_boolean_p_proc ) )
# 5767 "./../src/optimize.c"
          ; 
        }
      }
    }
    i = (scheme_hash_tree_next (f_types , i ) ) ; 
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 5775 "./../src/optimize.c"
static int relevant_predicate (Scheme_Object * pred ) 
{
  /* No conversion */




  if (((pred ) == (scheme_pair_p_proc ) ) 
  || ((pred ) == (scheme_list_p_proc ) ) 
  || ((pred ) == (scheme_list_pair_p_proc ) ) 
  || ((pred ) == (scheme_mpair_p_proc ) ) 
  || ((pred ) == (scheme_box_p_proc ) ) 
  || ((pred ) == (scheme_string_p_proc ) ) 
  || ((pred ) == (scheme_byte_string_p_proc ) ) 
  || ((pred ) == (scheme_vector_p_proc ) ) 
  || ((pred ) == (scheme_procedure_p_proc ) ) 
  || ((pred ) == (scheme_syntax_p_proc ) ) )
    return 1 ; 
  if (((pred ) == (scheme_char_p_proc ) ) 
  || ((pred ) == (scheme_flonum_p_proc ) ) 
  || ((pred ) == (scheme_number_p_proc ) ) 
  || ((pred ) == (scheme_real_p_proc ) ) 
  || ((pred ) == (scheme_extflonum_p_proc ) ) )
    return 2 ; 
  if (((pred ) == (scheme_symbol_p_proc ) ) 
  || ((pred ) == (scheme_keyword_p_proc ) ) 
  || ((pred ) == (scheme_fixnum_p_proc ) ) 
  || ((pred ) == (scheme_interned_char_p_proc ) ) 
  || ((pred ) == (scheme_boolean_p_proc ) ) )
    return 3 ; 
  if (((pred ) == (scheme_null_p_proc ) ) 
  || ((pred ) == (scheme_void_p_proc ) ) 
  || ((pred ) == (scheme_eof_object_p_proc ) ) 
  || ((pred ) == (scheme_true_object_p_proc ) ) 
  || ((pred ) == (scheme_not_proc ) ) )
    return 4 ; 

  return 0 ; 
}

static int predicate_implies (Scheme_Object * pred1 , Scheme_Object * pred2 ) 
{
  /* No conversion */

# 5817 "./../src/optimize.c"
  if (! pred1 || ! pred2 )
    return 0 ; 


  if ((pred1 ) == (pred2 ) ) 
    return 1 ; 


  if (((pred2 ) == (scheme_list_p_proc ) ) 
  && ((pred1 ) == (scheme_null_p_proc ) ) )
    return 1 ; 


  if (((pred2 ) == (scheme_list_p_proc ) ) 
  && ((pred1 ) == (scheme_list_pair_p_proc ) ) )
    return 1 ; 


  if (((pred2 ) == (scheme_pair_p_proc ) ) 
  && ((pred1 ) == (scheme_list_pair_p_proc ) ) )
    return 1 ; 


  if (((pred2 ) == (scheme_char_p_proc ) ) 
  && ((pred1 ) == (scheme_interned_char_p_proc ) ) )
    return 1 ; 


  if (((pred2 ) == (scheme_boolean_p_proc ) ) 
  && (((pred1 ) == (scheme_not_proc ) ) 
  || ((pred1 ) == (scheme_true_object_p_proc ) ) ) )
    return 1 ; 


  if (((pred2 ) == (scheme_number_p_proc ) ) 
  && (((pred1 ) == (scheme_real_p_proc ) ) 
  || ((pred1 ) == (scheme_fixnum_p_proc ) ) 
  || ((pred1 ) == (scheme_flonum_p_proc ) ) ) )
    return 1 ; 


  if (((pred2 ) == (scheme_real_p_proc ) ) 
  && (((pred1 ) == (scheme_fixnum_p_proc ) ) 
  || ((pred1 ) == (scheme_flonum_p_proc ) ) ) )
    return 1 ; 


  if (((Scheme_Type ) (((((intptr_t ) (pred1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pred1 ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) ) 
  && ((Scheme_Type ) (((((intptr_t ) (pred2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pred2 ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) ) 
  && is_struct_identity_subtype (((Scheme_Struct_Proc_Shape * ) pred1 ) -> identity , 
  ((Scheme_Struct_Proc_Shape * ) pred2 ) -> identity ) )
    return 1 ; 

  return 0 ; 
}

static int predicate_implies_not (Scheme_Object * pred1 , Scheme_Object * pred2 ) 
{
  /* No conversion */

# 5875 "./../src/optimize.c"
  if (((pred1 ) == (scheme_pair_p_proc ) ) && ((pred2 ) == (scheme_list_p_proc ) ) )
    return 0 ; 
  if (((pred1 ) == (scheme_list_p_proc ) ) && ((pred2 ) == (scheme_pair_p_proc ) ) )
    return 0 ; 





  if ((((Scheme_Type ) (((((intptr_t ) (pred1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pred1 ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) ) 
  || ((pred1 ) == (scheme_procedure_p_proc ) ) ) 
  && (((Scheme_Type ) (((((intptr_t ) (pred2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pred2 ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) ) 
  || ((pred2 ) == (scheme_procedure_p_proc ) ) ) )
    return 0 ; 



  return (! predicate_implies (pred1 , pred2 ) && ! predicate_implies (pred2 , pred1 ) ) ; 
}

static void add_types_for_t_branch (Scheme_Object * t , Optimize_Info * info , int fuel ) 
{
  PREPARE_VAR_STACK(5);
  BLOCK_SETUP_TOP((PUSH(t, 0), PUSH(info, 1)));
# define XfOrM848_COUNT (2)
# define SETUP_XfOrM848(x) SETUP(XfOrM848_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 5897 "./../src/optimize.c"
  if (fuel < 0 )
    RET_NOTHING ; 

  if ((Scheme_Type ) (((((intptr_t ) (t ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (t ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) )  {
#   define XfOrM863_COUNT (0+XfOrM848_COUNT)
#   define SETUP_XfOrM863(x) SETUP_XfOrM848(x)
    FUNCCALL(SETUP_XfOrM863(_), add_type_no (info , t , scheme_not_proc ) )
# 5901 "./../src/optimize.c"
    ; 
  }

# 5902 "./../src/optimize.c"
  else if ((Scheme_Type ) (((((intptr_t ) (t ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (t ) ) -> type ) ) == (Scheme_Type ) (scheme_application2_type ) )  {
    Scheme_App2_Rec * app = (Scheme_App2_Rec * ) t ; 
    BLOCK_SETUP((PUSH(app, 0+XfOrM848_COUNT)));
#   define XfOrM858_COUNT (1+XfOrM848_COUNT)
#   define SETUP_XfOrM858(x) SETUP(XfOrM858_COUNT)

# 5904 "./../src/optimize.c"
    if (((Scheme_Type ) (((((intptr_t ) (app -> rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
    && ((Scheme_Type ) (((((intptr_t ) (app -> rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) 
    && (relevant_predicate (app -> rator ) ) ) {
#     define XfOrM862_COUNT (0+XfOrM858_COUNT)
#     define SETUP_XfOrM862(x) SETUP_XfOrM858(x)
      FUNCCALL(SETUP_XfOrM862(_), add_type (info , app -> rand , app -> rator ) )
      ; 
    }
    if ((app -> rator ) == (scheme_not_proc ) )  {
#     define XfOrM861_COUNT (0+XfOrM858_COUNT)
#     define SETUP_XfOrM861(x) SETUP_XfOrM858(x)
      FUNCCALL(SETUP_XfOrM861(_), add_types_for_f_branch (app -> rand , info , fuel - 1 ) )
# 5913 "./../src/optimize.c"
      ; 
    }

    if ((Scheme_Type ) (((((intptr_t ) (app -> rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) )  {
      Scheme_Object * shape ; 
      BLOCK_SETUP((PUSH(shape, 0+XfOrM858_COUNT)));
#     define XfOrM859_COUNT (1+XfOrM858_COUNT)
#     define SETUP_XfOrM859(x) SETUP(XfOrM859_COUNT)
      shape = NULLED_OUT ; 

# 5918 "./../src/optimize.c"
      shape = FUNCCALL(SETUP_XfOrM859(_), get_struct_proc_shape (app -> rator , info , 0 ) ); 
      if (shape 
      && ((((Scheme_Struct_Proc_Shape * ) shape ) -> mode & 0x7 ) == 2 ) 
      && ! ((((Scheme_Struct_Proc_Shape * ) shape ) -> identity ) == (scheme_null ) ) ) {
#       define XfOrM860_COUNT (0+XfOrM859_COUNT)
#       define SETUP_XfOrM860(x) SETUP_XfOrM859(x)
        FUNCCALL(SETUP_XfOrM860(_), add_type (info , app -> rand , shape ) )
# 5922 "./../src/optimize.c"
        ; 
      }
    }
  }

# 5925 "./../src/optimize.c"
  else if ((Scheme_Type ) (((((intptr_t ) (t ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (t ) ) -> type ) ) == (Scheme_Type ) (scheme_application3_type ) )  {
    Scheme_App3_Rec * app = (Scheme_App3_Rec * ) t ; 
    Scheme_Object * pred1 , * pred2 ; 
    BLOCK_SETUP((PUSH(pred2, 0+XfOrM848_COUNT), PUSH(app, 1+XfOrM848_COUNT), PUSH(pred1, 2+XfOrM848_COUNT)));
#   define XfOrM852_COUNT (3+XfOrM848_COUNT)
#   define SETUP_XfOrM852(x) SETUP(XfOrM852_COUNT)
    pred1 = NULLED_OUT ; 
    pred2 = NULLED_OUT ; 

# 5928 "./../src/optimize.c"
    if (((app -> rator ) == (scheme_eq_proc ) ) 
    || ((app -> rator ) == (scheme_eqv_proc ) ) 
    || ((app -> rator ) == (scheme_equal_proc ) ) ) {
#     define XfOrM853_COUNT (0+XfOrM852_COUNT)
#     define SETUP_XfOrM853(x) SETUP_XfOrM852(x)

# 5931 "./../src/optimize.c"
      if ((Scheme_Type ) (((((intptr_t ) (app -> rand1 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand1 ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) )  {
#       define XfOrM856_COUNT (0+XfOrM853_COUNT)
#       define SETUP_XfOrM856(x) SETUP_XfOrM853(x)

# 5932 "./../src/optimize.c"
        pred1 = FUNCCALL(SETUP_XfOrM856(_), expr_implies_predicate (app -> rand1 , info ) ); 
        if (! pred1 ) {
#         define XfOrM857_COUNT (0+XfOrM856_COUNT)
#         define SETUP_XfOrM857(x) SETUP_XfOrM856(x)

# 5934 "./../src/optimize.c"
          pred2 = FUNCCALL(SETUP_XfOrM857(_), expr_implies_predicate (app -> rand2 , info ) ); 
          if (pred2 )
            FUNCCALL(SETUP_XfOrM857(_), add_type (info , app -> rand1 , pred2 ) ); 
        }
      }
      if ((Scheme_Type ) (((((intptr_t ) (app -> rand2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand2 ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) )  {
#       define XfOrM854_COUNT (0+XfOrM853_COUNT)
#       define SETUP_XfOrM854(x) SETUP_XfOrM853(x)

# 5940 "./../src/optimize.c"
        pred2 = FUNCCALL(SETUP_XfOrM854(_), expr_implies_predicate (app -> rand2 , info ) ); 
        if (! pred2 ) {
#         define XfOrM855_COUNT (0+XfOrM854_COUNT)
#         define SETUP_XfOrM855(x) SETUP_XfOrM854(x)

# 5942 "./../src/optimize.c"
          pred1 = FUNCCALL(SETUP_XfOrM855(_), expr_implies_predicate (app -> rand1 , info ) ); 
          if (pred1 )
            FUNCCALL(SETUP_XfOrM855(_), add_type (info , app -> rand2 , pred1 ) ); 
        }
      }
    }
  }
  else if ((Scheme_Type ) (((((intptr_t ) (t ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (t ) ) -> type ) ) == (Scheme_Type ) (scheme_branch_type ) )  {
    Scheme_Branch_Rec * b = (Scheme_Branch_Rec * ) t ; 
    BLOCK_SETUP((PUSH(b, 0+XfOrM848_COUNT)));
#   define XfOrM849_COUNT (1+XfOrM848_COUNT)
#   define SETUP_XfOrM849(x) SETUP(XfOrM849_COUNT)

# 5951 "./../src/optimize.c"
    if (((b -> fbranch ) ) == (scheme_false ) )  {
#     define XfOrM851_COUNT (0+XfOrM849_COUNT)
#     define SETUP_XfOrM851(x) SETUP_XfOrM849(x)
      FUNCCALL(SETUP_XfOrM851(_), add_types_for_t_branch (b -> test , info , fuel - 1 ) )
# 5952 "./../src/optimize.c"
      ; 
      FUNCCALL(SETUP_XfOrM851(_), add_types_for_t_branch (b -> tbranch , info , fuel - 1 ) ); 
    }
    if (((b -> tbranch ) ) == (scheme_false ) )  {
#     define XfOrM850_COUNT (0+XfOrM849_COUNT)
#     define SETUP_XfOrM850(x) SETUP_XfOrM849(x)
      FUNCCALL(SETUP_XfOrM850(_), add_types_for_f_branch (b -> test , info , fuel - 1 ) )
# 5956 "./../src/optimize.c"
      ; 
      FUNCCALL_EMPTY(add_types_for_t_branch (b -> fbranch , info , fuel - 1 ) ); 
    }
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 5962 "./../src/optimize.c"
static void add_types_for_f_branch (Scheme_Object * t , Optimize_Info * info , int fuel ) 
{
  PREPARE_VAR_STACK(3);
  BLOCK_SETUP_TOP((PUSH(t, 0), PUSH(info, 1)));
# define XfOrM864_COUNT (2)
# define SETUP_XfOrM864(x) SETUP(XfOrM864_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 5964 "./../src/optimize.c"
  if (fuel < 0 )
    RET_NOTHING ; 

  if ((Scheme_Type ) (((((intptr_t ) (t ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (t ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) )  {
#   define XfOrM870_COUNT (0+XfOrM864_COUNT)
#   define SETUP_XfOrM870(x) SETUP_XfOrM864(x)
    FUNCCALL(SETUP_XfOrM870(_), add_type (info , t , scheme_not_proc ) )
# 5968 "./../src/optimize.c"
    ; 
  }
  else if ((Scheme_Type ) (((((intptr_t ) (t ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (t ) ) -> type ) ) == (Scheme_Type ) (scheme_application2_type ) )  {
    Scheme_App2_Rec * app = (Scheme_App2_Rec * ) t ; 
#   define XfOrM868_COUNT (0+XfOrM864_COUNT)
#   define SETUP_XfOrM868(x) SETUP_XfOrM864(x)

# 5972 "./../src/optimize.c"
    if (((Scheme_Type ) (((((intptr_t ) (app -> rator ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rator ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) ) 
    && ((Scheme_Type ) (((((intptr_t ) (app -> rand ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> rand ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) 
    && (relevant_predicate (app -> rator ) ) ) {
#     define XfOrM869_COUNT (0+XfOrM868_COUNT)
#     define SETUP_XfOrM869(x) SETUP_XfOrM868(x)
      FUNCCALL(SETUP_XfOrM869(_), add_type_no (info , app -> rand , app -> rator ) ); 
    }
  }
  else if ((Scheme_Type ) (((((intptr_t ) (t ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (t ) ) -> type ) ) == (Scheme_Type ) (scheme_branch_type ) )  {
    Scheme_Branch_Rec * b = (Scheme_Branch_Rec * ) t ; 
    BLOCK_SETUP((PUSH(b, 0+XfOrM864_COUNT)));
#   define XfOrM865_COUNT (1+XfOrM864_COUNT)
#   define SETUP_XfOrM865(x) SETUP(XfOrM865_COUNT)

# 5982 "./../src/optimize.c"
    if ((b -> fbranch ) == (scheme_true ) )  {
#     define XfOrM867_COUNT (0+XfOrM865_COUNT)
#     define SETUP_XfOrM867(x) SETUP_XfOrM865(x)
      FUNCCALL(SETUP_XfOrM867(_), add_types_for_t_branch (b -> test , info , fuel - 1 ) )
# 5983 "./../src/optimize.c"
      ; 
      FUNCCALL(SETUP_XfOrM867(_), add_types_for_f_branch (b -> tbranch , info , fuel - 1 ) ); 
    }
    if ((b -> tbranch ) == (scheme_true ) )  {
#     define XfOrM866_COUNT (0+XfOrM865_COUNT)
#     define SETUP_XfOrM866(x) SETUP_XfOrM865(x)
      FUNCCALL(SETUP_XfOrM866(_), add_types_for_f_branch (b -> test , info , fuel - 1 ) )
# 5987 "./../src/optimize.c"
      ; 
      FUNCCALL_EMPTY(add_types_for_f_branch (b -> fbranch , info , fuel - 1 ) ); 
    }
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 5993 "./../src/optimize.c"
static int or_tentative (int x , int y ) 
{
  /* No conversion */

# 5995 "./../src/optimize.c"
  if (x && y ) {
    if ((x < 0 ) || (y < 0 ) )
      return - 1 ; 
    else 
    return 1 ; 
  }

# 6000 "./../src/optimize.c"
  else {
    return 0 ; 
  }
}

static Scheme_Object * optimize_branch (Scheme_Object * o , Optimize_Info * info , int context ) 
{
  Scheme_Branch_Rec * b ; 
  Scheme_Object * t , * tb , * fb ; 
  int init_vclock , init_aclock , init_kclock , init_sclock ; 
  Optimize_Info * then_info , * else_info ; 
  Optimize_Info * then_info_init , * else_info_init ; 
  Optimize_Info_Sequence info_seq ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(14);
  BLOCK_SETUP_TOP((PUSH(b, 0), PUSH(then_info_init, 1), PUSH(info, 2), PUSH(o, 3), PUSH(else_info, 4), PUSH(t, 5), PUSH(else_info_init, 6), PUSH(then_info, 7), PUSH(tb, 8), PUSH(fb, 9)));
# define XfOrM874_COUNT (10)
# define SETUP_XfOrM874(x) SETUP(XfOrM874_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  b = NULLED_OUT ; 
  t = NULLED_OUT ; 
  tb = NULLED_OUT ; 
  fb = NULLED_OUT ; 
  then_info = NULLED_OUT ; 
  else_info = NULLED_OUT ; 
  then_info_init = NULLED_OUT ; 
  else_info_init = NULLED_OUT ; 

# 6014 "./../src/optimize.c"
  b = (Scheme_Branch_Rec * ) o ; 

  t = b -> test ; 
  tb = b -> tbranch ; 
  fb = b -> fbranch ; 


  if (FUNCCALL(SETUP_XfOrM874(_), equivalent_exprs (t , fb , ((void * ) 0 ) , ((void * ) 0 ) , 0 ) )) {
    fb = scheme_false ; 
  }


  if ((context & 0x1 ) 
  && FUNCCALL(SETUP_XfOrM874(_), equivalent_exprs (t , tb , ((void * ) 0 ) , ((void * ) 0 ) , 0 ) )) {
    tb = scheme_true ; 
  }
  FUNCCALL(SETUP_XfOrM874(_), optimize_info_seq_init (info , & info_seq ) )
  ; 

  t = FUNCCALL(SETUP_XfOrM874(_), optimize_expr (t , info , 0x1 | 0x4 ) ); 

  if (info -> escapes ) {
#   define XfOrM907_COUNT (0+XfOrM874_COUNT)
#   define SETUP_XfOrM907(x) SETUP_XfOrM874(x)
    FUNCCALL(SETUP_XfOrM907(_), optimize_info_seq_done (info , & info_seq ) )
# 6036 "./../src/optimize.c"
    ; 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_noncm (t , info ) )) RET_VALUE_EMPTY_END ; 
  }


  {
    Scheme_Object * inside = ((void * ) 0 ) , * t2 = t ; 
    BLOCK_SETUP((PUSH(inside, 0+XfOrM874_COUNT), PUSH(t2, 1+XfOrM874_COUNT)));
#   define XfOrM894_COUNT (2+XfOrM874_COUNT)
#   define SETUP_XfOrM894(x) SETUP(XfOrM894_COUNT)

# 6044 "./../src/optimize.c"
    while (1 ) {
#     define XfOrM904_COUNT (0+XfOrM894_COUNT)
#     define SETUP_XfOrM904(x) SETUP_XfOrM894(x)
      FUNCCALL(SETUP_XfOrM904(_), extract_tail_inside (& t2 , & inside , 0 ) )
# 6045 "./../src/optimize.c"
      ; 


      if ((Scheme_Type ) (((((intptr_t ) (t2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (t2 ) ) -> type ) ) == (Scheme_Type ) (scheme_application2_type ) )  {
        Scheme_App2_Rec * app = (Scheme_App2_Rec * ) t2 ; 
#       define XfOrM905_COUNT (0+XfOrM904_COUNT)
#       define SETUP_XfOrM905(x) SETUP_XfOrM904(x)

# 6051 "./../src/optimize.c"
        if ((scheme_not_proc ) == (app -> rator ) )  {
#         define XfOrM906_COUNT (0+XfOrM905_COUNT)
#         define SETUP_XfOrM906(x) SETUP_XfOrM905(x)

# 6052 "./../src/optimize.c"
          t2 = tb ; 
          tb = fb ; 
          fb = t2 ; 

          t2 = app -> rand ; 
          t = FUNCCALL(SETUP_XfOrM906(_), replace_tail_inside (t2 , inside , t ) ); 
        }

# 6058 "./../src/optimize.c"
        else 
        break ; 
      }

# 6060 "./../src/optimize.c"
      else 
      break ; 
    }

    if (! (((((intptr_t ) (t2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (t2 ) ) -> type ) > _scheme_ir_values_types_ ) ) {


      Scheme_Object * pred ; 
      BLOCK_SETUP((PUSH(pred, 0+XfOrM894_COUNT)));
#     define XfOrM896_COUNT (1+XfOrM894_COUNT)
#     define SETUP_XfOrM896(x) SETUP(XfOrM896_COUNT)
      pred = NULLED_OUT ; 

# 6069 "./../src/optimize.c"
      pred = FUNCCALL(SETUP_XfOrM896(_), expr_implies_predicate (t2 , info ) ); 
      if (pred ) {
        Scheme_Object * test_val = ((void * ) 0 ) ; 
        BLOCK_SETUP((PUSH(test_val, 0+XfOrM896_COUNT)));
#       define XfOrM897_COUNT (1+XfOrM896_COUNT)
#       define SETUP_XfOrM897(x) SETUP(XfOrM897_COUNT)

# 6073 "./../src/optimize.c"
        if (predicate_implies (pred , scheme_not_proc ) ) 
          test_val = scheme_false ; 
        else if (predicate_implies_not (pred , scheme_not_proc ) ) 
          test_val = scheme_true ; 

        if (test_val ) {
#         define XfOrM898_COUNT (0+XfOrM897_COUNT)
#         define SETUP_XfOrM898(x) SETUP_XfOrM897(x)

# 6079 "./../src/optimize.c"
          t2 = FUNCCALL(SETUP_XfOrM898(_), optimize_ignored (t2 , info , 1 , 0 , 5 ) ); 
          t = FUNCCALL(SETUP_XfOrM898(_), replace_tail_inside (t2 , inside , t ) ); 

          t2 = test_val ; 
          if (FUNCCALL(SETUP_XfOrM898(_), scheme_omittable_expr (t , 1 , 5 , 0 , info , ((void * ) 0 ) ) )) {
            t = test_val ; 
            inside = ((void * ) 0 ) ; 
          }

# 6086 "./../src/optimize.c"
          else {
#           define XfOrM899_COUNT (0+XfOrM898_COUNT)
#           define SETUP_XfOrM899(x) SETUP_XfOrM898(x)

# 6087 "./../src/optimize.c"
            t = FUNCCALL(SETUP_XfOrM899(_), make_sequence_2 (t , test_val ) ); 
            inside = t ; 
          }
        }
      }
    }

    if (((((intptr_t ) (t2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (t2 ) ) -> type ) > _scheme_ir_values_types_ ) {

      Scheme_Object * xb ; 
      BLOCK_SETUP((PUSH(xb, 0+XfOrM894_COUNT)));
#     define XfOrM895_COUNT (1+XfOrM894_COUNT)
#     define SETUP_XfOrM895(x) SETUP(XfOrM895_COUNT)
      xb = NULLED_OUT ; 
      FUNCCALL(SETUP_XfOrM895(_), optimize_info_seq_done (info , & info_seq ) )
# 6098 "./../src/optimize.c"
      ; 
      info -> size -= 1 ; 

      if (((t2 ) ) == (scheme_false ) ) 
        xb = FUNCCALL(SETUP_XfOrM895(_), optimize_expr (fb , info , (context & (~ ((0x3 << 4 ) | 0x2 | 0x4 ) ) ) ) ); 
      else 
      xb = FUNCCALL(SETUP_XfOrM895(_), optimize_expr (tb , info , (context & (~ ((0x3 << 4 ) | 0x2 | 0x4 ) ) ) ) ); 
      FUNCCALL(SETUP_XfOrM895(_), optimize_info_seq_done (info , & info_seq ) )
      ; 
      RET_VALUE_START (FUNCCALL(SETUP_XfOrM895(_), replace_tail_inside (xb , inside , t ) )) RET_VALUE_END ; 
    }
  }
  FUNCCALL(SETUP_XfOrM874(_), optimize_info_seq_step (info , & info_seq ) )
  ; 

  info -> vclock += 1 ; 

  init_vclock = info -> vclock ; 
  init_aclock = info -> aclock ; 
  init_kclock = info -> kclock ; 
  init_sclock = info -> sclock ; 

  then_info = FUNCCALL(SETUP_XfOrM874(_), optimize_info_add_frame (info , 0 ) ); 
  FUNCCALL_AGAIN(add_types_for_t_branch (t , then_info , 5 ) ); 
  then_info_init = FUNCCALL_AGAIN(optimize_info_add_frame (then_info , 0 ) ); 
  tb = FUNCCALL_AGAIN(optimize_expr (tb , then_info , (context & (~ ((0x3 << 4 ) | 0x2 | 0x4 ) ) ) ) ); 
  FUNCCALL_AGAIN(optimize_info_done (then_info , ((void * ) 0 ) ) ); 

  info -> escapes = 0 ; 
  info -> vclock = init_vclock ; 
  info -> aclock = init_aclock ; 
  info -> kclock = init_kclock ; 
  info -> sclock = init_sclock ; 
  FUNCCALL(SETUP_XfOrM874(_), optimize_info_seq_step (info , & info_seq ) )
  ; 

  else_info = FUNCCALL_AGAIN(optimize_info_add_frame (info , 0 ) ); 
  FUNCCALL_AGAIN(add_types_for_f_branch (t , else_info , 5 ) ); 
  else_info_init = FUNCCALL_AGAIN(optimize_info_add_frame (else_info , 0 ) ); 
  fb = FUNCCALL_AGAIN(optimize_expr (fb , else_info , (context & (~ ((0x3 << 4 ) | 0x2 | 0x4 ) ) ) ) ); 
  FUNCCALL_AGAIN(optimize_info_done (else_info , ((void * ) 0 ) ) ); 

  if (then_info -> escapes && else_info -> escapes ) {

    info -> preserves_marks = 1 ; 
    info -> single_result = 1 ; 
    info -> kclock = init_kclock ; 
  }
  else if (else_info -> escapes ) {
#   define XfOrM892_COUNT (0+XfOrM874_COUNT)
#   define SETUP_XfOrM892(x) SETUP_XfOrM874(x)

# 6147 "./../src/optimize.c"
    info -> preserves_marks = then_info -> preserves_marks ; 
    info -> single_result = then_info -> single_result ; 
    info -> kclock = then_info -> kclock ; 
    FUNCCALL(SETUP_XfOrM892(_), merge_types (then_info , info , ((void * ) 0 ) ) ); 
    info -> escapes = 0 ; 
  }
  else if (then_info -> escapes ) {
#   define XfOrM891_COUNT (0+XfOrM874_COUNT)
#   define SETUP_XfOrM891(x) SETUP_XfOrM874(x)

# 6154 "./../src/optimize.c"
    info -> preserves_marks = else_info -> preserves_marks ; 
    info -> single_result = else_info -> single_result ; 
    FUNCCALL(SETUP_XfOrM891(_), merge_types (else_info , info , ((void * ) 0 ) ) ); 
    info -> escapes = 0 ; 
  }
  else {
    int new_preserves_marks , new_single_result ; 
#   define XfOrM890_COUNT (0+XfOrM874_COUNT)
#   define SETUP_XfOrM890(x) SETUP_XfOrM874(x)

# 6162 "./../src/optimize.c"
    new_preserves_marks = FUNCCALL(SETUP_XfOrM890(_), or_tentative (then_info -> preserves_marks , else_info -> preserves_marks ) ); 
    info -> preserves_marks = new_preserves_marks ; 
    new_single_result = FUNCCALL(SETUP_XfOrM890(_), or_tentative (then_info -> single_result , else_info -> single_result ) ); 
    info -> single_result = new_single_result ; 
    if (then_info -> kclock > info -> kclock )
      info -> kclock = then_info -> kclock ; 
    FUNCCALL(SETUP_XfOrM890(_), merge_branchs_types (then_info , else_info , info ) ); 
  }

  if (then_info -> sclock > info -> sclock )
    info -> sclock = then_info -> sclock ; 
  if (then_info -> aclock > info -> aclock )
    info -> aclock = then_info -> aclock ; 

  if ((init_vclock == then_info -> vclock ) && (init_vclock == info -> vclock ) ) {


    info -> vclock -- ; 
  }
  FUNCCALL(SETUP_XfOrM874(_), optimize_info_seq_done (info , & info_seq ) )
  ; 


  if ((((tb ) ) == (scheme_false ) ) 
  && ((fb ) == (scheme_true ) ) ) {
#   define XfOrM888_COUNT (0+XfOrM874_COUNT)
#   define SETUP_XfOrM888(x) SETUP_XfOrM874(x)

# 6187 "./../src/optimize.c"
    info -> size -= 2 ; 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_optimize_prim_application2 (scheme_not_proc , t , info , context ) )) RET_VALUE_EMPTY_END ; 
  }



  if (((tb ) == (scheme_true ) ) && ((fb ) == (scheme_false ) ) ) {
    Scheme_Object * pred ; 
    BLOCK_SETUP((PUSH(pred, 0+XfOrM874_COUNT)));
#   define XfOrM886_COUNT (1+XfOrM874_COUNT)
#   define SETUP_XfOrM886(x) SETUP(XfOrM886_COUNT)
    pred = NULLED_OUT ; 

# 6196 "./../src/optimize.c"
    if (context & 0x1 )

    pred = scheme_boolean_p_proc ; 
    else 
    pred = FUNCCALL(SETUP_XfOrM886(_), expr_implies_predicate (t , info ) ); 

    if (pred && (predicate_implies (pred , scheme_boolean_p_proc ) ) ) {
#     define XfOrM887_COUNT (0+XfOrM886_COUNT)
#     define SETUP_XfOrM887(x) SETUP_XfOrM886(x)

# 6203 "./../src/optimize.c"
      info -> size -= 2 ; 
      RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_single_value_noncm (t , info ) )) RET_VALUE_EMPTY_END ; 
    }
  }


  {
    Scheme_Object * nb ; 
    BLOCK_SETUP((PUSH(nb, 0+XfOrM874_COUNT)));
#   define XfOrM884_COUNT (1+XfOrM874_COUNT)
#   define SETUP_XfOrM884(x) SETUP(XfOrM884_COUNT)
    nb = NULLED_OUT ; 

# 6212 "./../src/optimize.c"
    nb = FUNCCALL(SETUP_XfOrM884(_), equivalent_exprs (tb , fb , then_info_init , else_info_init , context ) ); 
    if (nb ) {
#     define XfOrM885_COUNT (0+XfOrM884_COUNT)
#     define SETUP_XfOrM885(x) SETUP_XfOrM884(x)

# 6214 "./../src/optimize.c"
      info -> size -= 1 ; 
      RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_first_sequence (t , nb , info ) )) RET_VALUE_EMPTY_END ; 
    }
  }




  if ((((fb ) ) == (scheme_false ) ) 
  && FUNCCALL(SETUP_XfOrM874(_), equivalent_exprs (t , tb , ((void * ) 0 ) , ((void * ) 0 ) , 0 ) )) {
#   define XfOrM883_COUNT (0+XfOrM874_COUNT)
#   define SETUP_XfOrM883(x) SETUP_XfOrM874(x)

# 6224 "./../src/optimize.c"
    info -> size -= 2 ; 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_single_value (t , info ) )) RET_VALUE_EMPTY_END ; 
  }





  if ((Scheme_Type ) (((((intptr_t ) (t ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (t ) ) -> type ) ) == (Scheme_Type ) (scheme_branch_type ) )  {
    Scheme_Branch_Rec * b2 = (Scheme_Branch_Rec * ) t ; 
    Scheme_Object * ntb , * nfb , * nt2 = ((void * ) 0 ) ; 
    BLOCK_SETUP((PUSH(ntb, 0+XfOrM874_COUNT), PUSH(b2, 1+XfOrM874_COUNT), PUSH(nfb, 2+XfOrM874_COUNT), PUSH(nt2, 3+XfOrM874_COUNT)));
#   define XfOrM877_COUNT (4+XfOrM874_COUNT)
#   define SETUP_XfOrM877(x) SETUP(XfOrM877_COUNT)
    ntb = NULLED_OUT ; 
    nfb = NULLED_OUT ; 

# 6235 "./../src/optimize.c"
    if ((((b2 -> fbranch ) ) == (scheme_false ) ) 
    && FUNCCALL(SETUP_XfOrM877(_), scheme_ir_duplicate_ok (fb , 0 ) )) {
#     define XfOrM882_COUNT (0+XfOrM877_COUNT)
#     define SETUP_XfOrM882(x) SETUP_XfOrM877(x)

# 6238 "./../src/optimize.c"
      ntb = (Scheme_Object * ) b2 ; 
      nfb = FUNCCALL(SETUP_XfOrM882(_), optimize_clone (0 , fb , info , empty_eq_hash_tree , 0 ) ); 
      nt2 = b2 -> tbranch ; 
    }

# 6241 "./../src/optimize.c"
    else if ((((b2 -> tbranch ) ) == (scheme_false ) ) 
    && FUNCCALL(SETUP_XfOrM877(_), scheme_ir_duplicate_ok (fb , 0 ) )) {
#     define XfOrM881_COUNT (0+XfOrM877_COUNT)
#     define SETUP_XfOrM881(x) SETUP_XfOrM877(x)

# 6244 "./../src/optimize.c"
      ntb = FUNCCALL(SETUP_XfOrM881(_), optimize_clone (0 , fb , info , empty_eq_hash_tree , 0 ) ); 
      nfb = (Scheme_Object * ) b2 ; 
      nt2 = b2 -> fbranch ; 
    }

# 6247 "./../src/optimize.c"
    else if (((b2 -> fbranch ) == (scheme_true ) ) 
    && FUNCCALL(SETUP_XfOrM877(_), scheme_ir_duplicate_ok (tb , 0 ) )) {
#     define XfOrM880_COUNT (0+XfOrM877_COUNT)
#     define SETUP_XfOrM880(x) SETUP_XfOrM877(x)

# 6250 "./../src/optimize.c"
      ntb = (Scheme_Object * ) b2 ; 
      nfb = FUNCCALL(SETUP_XfOrM880(_), optimize_clone (0 , tb , info , empty_eq_hash_tree , 0 ) ); 
      nt2 = b2 -> tbranch ; 
    }

# 6253 "./../src/optimize.c"
    else if (((b2 -> tbranch ) == (scheme_true ) ) 
    && FUNCCALL(SETUP_XfOrM877(_), scheme_ir_duplicate_ok (tb , 0 ) )) {
#     define XfOrM879_COUNT (0+XfOrM877_COUNT)
#     define SETUP_XfOrM879(x) SETUP_XfOrM877(x)

# 6256 "./../src/optimize.c"
      ntb = FUNCCALL(SETUP_XfOrM879(_), optimize_clone (0 , tb , info , empty_eq_hash_tree , 0 ) ); 
      nfb = (Scheme_Object * ) b2 ; 
      nt2 = b2 -> fbranch ; 
    }
    if (nt2 ) {
      t = b2 -> test ; 
      b2 -> test = nt2 ; 
      b2 -> tbranch = tb ; 
      b2 -> fbranch = fb ; 
      tb = ntb ; 
      fb = nfb ; 
    }
  }

  b -> test = t ; 
  b -> tbranch = tb ; 
  b -> fbranch = fb ; 

  if (1 ) {
  }

  else {
    info -> size += 1 ; 
  }

  RET_VALUE_START (o ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

static int omittable_key (Scheme_Object * k , Optimize_Info * info ) 
{
  /* No conversion */

  return scheme_omittable_expr (k , 1 , 20 , 0x2 , info , info ) ; 
}

static Scheme_Object * optimize_wcm (Scheme_Object * o , Optimize_Info * info , int context ) 
{
  Scheme_With_Continuation_Mark * wcm = (Scheme_With_Continuation_Mark * ) o ; 
  Scheme_Object * k , * v , * b ; 
  int init_vclock , can_omit_key ; 
  Optimize_Info_Sequence info_seq ; 

# 1027 "./../src/optimize.c"
  Scheme_Object * __funcarg100 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(5);
  BLOCK_SETUP_TOP((PUSH(b, 0), PUSH(info, 1), PUSH(v, 2), PUSH(wcm, 3), PUSH(k, 4)));
# define XfOrM911_COUNT (5)
# define SETUP_XfOrM911(x) SETUP(XfOrM911_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  k = NULLED_OUT ; 
  v = NULLED_OUT ; 
  b = NULLED_OUT ; 
  FUNCCALL(SETUP_XfOrM911(_), optimize_info_seq_init (info , & info_seq ) )
# 6302 "./../src/optimize.c"
  ; 

  k = FUNCCALL_AGAIN(optimize_expr (wcm -> key , info , 0x4 ) ); 

  if (info -> escapes ) {
#   define XfOrM914_COUNT (0+XfOrM911_COUNT)
#   define SETUP_XfOrM914(x) SETUP_XfOrM911(x)
    FUNCCALL(SETUP_XfOrM914(_), optimize_info_seq_done (info , & info_seq ) )
# 6307 "./../src/optimize.c"
    ; 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_noncm (k , info ) )) RET_VALUE_EMPTY_END ; 
  }
  FUNCCALL(SETUP_XfOrM911(_), optimize_info_seq_step (info , & info_seq ) )
  ; 

  v = FUNCCALL(SETUP_XfOrM911(_), optimize_expr (wcm -> val , info , 0x4 ) ); 

  if (info -> escapes ) {
#   define XfOrM913_COUNT (0+XfOrM911_COUNT)
#   define SETUP_XfOrM913(x) SETUP_XfOrM911(x)
    FUNCCALL(SETUP_XfOrM913(_), optimize_info_seq_done (info , & info_seq ) )
# 6316 "./../src/optimize.c"
    ; 
    info -> size += 1 ; 
    RET_VALUE_START ((__funcarg100 = FUNCCALL(SETUP_XfOrM913(_), make_discarding_first_sequence (k , v , info ) ), FUNCCALL_EMPTY(ensure_noncm (__funcarg100 , info ) )) ) RET_VALUE_END ; 
  }




  info -> vclock ++ ; 
  init_vclock = info -> vclock ; 
  FUNCCALL(SETUP_XfOrM911(_), optimize_info_seq_step (info , & info_seq ) )
  ; 

  b = FUNCCALL_AGAIN(optimize_expr (wcm -> body , info , (context & (~ ((0x3 << 4 ) | 0x2 | 0x4 ) ) ) ) ); 

  if (init_vclock == info -> vclock ) {

    info -> vclock -- ; 
  }
  FUNCCALL(SETUP_XfOrM911(_), optimize_info_seq_done (info , & info_seq ) )
  ; 



  can_omit_key = FUNCCALL(SETUP_XfOrM911(_), omittable_key (k , info ) ); 
  if (can_omit_key 
  && FUNCCALL(SETUP_XfOrM911(_), scheme_omittable_expr (b , - 1 , 20 , 0 , info , info ) ))
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_first_sequence (v , b , info ) )) RET_VALUE_EMPTY_END ; 


  info -> preserves_marks = 0 ; 

  wcm -> key = k ; 
  wcm -> val = v ; 
  wcm -> body = b ; 

  info -> size += 1 ; 









  if (can_omit_key 
  && ((Scheme_Type ) (((((intptr_t ) (wcm -> body ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (wcm -> body ) ) -> type ) ) == (Scheme_Type ) (scheme_with_cont_mark_type ) ) 
  && FUNCCALL(SETUP_XfOrM911(_), equivalent_exprs (wcm -> key , ((Scheme_With_Continuation_Mark * ) wcm -> body ) -> key , ((void * ) 0 ) , ((void * ) 0 ) , 0 ) )
  && FUNCCALL(SETUP_XfOrM911(_), scheme_omittable_expr (((Scheme_With_Continuation_Mark * ) wcm -> body ) -> val , 1 , 20 , 0 , info , info ) ))
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(make_discarding_first_sequence (wcm -> val , wcm -> body , info ) )) RET_VALUE_EMPTY_END ; 

  RET_VALUE_START ((Scheme_Object * ) wcm ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

static Scheme_Object * 
define_values_optimize (Scheme_Object * data , Optimize_Info * info , int context ) 
{
  Scheme_Object * val = ((((Scheme_Vector * ) (data ) ) -> els ) [0 ] ) ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSH(data, 0), PUSH(info, 1), PUSH(val, 2)));
# define XfOrM915_COUNT (3)
# define SETUP_XfOrM915(x) SETUP(XfOrM915_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  FUNCCALL(SETUP_XfOrM915(_), optimize_info_used_top (info ) )
# 6380 "./../src/optimize.c"
  ; 
  val = FUNCCALL(SETUP_XfOrM915(_), optimize_expr (val , info , 0 ) ); 

  ((((Scheme_Vector * ) (data ) ) -> els ) [0 ] ) = val ; 

  RET_VALUE_START (data ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 6388 "./../src/optimize.c"
static Scheme_Object * 
set_optimize (Scheme_Object * data , Optimize_Info * info , int context ) 
{
  Scheme_Set_Bang * sb = (Scheme_Set_Bang * ) data ; 
  Scheme_Object * var , * val ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(4);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(val, 1), PUSH(var, 2), PUSH(sb, 3)));
# define XfOrM916_COUNT (4)
# define SETUP_XfOrM916(x) SETUP(XfOrM916_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  var = NULLED_OUT ; 
  val = NULLED_OUT ; 

# 6394 "./../src/optimize.c"
  var = sb -> var ; 
  val = sb -> val ; 

  val = FUNCCALL(SETUP_XfOrM916(_), optimize_expr (val , info , 0x4 ) ); 

  if (info -> escapes )
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_noncm (val , info ) )) RET_VALUE_EMPTY_END ; 

  info -> preserves_marks = 1 ; 
  info -> single_result = 1 ; 

  if ((Scheme_Type ) (((((intptr_t ) (var ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (var ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) )  {
#   define XfOrM919_COUNT (0+XfOrM916_COUNT)
#   define SETUP_XfOrM919(x) SETUP_XfOrM916(x)
    FUNCCALL(SETUP_XfOrM919(_), register_use (((Scheme_IR_Local * ) var ) , info ) )
# 6406 "./../src/optimize.c"
    ; 
  }

# 6407 "./../src/optimize.c"
  else {
#   define XfOrM917_COUNT (0+XfOrM916_COUNT)
#   define SETUP_XfOrM917(x) SETUP_XfOrM916(x)

# 6408 "./../src/optimize.c"
    ((void ) sizeof ((((Scheme_IR_Toplevel * ) var ) -> instance_pos == - 1 ) ? 1 : 0 ) , __extension__ ({
#     define XfOrM918_COUNT (0+XfOrM917_COUNT)
#     define SETUP_XfOrM918(x) SETUP_XfOrM917(x)

# 6408 "./../src/optimize.c" 3
      if (((Scheme_IR_Toplevel * ) var ) -> instance_pos == - 1 )

# 6408 "./../src/optimize.c" 3
      ; 

# 6408 "./../src/optimize.c" 3
      else FUNCCALL(SETUP_XfOrM918(_), __assert_fail ("((Scheme_IR_Toplevel *)var)->instance_pos == -1" , "./../src/optimize.c" , 6408 , __extension__ __PRETTY_FUNCTION__ ) ); 
    }
    ) ) 
# 6408 "./../src/optimize.c"
    ; 
    FUNCCALL(SETUP_XfOrM917(_), optimize_info_used_top (info ) ); 
  }

  info -> vclock ++ ; 

  sb -> var = var ; 
  sb -> val = val ; 

  RET_VALUE_START ((Scheme_Object * ) sb ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 6420 "./../src/optimize.c"
static Scheme_Object * 
set_clone (int single_use , Scheme_Object * data , Optimize_Info * info , Scheme_Hash_Tree * var_map ) 
{
  Scheme_Set_Bang * sb = (Scheme_Set_Bang * ) data , * naya ; 
  Scheme_Object * var , * val ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(6);
  BLOCK_SETUP_TOP((PUSH(naya, 0), PUSH(val, 1), PUSH(info, 2), PUSH(var_map, 3), PUSH(var, 4), PUSH(sb, 5)));
# define XfOrM920_COUNT (6)
# define SETUP_XfOrM920(x) SETUP(XfOrM920_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  naya = NULLED_OUT ; 
  var = NULLED_OUT ; 
  val = NULLED_OUT ; 

# 6426 "./../src/optimize.c"
  naya = ((Scheme_Set_Bang * ) FUNCCALL(SETUP_XfOrM920(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_Set_Bang ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
  (memcpy (naya , sb , sizeof (Scheme_Set_Bang ) ) ) ; 

  var = naya -> var ; 
  val = naya -> val ; 

  val = FUNCCALL(SETUP_XfOrM920(_), optimize_clone (single_use , val , info , var_map , 0 ) ); 
  if (! val )

# 6433 "./../src/optimize.c"
  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
  if ((Scheme_Type ) (((((intptr_t ) (var ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (var ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) )  {
#   define XfOrM921_COUNT (0+XfOrM920_COUNT)
#   define SETUP_XfOrM921(x) SETUP_XfOrM920(x)

# 6435 "./../src/optimize.c"
    var = FUNCCALL(SETUP_XfOrM921(_), optimize_clone (single_use , var , info , var_map , 0 ) ); 
    if (! var )

# 6436 "./../src/optimize.c"
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
  }

  naya -> var = var ; 
  naya -> val = val ; 

  RET_VALUE_START ((Scheme_Object * ) naya ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 6445 "./../src/optimize.c"
static Scheme_Object * 
ref_optimize (Scheme_Object * data , Optimize_Info * info , int context ) 
{
  Scheme_Object * v ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSH(data, 0), PUSH(info, 1), PUSH(v, 2)));
# define XfOrM922_COUNT (3)
# define SETUP_XfOrM922(x) SETUP(XfOrM922_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  v = NULLED_OUT ; 
  FUNCCALL(SETUP_XfOrM922(_), optimize_info_used_top (info ) )
# 6450 "./../src/optimize.c"
  ; 

  v = (((Scheme_Simple_Object * ) (data ) ) -> u . two_ptr_val . ptr1 ) ; 
  if ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) )  {
    (((Scheme_Simple_Object * ) (data ) ) -> u . two_ptr_val . ptr1 ) = (((Scheme_IR_Local * ) v ) -> mutated ? scheme_false : scheme_true ) ; 
  }

# 6455 "./../src/optimize.c"
  else if ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_toplevel_type ) )  {
#   define XfOrM923_COUNT (0+XfOrM922_COUNT)
#   define SETUP_XfOrM923(x) SETUP_XfOrM922(x)
    if (FUNCCALL(SETUP_XfOrM923(_), get_defn_shape (info , (Scheme_IR_Toplevel * ) v ) )
    || FUNCCALL(SETUP_XfOrM923(_), get_import_shape (info , (Scheme_IR_Toplevel * ) v ) )) {
#     define XfOrM924_COUNT (0+XfOrM923_COUNT)
#     define SETUP_XfOrM924(x) SETUP_XfOrM923(x)

# 6460 "./../src/optimize.c"
      v = FUNCCALL(SETUP_XfOrM924(_), scheme_ir_toplevel_to_flagged_toplevel (v , 2 ) ); 
      (((Scheme_Simple_Object * ) (data ) ) -> u . two_ptr_val . ptr1 ) = v ; 
    }
    FUNCCALL(SETUP_XfOrM923(_), register_import_used (info , (Scheme_IR_Toplevel * ) v ) ); 
  }

  info -> preserves_marks = 1 ; 
  info -> single_result = 1 ; 
  info -> size ++ ; 

  RET_VALUE_START (data ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 6473 "./../src/optimize.c"
static Scheme_Object * 
ref_clone (int single_use , Scheme_Object * data , Optimize_Info * info , Scheme_Hash_Tree * var_map ) 
{
  Scheme_Object * naya ; 
  Scheme_Object * a , * b ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(6);
  BLOCK_SETUP_TOP((PUSH(b, 0), PUSH(data, 1), PUSH(info, 2), PUSH(var_map, 3), PUSH(naya, 4), PUSH(a, 5)));
# define XfOrM926_COUNT (6)
# define SETUP_XfOrM926(x) SETUP(XfOrM926_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  naya = NULLED_OUT ; 
  a = NULLED_OUT ; 
  b = NULLED_OUT ; 

# 6479 "./../src/optimize.c"
  a = (((Scheme_Simple_Object * ) (data ) ) -> u . two_ptr_val . ptr1 ) ; 
  a = FUNCCALL(SETUP_XfOrM926(_), optimize_clone (single_use , a , info , var_map , 0 ) ); 
  if (! a )

# 6481 "./../src/optimize.c"
  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

  b = (((Scheme_Simple_Object * ) (data ) ) -> u . two_ptr_val . ptr2 ) ; 
  b = FUNCCALL(SETUP_XfOrM926(_), optimize_clone (single_use , b , info , var_map , 0 ) ); 
  if (! b )

# 6485 "./../src/optimize.c"
  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

  naya = ((Scheme_Object * ) FUNCCALL(SETUP_XfOrM926(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_Simple_Object ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
  naya -> type = scheme_varref_form_type ; 
  (((Scheme_Simple_Object * ) (naya ) ) -> u . two_ptr_val . ptr1 ) = a ; 
  (((Scheme_Simple_Object * ) (naya ) ) -> u . two_ptr_val . ptr2 ) = b ; 

  RET_VALUE_START (naya ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 6495 "./../src/optimize.c"
static Scheme_Object * 
apply_values_optimize (Scheme_Object * data , Optimize_Info * info , int context ) 
{
  Scheme_Object * f , * e ; 
  Optimize_Info_Sequence info_seq ; 

# 1027 "./../src/optimize.c"
  Scheme_Object * __funcarg101 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(f, 1), PUSH(e, 2)));
# define XfOrM927_COUNT (3)
# define SETUP_XfOrM927(x) SETUP(XfOrM927_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  f = NULLED_OUT ; 
  e = NULLED_OUT ; 

# 6501 "./../src/optimize.c"
  f = (((Scheme_Simple_Object * ) (data ) ) -> u . two_ptr_val . ptr1 ) ; 
  e = (((Scheme_Simple_Object * ) (data ) ) -> u . two_ptr_val . ptr2 ) ; 
  FUNCCALL(SETUP_XfOrM927(_), optimize_info_seq_init (info , & info_seq ) )
  ; 

  f = FUNCCALL_AGAIN(optimize_expr (f , info , 0x4 ) ); 

  if (info -> escapes ) {
#   define XfOrM929_COUNT (0+XfOrM927_COUNT)
#   define SETUP_XfOrM929(x) SETUP_XfOrM927(x)
    FUNCCALL(SETUP_XfOrM929(_), optimize_info_seq_done (info , & info_seq ) )
# 6509 "./../src/optimize.c"
    ; 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_noncm (f , info ) )) RET_VALUE_EMPTY_END ; 
  }
  FUNCCALL(SETUP_XfOrM927(_), optimize_info_seq_step (info , & info_seq ) ); 

  e = FUNCCALL(SETUP_XfOrM927(_), optimize_expr (e , info , 0 ) ); 
  FUNCCALL_AGAIN(optimize_info_seq_done (info , & info_seq ) )
  ; 

  if (info -> escapes ) {
#   define XfOrM928_COUNT (0+XfOrM927_COUNT)
#   define SETUP_XfOrM928(x) SETUP_XfOrM927(x)

# 6519 "./../src/optimize.c"
    info -> size += 1 ; 
    RET_VALUE_START ((__funcarg101 = FUNCCALL(SETUP_XfOrM928(_), make_discarding_first_sequence (f , e , info ) ), FUNCCALL_EMPTY(ensure_noncm (__funcarg101 , info ) )) ) RET_VALUE_END ; 
  }

  info -> size += 1 ; 
  info -> vclock += 1 ; 
  info -> kclock += 1 ; 
  info -> sclock += 1 ; 

  RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_apply_values (f , e , info , info -> single_result , context ) )) RET_VALUE_EMPTY_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 6531 "./../src/optimize.c"
static Scheme_Object * 
apply_values_clone (int single_use , Scheme_Object * data , Optimize_Info * info , Scheme_Hash_Tree * var_map ) 
{
  Scheme_Object * f , * e ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(5);
  BLOCK_SETUP_TOP((PUSH(data, 0), PUSH(info, 1), PUSH(var_map, 2), PUSH(f, 3), PUSH(e, 4)));
# define XfOrM930_COUNT (5)
# define SETUP_XfOrM930(x) SETUP(XfOrM930_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  f = NULLED_OUT ; 
  e = NULLED_OUT ; 

# 6536 "./../src/optimize.c"
  f = (((Scheme_Simple_Object * ) (data ) ) -> u . two_ptr_val . ptr1 ) ; 
  e = (((Scheme_Simple_Object * ) (data ) ) -> u . two_ptr_val . ptr2 ) ; 

  f = FUNCCALL(SETUP_XfOrM930(_), optimize_clone (single_use , f , info , var_map , 0 ) ); 
  if (! f )

# 6540 "./../src/optimize.c"
  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
  e = FUNCCALL(SETUP_XfOrM930(_), optimize_clone (single_use , e , info , var_map , 0 ) ); 
  if (! e )

# 6542 "./../src/optimize.c"
  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

  data = ((Scheme_Object * ) FUNCCALL(SETUP_XfOrM930(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_Simple_Object ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
  data -> type = scheme_apply_values_type ; 
  (((Scheme_Simple_Object * ) (data ) ) -> u . two_ptr_val . ptr1 ) = f ; 
  (((Scheme_Simple_Object * ) (data ) ) -> u . two_ptr_val . ptr2 ) = e ; 

  RET_VALUE_START (data ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 6552 "./../src/optimize.c"
static Scheme_Object * 
with_immed_mark_optimize (Scheme_Object * data , Optimize_Info * info , int context ) 
{
  Scheme_With_Continuation_Mark * wcm = (Scheme_With_Continuation_Mark * ) data ; 
  Scheme_Object * key , * val , * body ; 
  Optimize_Info_Sequence info_seq ; 
  Optimize_Info * body_info ; 
  Scheme_IR_Local * var ; 

# 1027 "./../src/optimize.c"
  Scheme_Object * __funcarg102 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(8);
  BLOCK_SETUP_TOP((PUSH(body, 0), PUSH(info, 1), PUSH(val, 2), PUSH(body_info, 3), PUSH(data, 4), PUSH(key, 5), PUSH(wcm, 6), PUSH(var, 7)));
# define XfOrM931_COUNT (8)
# define SETUP_XfOrM931(x) SETUP(XfOrM931_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  key = NULLED_OUT ; 
  val = NULLED_OUT ; 
  body = NULLED_OUT ; 
  body_info = NULLED_OUT ; 
  var = NULLED_OUT ; 
  FUNCCALL(SETUP_XfOrM931(_), optimize_info_seq_init (info , & info_seq ) )
# 6561 "./../src/optimize.c"
  ; 

  key = FUNCCALL_AGAIN(optimize_expr (wcm -> key , info , 0x4 ) ); 
  FUNCCALL_AGAIN(optimize_info_seq_step (info , & info_seq ) ); 
  if (info -> escapes ) {
#   define XfOrM933_COUNT (0+XfOrM931_COUNT)
#   define SETUP_XfOrM933(x) SETUP_XfOrM931(x)
    FUNCCALL(SETUP_XfOrM933(_), optimize_info_seq_done (info , & info_seq ) )
# 6566 "./../src/optimize.c"
    ; 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ensure_noncm (key , info ) )) RET_VALUE_EMPTY_END ; 
  }

  val = FUNCCALL(SETUP_XfOrM931(_), optimize_expr (wcm -> val , info , 0x4 ) ); 
  FUNCCALL(SETUP_XfOrM931(_), optimize_info_seq_step (info , & info_seq ) ); 
  if (info -> escapes ) {
#   define XfOrM932_COUNT (0+XfOrM931_COUNT)
#   define SETUP_XfOrM932(x) SETUP_XfOrM931(x)
    FUNCCALL(SETUP_XfOrM932(_), optimize_info_seq_done (info , & info_seq ) )
# 6573 "./../src/optimize.c"
    ; 
    RET_VALUE_START ((__funcarg102 = FUNCCALL(SETUP_XfOrM932(_), make_discarding_first_sequence (key , val , info ) ), FUNCCALL_EMPTY(ensure_noncm (__funcarg102 , info ) )) ) RET_VALUE_END ; 
  }
  FUNCCALL(SETUP_XfOrM931(_), optimize_info_seq_done (info , & info_seq ) )
  ; 

  body_info = FUNCCALL(SETUP_XfOrM931(_), optimize_info_add_frame (info , 0 ) ); 
  var = ((Scheme_IR_Local * ) (((Scheme_Simple_Object * ) (wcm -> body ) ) -> u . pair_val . car ) ) ; 
  FUNCCALL(SETUP_XfOrM931(_), set_optimize_mode (var ) ); 
  var -> optimize . lambda_depth = body_info -> lambda_depth ; 
  var -> optimize_used = 0 ; 
  var -> optimize . init_kclock = info -> kclock ; 

  body = FUNCCALL(SETUP_XfOrM931(_), optimize_expr ((((Scheme_Simple_Object * ) (wcm -> body ) ) -> u . pair_val . cdr ) , body_info , 0 ) ); 
  FUNCCALL_AGAIN(optimize_info_done (body_info , ((void * ) 0 ) ) )
  ; 

  wcm -> key = key ; 
  wcm -> val = val ; 
  (((Scheme_Simple_Object * ) (wcm -> body ) ) -> u . pair_val . cdr ) = body ; 

  info -> preserves_marks = 0 ; 

  RET_VALUE_START (data ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 6599 "./../src/optimize.c"
static Scheme_Object * 
with_immed_mark_clone (int single_use , Scheme_Object * data , Optimize_Info * info , Scheme_Hash_Tree * var_map ) 
{
  Scheme_With_Continuation_Mark * wcm = (Scheme_With_Continuation_Mark * ) data ; 
  Scheme_With_Continuation_Mark * wcm2 ; 
  Scheme_Object * e ; 
  Scheme_IR_Local * var ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(6);
  BLOCK_SETUP_TOP((PUSH(var, 0), PUSH(wcm, 1), PUSH(info, 2), PUSH(var_map, 3), PUSH(wcm2, 4), PUSH(e, 5)));
# define XfOrM934_COUNT (6)
# define SETUP_XfOrM934(x) SETUP(XfOrM934_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  wcm2 = NULLED_OUT ; 
  e = NULLED_OUT ; 
  var = NULLED_OUT ; 

# 6607 "./../src/optimize.c"
  wcm2 = ((Scheme_With_Continuation_Mark * ) FUNCCALL(SETUP_XfOrM934(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_With_Continuation_Mark ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
  wcm2 -> so . type = scheme_with_immed_mark_type ; 

  e = FUNCCALL(SETUP_XfOrM934(_), optimize_clone (single_use , wcm -> key , info , var_map , 0 ) ); 
  if (! e )

# 6611 "./../src/optimize.c"
  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
  wcm2 -> key = e ; 

  e = FUNCCALL(SETUP_XfOrM934(_), optimize_clone (single_use , wcm -> val , info , var_map , 0 ) ); 
  if (! e )

# 6615 "./../src/optimize.c"
  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
  wcm2 -> val = e ; 

  var = FUNCCALL(SETUP_XfOrM934(_), clone_variable (((Scheme_IR_Local * ) (((Scheme_Simple_Object * ) (wcm -> body ) ) -> u . pair_val . car ) ) ) ); 
  var_map = FUNCCALL_AGAIN(scheme_hash_tree_set (var_map , (((Scheme_Simple_Object * ) (wcm -> body ) ) -> u . pair_val . car ) , (Scheme_Object * ) var ) ); 

  e = FUNCCALL_AGAIN(optimize_clone (single_use , (((Scheme_Simple_Object * ) (wcm -> body ) ) -> u . pair_val . cdr ) , info , var_map , 0 ) ); 
  if (! e )

# 6622 "./../src/optimize.c"
  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
  e = FUNCCALL(SETUP_XfOrM934(_), scheme_make_mutable_pair ((Scheme_Object * ) var , e ) ); 
  wcm2 -> body = e ; 

  RET_VALUE_START ((Scheme_Object * ) wcm2 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 6629 "./../src/optimize.c"
static Scheme_Object * 
case_lambda_optimize (Scheme_Object * expr , Optimize_Info * info , int context ) 
{
  Scheme_Object * le ; 
  int i ; 
  Scheme_Case_Lambda * seq = (Scheme_Case_Lambda * ) expr ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(4);
  BLOCK_SETUP_TOP((PUSH(expr, 0), PUSH(info, 1), PUSH(le, 2), PUSH(seq, 3)));
# define XfOrM935_COUNT (4)
# define SETUP_XfOrM935(x) SETUP(XfOrM935_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  le = NULLED_OUT ; 

# 6636 "./../src/optimize.c"
  for (i = 0 ; i < seq -> count ; i ++ ) {
#   define XfOrM937_COUNT (0+XfOrM935_COUNT)
#   define SETUP_XfOrM937(x) SETUP_XfOrM935(x)

# 6637 "./../src/optimize.c"
    le = seq -> array [i ] ; 
    le = FUNCCALL(SETUP_XfOrM937(_), optimize_expr (le , info , 0 ) ); 
    seq -> array [i ] = le ; 
  }

  info -> preserves_marks = 1 ; 
  info -> single_result = 1 ; 
  info -> size += 1 ; 

  RET_VALUE_START (expr ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 6649 "./../src/optimize.c"
static Scheme_Object * 
case_lambda_clone (int single_use , Scheme_Object * data , Optimize_Info * info , Scheme_Hash_Tree * var_map ) 
{
  Scheme_Object * le ; 
  int i , sz ; 
  Scheme_Case_Lambda * seq = (Scheme_Case_Lambda * ) data ; 
  Scheme_Case_Lambda * seq2 ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(5);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(var_map, 1), PUSH(le, 2), PUSH(seq2, 3), PUSH(seq, 4)));
# define XfOrM938_COUNT (5)
# define SETUP_XfOrM938(x) SETUP(XfOrM938_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  le = NULLED_OUT ; 
  seq2 = NULLED_OUT ; 

# 6657 "./../src/optimize.c"
  sz = sizeof (Scheme_Case_Lambda ) + ((seq -> count - 0 ) * sizeof (Scheme_Object * ) ) ; 
  seq2 = (Scheme_Case_Lambda * ) FUNCCALL(SETUP_XfOrM938(_), GC_malloc_one_tagged (sz ) ); 
  (memcpy (seq2 , seq , sz ) ) ; 

  for (i = 0 ; i < seq -> count ; i ++ ) {
#   define XfOrM940_COUNT (0+XfOrM938_COUNT)
#   define SETUP_XfOrM940(x) SETUP_XfOrM938(x)

# 6662 "./../src/optimize.c"
    le = seq -> array [i ] ; 
    le = FUNCCALL(SETUP_XfOrM940(_), optimize_clone (single_use , le , info , var_map , 0 ) ); 
    if (! le )

# 6664 "./../src/optimize.c"
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
    seq2 -> array [i ] = le ; 
  }

  RET_VALUE_START ((Scheme_Object * ) seq2 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 6671 "./../src/optimize.c"
static Scheme_Object * begin0_optimize (Scheme_Object * obj , Optimize_Info * info , int context ) 
{
  int i , count , drop = 0 , prev_size , single_result = 0 , preserves_marks = 0 , kclock = 0 , sclock = 0 ; 
  Scheme_Sequence * s = (Scheme_Sequence * ) obj ; 
  Scheme_Object * inside = ((void * ) 0 ) , * expr , * orig_first ; 
  Scheme_Object * le ; 
  Optimize_Info_Sequence info_seq ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(7);
  BLOCK_SETUP_TOP((PUSH(expr, 0), PUSH(info, 1), PUSH(le, 2), PUSH(s, 3), PUSH(orig_first, 4), PUSH(inside, 5)));
# define XfOrM941_COUNT (6)
# define SETUP_XfOrM941(x) SETUP(XfOrM941_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  expr = NULLED_OUT ; 
  orig_first = NULLED_OUT ; 
  le = NULLED_OUT ; 

# 6679 "./../src/optimize.c"
  count = s -> count ; 
  FUNCCALL(SETUP_XfOrM941(_), optimize_info_seq_init (info , & info_seq ) ); 

  for (i = 0 ; i < count ; i ++ ) {
#   define XfOrM974_COUNT (0+XfOrM941_COUNT)
#   define SETUP_XfOrM974(x) SETUP_XfOrM941(x)

# 6683 "./../src/optimize.c"
    prev_size = info -> size ; 
    FUNCCALL(SETUP_XfOrM974(_), optimize_info_seq_step (info , & info_seq ) )
    ; 

    le = FUNCCALL_AGAIN(optimize_expr (s -> array [i ] , info , (! i ? (context & (~ ((0x3 << 4 ) | 0x2 | 0x4 ) ) ) : 0 ) ) )



    ; 

    if (! i ) {
      single_result = info -> single_result ; 
      preserves_marks = info -> preserves_marks ; 
      kclock = info -> kclock ; 
      sclock = info -> sclock ; 
      s -> array [0 ] = le ; 
    }

# 6699 "./../src/optimize.c"
    else {
#     define XfOrM978_COUNT (0+XfOrM974_COUNT)
#     define SETUP_XfOrM978(x) SETUP_XfOrM974(x)

# 6701 "./../src/optimize.c"
      le = FUNCCALL(SETUP_XfOrM978(_), optimize_ignored (le , info , - 1 , 1 , 5 ) ); 
      if (! le ) {
        drop ++ ; 
        info -> size = prev_size ; 
        s -> array [i ] = ((void * ) 0 ) ; 
      }

# 6706 "./../src/optimize.c"
      else {
        s -> array [i ] = le ; 
      }
    }

    if (info -> escapes ) {
      int j ; 
      single_result = info -> single_result ; 
      preserves_marks = info -> preserves_marks ; 
      for (j = i + 1 ; j < count ; j ++ ) {
        drop ++ ; 
        s -> array [j ] = ((void * ) 0 ) ; 
      }
      break ; 
    }
  }
  FUNCCALL(SETUP_XfOrM941(_), optimize_info_seq_done (info , & info_seq ) )
  ; 

  if (info -> escapes ) {

    Scheme_Sequence * s2 ; 
    int j = 0 ; 
    BLOCK_SETUP((PUSH(s2, 0+XfOrM941_COUNT)));
#   define XfOrM957_COUNT (1+XfOrM941_COUNT)
#   define SETUP_XfOrM957(x) SETUP(XfOrM957_COUNT)
    s2 = NULLED_OUT ; 

# 6730 "./../src/optimize.c"
    info -> single_result = 1 ; 
    info -> preserves_marks = 1 ; 

    if (i != 0 ) {
#     define XfOrM963_COUNT (0+XfOrM957_COUNT)
#     define SETUP_XfOrM963(x) SETUP_XfOrM957(x)

# 6735 "./../src/optimize.c"
      le = FUNCCALL(SETUP_XfOrM963(_), optimize_ignored (s -> array [0 ] , info , - 1 , 1 , 5 ) ); 
      if (! le ) {
        drop ++ ; 
        info -> size = prev_size ; 
        s -> array [0 ] = ((void * ) 0 ) ; 
      }

# 6740 "./../src/optimize.c"
      else {
        s -> array [0 ] = le ; 
      }
    }

    if ((count - drop ) == 1 ) {
#     define XfOrM962_COUNT (0+XfOrM957_COUNT)
#     define SETUP_XfOrM962(x) SETUP_XfOrM957(x)

# 6747 "./../src/optimize.c"
      RET_VALUE_START (FUNCCALL(SETUP_XfOrM962(_), ensure_noncm (s -> array [i ] , info ) )) RET_VALUE_END ; 
    }

    s2 = FUNCCALL(SETUP_XfOrM957(_), scheme_malloc_sequence (count - drop ) ); 
    s2 -> so . type = scheme_sequence_type ; 
    s2 -> count = count - drop ; 

    for (i = 0 ; i < count ; i ++ ) {
      if (s -> array [i ] ) {
        s2 -> array [j ++ ] = s -> array [i ] ; 
      }
    }
    RET_VALUE_START (FUNCCALL(SETUP_XfOrM957(_), flatten_sequence ((Scheme_Object * ) s2 , info , context ) )) RET_VALUE_END ; 
  }

  info -> preserves_marks = 1 ; 
  info -> single_result = single_result ; 

  if ((s -> count - drop ) == 1 && (preserves_marks == 1 ) ) {

    RET_VALUE_START (s -> array [0 ] ) RET_VALUE_END ; 
  }

  expr = s -> array [0 ] ; 
  orig_first = s -> array [0 ] ; 
  FUNCCALL(SETUP_XfOrM941(_), extract_tail_inside (& expr , & inside , 0 ) ); 


  if (FUNCCALL(SETUP_XfOrM941(_), movable_expression (expr , info , 0 , kclock != info -> kclock , sclock != info -> sclock , 0 , 50 ) )) 
  {
#   define XfOrM949_COUNT (0+XfOrM941_COUNT)
#   define SETUP_XfOrM949(x) SETUP_XfOrM941(x)

# 6777 "./../src/optimize.c"
    if ((s -> count - drop ) == 1 ) {

      info -> size -= 1 ; 
    }
    else {
      Scheme_Sequence * s2 ; 
      int j = 0 ; 
      BLOCK_SETUP((PUSH(s2, 0+XfOrM949_COUNT)));
#     define XfOrM950_COUNT (1+XfOrM949_COUNT)
#     define SETUP_XfOrM950(x) SETUP(XfOrM950_COUNT)
      s2 = NULLED_OUT ; 

# 6785 "./../src/optimize.c"
      s2 = FUNCCALL(SETUP_XfOrM950(_), scheme_malloc_sequence (s -> count - drop ) ); 
      s2 -> so . type = scheme_sequence_type ; 
      s2 -> count = s -> count - drop ; 

      for (i = 1 ; i < s -> count ; i ++ ) {
        if (s -> array [i ] ) {
          s2 -> array [j ++ ] = s -> array [i ] ; 
        }
      }
      s2 -> array [j ++ ] = expr ; 

      expr = (Scheme_Object * ) s2 ; 
    }
  }

# 6798 "./../src/optimize.c"
  else {
#   define XfOrM942_COUNT (0+XfOrM941_COUNT)
#   define SETUP_XfOrM942(x) SETUP_XfOrM941(x)

# 6799 "./../src/optimize.c"
    if (drop ) {
      Scheme_Sequence * s2 ; 
      int j = 0 ; 
      BLOCK_SETUP((PUSH(s2, 0+XfOrM942_COUNT)));
#     define XfOrM944_COUNT (1+XfOrM942_COUNT)
#     define SETUP_XfOrM944(x) SETUP(XfOrM944_COUNT)
      s2 = NULLED_OUT ; 

# 6803 "./../src/optimize.c"
      s2 = FUNCCALL(SETUP_XfOrM944(_), scheme_malloc_sequence (s -> count - drop ) ); 
      s2 -> so . type = s -> so . type ; 
      s2 -> count = s -> count - drop ; 

      s2 -> array [j ++ ] = expr ; 
      for (i = 1 ; i < s -> count ; i ++ ) {
        if (s -> array [i ] ) {
          s2 -> array [j ++ ] = s -> array [i ] ; 
        }
      }

      expr = (Scheme_Object * ) s2 ; 
    }

# 6815 "./../src/optimize.c"
    else {
      s -> array [0 ] = expr ; 
      expr = (Scheme_Object * ) s ; 
    }
  }

  info -> size += 1 ; 
  expr = FUNCCALL(SETUP_XfOrM941(_), flatten_sequence (expr , info , context ) ); 
  RET_VALUE_START (FUNCCALL(SETUP_XfOrM941(_), replace_tail_inside (expr , inside , orig_first ) )) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

static int is_liftable_prim (Scheme_Object * v , int or_escape ) 

{
  /* No conversion */

# 6833 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_prim_type ) )  {
    int opt = (((Scheme_Primitive_Proc * ) v ) -> pp . flags & (1 | 2 ) ) ; 
    if (opt >= 2 )
      return 1 ; 
    if (or_escape && (opt >= 1 ) ) {
      if (scheme_prim_opt_flags [((((Scheme_Prim_Proc_Header * ) v ) -> flags ) >> 9 ) & ((1 << 7 ) - 1 ) ] & (1 << 16 ) )
        return 1 ; 
    }
  }

  if ((v ) == (scheme_values_proc ) ) 
    return 1 ; 

  return 0 ; 
}

int scheme_is_liftable (Scheme_Object * o , Scheme_Hash_Tree * exclude_vars , int fuel , int as_rator , int or_escape ) 


{
  Scheme_Type t = ((((intptr_t ) (o ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o ) ) -> type ) ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK(3);
  BLOCK_SETUP_TOP((PUSH(o, 0), PUSH(exclude_vars, 1)));
# define XfOrM985_COUNT (2)
# define SETUP_XfOrM985(x) SETUP(XfOrM985_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 6855 "./../src/optimize.c"
  if (! fuel )

# 6855 "./../src/optimize.c"
  RET_VALUE_START (0 ) RET_VALUE_END ; 

  switch (t ) {
#   define XfOrM986_COUNT (0+XfOrM985_COUNT)
#   define SETUP_XfOrM986(x) SETUP_XfOrM985(x)

# 6858 "./../src/optimize.c"
    case scheme_ir_lambda_type : 
    RET_VALUE_START (! as_rator ) RET_VALUE_END ; 
    case scheme_case_lambda_sequence_type : 
    RET_VALUE_START (! as_rator ) RET_VALUE_END ; 
    case scheme_ir_toplevel_type : 
    RET_VALUE_START (1 ) RET_VALUE_END ; 
    case scheme_ir_local_type : 
    if (! (scheme_eq_hash_tree_get (exclude_vars , o ) ) )
      RET_VALUE_START (1 ) RET_VALUE_END ; 
    break ; 
    case scheme_branch_type : 
    {
      Scheme_Branch_Rec * b = (Scheme_Branch_Rec * ) o ; 
      BLOCK_SETUP((PUSH(b, 0+XfOrM986_COUNT)));
#     define XfOrM995_COUNT (1+XfOrM986_COUNT)
#     define SETUP_XfOrM995(x) SETUP(XfOrM995_COUNT)

# 6871 "./../src/optimize.c"
      if (FUNCCALL(SETUP_XfOrM995(_), scheme_is_liftable (b -> test , exclude_vars , fuel - 1 , 0 , or_escape ) )
      && FUNCCALL(SETUP_XfOrM995(_), scheme_is_liftable (b -> tbranch , exclude_vars , fuel - 1 , as_rator , or_escape ) )
      && FUNCCALL(SETUP_XfOrM995(_), scheme_is_liftable (b -> fbranch , exclude_vars , fuel - 1 , as_rator , or_escape ) ))
        RET_VALUE_START (1 ) RET_VALUE_END ; 
    }
    break ; 
    case scheme_application_type : 
    {
      Scheme_App_Rec * app = (Scheme_App_Rec * ) o ; 
      int i ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM986_COUNT)));
#     define XfOrM992_COUNT (1+XfOrM986_COUNT)
#     define SETUP_XfOrM992(x) SETUP(XfOrM992_COUNT)

# 6881 "./../src/optimize.c"
      if (! FUNCCALL(SETUP_XfOrM992(_), is_liftable_prim (app -> args [0 ] , or_escape ) ))
        RET_VALUE_START (0 ) RET_VALUE_END ; 
      for (i = app -> num_args + 1 ; i -- ; ) {
#       define XfOrM994_COUNT (0+XfOrM992_COUNT)
#       define SETUP_XfOrM994(x) SETUP_XfOrM992(x)

# 6884 "./../src/optimize.c"
        if (! FUNCCALL(SETUP_XfOrM994(_), scheme_is_liftable (app -> args [i ] , exclude_vars , fuel - 1 , 1 , or_escape ) ))
          RET_VALUE_START (0 ) RET_VALUE_END ; 
      }
      RET_VALUE_START (1 ) RET_VALUE_END ; 
    }
    break ; 
    case scheme_application2_type : 
    {
      Scheme_App2_Rec * app = (Scheme_App2_Rec * ) o ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM986_COUNT)));
#     define XfOrM991_COUNT (1+XfOrM986_COUNT)
#     define SETUP_XfOrM991(x) SETUP(XfOrM991_COUNT)

# 6893 "./../src/optimize.c"
      if (! FUNCCALL(SETUP_XfOrM991(_), is_liftable_prim (app -> rator , or_escape ) ))
        RET_VALUE_START (0 ) RET_VALUE_END ; 
      if (FUNCCALL(SETUP_XfOrM991(_), scheme_is_liftable (app -> rator , exclude_vars , fuel - 1 , 1 , or_escape ) )
      && FUNCCALL(SETUP_XfOrM991(_), scheme_is_liftable (app -> rand , exclude_vars , fuel - 1 , 1 , or_escape ) ))
        RET_VALUE_START (1 ) RET_VALUE_END ; 
    }
    break ; 
    case scheme_application3_type : 
    {
      Scheme_App3_Rec * app = (Scheme_App3_Rec * ) o ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM986_COUNT)));
#     define XfOrM990_COUNT (1+XfOrM986_COUNT)
#     define SETUP_XfOrM990(x) SETUP(XfOrM990_COUNT)

# 6903 "./../src/optimize.c"
      if (! FUNCCALL(SETUP_XfOrM990(_), is_liftable_prim (app -> rator , or_escape ) ))
        RET_VALUE_START (0 ) RET_VALUE_END ; 
      if (FUNCCALL(SETUP_XfOrM990(_), scheme_is_liftable (app -> rator , exclude_vars , fuel - 1 , 1 , or_escape ) )
      && FUNCCALL(SETUP_XfOrM990(_), scheme_is_liftable (app -> rand1 , exclude_vars , fuel - 1 , 1 , or_escape ) )
      && FUNCCALL(SETUP_XfOrM990(_), scheme_is_liftable (app -> rand2 , exclude_vars , fuel - 1 , 1 , or_escape ) ))
        RET_VALUE_START (1 ) RET_VALUE_END ; 
    }
    break ; 
    case scheme_ir_let_header_type : 
    {
      Scheme_IR_Let_Header * lh = (Scheme_IR_Let_Header * ) o ; 
      int i ; 
      BLOCK_SETUP((PUSH(lh, 0+XfOrM986_COUNT)));
#     define XfOrM987_COUNT (1+XfOrM986_COUNT)
#     define SETUP_XfOrM987(x) SETUP(XfOrM987_COUNT)

# 6916 "./../src/optimize.c"
      o = lh -> body ; 
      for (i = lh -> num_clauses ; i -- ; ) {
#       define XfOrM989_COUNT (0+XfOrM987_COUNT)
#       define SETUP_XfOrM989(x) SETUP_XfOrM987(x)

# 6918 "./../src/optimize.c"
        if (! FUNCCALL(SETUP_XfOrM989(_), scheme_is_liftable (((Scheme_IR_Let_Value * ) o ) -> value , exclude_vars , fuel - 1 , as_rator , or_escape ) ))
          RET_VALUE_START (0 ) RET_VALUE_END ; 
        o = ((Scheme_IR_Let_Value * ) o ) -> body ; 
      }
      if (FUNCCALL_EMPTY(scheme_is_liftable (o , exclude_vars , fuel - 1 , as_rator , or_escape ) ))
        RET_VALUE_START (1 ) RET_VALUE_END ; 
      break ; 
    }
    default : 
    if (t > _scheme_ir_values_types_ )
      RET_VALUE_START (1 ) RET_VALUE_END ; 
  }

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 6934 "./../src/optimize.c"
int ir_propagate_ok (Scheme_Object * value , Optimize_Info * info , int used_once , Scheme_IR_Local * once_var ) 




{

# 251 "./../src/optimize.c"
  char * __funcarg106 = NULLED_OUT ; 
  char * __funcarg105 = NULLED_OUT ; 

# 251 "./../src/optimize.c"
  char * __funcarg104 = NULLED_OUT ; 
  char * __funcarg103 = NULLED_OUT ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK(6);
  BLOCK_SETUP_TOP((PUSH(once_var, 0), PUSH(info, 1), PUSH(__funcarg103, 2), PUSH(value, 3), PUSH(__funcarg105, 4)));
# define XfOrM996_COUNT (5)
# define SETUP_XfOrM996(x) SETUP(XfOrM996_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 6940 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) )  {
    int sz ; 
#   define XfOrM1001_COUNT (0+XfOrM996_COUNT)
#   define SETUP_XfOrM1001(x) SETUP_XfOrM996(x)

# 6942 "./../src/optimize.c"
    sz = FUNCCALL(SETUP_XfOrM1001(_), lambda_body_size_plus_info ((Scheme_Lambda * ) value , 1 , info , ((void * ) 0 ) ) ); 
    if ((sz >= 0 ) && (sz <= 256 ) )
      RET_VALUE_START (1 ) RET_VALUE_END ; 
    else if (used_once ) {
      if (once_var ) {




        once_var -> optimize . clear_known_on_multi_use = 1 ; 
      }
      RET_VALUE_START (1 ) RET_VALUE_END ; 
    }

# 6954 "./../src/optimize.c"
    else {
#     define XfOrM1002_COUNT (0+XfOrM1001_COUNT)
#     define SETUP_XfOrM1002(x) SETUP_XfOrM1001(x)

# 6955 "./../src/optimize.c"
      if (FUNCCALL(SETUP_XfOrM1002(_), scheme_log_level_p (info -> logger , 5 ) )) {
        Scheme_Lambda * lam = (Scheme_Lambda * ) value ; 
        BLOCK_SETUP((PUSH(lam, 0+XfOrM1002_COUNT)));
#       define XfOrM1003_COUNT (1+XfOrM1002_COUNT)
#       define SETUP_XfOrM1003(x) SETUP(XfOrM1003_COUNT)

# 6957 "./../src/optimize.c"
        if (sz < 0 )
          (__funcarg105 = FUNCCALL(SETUP_XfOrM1003(_), scheme_write_to_string (lam -> name ? lam -> name : scheme_false , ((void * ) 0 ) ) )




        , __funcarg106 = FUNCCALL(SETUP_XfOrM1003(_), scheme_optimize_context_to_string (info -> context ) ), FUNCCALL_AGAIN(scheme_log (info -> logger , 5 , 0 , "non-copyable %s size: %d threshold: %d#<separator>%s" , __funcarg105 , sz , 0 , __funcarg106 ) )) 


        ; 
        else (__funcarg103 = FUNCCALL(SETUP_XfOrM1003(_), scheme_write_to_string (lam -> name ? lam -> name : scheme_false , ((void * ) 0 ) ) )





        , __funcarg104 = FUNCCALL(SETUP_XfOrM1003(_), scheme_optimize_context_to_string (info -> context ) ), FUNCCALL_AGAIN(scheme_log (info -> logger , 5 , 0 , "too-large %s size: %d threshold: %d#<separator>%s" , __funcarg103 , sz , 0 , __funcarg104 ) )) 


        ; 
      }
      RET_VALUE_START (0 ) RET_VALUE_END ; 
    }
  }

  if ((Scheme_Type ) (scheme_case_lambda_sequence_type ) == (Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) )  {
    Scheme_Case_Lambda * cl = (Scheme_Case_Lambda * ) value ; 
    int i ; 
    BLOCK_SETUP((PUSH(cl, 0+XfOrM996_COUNT)));
#   define XfOrM998_COUNT (1+XfOrM996_COUNT)
#   define SETUP_XfOrM998(x) SETUP(XfOrM998_COUNT)

# 6985 "./../src/optimize.c"
    for (i = cl -> count ; i -- ; ) {
#     define XfOrM1000_COUNT (0+XfOrM998_COUNT)
#     define SETUP_XfOrM1000(x) SETUP_XfOrM998(x)

# 6986 "./../src/optimize.c"
      if (! FUNCCALL(SETUP_XfOrM1000(_), ir_propagate_ok (cl -> array [i ] , info , used_once , once_var ) ))
        RET_VALUE_START (0 ) RET_VALUE_END ; 
    }
    RET_VALUE_START (1 ) RET_VALUE_END ; 
  }

  if ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_toplevel_type ) )  {
#   define XfOrM997_COUNT (0+XfOrM996_COUNT)
#   define SETUP_XfOrM997(x) SETUP_XfOrM996(x)

# 6993 "./../src/optimize.c"
    if (((& ((Scheme_IR_Toplevel * ) value ) -> iso ) -> so . keyex & 0x3 ) >= 2 )
      RET_VALUE_START (1 ) RET_VALUE_END ; 
    if (FUNCCALL(SETUP_XfOrM997(_), get_import_shape (info , (Scheme_IR_Toplevel * ) value ) ))
      RET_VALUE_START (1 ) RET_VALUE_END ; 

    value = FUNCCALL(SETUP_XfOrM997(_), get_defn_shape (info , (Scheme_IR_Toplevel * ) value ) ); 
    value = FUNCCALL_AGAIN(no_potential_size (value ) ); 
    if (((value ) == (scheme_stack_dump_key ) ) 
    || (value && ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) ) ) )
      RET_VALUE_START (0 ) RET_VALUE_END ; 
    else if (value )
      RET_VALUE_START (1 ) RET_VALUE_END ; 
    else 
    RET_VALUE_START (0 ) RET_VALUE_END ; 
  }



  if (FUNCCALL_EMPTY(scheme_ir_duplicate_ok (value , 0 ) ))
    RET_VALUE_START (1 ) RET_VALUE_END ; 

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 7017 "./../src/optimize.c"
int scheme_is_statically_proc (Scheme_Object * value , Optimize_Info * info , int flags ) 





{
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK(3);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(value, 1)));
# define XfOrM1006_COUNT (2)
# define SETUP_XfOrM1006(x) SETUP(XfOrM1006_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 7024 "./../src/optimize.c"
  while (1 ) {
#   define XfOrM1018_COUNT (0+XfOrM1006_COUNT)
#   define SETUP_XfOrM1018(x) SETUP_XfOrM1006(x)

# 7025 "./../src/optimize.c"
    if ((((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) 
    || (! (((intptr_t ) (value ) ) & 0x1 ) && ((((value ) -> type ) >= scheme_prim_type ) && (((value ) -> type ) <= scheme_proc_chaperone_type ) ) ) 
    || ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_lambda_type ) ) 
    || ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) 
    || ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_inline_variant_type ) ) )
      RET_VALUE_START (1 ) RET_VALUE_END ; 
    else if ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_let_header_type ) )  {

      Scheme_IR_Let_Header * lh = (Scheme_IR_Let_Header * ) value ; 
#     define XfOrM1026_COUNT (0+XfOrM1018_COUNT)
#     define SETUP_XfOrM1026(x) SETUP_XfOrM1018(x)

# 7034 "./../src/optimize.c"
      if (lh -> num_clauses == 1 ) {
        Scheme_IR_Let_Value * lv = (Scheme_IR_Let_Value * ) lh -> body ; 
        BLOCK_SETUP((PUSH(lv, 0+XfOrM1026_COUNT)));
#       define XfOrM1027_COUNT (1+XfOrM1026_COUNT)
#       define SETUP_XfOrM1027(x) SETUP(XfOrM1027_COUNT)

# 7036 "./../src/optimize.c"
        if (FUNCCALL(SETUP_XfOrM1027(_), scheme_omittable_expr (lv -> value , lv -> count , 20 , flags , info , ((void * ) 0 ) ) )) {
          value = lv -> body ; 
        }

# 7038 "./../src/optimize.c"
        else 
        break ; 
      }

# 7040 "./../src/optimize.c"
      else 
      break ; 
    }

# 7042 "./../src/optimize.c"
    else if ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_let_one_type ) )  {
      Scheme_Let_One * lo = (Scheme_Let_One * ) value ; 
      BLOCK_SETUP((PUSH(lo, 0+XfOrM1018_COUNT)));
#     define XfOrM1024_COUNT (1+XfOrM1018_COUNT)
#     define SETUP_XfOrM1024(x) SETUP(XfOrM1024_COUNT)

# 7044 "./../src/optimize.c"
      if (FUNCCALL(SETUP_XfOrM1024(_), scheme_omittable_expr (lo -> value , 1 , 20 , flags , info , ((void * ) 0 ) ) )) {
        value = lo -> body ; 
      }

# 7046 "./../src/optimize.c"
      else 
      break ; 
    }

# 7048 "./../src/optimize.c"
    else if ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_boxenv_type ) )  {
      value = (((Scheme_Simple_Object * ) (value ) ) -> u . two_ptr_val . ptr2 ) ; 
    }

# 7050 "./../src/optimize.c"
    else if (((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_sequence_type ) ) 


    && (flags & 0x1 ) ) {
      Scheme_Sequence * seq = (Scheme_Sequence * ) value ; 
      int i ; 
      BLOCK_SETUP((PUSH(seq, 0+XfOrM1018_COUNT)));
#     define XfOrM1019_COUNT (1+XfOrM1018_COUNT)
#     define SETUP_XfOrM1019(x) SETUP(XfOrM1019_COUNT)

# 7056 "./../src/optimize.c"
      for (i = 0 ; i < seq -> count - 1 ; i ++ ) {
#       define XfOrM1022_COUNT (0+XfOrM1019_COUNT)
#       define SETUP_XfOrM1022(x) SETUP_XfOrM1019(x)

# 7057 "./../src/optimize.c"
        if (! FUNCCALL(SETUP_XfOrM1022(_), scheme_omittable_expr (seq -> array [i ] , 1 , 5 , flags , info , ((void * ) 0 ) ) ))
          break ; 
      }
      if (i == seq -> count - 1 ) {
        value = seq -> array [i ] ; 
      }

# 7062 "./../src/optimize.c"
      else 
      break ; 
    }

# 7064 "./../src/optimize.c"
    else 
    break ; 
  }

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 7071 "./../src/optimize.c"
Scheme_Object * scheme_make_noninline_proc (Scheme_Object * e ) 


{
  Scheme_Object * ni ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(4);
  BLOCK_SETUP_TOP((PUSH(ni, 0), PUSH(e, 1)));
# define XfOrM1029_COUNT (2)
# define SETUP_XfOrM1029(x) SETUP(XfOrM1029_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  ni = NULLED_OUT ; 

# 7077 "./../src/optimize.c"
  while (((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_let_header_type ) ) ) {

    Scheme_IR_Let_Header * lh = (Scheme_IR_Let_Header * ) e ; 
    Scheme_IR_Let_Value * lv = (Scheme_IR_Let_Value * ) lh -> body ; 
    BLOCK_SETUP((PUSH(lv, 0+XfOrM1029_COUNT), PUSH(lh, 1+XfOrM1029_COUNT)));
#   define XfOrM1032_COUNT (2+XfOrM1029_COUNT)
#   define SETUP_XfOrM1032(x) SETUP(XfOrM1032_COUNT)

# 7081 "./../src/optimize.c"
    ((void ) sizeof ((lh -> num_clauses == 1 ) ? 1 : 0 ) , __extension__ ({
#     define XfOrM1033_COUNT (0+XfOrM1032_COUNT)
#     define SETUP_XfOrM1033(x) SETUP_XfOrM1032(x)

# 7081 "./../src/optimize.c" 3
      if (lh -> num_clauses == 1 )

# 7081 "./../src/optimize.c" 3
      ; 

# 7081 "./../src/optimize.c" 3
      else FUNCCALL(SETUP_XfOrM1033(_), __assert_fail ("lh->num_clauses == 1" , "./../src/optimize.c" , 7081 , __extension__ __PRETTY_FUNCTION__ ) ); 
    }
    ) ) 
# 7081 "./../src/optimize.c"
    ; 
    e = lv -> body ; 
  }

  ni = ((Scheme_Object * ) FUNCCALL(SETUP_XfOrM1029(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_Small_Object ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
  ni -> type = scheme_noninline_proc_type ; 
  (((Scheme_Small_Object * ) (ni ) ) -> u . ptr_val ) = e ; 

  RET_VALUE_START (ni ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 7092 "./../src/optimize.c"
static int is_values_apply (Scheme_Object * e , int n , Optimize_Info * info , Scheme_Hash_Tree * except_vars , int fuel ) 



{
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK(3);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(except_vars, 1)));
# define XfOrM1034_COUNT (2)
# define SETUP_XfOrM1034(x) SETUP(XfOrM1034_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 7097 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_application_type ) )  {
    Scheme_App_Rec * app = (Scheme_App_Rec * ) e ; 
    if (n != app -> num_args )

# 7099 "./../src/optimize.c"
    RET_VALUE_START (0 ) RET_VALUE_END ; 
    RET_VALUE_START (((scheme_values_proc ) == (app -> args [0 ] ) ) ) RET_VALUE_END ; 
  }

# 7101 "./../src/optimize.c"
  else if ((n == 1 ) && ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_application2_type ) ) ) {
    Scheme_App2_Rec * app = (Scheme_App2_Rec * ) e ; 
    RET_VALUE_START (((scheme_values_proc ) == (app -> rator ) ) ) RET_VALUE_END ; 
  }

# 7104 "./../src/optimize.c"
  else if ((n == 2 ) && ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_application3_type ) ) ) {
    Scheme_App3_Rec * app = (Scheme_App3_Rec * ) e ; 
    RET_VALUE_START (((scheme_values_proc ) == (app -> rator ) ) ) RET_VALUE_END ; 
  }

# 7107 "./../src/optimize.c"
  else if (fuel && ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_branch_type ) ) ) {
    Scheme_Branch_Rec * b = (Scheme_Branch_Rec * ) e ; 
    BLOCK_SETUP((PUSH(b, 0+XfOrM1034_COUNT)));
#   define XfOrM1035_COUNT (1+XfOrM1034_COUNT)
#   define SETUP_XfOrM1035(x) SETUP(XfOrM1035_COUNT)

# 7109 "./../src/optimize.c"
    if (((Scheme_Type ) (((((intptr_t ) (b -> test ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (b -> test ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) 
    && ! (scheme_eq_hash_tree_get (except_vars , b -> test ) ) 
    && ! ((Scheme_IR_Local * ) b -> test ) -> mutated ) {
#     define XfOrM1036_COUNT (0+XfOrM1035_COUNT)
#     define SETUP_XfOrM1036(x) SETUP_XfOrM1035(x)

# 7112 "./../src/optimize.c"
      RET_VALUE_START ((FUNCCALL(SETUP_XfOrM1036(_), is_values_apply (b -> tbranch , n , info , except_vars , 0 ) )
      && FUNCCALL_EMPTY(is_values_apply (b -> fbranch , n , info , except_vars , 0 ) )) ) 
# 7112 "./../src/optimize.c"
      RET_VALUE_END 
      ; 
    }
  }

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 7120 "./../src/optimize.c"
static int no_mutable_bindings (Scheme_IR_Let_Value * irlv ) 

{
  /* No conversion */

# 7123 "./../src/optimize.c"
  int i ; 

  for (i = irlv -> count ; i -- ; ) {
    if (irlv -> vars [i ] -> mutated )
      return 0 ; 
  }

  return 1 ; 
}

static void update_rhs_value (Scheme_IR_Let_Value * naya , Scheme_Object * e , 
Optimize_Info * info , Scheme_IR_Local * tst ) 

{
  PREPARE_VAR_STACK(5);
  BLOCK_SETUP_TOP((PUSH(naya, 0), PUSH(tst, 1), PUSH(e, 2)));
# define XfOrM1043_COUNT (3)
# define SETUP_XfOrM1043(x) SETUP(XfOrM1043_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 7137 "./../src/optimize.c"
  if (tst ) {
    Scheme_Object * n ; 
    BLOCK_SETUP((PUSH(n, 0+XfOrM1043_COUNT)));
#   define XfOrM1044_COUNT (1+XfOrM1043_COUNT)
#   define SETUP_XfOrM1044(x) SETUP(XfOrM1044_COUNT)
    n = NULLED_OUT ; 

# 7140 "./../src/optimize.c"
    n = FUNCCALL(SETUP_XfOrM1044(_), equivalent_exprs (naya -> value , e , ((void * ) 0 ) , ((void * ) 0 ) , 0 ) ); 
    if (! n ) {
      Scheme_Branch_Rec * b ; 
      BLOCK_SETUP((PUSH(b, 0+XfOrM1044_COUNT)));
#     define XfOrM1045_COUNT (1+XfOrM1044_COUNT)
#     define SETUP_XfOrM1045(x) SETUP(XfOrM1045_COUNT)
      b = NULLED_OUT ; 
      FUNCCALL(SETUP_XfOrM1045(_), increment_use_count (tst , 0 ) )
# 7145 "./../src/optimize.c"
      ; 

      b = ((Scheme_Branch_Rec * ) FUNCCALL(SETUP_XfOrM1045(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_Branch_Rec ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
      b -> so . type = scheme_branch_type ; 
      b -> test = (Scheme_Object * ) tst ; 
      b -> tbranch = naya -> value ; 
      b -> fbranch = e ; 

      naya -> value = (Scheme_Object * ) b ; 
    }

# 7154 "./../src/optimize.c"
    else 
    naya -> value = n ; 
  }

# 7156 "./../src/optimize.c"
  else 
  naya -> value = e ; 
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 7160 "./../src/optimize.c"
static void unpack_values_application (Scheme_Object * e , Scheme_IR_Let_Value * naya , 
Optimize_Info * info , Scheme_IR_Local * branch_test ) 

{
  PREPARE_VAR_STACK(5);
  BLOCK_SETUP_TOP((PUSH(naya, 0), PUSH(info, 1), PUSH(e, 2), PUSH(branch_test, 3)));
# define XfOrM1046_COUNT (4)
# define SETUP_XfOrM1046(x) SETUP(XfOrM1046_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 7164 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_application_type ) )  {
    Scheme_App_Rec * app = (Scheme_App_Rec * ) e ; 
    int i ; 
    BLOCK_SETUP((PUSH(app, 0+XfOrM1046_COUNT)));
#   define XfOrM1051_COUNT (1+XfOrM1046_COUNT)
#   define SETUP_XfOrM1051(x) SETUP(XfOrM1051_COUNT)

# 7167 "./../src/optimize.c"
    for (i = 0 ; i < app -> num_args ; i ++ ) {
#     define XfOrM1053_COUNT (0+XfOrM1051_COUNT)
#     define SETUP_XfOrM1053(x) SETUP_XfOrM1051(x)
      FUNCCALL(SETUP_XfOrM1053(_), update_rhs_value (naya , app -> args [i + 1 ] , info , branch_test ) )
# 7168 "./../src/optimize.c"
      ; 
      naya = (Scheme_IR_Let_Value * ) naya -> body ; 
    }
  }

# 7171 "./../src/optimize.c"
  else if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_application2_type ) )  {
    Scheme_App2_Rec * app = (Scheme_App2_Rec * ) e ; 
#   define XfOrM1050_COUNT (0+XfOrM1046_COUNT)
#   define SETUP_XfOrM1050(x) SETUP_XfOrM1046(x)
    FUNCCALL(SETUP_XfOrM1050(_), update_rhs_value (naya , app -> rand , info , branch_test ) )
# 7173 "./../src/optimize.c"
    ; 
  }

# 7174 "./../src/optimize.c"
  else if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_application3_type ) )  {
    Scheme_App3_Rec * app = (Scheme_App3_Rec * ) e ; 
    BLOCK_SETUP((PUSH(app, 0+XfOrM1046_COUNT)));
#   define XfOrM1049_COUNT (1+XfOrM1046_COUNT)
#   define SETUP_XfOrM1049(x) SETUP(XfOrM1049_COUNT)
    FUNCCALL(SETUP_XfOrM1049(_), update_rhs_value (naya , app -> rand1 , info , branch_test ) )
# 7176 "./../src/optimize.c"
    ; 
    naya = (Scheme_IR_Let_Value * ) naya -> body ; 
    FUNCCALL(SETUP_XfOrM1049(_), update_rhs_value (naya , app -> rand2 , info , branch_test ) ); 
  }

# 7179 "./../src/optimize.c"
  else if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_branch_type ) )  {
    Scheme_Branch_Rec * b = (Scheme_Branch_Rec * ) e ; 
    BLOCK_SETUP((PUSH(b, 0+XfOrM1046_COUNT)));
#   define XfOrM1047_COUNT (1+XfOrM1046_COUNT)
#   define SETUP_XfOrM1047(x) SETUP(XfOrM1047_COUNT)

# 7182 "./../src/optimize.c"
    ((void ) sizeof ((((Scheme_Type ) (((((intptr_t ) (b -> test ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (b -> test ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) ) ? 1 : 0 ) , __extension__ ({
#     define XfOrM1048_COUNT (0+XfOrM1047_COUNT)
#     define SETUP_XfOrM1048(x) SETUP_XfOrM1047(x)

# 7182 "./../src/optimize.c" 3
      if ((Scheme_Type ) (((((intptr_t ) (b -> test ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (b -> test ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) 

# 7182 "./../src/optimize.c" 3
      ; 

# 7182 "./../src/optimize.c" 3
      else FUNCCALL(SETUP_XfOrM1048(_), __assert_fail ("((Scheme_Type)(((((intptr_t)(b->test)) & 0x1)?(Scheme_Type)scheme_integer_type:((Scheme_Object *)(b->test))->type)) == (Scheme_Type)(scheme_ir_local_type))" , "./../src/optimize.c" , 7182 , __extension__ __PRETTY_FUNCTION__ ) ); 
    }
    ) ) 
# 7182 "./../src/optimize.c"
    ; 
    FUNCCALL(SETUP_XfOrM1047(_), unpack_values_application (b -> tbranch , naya , info , ((void * ) 0 ) ) )
    ; 
    FUNCCALL_EMPTY(unpack_values_application (b -> fbranch , naya , info , ((Scheme_IR_Local * ) b -> test ) ) ); 
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 7189 "./../src/optimize.c"
static Scheme_Object * make_clones (Scheme_IR_Let_Value * retry_start , 
Scheme_IR_Let_Value * pre_body , 
Optimize_Info * body_info ) 


{
  Scheme_IR_Let_Value * irlv ; 
  Scheme_Object * value , * clone , * pr ; 
  Scheme_Object * last = ((void * ) 0 ) , * first = ((void * ) 0 ) ; 
  Scheme_Object * __funcarg108 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(8);
  BLOCK_SETUP_TOP((PUSH(value, 0), PUSH(last, 1), PUSH(irlv, 2), PUSH(body_info, 3), PUSH(pre_body, 4), PUSH(first, 5), PUSH(pr, 6), PUSH(clone, 7)));
# define XfOrM1054_COUNT (8)
# define SETUP_XfOrM1054(x) SETUP(XfOrM1054_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  irlv = NULLED_OUT ; 
  value = NULLED_OUT ; 
  clone = NULLED_OUT ; 
  pr = NULLED_OUT ; 

# 7199 "./../src/optimize.c"
  irlv = retry_start ; 
  while (1 ) {
#   define XfOrM1058_COUNT (0+XfOrM1054_COUNT)
#   define SETUP_XfOrM1058(x) SETUP_XfOrM1054(x)

# 7201 "./../src/optimize.c"
    value = irlv -> value ; 
    if (((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) )  {
#     define XfOrM1059_COUNT (0+XfOrM1058_COUNT)
#     define SETUP_XfOrM1059(x) SETUP_XfOrM1058(x)

# 7203 "./../src/optimize.c"
      clone = FUNCCALL(SETUP_XfOrM1059(_), optimize_clone (1 , value , body_info , empty_eq_hash_tree , 0 ) ); 
      if (clone ) {
#       define XfOrM1060_COUNT (0+XfOrM1059_COUNT)
#       define SETUP_XfOrM1060(x) SETUP_XfOrM1059(x)

# 7205 "./../src/optimize.c"
        pr = (__funcarg108 = FUNCCALL(SETUP_XfOrM1060(_), scheme_make_raw_pair (value , clone ) ), FUNCCALL_AGAIN(scheme_make_raw_pair (__funcarg108 , ((void * ) 0 ) ) )) ; 
      }

# 7206 "./../src/optimize.c"
      else 
      pr = FUNCCALL(SETUP_XfOrM1059(_), scheme_make_raw_pair (((void * ) 0 ) , ((void * ) 0 ) ) ); 
      if (last )
        (((Scheme_Simple_Object * ) (last ) ) -> u . pair_val . cdr ) = pr ; 
      else 
      first = pr ; 
      last = pr ; 
    }
    if (irlv == pre_body )
      break ; 
    irlv = (Scheme_IR_Let_Value * ) irlv -> body ; 
  }

  RET_VALUE_START (first ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 7222 "./../src/optimize.c"
static int set_one_code_flags (Scheme_Object * value , int flags , 
Scheme_Object * first , Scheme_Object * second , 
int set_flags , int mask_flags , int just_tentative , 
int merge_local_typed ) 

{
  Scheme_Case_Lambda * cl , * cl2 , * cl3 ; 
  Scheme_Lambda * lam , * lam2 , * lam3 ; 
  int i , count ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(9);
  BLOCK_SETUP_TOP((PUSH(lam2, 0), PUSH(lam3, 1), PUSH(value, 2), PUSH(first, 3), PUSH(lam, 4), PUSH(cl3, 5), PUSH(second, 6), PUSH(cl, 7), PUSH(cl2, 8)));
# define XfOrM1061_COUNT (9)
# define SETUP_XfOrM1061(x) SETUP(XfOrM1061_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  cl = NULLED_OUT ; 
  cl2 = NULLED_OUT ; 
  cl3 = NULLED_OUT ; 
  lam = NULLED_OUT ; 
  lam2 = NULLED_OUT ; 
  lam3 = NULLED_OUT ; 

# 7232 "./../src/optimize.c"
  if ((Scheme_Type ) (scheme_ir_lambda_type ) == (Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) )  {
    count = 1 ; 
    cl = ((void * ) 0 ) ; 
    cl2 = ((void * ) 0 ) ; 
    cl3 = ((void * ) 0 ) ; 
  }

# 7237 "./../src/optimize.c"
  else {
    cl = (Scheme_Case_Lambda * ) value ; 
    cl2 = (Scheme_Case_Lambda * ) first ; 
    cl3 = (Scheme_Case_Lambda * ) second ; 
    count = cl -> count ; 
  }

  for (i = 0 ; i < count ; i ++ ) {
#   define XfOrM1067_COUNT (0+XfOrM1061_COUNT)
#   define SETUP_XfOrM1067(x) SETUP_XfOrM1061(x)

# 7245 "./../src/optimize.c"
    if (cl ) {
      lam = (Scheme_Lambda * ) cl -> array [i ] ; 
      lam2 = (Scheme_Lambda * ) cl2 -> array [i ] ; 
      lam3 = (Scheme_Lambda * ) cl3 -> array [i ] ; 
    }

# 7249 "./../src/optimize.c"
    else {
      lam = (Scheme_Lambda * ) value ; 
      lam2 = (Scheme_Lambda * ) first ; 
      lam3 = (Scheme_Lambda * ) second ; 
    }

    if (merge_local_typed ) {
#     define XfOrM1069_COUNT (0+XfOrM1067_COUNT)
#     define SETUP_XfOrM1069(x) SETUP_XfOrM1067(x)
      FUNCCALL(SETUP_XfOrM1069(_), merge_lambda_arg_types (lam , lam2 ) )
# 7256 "./../src/optimize.c"
      ; 
      FUNCCALL(SETUP_XfOrM1069(_), merge_lambda_arg_types (lam , lam3 ) ); 
      FUNCCALL_AGAIN(merge_lambda_arg_types (lam , lam2 ) ); 
    }

    if (! just_tentative || (((& (lam ) -> iso ) -> so . keyex & (64 | 128 ) ) == 64 ) ) {
      flags = (flags & (& (lam ) -> iso ) -> so . keyex ) ; 
      (& (lam2 ) -> iso ) -> so . keyex = set_flags | ((& (lam2 ) -> iso ) -> so . keyex & mask_flags ) ; 
      (& (lam3 ) -> iso ) -> so . keyex = set_flags | ((& (lam3 ) -> iso ) -> so . keyex & mask_flags ) ; 
    }
  }

  RET_VALUE_START (flags ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 7271 "./../src/optimize.c"
static int set_code_flags (Scheme_IR_Let_Value * retry_start , 
Scheme_IR_Let_Value * pre_body , 
Scheme_Object * clones , 
int set_flags , int mask_flags , int just_tentative , 
int merge_local_typed ) 

{
  Scheme_IR_Let_Value * irlv ; 
  Scheme_Object * value , * first ; 
  int flags = 32 | 4 ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(5);
  BLOCK_SETUP_TOP((PUSH(clones, 0), PUSH(irlv, 1), PUSH(value, 2), PUSH(pre_body, 3), PUSH(first, 4)));
# define XfOrM1074_COUNT (5)
# define SETUP_XfOrM1074(x) SETUP(XfOrM1074_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  irlv = NULLED_OUT ; 
  value = NULLED_OUT ; 
  first = NULLED_OUT ; 

# 7287 "./../src/optimize.c"
  irlv = retry_start ; 
  while (clones ) {
#   define XfOrM1077_COUNT (0+XfOrM1074_COUNT)
#   define SETUP_XfOrM1077(x) SETUP_XfOrM1074(x)

# 7289 "./../src/optimize.c"
    value = irlv -> value ; 
    if (((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) )  {
#     define XfOrM1078_COUNT (0+XfOrM1077_COUNT)
#     define SETUP_XfOrM1078(x) SETUP_XfOrM1077(x)

# 7291 "./../src/optimize.c"
      first = (((Scheme_Simple_Object * ) (clones ) ) -> u . pair_val . car ) ; 

      if (first )
        flags = FUNCCALL(SETUP_XfOrM1078(_), set_one_code_flags (value , flags , (((Scheme_Simple_Object * ) (first ) ) -> u . pair_val . car ) , (((Scheme_Simple_Object * ) (first ) ) -> u . pair_val . cdr ) , set_flags , mask_flags , just_tentative , merge_local_typed ) )


      ; 

      clones = (((Scheme_Simple_Object * ) (clones ) ) -> u . pair_val . cdr ) ; 
    }

    if (irlv == pre_body )
      break ; 
    irlv = (Scheme_IR_Let_Value * ) irlv -> body ; 
  }

  RET_VALUE_START (flags ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 7310 "./../src/optimize.c"
static int lambda_body_size (Scheme_Object * o , int less_args ) 
{
  int bsz ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK(2);
  BLOCK_SETUP_TOP((PUSH(o, 0)));
# define XfOrM1079_COUNT (1)
# define SETUP_XfOrM1079(x) SETUP(XfOrM1079_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 7314 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (o ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) )  {
#   define XfOrM1085_COUNT (0+XfOrM1079_COUNT)
#   define SETUP_XfOrM1085(x) SETUP_XfOrM1079(x)

# 7315 "./../src/optimize.c"
    bsz = FUNCCALL(SETUP_XfOrM1085(_), lambda_body_size_plus_info ((Scheme_Lambda * ) o , 0 , ((void * ) 0 ) , ((void * ) 0 ) ) ); 
    if (less_args )

# 7316 "./../src/optimize.c"
    bsz -= ((Scheme_Lambda * ) o ) -> num_params ; 
    RET_VALUE_START (bsz ) RET_VALUE_END ; 
  }

# 7318 "./../src/optimize.c"
  else if ((Scheme_Type ) (((((intptr_t ) (o ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) )  {
    Scheme_Case_Lambda * cl = (Scheme_Case_Lambda * ) o ; 
    int i , sz = 0 ; 
    BLOCK_SETUP((PUSH(cl, 0+XfOrM1079_COUNT)));
#   define XfOrM1080_COUNT (1+XfOrM1079_COUNT)
#   define SETUP_XfOrM1080(x) SETUP(XfOrM1080_COUNT)

# 7321 "./../src/optimize.c"
    for (i = cl -> count ; i -- ; ) {
#     define XfOrM1083_COUNT (0+XfOrM1080_COUNT)
#     define SETUP_XfOrM1083(x) SETUP_XfOrM1080(x)

# 7322 "./../src/optimize.c"
      bsz = FUNCCALL(SETUP_XfOrM1083(_), lambda_body_size_plus_info ((Scheme_Lambda * ) cl -> array [i ] , 0 , ((void * ) 0 ) , ((void * ) 0 ) ) ); 
      if (less_args ) {
        bsz -= ((Scheme_Lambda * ) cl -> array [i ] ) -> num_params ; 
        if (bsz > sz )

# 7325 "./../src/optimize.c"
        sz = bsz ; 
      }

# 7326 "./../src/optimize.c"
      else 
      sz += bsz ; 
    }
    RET_VALUE_START (sz ) RET_VALUE_END ; 
  }

# 7330 "./../src/optimize.c"
  else 
  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 7334 "./../src/optimize.c"
static int expr_size (Scheme_Object * o ) 
{
  /* No conversion */

# 7336 "./../src/optimize.c"
  return lambda_body_size (o , 0 ) + 1 ; 
}

int scheme_might_invoke_call_cc (Scheme_Object * value ) 
{
  /* No conversion */

# 7341 "./../src/optimize.c"
  return ! scheme_is_liftable (value , empty_eq_hash_tree , 10 , 0 , 1 ) ; 
}



void advance_clocks_for_optimized (Scheme_Object * o , 
GC_CAN_IGNORE int * _vclock , 
GC_CAN_IGNORE int * _aclock , 
GC_CAN_IGNORE int * _kclock , 
GC_CAN_IGNORE int * _sclock , 
Optimize_Info * info , 
int fuel ) 


{
  Scheme_Object * rator = ((void * ) 0 ) ; 
  int argc = 0 ; 
  PREPARE_VAR_STACK(4);
  BLOCK_SETUP_TOP((PUSH(o, 0), PUSH(info, 1), PUSH(rator, 2)));
# define XfOrM1088_COUNT (3)
# define SETUP_XfOrM1088(x) SETUP(XfOrM1088_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 7359 "./../src/optimize.c"
  if (! fuel )

# 7359 "./../src/optimize.c"
  RET_NOTHING ; 

  switch (((((intptr_t ) (o ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o ) ) -> type ) ) {
#   define XfOrM1089_COUNT (0+XfOrM1088_COUNT)
#   define SETUP_XfOrM1089(x) SETUP_XfOrM1088(x)

# 7362 "./../src/optimize.c"
    case scheme_application_type : 
    {
      Scheme_App_Rec * app = (Scheme_App_Rec * ) o ; 
      int i ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM1089_COUNT)));
#     define XfOrM1092_COUNT (1+XfOrM1089_COUNT)
#     define SETUP_XfOrM1092(x) SETUP(XfOrM1092_COUNT)

# 7366 "./../src/optimize.c"
      for (i = 0 ; i < app -> num_args ; i ++ ) {
#       define XfOrM1094_COUNT (0+XfOrM1092_COUNT)
#       define SETUP_XfOrM1094(x) SETUP_XfOrM1092(x)
        FUNCCALL(SETUP_XfOrM1094(_), advance_clocks_for_optimized (app -> args [i + 1 ] , _vclock , _aclock , _kclock , _sclock , info , fuel - 1 ) ); 
      }
      rator = app -> args [0 ] ; 
      argc = app -> num_args ; 
    }
    break ; 
    case scheme_application2_type : 
    {
      Scheme_App2_Rec * app = (Scheme_App2_Rec * ) o ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM1089_COUNT)));
#     define XfOrM1091_COUNT (1+XfOrM1089_COUNT)
#     define SETUP_XfOrM1091(x) SETUP(XfOrM1091_COUNT)
      FUNCCALL(SETUP_XfOrM1091(_), advance_clocks_for_optimized (app -> rand , _vclock , _aclock , _kclock , _sclock , info , fuel - 1 ) )
# 7380 "./../src/optimize.c"
      ; 
      rator = app -> rator ; 
      argc = 1 ; 
      break ; 
    }
    case scheme_application3_type : 
    {
      Scheme_App3_Rec * app = (Scheme_App3_Rec * ) o ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM1089_COUNT)));
#     define XfOrM1090_COUNT (1+XfOrM1089_COUNT)
#     define SETUP_XfOrM1090(x) SETUP(XfOrM1090_COUNT)
      FUNCCALL(SETUP_XfOrM1090(_), advance_clocks_for_optimized (app -> rand1 , _vclock , _aclock , _kclock , _sclock , info , fuel - 1 ) )
# 7390 "./../src/optimize.c"
      ; 
      FUNCCALL(SETUP_XfOrM1090(_), advance_clocks_for_optimized (app -> rand2 , _vclock , _aclock , _kclock , _sclock , info , fuel - 1 ) )

      ; 
      rator = app -> rator ; 
      argc = 2 ; 
    }
    break ; 
    default : 
    break ; 
  }

  if (rator )
    FUNCCALL(SETUP_XfOrM1088(_), increment_clock_counts_for_application (_vclock , _aclock , _kclock , _sclock , rator , argc ) ); 

  if ((* _vclock > info -> vclock ) 
  || (* _aclock > info -> aclock ) 
  || (* _kclock > info -> kclock ) 
  || (* _sclock > info -> sclock ) )
    FUNCCALL_EMPTY(scheme_signal_error ("internal error: optimizer clock tracking has gone wrong" ) ); 
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 7412 "./../src/optimize.c"
static void set_application_types (Scheme_Object * o , Optimize_Info * info , int fuel ) 



{
  PREPARE_VAR_STACK(3);
  BLOCK_SETUP_TOP((PUSH(o, 0), PUSH(info, 1)));
# define XfOrM1095_COUNT (2)
# define SETUP_XfOrM1095(x) SETUP(XfOrM1095_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 7417 "./../src/optimize.c"
  if (! fuel )

# 7417 "./../src/optimize.c"
  RET_NOTHING ; 

  switch (((((intptr_t ) (o ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (o ) ) -> type ) ) {
#   define XfOrM1096_COUNT (0+XfOrM1095_COUNT)
#   define SETUP_XfOrM1096(x) SETUP_XfOrM1095(x)

# 7420 "./../src/optimize.c"
    case scheme_application_type : 
    {
      Scheme_App_Rec * app = (Scheme_App_Rec * ) o ; 
      int i ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM1096_COUNT)));
#     define XfOrM1103_COUNT (1+XfOrM1096_COUNT)
#     define SETUP_XfOrM1103(x) SETUP(XfOrM1103_COUNT)
      FUNCCALL(SETUP_XfOrM1103(_), register_local_argument_types (app , ((void * ) 0 ) , ((void * ) 0 ) , info ) )
# 7424 "./../src/optimize.c"
      ; 
      for (i = 0 ; i < app -> num_args + 1 ; i ++ ) {
#       define XfOrM1105_COUNT (0+XfOrM1103_COUNT)
#       define SETUP_XfOrM1105(x) SETUP_XfOrM1103(x)
        FUNCCALL(SETUP_XfOrM1105(_), set_application_types (app -> args [i ] , info , fuel - 1 ) )
# 7426 "./../src/optimize.c"
        ; 
      }
    }
    break ; 
    case scheme_application2_type : 
    {
      Scheme_App2_Rec * app = (Scheme_App2_Rec * ) o ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM1096_COUNT)));
#     define XfOrM1102_COUNT (1+XfOrM1096_COUNT)
#     define SETUP_XfOrM1102(x) SETUP(XfOrM1102_COUNT)
      FUNCCALL(SETUP_XfOrM1102(_), register_local_argument_types (((void * ) 0 ) , app , ((void * ) 0 ) , info ) )
# 7433 "./../src/optimize.c"
      ; 
      FUNCCALL(SETUP_XfOrM1102(_), set_application_types (app -> rator , info , fuel - 1 ) ); 
      FUNCCALL_AGAIN(set_application_types (app -> rand , info , fuel - 1 ) ); 
      break ; 
    }
    case scheme_application3_type : 
    {
      Scheme_App3_Rec * app = (Scheme_App3_Rec * ) o ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM1096_COUNT)));
#     define XfOrM1101_COUNT (1+XfOrM1096_COUNT)
#     define SETUP_XfOrM1101(x) SETUP(XfOrM1101_COUNT)
      FUNCCALL(SETUP_XfOrM1101(_), register_local_argument_types (((void * ) 0 ) , ((void * ) 0 ) , app , info ) )
# 7441 "./../src/optimize.c"
      ; 
      FUNCCALL(SETUP_XfOrM1101(_), set_application_types (app -> rator , info , fuel - 1 ) ); 
      FUNCCALL_AGAIN(set_application_types (app -> rand1 , info , fuel - 1 ) ); 
      FUNCCALL_AGAIN(set_application_types (app -> rand2 , info , fuel - 1 ) ); 
    }
    break ; 
    case scheme_sequence_type : 
    case scheme_begin0_sequence_type : 
    {
      Scheme_Sequence * seq = (Scheme_Sequence * ) o ; 
      int i ; 
      BLOCK_SETUP((PUSH(seq, 0+XfOrM1096_COUNT)));
#     define XfOrM1098_COUNT (1+XfOrM1096_COUNT)
#     define SETUP_XfOrM1098(x) SETUP(XfOrM1098_COUNT)

# 7453 "./../src/optimize.c"
      for (i = 0 ; i < seq -> count ; i ++ ) {
#       define XfOrM1100_COUNT (0+XfOrM1098_COUNT)
#       define SETUP_XfOrM1100(x) SETUP_XfOrM1098(x)
        FUNCCALL(SETUP_XfOrM1100(_), set_application_types (seq -> array [i ] , info , fuel - 1 ) )
# 7454 "./../src/optimize.c"
        ; 
      }
    }
    break ; 
    case scheme_branch_type : 
    {
      Scheme_Branch_Rec * b = (Scheme_Branch_Rec * ) o ; 
      BLOCK_SETUP((PUSH(b, 0+XfOrM1096_COUNT)));
#     define XfOrM1097_COUNT (1+XfOrM1096_COUNT)
#     define SETUP_XfOrM1097(x) SETUP(XfOrM1097_COUNT)
      FUNCCALL(SETUP_XfOrM1097(_), set_application_types (b -> test , info , fuel - 1 ) )
# 7461 "./../src/optimize.c"
      ; 
      FUNCCALL(SETUP_XfOrM1097(_), set_application_types (b -> tbranch , info , fuel - 1 ) ); 
      FUNCCALL_EMPTY(set_application_types (b -> fbranch , info , fuel - 1 ) ); 
    }
    break ; 
    default : 
    break ; 
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 7471 "./../src/optimize.c"
static void flip_transitive (Scheme_Hash_Table * ht , int on ) 

{
  Scheme_IR_Local * tvar ; 
  int j ; 
  Scheme_Object * to_remove = scheme_null ; 
  PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSH(tvar, 0), PUSH(ht, 1), PUSH(to_remove, 2)));
# define XfOrM1106_COUNT (3)
# define SETUP_XfOrM1106(x) SETUP(XfOrM1106_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  tvar = NULLED_OUT ; 

# 7478 "./../src/optimize.c"
  for (j = 0 ; j < ht -> size ; j ++ ) {
#   define XfOrM1114_COUNT (0+XfOrM1106_COUNT)
#   define SETUP_XfOrM1114(x) SETUP_XfOrM1106(x)

# 7479 "./../src/optimize.c"
    if (ht -> vals [j ] ) {
#     define XfOrM1115_COUNT (0+XfOrM1114_COUNT)
#     define SETUP_XfOrM1115(x) SETUP_XfOrM1114(x)

# 7480 "./../src/optimize.c"
      tvar = ((Scheme_IR_Local * ) ht -> keys [j ] ) ; 
      if (on ) {
#       define XfOrM1117_COUNT (0+XfOrM1115_COUNT)
#       define SETUP_XfOrM1117(x) SETUP_XfOrM1115(x)

# 7482 "./../src/optimize.c"
        if (tvar -> optimize_used ) {
#         define XfOrM1118_COUNT (0+XfOrM1117_COUNT)
#         define SETUP_XfOrM1118(x) SETUP_XfOrM1117(x)

# 7484 "./../src/optimize.c"
          to_remove = FUNCCALL(SETUP_XfOrM1118(_), scheme_make_pair ((Scheme_Object * ) tvar , to_remove ) )
          ; 
        }

# 7486 "./../src/optimize.c"
        else 
        tvar -> optimize_used = 1 ; 
      }

# 7488 "./../src/optimize.c"
      else {


        tvar -> optimize_used = 0 ; 
      }
    }
  }

  while (! ((to_remove ) == (scheme_null ) ) ) {
#   define XfOrM1108_COUNT (0+XfOrM1106_COUNT)
#   define SETUP_XfOrM1108(x) SETUP_XfOrM1106(x)
    FUNCCALL(SETUP_XfOrM1108(_), scheme_hash_set (ht , (((Scheme_Simple_Object * ) (to_remove ) ) -> u . pair_val . car ) , ((void * ) 0 ) ) )
# 7497 "./../src/optimize.c"
    ; 
    to_remove = (((Scheme_Simple_Object * ) (to_remove ) ) -> u . pair_val . cdr ) ; 
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 7502 "./../src/optimize.c"
static void start_transitive_use_record (Optimize_Info * to_info , Optimize_Info * info , Scheme_IR_Local * var ) 



{
  /* No conversion */

# 7507 "./../src/optimize.c"
  if (var -> optimize_used )
    return ; 

  info -> transitive_use_var = var ; 


  if (var -> optimize . transitive_uses )
    flip_transitive (var -> optimize . transitive_uses , 1 ) ; 
}

static void end_transitive_use_record (Optimize_Info * info ) 

{
  /* No conversion */

# 7520 "./../src/optimize.c"
  Scheme_IR_Local * var = info -> transitive_use_var ; 

  if (var != info -> next -> transitive_use_var ) {
    info -> transitive_use_var = info -> next -> transitive_use_var ; 

    if (var -> optimize . transitive_uses )
      flip_transitive (var -> optimize . transitive_uses , 0 ) ; 
  }
}




static Scheme_Object * convert_leading_zero_bindings_to_begin (Scheme_IR_Let_Header * head , 
Scheme_Object * start_body , 
int c ) 
{
  Scheme_Object * body ; 
  Scheme_IR_Let_Value * irlv ; 
  Scheme_Sequence * seq ; 
  int i , n = 0 ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(5);
  BLOCK_SETUP_TOP((PUSH(seq, 0), PUSH(body, 1), PUSH(irlv, 2), PUSH(head, 3), PUSH(start_body, 4)));
# define XfOrM1122_COUNT (5)
# define SETUP_XfOrM1122(x) SETUP(XfOrM1122_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  body = NULLED_OUT ; 
  irlv = NULLED_OUT ; 
  seq = NULLED_OUT ; 

# 7542 "./../src/optimize.c"
  body = start_body ; 
  for (i = 0 ; i < c ; i ++ ) {
    irlv = (Scheme_IR_Let_Value * ) body ; 
    if (irlv -> count )
      break ; 
    n ++ ; 
    body = irlv -> body ; 
  }

  seq = FUNCCALL(SETUP_XfOrM1122(_), scheme_malloc_sequence (n + 1 ) ); 
  seq -> so . type = scheme_sequence_type ; 
  seq -> count = n + 1 ; 
  body = start_body ; 
  for (i = 0 ; i < n ; i ++ ) {
    irlv = (Scheme_IR_Let_Value * ) body ; 
    seq -> array [i ] = irlv -> value ; 
    body = irlv -> body ; 
  }

  if (n < c ) {
    head -> num_clauses -= n ; 
    head -> body = body ; 
    seq -> array [n ] = (Scheme_Object * ) head ; 
  }

# 7565 "./../src/optimize.c"
  else 
  seq -> array [n ] = body ; 

  RET_VALUE_START ((Scheme_Object * ) seq ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 7571 "./../src/optimize.c"
static Scheme_Object * optimize_lets (Scheme_Object * form , Optimize_Info * info , int context ) 

{
  Optimize_Info * body_info , * rhs_info ; 
  Optimize_Info_Sequence info_seq ; 
  Scheme_IR_Let_Header * head = (Scheme_IR_Let_Header * ) form ; 
  Scheme_IR_Let_Value * irlv , * pre_body , * retry_start , * prev_body ; 
  Scheme_Object * body , * value , * ready_pairs = ((void * ) 0 ) , * rp_last = ((void * ) 0 ) , * ready_pairs_start ; 
  Scheme_Object * escape_body = scheme_false ; 
  Scheme_Once_Used * once_used ; 
  Scheme_Hash_Tree * merge_skip_vars ; 
  int i , j , is_rec , not_simply_let_star = 0 , undiscourage , skip_opts = 0 ; 
  int did_set_value , found_escapes ; 
  int remove_last_one = 0 , inline_fuel ; 
  int pre_vclock , pre_aclock , pre_kclock , pre_sclock , increments_kclock = 0 ; 
  int once_vclock , once_aclock , once_kclock , once_sclock , once_increments_kclock = 0 ; 
  Scheme_Object * __funcarg112 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(22);
  BLOCK_SETUP_TOP((PUSH(prev_body, 0), PUSH(info, 1), PUSH(rp_last, 2), PUSH(irlv, 3), PUSH(body_info, 4), PUSH(once_used, 5), PUSH(ready_pairs_start, 6), PUSH(ready_pairs, 7), PUSH(rhs_info, 8), PUSH(body, 9), PUSH(merge_skip_vars, 10), PUSH(value, 11), PUSH(pre_body, 12), PUSH(form, 13), PUSH(escape_body, 14), PUSH(head, 15), PUSH(retry_start, 16)));
# define XfOrM1128_COUNT (17)
# define SETUP_XfOrM1128(x) SETUP(XfOrM1128_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  body_info = NULLED_OUT ; 
  rhs_info = NULLED_OUT ; 
  irlv = NULLED_OUT ; 
  pre_body = NULLED_OUT ; 
  retry_start = NULLED_OUT ; 
  prev_body = NULLED_OUT ; 
  body = NULLED_OUT ; 
  value = NULLED_OUT ; 
  ready_pairs_start = NULLED_OUT ; 
  once_used = NULLED_OUT ; 
  merge_skip_vars = NULLED_OUT ; 

# 7591 "./../src/optimize.c"
  if (! ((& head -> iso ) -> so . keyex & 0x1 ) 
  && (head -> count == 1 ) 
  && (head -> num_clauses == 1 ) ) {
#   define XfOrM1438_COUNT (0+XfOrM1128_COUNT)
#   define SETUP_XfOrM1438(x) SETUP_XfOrM1128(x)

# 7594 "./../src/optimize.c"
    irlv = (Scheme_IR_Let_Value * ) head -> body ; 
    if (((Scheme_Type ) (((((intptr_t ) (irlv -> body ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (irlv -> body ) ) -> type ) ) == (Scheme_Type ) (scheme_branch_type ) ) 
    && (irlv -> vars [0 ] -> use_count == 2 ) ) {
      Scheme_Branch_Rec * b = (Scheme_Branch_Rec * ) irlv -> body ; 
      BLOCK_SETUP((PUSH(b, 0+XfOrM1438_COUNT)));
#     define XfOrM1439_COUNT (1+XfOrM1438_COUNT)
#     define SETUP_XfOrM1439(x) SETUP(XfOrM1439_COUNT)

# 7598 "./../src/optimize.c"
      if (((b -> test ) == ((Scheme_Object * ) irlv -> vars [0 ] ) ) 
      && ((b -> tbranch ) == ((Scheme_Object * ) irlv -> vars [0 ] ) ) ) {
        Scheme_Object * pred ; 
        BLOCK_SETUP((PUSH(pred, 0+XfOrM1439_COUNT)));
#       define XfOrM1440_COUNT (1+XfOrM1439_COUNT)
#       define SETUP_XfOrM1440(x) SETUP(XfOrM1440_COUNT)
        pred = NULLED_OUT ; 

# 7602 "./../src/optimize.c"
        if (context & 0x1 )

        pred = scheme_boolean_p_proc ; 
        else 
        pred = FUNCCALL(SETUP_XfOrM1440(_), expr_implies_predicate (irlv -> value , info ) ); 

        if (pred && (predicate_implies (pred , scheme_boolean_p_proc ) ) ) {
          Scheme_Branch_Rec * b3 ; 
          BLOCK_SETUP((PUSH(b3, 0+XfOrM1440_COUNT)));
#         define XfOrM1441_COUNT (1+XfOrM1440_COUNT)
#         define SETUP_XfOrM1441(x) SETUP(XfOrM1441_COUNT)
          b3 = NULLED_OUT ; 

# 7611 "./../src/optimize.c"
          b3 = ((Scheme_Branch_Rec * ) FUNCCALL(SETUP_XfOrM1441(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_Branch_Rec ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
          b3 -> so . type = scheme_branch_type ; 
          b3 -> test = irlv -> value ; 
          b3 -> tbranch = scheme_true ; 
          b3 -> fbranch = b -> fbranch ; 

          form = FUNCCALL(SETUP_XfOrM1441(_), optimize_expr ((Scheme_Object * ) b3 , info , context ) ); 

          RET_VALUE_START (form ) RET_VALUE_END ; 
        }
      }
    }
  }

  is_rec = ((& head -> iso ) -> so . keyex & 0x1 ) ; 


  if (! is_rec 
  && (head -> count == 1 ) 
  && (head -> num_clauses == 1 ) ) {
#   define XfOrM1436_COUNT (0+XfOrM1128_COUNT)
#   define SETUP_XfOrM1436(x) SETUP_XfOrM1128(x)

# 7631 "./../src/optimize.c"
    irlv = (Scheme_IR_Let_Value * ) head -> body ; 
    if (((Scheme_Object * ) irlv -> vars [0 ] ) == (irlv -> body ) )  {
#     define XfOrM1437_COUNT (0+XfOrM1436_COUNT)
#     define SETUP_XfOrM1437(x) SETUP_XfOrM1436(x)

# 7633 "./../src/optimize.c"
      body = irlv -> value ; 
      body = FUNCCALL(SETUP_XfOrM1437(_), ensure_single_value_noncm (body , info ) ); 
      RET_VALUE_START (FUNCCALL(SETUP_XfOrM1437(_), optimize_expr (body , info , context ) )) RET_VALUE_END ; 
    }
  }



  if (! is_rec && info -> unsafe_mode && head -> num_clauses 
  && ! ((Scheme_IR_Let_Value * ) head -> body ) -> count ) {
#   define XfOrM1435_COUNT (0+XfOrM1128_COUNT)
#   define SETUP_XfOrM1435(x) SETUP_XfOrM1128(x)

# 7643 "./../src/optimize.c"
    body = FUNCCALL(SETUP_XfOrM1435(_), convert_leading_zero_bindings_to_begin (head , head -> body , head -> num_clauses ) ); 
    RET_VALUE_START (FUNCCALL(SETUP_XfOrM1435(_), optimize_expr (body , info , context ) )) RET_VALUE_END ; 
  }

  if (! is_rec ) {
    int try_again ; 
#   define XfOrM1418_COUNT (0+XfOrM1128_COUNT)
#   define SETUP_XfOrM1418(x) SETUP_XfOrM1128(x)

# 7649 "./../src/optimize.c"
    do {
#     define XfOrM1427_COUNT (0+XfOrM1418_COUNT)
#     define SETUP_XfOrM1427(x) SETUP_XfOrM1418(x)

# 7650 "./../src/optimize.c"
      try_again = 0 ; 


      if (head -> num_clauses ) {
#       define XfOrM1428_COUNT (0+XfOrM1427_COUNT)
#       define SETUP_XfOrM1428(x) SETUP_XfOrM1427(x)

# 7654 "./../src/optimize.c"
        irlv = (Scheme_IR_Let_Value * ) head -> body ; 
        if ((Scheme_Type ) (((((intptr_t ) (irlv -> value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (irlv -> value ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_let_header_type ) )  {
          Scheme_IR_Let_Header * lh = (Scheme_IR_Let_Header * ) irlv -> value ; 

          if (! lh -> num_clauses ) {
            irlv -> value = lh -> body ; 
            lh -> body = (Scheme_Object * ) head ; 
          }

# 7661 "./../src/optimize.c"
          else {
            body = lh -> body ; 
            for (i = lh -> num_clauses - 1 ; i -- ; ) {
              body = ((Scheme_IR_Let_Value * ) body ) -> body ; 
            }
            irlv -> value = ((Scheme_IR_Let_Value * ) body ) -> body ; 
            ((Scheme_IR_Let_Value * ) body ) -> body = (Scheme_Object * ) head ; 
          }

          head = lh ; 
          form = (Scheme_Object * ) head ; 
          is_rec = ((& head -> iso ) -> so . keyex & 0x1 ) ; 
          try_again = ! is_rec ; 
        }

# 7674 "./../src/optimize.c"
        else if ((Scheme_Type ) (((((intptr_t ) (irlv -> value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (irlv -> value ) ) -> type ) ) == (Scheme_Type ) (scheme_sequence_type ) )  {
          Scheme_Sequence * seq = (Scheme_Sequence * ) irlv -> value ; 
#         define XfOrM1429_COUNT (0+XfOrM1428_COUNT)
#         define SETUP_XfOrM1429(x) SETUP_XfOrM1428(x)

# 7677 "./../src/optimize.c"
          irlv -> value = seq -> array [seq -> count - 1 ] ; 
          seq -> array [seq -> count - 1 ] = (Scheme_Object * ) head ; 

          RET_VALUE_START (FUNCCALL(SETUP_XfOrM1429(_), optimize_expr ((Scheme_Object * ) seq , info , context ) )) RET_VALUE_END ; 
        }
      }
    }

# 7683 "./../src/optimize.c"
    while (try_again ) ; 
  }

  body_info = FUNCCALL(SETUP_XfOrM1128(_), optimize_info_add_frame (info , 0 ) ); 
  rhs_info = body_info ; 

  merge_skip_vars = (scheme_make_hash_tree (SCHEME_hashtr_eq ) ) ; 
  body = head -> body ; 
  for (i = head -> num_clauses ; i -- ; ) {
#   define XfOrM1413_COUNT (0+XfOrM1128_COUNT)
#   define SETUP_XfOrM1413(x) SETUP_XfOrM1128(x)

# 7692 "./../src/optimize.c"
    pre_body = (Scheme_IR_Let_Value * ) body ; 
    for (j = pre_body -> count ; j -- ; ) {
#     define XfOrM1416_COUNT (0+XfOrM1413_COUNT)
#     define SETUP_XfOrM1416(x) SETUP_XfOrM1413(x)

# 7694 "./../src/optimize.c"
      merge_skip_vars = FUNCCALL(SETUP_XfOrM1416(_), scheme_hash_tree_set (merge_skip_vars , (Scheme_Object * ) pre_body -> vars [j ] , scheme_true ) ); 
      FUNCCALL(SETUP_XfOrM1416(_), set_optimize_mode (pre_body -> vars [j ] ) ); 
      pre_body -> vars [j ] -> optimize . lambda_depth = body_info -> lambda_depth ; 
      pre_body -> vars [j ] -> optimize_used = 0 ; 
      pre_body -> vars [j ] -> optimize_outside_binding = 0 ; 
      if (! pre_body -> vars [j ] -> mutated && is_rec ) {

        Scheme_Object * rp ; 
        BLOCK_SETUP((PUSH(rp, 0+XfOrM1416_COUNT)));
#       define XfOrM1417_COUNT (1+XfOrM1416_COUNT)
#       define SETUP_XfOrM1417(x) SETUP(XfOrM1417_COUNT)
        rp = NULLED_OUT ; 

# 7702 "./../src/optimize.c"
        pre_body -> vars [j ] -> optimize_unready = 1 ; 
        rp = FUNCCALL(SETUP_XfOrM1417(_), scheme_make_raw_pair ((Scheme_Object * ) pre_body -> vars [j ] , ((void * ) 0 ) ) ); 
        if (rp_last )
          (((Scheme_Simple_Object * ) (rp_last ) ) -> u . pair_val . cdr ) = rp ; 
        else 
        ready_pairs = rp ; 
        rp_last = rp ; 
      }
    }
    body = pre_body -> body ; 
  }

  if (1 ) {
#   define XfOrM1402_COUNT (0+XfOrM1128_COUNT)
#   define SETUP_XfOrM1402(x) SETUP_XfOrM1128(x)

# 7715 "./../src/optimize.c"
    if (is_rec && ! body_info -> letrec_not_twice ) {
#     define XfOrM1403_COUNT (0+XfOrM1402_COUNT)
#     define SETUP_XfOrM1403(x) SETUP_XfOrM1402(x)

      body = head -> body ; 
      pre_body = ((void * ) 0 ) ; 
      for (i = head -> num_clauses ; i -- ; ) {
#       define XfOrM1406_COUNT (0+XfOrM1403_COUNT)
#       define SETUP_XfOrM1406(x) SETUP_XfOrM1403(x)

# 7722 "./../src/optimize.c"
        pre_body = (Scheme_IR_Let_Value * ) body ; 

        if ((pre_body -> count == 1 ) 
        && (((Scheme_Type ) (((((intptr_t ) (pre_body -> value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pre_body -> value ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (pre_body -> value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pre_body -> value ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) 
        && ! pre_body -> vars [0 ] -> mutated ) {
          Scheme_Object * sz ; 
          BLOCK_SETUP((PUSH(sz, 0+XfOrM1406_COUNT)));
#         define XfOrM1407_COUNT (1+XfOrM1406_COUNT)
#         define SETUP_XfOrM1407(x) SETUP(XfOrM1407_COUNT)
          sz = NULLED_OUT ; 

# 7728 "./../src/optimize.c"
          sz = FUNCCALL(SETUP_XfOrM1407(_), estimate_closure_size (pre_body -> value ) ); 
          pre_body -> vars [0 ] -> optimize . known_val = sz ; 
        }

        body = pre_body -> body ; 
      }
      rhs_info -> use_psize = 1 ; 
    }
  }
  FUNCCALL(SETUP_XfOrM1128(_), optimize_info_seq_init (rhs_info , & info_seq ) )
  ; 

  prev_body = ((void * ) 0 ) ; 
  body = head -> body ; 
  pre_body = ((void * ) 0 ) ; 
  retry_start = ((void * ) 0 ) ; 
  ready_pairs_start = ((void * ) 0 ) ; 
  did_set_value = 0 ; 
  found_escapes = 0 ; 
  for (i = head -> num_clauses ; i -- ; ) {
#   define XfOrM1315_COUNT (0+XfOrM1128_COUNT)
#   define SETUP_XfOrM1315(x) SETUP_XfOrM1128(x)

# 7748 "./../src/optimize.c"
    pre_body = (Scheme_IR_Let_Value * ) body ; 

    if ((pre_body -> count == 1 ) 
    && (((Scheme_Type ) (((((intptr_t ) (pre_body -> value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pre_body -> value ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (pre_body -> value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pre_body -> value ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) 
    && ! pre_body -> vars [0 ] -> optimize_used )
      FUNCCALL(SETUP_XfOrM1315(_), start_transitive_use_record (body_info , rhs_info , pre_body -> vars [0 ] ) ); 

    if (is_rec && 1 && ! rhs_info -> letrec_not_twice 
    && (((Scheme_Type ) (((((intptr_t ) (pre_body -> value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pre_body -> value ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (pre_body -> value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pre_body -> value ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) ) {
      inline_fuel = rhs_info -> inline_fuel ; 
      if (inline_fuel > 2 )
        rhs_info -> inline_fuel = 2 ; 
      rhs_info -> letrec_not_twice ++ ; 
      undiscourage = 1 ; 
    }

# 7762 "./../src/optimize.c"
    else {
      inline_fuel = 0 ; 
      undiscourage = 0 ; 
    }

    if (! skip_opts ) {
#     define XfOrM1397_COUNT (0+XfOrM1315_COUNT)
#     define SETUP_XfOrM1397(x) SETUP_XfOrM1315(x)

# 7768 "./../src/optimize.c"
      pre_vclock = rhs_info -> vclock ; 
      pre_aclock = rhs_info -> aclock ; 
      pre_kclock = rhs_info -> kclock ; 
      pre_sclock = rhs_info -> sclock ; 
      if (! found_escapes ) {
#       define XfOrM1399_COUNT (0+XfOrM1397_COUNT)
#       define SETUP_XfOrM1399(x) SETUP_XfOrM1397(x)
        FUNCCALL(SETUP_XfOrM1399(_), optimize_info_seq_step (rhs_info , & info_seq ) )
# 7773 "./../src/optimize.c"
        ; 
        value = FUNCCALL(SETUP_XfOrM1399(_), optimize_expr (pre_body -> value , rhs_info , (((pre_body -> count == 1 ) ? 0x4 : 0 ) | (((pre_body -> count == 1 ) && ! pre_body -> vars [0 ] -> non_app_count ) ? (pre_body -> vars [0 ] -> use_count << (4 + 2 ) ) : 0 ) ) ) )






        ; 
        pre_body -> value = value ; 
        if (rhs_info -> escapes )
          found_escapes = 1 ; 
      }

# 7785 "./../src/optimize.c"
      else {
#       define XfOrM1398_COUNT (0+XfOrM1397_COUNT)
#       define SETUP_XfOrM1398(x) SETUP_XfOrM1397(x)
        FUNCCALL(SETUP_XfOrM1398(_), optimize_info_seq_step (rhs_info , & info_seq ) )
# 7786 "./../src/optimize.c"
        ; 
        value = scheme_false ; 
        pre_body -> value = value ; 
        body_info -> single_result = 1 ; 
        body_info -> preserves_marks = 1 ; 
        body_info -> escapes = 1 ; 
        body_info -> size ++ ; 
      }
      once_vclock = rhs_info -> vclock ; 
      once_aclock = rhs_info -> aclock ; 
      once_kclock = rhs_info -> kclock ; 
      once_sclock = rhs_info -> sclock ; 
      increments_kclock = (once_kclock > pre_kclock ) ; 
      once_increments_kclock = increments_kclock ; 
    }

# 7800 "./../src/optimize.c"
    else {
#     define XfOrM1393_COUNT (0+XfOrM1315_COUNT)
#     define SETUP_XfOrM1393(x) SETUP_XfOrM1315(x)

# 7801 "./../src/optimize.c"
      value = pre_body -> value ; 
      -- skip_opts ; 
      if (skip_opts ) {
#       define XfOrM1396_COUNT (0+XfOrM1393_COUNT)
#       define SETUP_XfOrM1396(x) SETUP_XfOrM1393(x)



        if (! found_escapes )
          FUNCCALL(SETUP_XfOrM1396(_), advance_clocks_for_optimized (value , & pre_vclock , & pre_aclock , & pre_kclock , & pre_sclock , rhs_info , 3 ) )


        ; 
        once_vclock = pre_vclock ; 
        once_aclock = pre_aclock ; 
        once_kclock = pre_kclock ; 
        once_sclock = pre_sclock ; 
      }

# 7818 "./../src/optimize.c"
      else {

        once_vclock = rhs_info -> vclock ; 
        once_aclock = rhs_info -> aclock ; 
        once_kclock = rhs_info -> kclock ; 
        once_sclock = rhs_info -> sclock ; 
      }
      if (increments_kclock ) {


        once_increments_kclock = 1 ; 
      }
    }

    if (undiscourage ) {
      rhs_info -> inline_fuel = inline_fuel ; 
      -- rhs_info -> letrec_not_twice ; 
    }
    FUNCCALL(SETUP_XfOrM1315(_), end_transitive_use_record (rhs_info ) )
    ; 

    if (is_rec && ! not_simply_let_star ) {
#     define XfOrM1391_COUNT (0+XfOrM1315_COUNT)
#     define SETUP_XfOrM1391(x) SETUP_XfOrM1315(x)

# 7841 "./../src/optimize.c"
      if (FUNCCALL(SETUP_XfOrM1391(_), scheme_might_invoke_call_cc (value ) )
      || FUNCCALL(SETUP_XfOrM1391(_), optimize_any_uses (body_info , pre_body , i + 1 ) ))
        not_simply_let_star = 1 ; 
    }







    if ((pre_body -> count != 1 ) 
    && ((! is_rec && found_escapes ) 
    || (FUNCCALL(SETUP_XfOrM1315(_), is_values_apply (value , pre_body -> count , rhs_info , merge_skip_vars , 1 ) )
    && ((! is_rec && FUNCCALL(SETUP_XfOrM1315(_), no_mutable_bindings (pre_body ) )) 


    || FUNCCALL(SETUP_XfOrM1315(_), scheme_omittable_expr (value , pre_body -> count , - 1 , 0 , rhs_info , info ) )) ) ) ) {
#     define XfOrM1377_COUNT (0+XfOrM1315_COUNT)
#     define SETUP_XfOrM1377(x) SETUP_XfOrM1315(x)

# 7859 "./../src/optimize.c"
      if (! pre_body -> count && ! i ) {



        remove_last_one = 1 ; 




        if (! found_escapes )
          value = scheme_false ; 
        pre_body -> value = value ; 
      }

# 7871 "./../src/optimize.c"
      else {
        Scheme_IR_Let_Value * naya ; 
        Scheme_Object * rest = pre_body -> body ; 
        int j ; 
        BLOCK_SETUP((PUSH(naya, 0+XfOrM1377_COUNT), PUSH(rest, 1+XfOrM1377_COUNT)));
#       define XfOrM1378_COUNT (2+XfOrM1377_COUNT)
#       define SETUP_XfOrM1378(x) SETUP(XfOrM1378_COUNT)
        naya = NULLED_OUT ; 

# 7876 "./../src/optimize.c"
        for (j = pre_body -> count ; j -- ; ) {
          Scheme_IR_Local * * new_vars ; 
          BLOCK_SETUP((PUSH(new_vars, 0+XfOrM1378_COUNT)));
#         define XfOrM1389_COUNT (1+XfOrM1378_COUNT)
#         define SETUP_XfOrM1389(x) SETUP(XfOrM1389_COUNT)
          new_vars = NULLED_OUT ; 

# 7878 "./../src/optimize.c"
          naya = ((Scheme_IR_Let_Value * ) FUNCCALL(SETUP_XfOrM1389(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_IR_Let_Value ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
          naya -> iso . so . type = scheme_ir_let_value_type ; 
          naya -> body = rest ; 
          naya -> count = 1 ; 
          new_vars = ((Scheme_IR_Local * * ) FUNCCALL(SETUP_XfOrM1389(_), GC_malloc (sizeof (Scheme_IR_Local * ) * (1 ) ) )) ; 
          new_vars [0 ] = pre_body -> vars [j ] ; 
          naya -> vars = new_vars ; 
          rest = (Scheme_Object * ) naya ; 
        }

        naya = (Scheme_IR_Let_Value * ) rest ; 
        if (! found_escapes ) {
#         define XfOrM1387_COUNT (0+XfOrM1378_COUNT)
#         define SETUP_XfOrM1387(x) SETUP_XfOrM1378(x)
          FUNCCALL(SETUP_XfOrM1387(_), unpack_values_application (value , naya , rhs_info , ((void * ) 0 ) ) )
# 7890 "./../src/optimize.c"
          ; 
        }

# 7891 "./../src/optimize.c"
        else {
          Scheme_IR_Let_Value * naya2 = naya ; 
          for (j = 0 ; j < pre_body -> count ; j ++ ) {
            if (! j )
              naya2 -> value = value ; 
            else 
            naya2 -> value = scheme_false ; 
            naya2 = (Scheme_IR_Let_Value * ) naya2 -> body ; 
          }

          if (! pre_body -> count && ! (((value ) ) == (scheme_false ) ) ) {




            escape_body = value ; 
          }
        }

        if (prev_body )
          prev_body -> body = (Scheme_Object * ) naya ; 
        else 
        head -> body = (Scheme_Object * ) naya ; 
        head -> num_clauses += (pre_body -> count - 1 ) ; 
        i += (pre_body -> count - 1 ) ; 
        if (pre_body -> count ) {
#         define XfOrM1381_COUNT (0+XfOrM1378_COUNT)
#         define SETUP_XfOrM1381(x) SETUP_XfOrM1378(x)
          skip_opts = pre_body -> count - 1 ; 
          pre_body = naya ; 
          body = (Scheme_Object * ) naya ; 
          value = pre_body -> value ; 

          if (skip_opts ) {
#           define XfOrM1382_COUNT (0+XfOrM1381_COUNT)
#           define SETUP_XfOrM1382(x) SETUP_XfOrM1381(x)

# 7926 "./../src/optimize.c"
            if (! found_escapes )
              FUNCCALL(SETUP_XfOrM1382(_), advance_clocks_for_optimized (value , & pre_vclock , & pre_aclock , & pre_kclock , & pre_sclock , rhs_info , 3 ) )


            ; 
            once_vclock = pre_vclock ; 
            once_aclock = pre_aclock ; 
            once_kclock = pre_kclock ; 
            once_sclock = pre_sclock ; 
          }
        }

# 7936 "./../src/optimize.c"
        else {

          i ++ ; 
          if (i > 0 ) {
            body = (Scheme_Object * ) naya ; 
            continue ; 
          }

# 7942 "./../src/optimize.c"
          else 
          break ; 
        }
      }
    }

    if ((pre_body -> count == 1 ) && ! pre_body -> vars [0 ] -> mutated ) {
      int indirect = 0 , indirect_binding = 0 ; 
#     define XfOrM1354_COUNT (0+XfOrM1315_COUNT)
#     define SETUP_XfOrM1354(x) SETUP_XfOrM1315(x)



      while (indirect < 10 ) {
        if ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_sequence_type ) )  {
          Scheme_Sequence * seq = (Scheme_Sequence * ) value ; 
          value = seq -> array [seq -> count - 1 ] ; 
          indirect ++ ; 
        }

# 7960 "./../src/optimize.c"
        else if ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_with_cont_mark_type ) )  {
          Scheme_With_Continuation_Mark * wcm = (Scheme_With_Continuation_Mark * ) value ; 
          value = wcm -> body ; 
          indirect ++ ; 
        }

# 7964 "./../src/optimize.c"
        else if ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_let_header_type ) )  {
          Scheme_IR_Let_Header * head2 = (Scheme_IR_Let_Header * ) value ; 
          int i ; 

          if (head2 -> num_clauses < 10 ) {
            value = head2 -> body ; 
            for (i = head2 -> num_clauses ; i -- ; ) {
              value = ((Scheme_IR_Let_Value * ) value ) -> body ; 
            }
          }
          indirect ++ ; 
          if (head2 -> count )
            indirect_binding = 1 ; 
        }

# 7977 "./../src/optimize.c"
        else 
        break ; 
      }

      if (indirect_binding ) {

        if (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) < _scheme_ir_values_types_ )
          value = ((void * ) 0 ) ; 
      }

      if (value && ((Scheme_Type ) (((((intptr_t ) (value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (value ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) ) {



        if (((Scheme_IR_Local * ) value ) -> mutated || ((Scheme_IR_Local * ) value ) -> optimize_unready )
          value = ((void * ) 0 ) ; 
      }

      if (value )
        value = FUNCCALL(SETUP_XfOrM1354(_), extract_specialized_proc (value , value ) ); 

      if (value && FUNCCALL(SETUP_XfOrM1354(_), ir_propagate_ok (value , body_info , (! indirect && (pre_body -> vars [0 ] -> use_count == 1 ) ) , pre_body -> vars [0 ] ) )) 


      {
        pre_body -> vars [0 ] -> optimize . known_val = value ; 
        did_set_value = 1 ; 
      }

# 8004 "./../src/optimize.c"
      else if (value && ! is_rec ) {
        int cnt , ct , involves_k_cross ; 
        Scheme_Object * pred ; 
        BLOCK_SETUP((PUSH(pred, 0+XfOrM1354_COUNT)));
#       define XfOrM1355_COUNT (1+XfOrM1354_COUNT)
#       define SETUP_XfOrM1355(x) SETUP(XfOrM1355_COUNT)
        pred = NULLED_OUT ; 

# 8008 "./../src/optimize.c"
        ct = FUNCCALL(SETUP_XfOrM1355(_), scheme_expr_produces_local_type (value , & involves_k_cross ) ); 
        if (ct ) {
          ((Scheme_IR_Local * ) pre_body -> vars [0 ] ) -> val_type = ct ; 
          if (involves_k_cross ) {






            ((Scheme_IR_Local * ) pre_body -> vars [0 ] ) -> escapes_after_k_tick = 1 ; 
          }
        }

        pred = FUNCCALL(SETUP_XfOrM1355(_), expr_implies_predicate (value , rhs_info ) ); 

        if (pred )
          FUNCCALL(SETUP_XfOrM1355(_), add_type (body_info , (Scheme_Object * ) pre_body -> vars [0 ] , pred ) ); 

        if (! indirect ) {
#         define XfOrM1356_COUNT (0+XfOrM1355_COUNT)
#         define SETUP_XfOrM1356(x) SETUP_XfOrM1355(x)

# 8028 "./../src/optimize.c"
          cnt = pre_body -> vars [0 ] -> use_count ; 
          if (cnt == 1 ) {
#           define XfOrM1357_COUNT (0+XfOrM1356_COUNT)
#           define SETUP_XfOrM1357(x) SETUP_XfOrM1356(x)
            once_used = FUNCCALL(SETUP_XfOrM1357(_), make_once_used (value , pre_body -> vars [0 ] , once_vclock , once_aclock , once_kclock , once_sclock , once_increments_kclock ) )

            ; 
            pre_body -> vars [0 ] -> optimize . known_val = (Scheme_Object * ) once_used ; 
            pre_body -> vars [0 ] -> optimize . clear_known_on_multi_use = 1 ; 
          }
        }
      }
    }

    if (! retry_start ) {
      retry_start = pre_body ; 
      ready_pairs_start = ready_pairs ; 
    }


    if (is_rec 
    && ! body_info -> letrec_not_twice 
    && ((i < 1 ) 
    || (! FUNCCALL(SETUP_XfOrM1315(_), scheme_is_ir_lambda (((Scheme_IR_Let_Value * ) pre_body -> body ) -> value , 1 , 1 ) )
    && ! FUNCCALL(SETUP_XfOrM1315(_), scheme_is_liftable (((Scheme_IR_Let_Value * ) pre_body -> body ) -> value , merge_skip_vars , 5 , 1 , 0 ) )) ) ) {
      Scheme_Object * prop_later = ((void * ) 0 ) ; 
      BLOCK_SETUP((PUSH(prop_later, 0+XfOrM1315_COUNT)));
#     define XfOrM1325_COUNT (1+XfOrM1315_COUNT)
#     define SETUP_XfOrM1325(x) SETUP(XfOrM1325_COUNT)

# 8055 "./../src/optimize.c"
      if (did_set_value ) {




        int flags ; 
        Scheme_Object * clones , * cl , * cl_first ; 
        BLOCK_SETUP((PUSH(cl_first, 0+XfOrM1325_COUNT), PUSH(clones, 1+XfOrM1325_COUNT), PUSH(cl, 2+XfOrM1325_COUNT)));
#       define XfOrM1328_COUNT (3+XfOrM1325_COUNT)
#       define SETUP_XfOrM1328(x) SETUP(XfOrM1328_COUNT)
        clones = NULLED_OUT ; 
        cl = NULLED_OUT ; 
        cl_first = NULLED_OUT ; 

# 8065 "./../src/optimize.c"
        if (! i )
          FUNCCALL(SETUP_XfOrM1328(_), set_application_types (pre_body -> body , body_info , 5 ) ); 


        for (rp_last = ready_pairs_start ; ! ((rp_last ) == (ready_pairs ) ) ; rp_last = (((Scheme_Simple_Object * ) (rp_last ) ) -> u . pair_val . cdr ) ) {
          ((Scheme_IR_Local * ) (((Scheme_Simple_Object * ) (rp_last ) ) -> u . pair_val . car ) ) -> optimize_unready = 1 ; 
        }

        clones = FUNCCALL(SETUP_XfOrM1328(_), make_clones (retry_start , pre_body , rhs_info ) ); 
        (void ) FUNCCALL(SETUP_XfOrM1328(_), set_code_flags (retry_start , pre_body , clones , 32 | 4 | 64 , 0xFFFF , 0 , 0 ) )



        ; 

        irlv = retry_start ; 
        cl = clones ; 
        while (1 ) {
#         define XfOrM1340_COUNT (0+XfOrM1328_COUNT)
#         define SETUP_XfOrM1340(x) SETUP_XfOrM1328(x)

# 8083 "./../src/optimize.c"
          value = irlv -> value ; 
          if (cl ) {
            cl_first = (((Scheme_Simple_Object * ) (cl ) ) -> u . pair_val . car ) ; 
            if (! cl_first )
              cl = (((Scheme_Simple_Object * ) (cl ) ) -> u . pair_val . cdr ) ; 
          }

# 8088 "./../src/optimize.c"
          else 
          cl_first = ((void * ) 0 ) ; 
          if (cl_first && ((value ) == ((((Scheme_Simple_Object * ) (cl_first ) ) -> u . pair_val . car ) ) ) ) {

            Scheme_Object * self_value ; 
            int sz ; 
            char use_psize ; 
            BLOCK_SETUP((PUSH(self_value, 0+XfOrM1340_COUNT)));
#           define XfOrM1346_COUNT (1+XfOrM1340_COUNT)
#           define SETUP_XfOrM1346(x) SETUP(XfOrM1346_COUNT)
            self_value = NULLED_OUT ; 

# 8096 "./../src/optimize.c"
            if ((irlv -> count == 1 ) 
            && ! irlv -> vars [0 ] -> optimize_used )
              FUNCCALL(SETUP_XfOrM1346(_), start_transitive_use_record (body_info , rhs_info , irlv -> vars [0 ] ) ); 

            cl = (((Scheme_Simple_Object * ) (cl ) ) -> u . pair_val . cdr ) ; 
            self_value = (((Scheme_Simple_Object * ) (cl_first ) ) -> u . pair_val . cdr ) ; 


            sz = FUNCCALL(SETUP_XfOrM1346(_), lambda_body_size (value , 0 ) ); 
            rhs_info -> size -= (sz + 1 ) ; 





            if (! 1 )
              rhs_info -> letrec_not_twice ++ ; 
            inline_fuel = rhs_info -> inline_fuel ; 
            rhs_info -> inline_fuel >>= 1 ; 
            use_psize = rhs_info -> use_psize ; 
            rhs_info -> use_psize = info -> use_psize ; 
            FUNCCALL(SETUP_XfOrM1346(_), optimize_info_seq_step (rhs_info , & info_seq ) )
            ; 
            value = FUNCCALL_AGAIN(optimize_expr (self_value , rhs_info , (((irlv -> count == 1 ) ? 0x4 : 0 ) | (((irlv -> count == 1 ) && ! irlv -> vars [0 ] -> non_app_count ) ? (irlv -> vars [0 ] -> use_count << (4 + 2 ) ) : 0 ) ) ) )






            ; 

            if (! 1 )
              -- rhs_info -> letrec_not_twice ; 
            rhs_info -> inline_fuel = inline_fuel ; 
            rhs_info -> use_psize = use_psize ; 

            irlv -> value = value ; 

            if (! irlv -> vars [0 ] -> mutated ) {
#             define XfOrM1347_COUNT (0+XfOrM1346_COUNT)
#             define SETUP_XfOrM1347(x) SETUP_XfOrM1346(x)

# 8136 "./../src/optimize.c"
              if (FUNCCALL(SETUP_XfOrM1347(_), ir_propagate_ok (value , rhs_info , irlv -> vars [0 ] -> use_count == 1 , irlv -> vars [0 ] ) )) {


                int new_sz ; 
#               define XfOrM1348_COUNT (0+XfOrM1347_COUNT)
#               define SETUP_XfOrM1348(x) SETUP_XfOrM1347(x)

# 8140 "./../src/optimize.c"
                if (0 || 0 )
                  new_sz = FUNCCALL(SETUP_XfOrM1348(_), lambda_body_size (value , 0 ) ); 
                else 
                new_sz = 0 ; 
                if (new_sz <= sz ) {
                  irlv -> vars [0 ] -> optimize . known_val = value ; 
                }
                else if (! 0 
                || (new_sz < 4 * sz ) )
                  prop_later = (__funcarg112 = FUNCCALL(SETUP_XfOrM1348(_), scheme_make_pair ((Scheme_Object * ) irlv -> vars [0 ] , value ) )
                , FUNCCALL_AGAIN(scheme_make_raw_pair (__funcarg112 , prop_later ) )) 
                ; 
              }
            }
            FUNCCALL(SETUP_XfOrM1346(_), end_transitive_use_record (rhs_info ) )
            ; 
          }
          if (irlv == pre_body )
            break ; 
          {


            int i ; 
            for (i = irlv -> count ; i -- ; ) {
              if (! irlv -> vars [i ] -> mutated ) {
                ((Scheme_IR_Local * ) (((Scheme_Simple_Object * ) (ready_pairs_start ) ) -> u . pair_val . car ) ) -> optimize_unready = 0 ; 
                ready_pairs_start = (((Scheme_Simple_Object * ) (ready_pairs_start ) ) -> u . pair_val . cdr ) ; 
              }
            }
          }
          irlv = (Scheme_IR_Let_Value * ) irlv -> body ; 
        }

        flags = FUNCCALL(SETUP_XfOrM1328(_), set_code_flags (retry_start , pre_body , clones , 0 , 0xFFFF , 0 , 0 ) ); 

        (void ) FUNCCALL(SETUP_XfOrM1328(_), set_code_flags (retry_start , pre_body , clones , (flags & (32 | 4 ) ) , ~ (32 | 4 | (64 | 128 ) ) , 1 , 1 ) )



        ; 
      }
      retry_start = ((void * ) 0 ) ; 
      ready_pairs_start = ((void * ) 0 ) ; 
      did_set_value = 0 ; 

      while (prop_later ) {
        value = (((Scheme_Simple_Object * ) (prop_later ) ) -> u . pair_val . car ) ; 
        ((Scheme_IR_Local * ) (((Scheme_Simple_Object * ) (value ) ) -> u . pair_val . car ) ) -> optimize . known_val = (((Scheme_Simple_Object * ) (value ) ) -> u . pair_val . cdr ) ; 
        prop_later = (((Scheme_Simple_Object * ) (prop_later ) ) -> u . pair_val . cdr ) ; 
      }
    }

    if (is_rec ) {


      int i ; 
      for (i = pre_body -> count ; i -- ; ) {
        pre_body -> vars [i ] -> optimize . init_kclock = rhs_info -> kclock ; 
        if (! pre_body -> vars [i ] -> mutated ) {
          ((Scheme_IR_Local * ) (((Scheme_Simple_Object * ) (ready_pairs ) ) -> u . pair_val . car ) ) -> optimize_unready = 0 ; 
          ready_pairs = (((Scheme_Simple_Object * ) (ready_pairs ) ) -> u . pair_val . cdr ) ; 
        }
      }
    }

    if (remove_last_one ) {
      head -> num_clauses -= 1 ; 
      body = (Scheme_Object * ) pre_body -> body ; 

      if (found_escapes && ! (((pre_body -> value ) ) == (scheme_false ) ) ) {


        escape_body = pre_body -> value ; 
      }

      if (prev_body ) {
        prev_body -> body = body ; 
        pre_body = prev_body ; 
      }

# 8218 "./../src/optimize.c"
      else {
        head -> body = body ; 
        pre_body = ((void * ) 0 ) ; 
      }
      break ; 
    }

    prev_body = pre_body ; 
    body = pre_body -> body ; 
  }

  if (! is_rec ) {

    body = head -> body ; 
    for (i = head -> num_clauses ; i -- ; ) {
      pre_body = (Scheme_IR_Let_Value * ) body ; 
      for (j = pre_body -> count ; j -- ; ) {
        pre_body -> vars [j ] -> optimize . init_kclock = body_info -> kclock ; 
      }
      body = pre_body -> body ; 
    }
  }
  FUNCCALL(SETUP_XfOrM1128(_), optimize_info_seq_done (body_info , & info_seq ) )
  ; 

  if (! found_escapes ) {
#   define XfOrM1220_COUNT (0+XfOrM1128_COUNT)
#   define SETUP_XfOrM1220(x) SETUP_XfOrM1128(x)

# 8244 "./../src/optimize.c"
    body = FUNCCALL(SETUP_XfOrM1220(_), optimize_expr (body , body_info , (context & (~ ((0x3 << 4 ) | 0x2 | 0x4 ) ) ) ) ); 
  }

# 8245 "./../src/optimize.c"
  else {
#   define XfOrM1219_COUNT (0+XfOrM1128_COUNT)
#   define SETUP_XfOrM1219(x) SETUP_XfOrM1128(x)

# 8246 "./../src/optimize.c"
    body = FUNCCALL(SETUP_XfOrM1219(_), ensure_noncm (escape_body , body_info ) ); 
    body_info -> single_result = 1 ; 
    body_info -> preserves_marks = 1 ; 
    body_info -> escapes = 1 ; 
    body_info -> size ++ ; 
  }
  if (head -> num_clauses )
    pre_body -> body = body ; 
  else 
  head -> body = body ; 


  while (1 ) {
    int changed = 0 ; 
#   define XfOrM1208_COUNT (0+XfOrM1128_COUNT)
#   define SETUP_XfOrM1208(x) SETUP_XfOrM1128(x)

# 8260 "./../src/optimize.c"
    body = head -> body ; 
    for (i = head -> num_clauses ; i -- ; ) {
#     define XfOrM1214_COUNT (0+XfOrM1208_COUNT)
#     define SETUP_XfOrM1214(x) SETUP_XfOrM1208(x)

# 8262 "./../src/optimize.c"
      pre_body = (Scheme_IR_Let_Value * ) body ; 
      for (j = pre_body -> count ; j -- ; ) {
#       define XfOrM1217_COUNT (0+XfOrM1214_COUNT)
#       define SETUP_XfOrM1217(x) SETUP_XfOrM1214(x)

# 8264 "./../src/optimize.c"
        if (pre_body -> vars [j ] -> optimize_used 
        && pre_body -> vars [j ] -> optimize . transitive_uses ) {
#         define XfOrM1218_COUNT (0+XfOrM1217_COUNT)
#         define SETUP_XfOrM1218(x) SETUP_XfOrM1217(x)
          FUNCCALL(SETUP_XfOrM1218(_), register_transitive_uses (pre_body -> vars [j ] , body_info ) )
# 8266 "./../src/optimize.c"
          ; 
          changed = 1 ; 
          pre_body -> vars [j ] -> optimize . transitive_uses = ((void * ) 0 ) ; 
        }
      }
      body = pre_body -> body ; 
    }
    if (! changed )
      break ; 
  }

  info -> single_result = body_info -> single_result ; 
  info -> preserves_marks = body_info -> preserves_marks ; 
  info -> vclock = body_info -> vclock ; 
  info -> aclock = body_info -> aclock ; 
  info -> kclock = body_info -> kclock ; 
  info -> sclock = body_info -> sclock ; 


  body = head -> body ; 
  prev_body = ((void * ) 0 ) ; 
  for (i = head -> num_clauses ; i -- ; ) {
    int used = 0 , j ; 
#   define XfOrM1176_COUNT (0+XfOrM1128_COUNT)
#   define SETUP_XfOrM1176(x) SETUP_XfOrM1128(x)

# 8290 "./../src/optimize.c"
    pre_body = (Scheme_IR_Let_Value * ) body ; 

    if (pre_body -> count == 1 ) {
#     define XfOrM1195_COUNT (0+XfOrM1176_COUNT)
#     define SETUP_XfOrM1195(x) SETUP_XfOrM1176(x)
      if (pre_body -> vars [0 ] -> optimize . known_val 
      && ((Scheme_Type ) (((((intptr_t ) (pre_body -> vars [0 ] -> optimize . known_val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pre_body -> vars [0 ] -> optimize . known_val ) ) -> type ) ) == (Scheme_Type ) (scheme_lambda_type ) ) ) {
#       define XfOrM1196_COUNT (0+XfOrM1195_COUNT)
#       define SETUP_XfOrM1196(x) SETUP_XfOrM1195(x)
        FUNCCALL(SETUP_XfOrM1196(_), check_lambda_arg_types_registered ((Scheme_Lambda * ) pre_body -> vars [0 ] -> optimize . known_val , pre_body -> vars [0 ] -> use_count ) )
# 8298 "./../src/optimize.c"
        ; 
      }
    }

    for (j = pre_body -> count ; j -- ; ) {
      if (pre_body -> vars [j ] -> optimize_used ) {
        used = 1 ; 
        break ; 
      }
    }


    ((void ) sizeof ((! (used && (pre_body -> count == 1 ) && pre_body -> vars [0 ] -> optimize . known_val && ((Scheme_Type ) (scheme_once_used_type ) == (Scheme_Type ) (((((intptr_t ) (pre_body -> vars [0 ] -> optimize . known_val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pre_body -> vars [0 ] -> optimize . known_val ) ) -> type ) ) ) && ((Scheme_Once_Used * ) pre_body -> vars [0 ] -> optimize . known_val ) -> moved ) ) ? 1 : 0 ) , __extension__ ({
#     define XfOrM1190_COUNT (0+XfOrM1176_COUNT)
#     define SETUP_XfOrM1190(x) SETUP_XfOrM1176(x)

# 8310 "./../src/optimize.c" 3
      if (! (used && (pre_body -> count == 1 ) && pre_body -> vars [0 ] -> optimize . known_val && ((Scheme_Type ) (scheme_once_used_type ) == (Scheme_Type ) (((((intptr_t ) (pre_body -> vars [0 ] -> optimize . known_val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pre_body -> vars [0 ] -> optimize . known_val ) ) -> type ) ) ) && ((Scheme_Once_Used * ) pre_body -> vars [0 ] -> optimize . known_val ) -> moved ) )

# 8310 "./../src/optimize.c" 3
      ; 

# 8310 "./../src/optimize.c" 3
      else FUNCCALL(SETUP_XfOrM1190(_), __assert_fail ("!(used && (pre_body->count == 1) && pre_body->vars[0]->optimize.known_val && ((Scheme_Type)(scheme_once_used_type) == (Scheme_Type)(((((intptr_t)(pre_body->vars[0]->optimize.known_val)) & 0x1)?(Scheme_Type)scheme_integer_type:((Scheme_Object *)(pre_body->vars[0]->optimize.known_val))->type))) && ((Scheme_Once_Used *)pre_body->vars[0]->optimize.known_val)->moved)" , "./../src/optimize.c" , 8310 , __extension__ __PRETTY_FUNCTION__ ) ); 
    }
    ) ) 

    ; 

    if (! used 
    && (FUNCCALL(SETUP_XfOrM1176(_), scheme_omittable_expr (pre_body -> value , pre_body -> count , - 1 , 0 , info , info ) )
    || ((pre_body -> count == 1 ) 
    && pre_body -> vars [0 ] -> optimize . known_val 
    && ((Scheme_Type ) (scheme_once_used_type ) == (Scheme_Type ) (((((intptr_t ) (pre_body -> vars [0 ] -> optimize . known_val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pre_body -> vars [0 ] -> optimize . known_val ) ) -> type ) ) ) 
    && ((Scheme_Once_Used * ) pre_body -> vars [0 ] -> optimize . known_val ) -> moved ) ) ) {
#     define XfOrM1186_COUNT (0+XfOrM1176_COUNT)
#     define SETUP_XfOrM1186(x) SETUP_XfOrM1176(x)

# 8323 "./../src/optimize.c"
      for (j = pre_body -> count ; j -- ; ) {
        pre_body -> vars [j ] -> mode = 0 ; 
      }
      head -> num_clauses -= 1 ; 
      head -> count -= pre_body -> count ; 
      if (prev_body )
        prev_body -> body = pre_body -> body ; 
      else 
      head -> body = pre_body -> body ; 

      {
        int sz ; 
#       define XfOrM1187_COUNT (0+XfOrM1186_COUNT)
#       define SETUP_XfOrM1187(x) SETUP_XfOrM1186(x)

# 8335 "./../src/optimize.c"
        sz = FUNCCALL(SETUP_XfOrM1187(_), expr_size (pre_body -> value ) ); 
        body_info -> size -= sz ; 
      }
    }

# 8338 "./../src/optimize.c"
    else {
#     define XfOrM1177_COUNT (0+XfOrM1176_COUNT)
#     define SETUP_XfOrM1177(x) SETUP_XfOrM1176(x)

# 8339 "./../src/optimize.c"
      if (! used && (pre_body -> count == 1 ) ) {

        Scheme_Object * v2 = pre_body -> value , * inside ; 
        BLOCK_SETUP((PUSH(v2, 0+XfOrM1177_COUNT), PUSH(inside, 1+XfOrM1177_COUNT)));
#       define XfOrM1184_COUNT (2+XfOrM1177_COUNT)
#       define SETUP_XfOrM1184(x) SETUP(XfOrM1184_COUNT)
        inside = NULLED_OUT ; 
        FUNCCALL(SETUP_XfOrM1184(_), extract_tail_inside (& v2 , & inside , 1 ) )
# 8342 "./../src/optimize.c"
        ; 
        if (FUNCCALL(SETUP_XfOrM1184(_), scheme_omittable_expr (v2 , pre_body -> count , - 1 , 0 , info , info ) )) {
#         define XfOrM1185_COUNT (0+XfOrM1184_COUNT)
#         define SETUP_XfOrM1185(x) SETUP_XfOrM1184(x)
          FUNCCALL(SETUP_XfOrM1185(_), replace_tail_inside (scheme_false , inside , pre_body -> value ) )
# 8344 "./../src/optimize.c"
          ; 
        }
      }

      for (j = pre_body -> count ; j -- ; ) {
        int ct ; 

        pre_body -> vars [j ] -> optimize_outside_binding = 1 ; 
        if (pre_body -> vars [j ] -> optimize . known_val 
        && ((Scheme_Type ) (scheme_once_used_type ) == (Scheme_Type ) (((((intptr_t ) (pre_body -> vars [j ] -> optimize . known_val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (pre_body -> vars [j ] -> optimize . known_val ) ) -> type ) ) ) ) {


          pre_body -> vars [j ] -> optimize . known_val = ((void * ) 0 ) ; 
        }

        ct = pre_body -> vars [j ] -> arg_type ; 
        if (ct ) {
          if (((ct ) == 2 ) 
          || ! pre_body -> vars [j ] -> escapes_after_k_tick )
            pre_body -> vars [j ] -> arg_type = ct ; 
        }
      }
      info -> size += 1 ; 
      prev_body = pre_body ; 
    }
    body = pre_body -> body ; 
  }
  FUNCCALL(SETUP_XfOrM1128(_), optimize_info_done (body_info , ((void * ) 0 ) ) )
  ; 
  FUNCCALL(SETUP_XfOrM1128(_), merge_types (body_info , info , merge_skip_vars ) ); 

  if (is_rec && ! not_simply_let_star ) {
#   define XfOrM1154_COUNT (0+XfOrM1128_COUNT)
#   define SETUP_XfOrM1154(x) SETUP_XfOrM1128(x)

# 8377 "./../src/optimize.c"
    (& head -> iso ) -> so . keyex -= 0x1 ; 
    is_rec = 0 ; 
    FUNCCALL(SETUP_XfOrM1154(_), optimize_uses_of_mutable_imply_early_alloc ((Scheme_IR_Let_Value * ) head -> body , head -> num_clauses ) ); 
  }


  if (! head -> num_clauses ) {
    RET_VALUE_START (body ) RET_VALUE_END ; 
  }

  if (! is_rec 
  && ((((((intptr_t ) (body ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (body ) ) -> type ) > _scheme_ir_values_types_ ) 
  || ((Scheme_Type ) (((((intptr_t ) (body ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (body ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_toplevel_type ) ) 
  || ((Scheme_Type ) (((((intptr_t ) (body ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (body ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) ) ) {



    Scheme_Object * inside ; 
    BLOCK_SETUP((PUSH(inside, 0+XfOrM1128_COUNT)));
#   define XfOrM1146_COUNT (1+XfOrM1128_COUNT)
#   define SETUP_XfOrM1146(x) SETUP(XfOrM1146_COUNT)
    inside = NULLED_OUT ; 

# 8396 "./../src/optimize.c"
    inside = (Scheme_Object * ) head ; 
    pre_body = (Scheme_IR_Let_Value * ) head -> body ; 
    for (i = head -> num_clauses - 1 ; i -- ; ) {
      inside = (Scheme_Object * ) pre_body ; 
      pre_body = (Scheme_IR_Let_Value * ) pre_body -> body ; 
    }

    if (pre_body -> count == 1 ) {
#     define XfOrM1147_COUNT (0+XfOrM1146_COUNT)
#     define SETUP_XfOrM1147(x) SETUP_XfOrM1146(x)

# 8404 "./../src/optimize.c"
      if (! (((Scheme_Object * ) pre_body -> vars [0 ] ) == (body ) ) 
      && ! found_escapes ) {
#       define XfOrM1150_COUNT (0+XfOrM1147_COUNT)
#       define SETUP_XfOrM1150(x) SETUP_XfOrM1147(x)

# 8406 "./../src/optimize.c"
        body = FUNCCALL(SETUP_XfOrM1150(_), make_discarding_sequence (pre_body -> value , body , info ) ); 
      }

# 8407 "./../src/optimize.c"
      else {
#       define XfOrM1148_COUNT (0+XfOrM1147_COUNT)
#       define SETUP_XfOrM1148(x) SETUP_XfOrM1147(x)

# 8409 "./../src/optimize.c"
        body = pre_body -> value ; 
        body = FUNCCALL(SETUP_XfOrM1148(_), ensure_single_value_noncm (body , info ) ); 
        if (found_escapes ) {
#         define XfOrM1149_COUNT (0+XfOrM1148_COUNT)
#         define SETUP_XfOrM1149(x) SETUP_XfOrM1148(x)

# 8412 "./../src/optimize.c"
          found_escapes = 0 ; 
          body = FUNCCALL(SETUP_XfOrM1149(_), ensure_noncm (body , info ) ); 
        }
      }

      if (head -> num_clauses == 1 )
        RET_VALUE_START (body ) RET_VALUE_END ; 

      (void ) FUNCCALL(SETUP_XfOrM1147(_), replace_tail_inside (body , inside , ((void * ) 0 ) ) ); 
      head -> count -- ; 
      head -> num_clauses -- ; 
    }
  }

  if (! is_rec ) {

    Scheme_Object * prev = ((void * ) 0 ) , * rhs ; 
    BLOCK_SETUP((PUSH(prev, 0+XfOrM1128_COUNT), PUSH(rhs, 1+XfOrM1128_COUNT)));
#   define XfOrM1139_COUNT (2+XfOrM1128_COUNT)
#   define SETUP_XfOrM1139(x) SETUP(XfOrM1139_COUNT)
    rhs = NULLED_OUT ; 

# 8430 "./../src/optimize.c"
    body = head -> body ; 
    for (i = head -> num_clauses ; i -- ; ) {
#     define XfOrM1143_COUNT (0+XfOrM1139_COUNT)
#     define SETUP_XfOrM1143(x) SETUP_XfOrM1139(x)

# 8432 "./../src/optimize.c"
      pre_body = (Scheme_IR_Let_Value * ) body ; 
      if ((pre_body -> count == 1 ) 
      && ! pre_body -> vars [0 ] -> optimize_used ) {
        Scheme_Sequence * seq ; 
        Scheme_Object * new_body ; 
        BLOCK_SETUP((PUSH(new_body, 0+XfOrM1143_COUNT), PUSH(seq, 1+XfOrM1143_COUNT)));
#       define XfOrM1144_COUNT (2+XfOrM1143_COUNT)
#       define SETUP_XfOrM1144(x) SETUP(XfOrM1144_COUNT)
        seq = NULLED_OUT ; 
        new_body = NULLED_OUT ; 

# 8438 "./../src/optimize.c"
        pre_body -> vars [0 ] -> mode = 0 ; 

        seq = FUNCCALL(SETUP_XfOrM1144(_), scheme_malloc_sequence (2 ) ); 
        seq -> so . type = scheme_sequence_type ; 
        seq -> count = 2 ; 

        rhs = pre_body -> value ; 
        rhs = FUNCCALL(SETUP_XfOrM1144(_), ensure_single_value_noncm (rhs , info ) ); 
        seq -> array [0 ] = rhs ; 

        head -> count -- ; 
        head -> num_clauses -- ; 
        head -> body = pre_body -> body ; 

        new_body = (Scheme_Object * ) seq ; 

        if (head -> num_clauses )
          seq -> array [1 ] = (Scheme_Object * ) head ; 
        else if (found_escapes && (((head -> body ) ) == (scheme_false ) ) ) {
#         define XfOrM1145_COUNT (0+XfOrM1144_COUNT)
#         define SETUP_XfOrM1145(x) SETUP_XfOrM1144(x)

# 8458 "./../src/optimize.c"
          new_body = FUNCCALL(SETUP_XfOrM1145(_), ensure_noncm (rhs , info ) ); 
        }

# 8459 "./../src/optimize.c"
        else 
        seq -> array [1 ] = head -> body ; 

        if (prev )
          (void ) FUNCCALL(SETUP_XfOrM1144(_), replace_tail_inside (new_body , prev , ((void * ) 0 ) ) ); 
        else 
        form = new_body ; 
        prev = new_body ; 

        body = pre_body -> body ; 
      }

# 8469 "./../src/optimize.c"
      else 
      break ; 
    }

    if (prev && ((Scheme_Type ) (((((intptr_t ) (prev ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (prev ) ) -> type ) ) == (Scheme_Type ) (scheme_sequence_type ) ) )
      form = FUNCCALL(SETUP_XfOrM1139(_), optimize_sequence (form , info , context , 0 ) ); 
  }

  if (! is_rec && info -> unsafe_mode ) {
#   define XfOrM1129_COUNT (0+XfOrM1128_COUNT)
#   define SETUP_XfOrM1129(x) SETUP_XfOrM1128(x)

# 8479 "./../src/optimize.c"
    if ((Scheme_Type ) (((((intptr_t ) (form ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (form ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_let_header_type ) )  {
      int i , c , n ; 
#     define XfOrM1130_COUNT (0+XfOrM1129_COUNT)
#     define SETUP_XfOrM1130(x) SETUP_XfOrM1129(x)

# 8481 "./../src/optimize.c"
      head = (Scheme_IR_Let_Header * ) form ; 
      c = head -> num_clauses ; 
      n = head -> count ; 
      prev_body = ((void * ) 0 ) ; 
      body = head -> body ; 
      for (i = 0 ; i < c ; i ++ ) {
#       define XfOrM1135_COUNT (0+XfOrM1130_COUNT)
#       define SETUP_XfOrM1135(x) SETUP_XfOrM1130(x)

# 8487 "./../src/optimize.c"
        if (! n ) {
#         define XfOrM1137_COUNT (0+XfOrM1135_COUNT)
#         define SETUP_XfOrM1137(x) SETUP_XfOrM1135(x)
          body = FUNCCALL(SETUP_XfOrM1137(_), convert_leading_zero_bindings_to_begin (((void * ) 0 ) , body , c - i ) ); 
          if (prev_body ) {
            prev_body -> body = body ; 
            head -> num_clauses = i ; 
          }

# 8494 "./../src/optimize.c"
          else 
          form = body ; 
          break ; 
        }

# 8497 "./../src/optimize.c"
        else {
          irlv = (Scheme_IR_Let_Value * ) body ; 
          n -= irlv -> count ; 
          prev_body = irlv ; 
          body = irlv -> body ; 
        }
      }
    }
  }

  RET_VALUE_START (form ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

static Scheme_Object * 
optimize_lambda (Scheme_Object * _lam , Optimize_Info * info , int context ) 
{
  Scheme_Lambda * lam ; 
  Scheme_Object * code , * ctx , * to_remove ; 
  Scheme_IR_Lambda_Info * cl ; 
  int i , init_vclock , init_aclock , init_kclock , init_sclock ; 
  Scheme_Hash_Table * ht ; 
  int app_count = ((context >> (4 + 2 ) ) & 7 ) ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(7);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(ht, 1), PUSH(cl, 2), PUSH(to_remove, 3), PUSH(lam, 4), PUSH(ctx, 5), PUSH(code, 6)));
# define XfOrM1442_COUNT (7)
# define SETUP_XfOrM1442(x) SETUP(XfOrM1442_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  lam = NULLED_OUT ; 
  code = NULLED_OUT ; 
  ctx = NULLED_OUT ; 
  to_remove = NULLED_OUT ; 
  cl = NULLED_OUT ; 
  ht = NULLED_OUT ; 

# 8524 "./../src/optimize.c"
  lam = (Scheme_Lambda * ) _lam ; 

  info -> single_result = 1 ; 
  info -> preserves_marks = 1 ; 

  info = FUNCCALL(SETUP_XfOrM1442(_), optimize_info_add_frame (info , 1 ) ); 

  ht = FUNCCALL_AGAIN(scheme_make_hash_table (SCHEME_hash_ptr ) ); 
  info -> uses = ht ; 

  init_vclock = info -> vclock ; 
  init_aclock = info -> aclock ; 
  init_kclock = info -> kclock ; 
  init_sclock = info -> sclock ; 

  info -> vclock += 1 ; 
  info -> kclock += 1 ; 
  info -> sclock += 1 ; 


  if (info -> context && ((Scheme_Type ) (((((intptr_t ) (info -> context ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (info -> context ) ) -> type ) ) == (Scheme_Type ) (scheme_pair_type ) ) )
    ctx = FUNCCALL(SETUP_XfOrM1442(_), scheme_make_pair ((Scheme_Object * ) lam , (((Scheme_Simple_Object * ) (info -> context ) ) -> u . pair_val . cdr ) ) )
  ; 
  else if (info -> context )
    ctx = FUNCCALL(SETUP_XfOrM1442(_), scheme_make_pair ((Scheme_Object * ) lam , info -> context ) ); 
  else 
  ctx = (Scheme_Object * ) lam ; 
  info -> context = ctx ; 

  cl = lam -> ir_info ; 
  for (i = 0 ; i < lam -> num_params ; i ++ ) {
#   define XfOrM1453_COUNT (0+XfOrM1442_COUNT)
#   define SETUP_XfOrM1453(x) SETUP_XfOrM1442(x)
    FUNCCALL(SETUP_XfOrM1453(_), set_optimize_mode (cl -> vars [i ] ) )
# 8555 "./../src/optimize.c"
    ; 
    cl -> vars [i ] -> optimize . lambda_depth = info -> lambda_depth ; 
    cl -> vars [i ] -> optimize_used = 0 ; 
    cl -> vars [i ] -> optimize . init_kclock = info -> kclock ; 
    if (app_count 
    && (app_count < 7 ) 
    && cl -> arg_types 
    && cl -> arg_types [i ] 
    && (cl -> arg_type_contributors [i ] == ((1 << app_count ) - 1 ) ) ) {
#     define XfOrM1454_COUNT (0+XfOrM1453_COUNT)
#     define SETUP_XfOrM1454(x) SETUP_XfOrM1453(x)
      FUNCCALL(SETUP_XfOrM1454(_), add_type (info , (Scheme_Object * ) cl -> vars [i ] , cl -> arg_types [i ] ) )
# 8565 "./../src/optimize.c"
      ; 
    }
  }

  code = FUNCCALL(SETUP_XfOrM1442(_), optimize_expr (lam -> body , info , 0 ) ); 
  FUNCCALL(SETUP_XfOrM1442(_), propagate_used_variables (info ) )
  ; 

  if (info -> single_result )
    (& (lam ) -> iso ) -> so . keyex |= 32 ; 
  else if ((& (lam ) -> iso ) -> so . keyex & 32 )
    (& (lam ) -> iso ) -> so . keyex -= 32 ; 

  if (info -> preserves_marks )
    (& (lam ) -> iso ) -> so . keyex |= 4 ; 
  else if ((& (lam ) -> iso ) -> so . keyex & 4 )
    (& (lam ) -> iso ) -> so . keyex -= 4 ; 

  if ((info -> single_result > 0 ) && (info -> preserves_marks > 0 ) 
  && (((& (lam ) -> iso ) -> so . keyex & (64 | 128 ) ) == 64 ) )
    (& (lam ) -> iso ) -> so . keyex -= 64 ; 

  lam -> body = code ; 

# 8598 "./../src/optimize.c"
  to_remove = scheme_null ; 
  for (i = 0 ; i < ht -> size ; i ++ ) {
#   define XfOrM1448_COUNT (0+XfOrM1442_COUNT)
#   define SETUP_XfOrM1448(x) SETUP_XfOrM1442(x)

# 8600 "./../src/optimize.c"
    if (ht -> vals [i ] ) {
      Scheme_IR_Local * var = ((Scheme_IR_Local * ) ht -> keys [i ] ) ; 
#     define XfOrM1449_COUNT (0+XfOrM1448_COUNT)
#     define SETUP_XfOrM1449(x) SETUP_XfOrM1448(x)

# 8602 "./../src/optimize.c"
      if (! var -> optimize_used ) {
#       define XfOrM1450_COUNT (0+XfOrM1449_COUNT)
#       define SETUP_XfOrM1450(x) SETUP_XfOrM1449(x)

# 8604 "./../src/optimize.c"
        to_remove = FUNCCALL(SETUP_XfOrM1450(_), scheme_make_pair ((Scheme_Object * ) var , to_remove ) ); 
      }
    }
  }
  while (((Scheme_Type ) (((((intptr_t ) (to_remove ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (to_remove ) ) -> type ) ) == (Scheme_Type ) (scheme_pair_type ) ) ) {
#   define XfOrM1444_COUNT (0+XfOrM1442_COUNT)
#   define SETUP_XfOrM1444(x) SETUP_XfOrM1442(x)
    FUNCCALL(SETUP_XfOrM1444(_), scheme_hash_set (ht , (((Scheme_Simple_Object * ) (to_remove ) ) -> u . pair_val . car ) , ((void * ) 0 ) ) )
# 8609 "./../src/optimize.c"
    ; 
    to_remove = (((Scheme_Simple_Object * ) (to_remove ) ) -> u . pair_val . cdr ) ; 
  }


  cl -> base_closure = info -> uses ; 
  if (FUNCCALL(SETUP_XfOrM1442(_), env_uses_toplevel (info ) ))
    cl -> has_tl = 1 ; 
  else 
  cl -> has_tl = 0 ; 
  cl -> body_size = info -> size ; 
  cl -> body_psize = info -> psize ; 
  cl -> has_nonleaf = info -> has_nonleaf ; 


  info -> vclock = init_vclock ; 
  info -> aclock = init_aclock ; 
  info -> kclock = init_kclock ; 
  info -> sclock = init_sclock ; 
  info -> escapes = 0 ; 

  info -> size ++ ; 

  lam -> closure_size = (cl -> base_closure -> count 
  + (cl -> has_tl ? 1 : 0 ) ) ; 
  FUNCCALL(SETUP_XfOrM1442(_), optimize_info_done (info , ((void * ) 0 ) ) )
  ; 

  RET_VALUE_START ((Scheme_Object * ) lam ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 8640 "./../src/optimize.c"
static void merge_lambda_arg_types (Scheme_Lambda * lam1 , Scheme_Lambda * lam2 ) 
{
  /* No conversion */

# 8642 "./../src/optimize.c"
  Scheme_IR_Lambda_Info * cl1 = lam1 -> ir_info ; 
  Scheme_IR_Lambda_Info * cl2 = lam2 -> ir_info ; 
  int i ; 

  if (! cl1 -> arg_types ) {
    if (cl2 -> arg_types ) {
      cl1 -> arg_types = cl2 -> arg_types ; 
      cl1 -> arg_type_contributors = cl2 -> arg_type_contributors ; 
    }
  }

# 8651 "./../src/optimize.c"
  else {
    if (cl2 -> arg_types ) {
      for (i = lam1 -> num_params ; i -- ; ) {
        if (! cl1 -> arg_type_contributors [i ] ) {
          cl1 -> arg_types [i ] = cl2 -> arg_types [i ] ; 
          cl1 -> arg_type_contributors [i ] = cl2 -> arg_type_contributors [i ] ; 
        }

# 8657 "./../src/optimize.c"
        else if (cl2 -> arg_type_contributors [i ] ) {
          if (! cl2 -> arg_types [i ] )
            cl1 -> arg_types [i ] = ((void * ) 0 ) ; 
          else if (predicate_implies (cl1 -> arg_types [i ] , cl2 -> arg_types [i ] ) )
            cl1 -> arg_types [i ] = cl2 -> arg_types [i ] ; 
          else if (! predicate_implies (cl2 -> arg_types [i ] , cl1 -> arg_types [i ] ) ) {
            cl1 -> arg_types [i ] = ((void * ) 0 ) ; 
            cl1 -> arg_type_contributors [i ] |= (1 << (7 - 1 ) ) ; 
          }
          cl1 -> arg_type_contributors [i ] |= cl2 -> arg_type_contributors [i ] ; 
        }
      }
    }

    cl2 -> arg_types = cl1 -> arg_types ; 
    cl2 -> arg_type_contributors = cl1 -> arg_type_contributors ; 
  }
}

static void check_lambda_arg_types_registered (Scheme_Lambda * lam , int app_count ) 
{
  /* No conversion */

# 8678 "./../src/optimize.c"
  if (lam -> ir_info -> arg_types ) {
    int i ; 
    for (i = lam -> num_params ; i -- ; ) {
      if (lam -> ir_info -> arg_types [i ] ) {
        if ((lam -> ir_info -> arg_type_contributors [i ] & (1 << (7 - 1 ) ) ) 
        || (lam -> ir_info -> arg_type_contributors [i ] < ((1 << app_count ) - 1 ) ) ) {


          lam -> ir_info -> arg_types [i ] = ((void * ) 0 ) ; 
        }
      }
    }
  }
}

static Scheme_IR_Local * clone_variable (Scheme_IR_Local * var ) 
{
  Scheme_IR_Local * var2 ; 
  DECL_RET_SAVE (Scheme_IR_Local * ) PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(var2, 0), PUSH(var, 1)));
# define XfOrM1476_COUNT (2)
# define SETUP_XfOrM1476(x) SETUP(XfOrM1476_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  var2 = NULLED_OUT ; 

# 8696 "./../src/optimize.c"
  ((void ) sizeof ((((Scheme_Type ) (var -> so . type ) == (Scheme_Type ) (scheme_ir_local_type ) ) ) ? 1 : 0 ) , __extension__ ({
#   define XfOrM1477_COUNT (0+XfOrM1476_COUNT)
#   define SETUP_XfOrM1477(x) SETUP_XfOrM1476(x)

# 8696 "./../src/optimize.c" 3
    if ((Scheme_Type ) (var -> so . type ) == (Scheme_Type ) (scheme_ir_local_type ) ) 

# 8696 "./../src/optimize.c" 3
    ; 

# 8696 "./../src/optimize.c" 3
    else FUNCCALL(SETUP_XfOrM1477(_), __assert_fail ("((Scheme_Type)(var->so.type) == (Scheme_Type)(scheme_ir_local_type))" , "./../src/optimize.c" , 8696 , __extension__ __PRETTY_FUNCTION__ ) ); 
  }
  ) ) 
# 8696 "./../src/optimize.c"
  ; 
  var2 = ((Scheme_IR_Local * ) FUNCCALL(SETUP_XfOrM1476(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_IR_Local ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
  (memcpy (var2 , var , sizeof (Scheme_IR_Local ) ) ) ; 
  (scheme_set_distinct_eq_hash ((Scheme_Object * ) var2 ) ) ; 
  RET_VALUE_START (var2 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 8703 "./../src/optimize.c"
static Scheme_IR_Local * * clone_variable_array (Scheme_IR_Local * * vars , 
int sz , 
Scheme_Hash_Tree * * _var_map ) 
{
  Scheme_IR_Local * * new_vars , * var ; 
  Scheme_Hash_Tree * var_map = * _var_map ; 
  int j ; 
  DECL_RET_SAVE (Scheme_IR_Local * * ) PREPARE_VAR_STACK_ONCE(5);
  BLOCK_SETUP_TOP((PUSH(new_vars, 0), PUSH(var_map, 1), PUSH(var, 2), PUSH(_var_map, 3), PUSH(vars, 4)));
# define XfOrM1478_COUNT (5)
# define SETUP_XfOrM1478(x) SETUP(XfOrM1478_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  new_vars = NULLED_OUT ; 
  var = NULLED_OUT ; 

# 8711 "./../src/optimize.c"
  new_vars = ((Scheme_IR_Local * * ) FUNCCALL(SETUP_XfOrM1478(_), GC_malloc (sizeof (Scheme_IR_Local * ) * (sz ) ) )) ; 
  for (j = sz ; j -- ; ) {
#   define XfOrM1480_COUNT (0+XfOrM1478_COUNT)
#   define SETUP_XfOrM1480(x) SETUP_XfOrM1478(x)

# 8713 "./../src/optimize.c"
    var = FUNCCALL(SETUP_XfOrM1480(_), clone_variable (vars [j ] ) ); 
    var -> mode = 0 ; 
    new_vars [j ] = var ; 
    var_map = FUNCCALL(SETUP_XfOrM1480(_), scheme_hash_tree_set (var_map , (Scheme_Object * ) vars [j ] , (Scheme_Object * ) new_vars [j ] ) ); 
  }

  * _var_map = var_map ; 
  RET_VALUE_START (new_vars ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 8723 "./../src/optimize.c"
static Scheme_Object * clone_lambda (int single_use , Scheme_Object * _lam , Optimize_Info * info , Scheme_Hash_Tree * var_map ) 
{
  Scheme_Lambda * lam , * lam2 ; 
  Scheme_Object * body , * var ; 
  Scheme_Hash_Table * ht ; 
  Scheme_IR_Lambda_Info * cl ; 
  Scheme_IR_Local * * vars ; 
  int sz ; 
  Scheme_Object * * arg_types ; 
  short * arg_type_contributors ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(11);
  BLOCK_SETUP_TOP((PUSH(body, 0), PUSH(info, 1), PUSH(var_map, 2), PUSH(ht, 3), PUSH(vars, 4), PUSH(lam, 5), PUSH(arg_type_contributors, 6), PUSH(lam2, 7), PUSH(var, 8), PUSH(arg_types, 9), PUSH(cl, 10)));
# define XfOrM1481_COUNT (11)
# define SETUP_XfOrM1481(x) SETUP(XfOrM1481_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  lam = NULLED_OUT ; 
  lam2 = NULLED_OUT ; 
  body = NULLED_OUT ; 
  var = NULLED_OUT ; 
  ht = NULLED_OUT ; 
  cl = NULLED_OUT ; 
  vars = NULLED_OUT ; 
  arg_types = NULLED_OUT ; 
  arg_type_contributors = NULLED_OUT ; 

# 8734 "./../src/optimize.c"
  lam = (Scheme_Lambda * ) _lam ; 

  lam2 = ((Scheme_Lambda * ) FUNCCALL(SETUP_XfOrM1481(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_Lambda ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
  (memcpy (lam2 , lam , sizeof (Scheme_Lambda ) ) ) ; 

  cl = ((Scheme_IR_Lambda_Info * ) FUNCCALL(SETUP_XfOrM1481(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_IR_Lambda_Info ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
  (memcpy (cl , lam -> ir_info , sizeof (Scheme_IR_Lambda_Info ) ) ) ; 
  lam2 -> ir_info = cl ; 

  vars = FUNCCALL(SETUP_XfOrM1481(_), clone_variable_array (cl -> vars , lam2 -> num_params , & var_map ) ); 
  cl -> vars = vars ; 

  cl -> is_dup |= ! single_use ; 

  body = FUNCCALL(SETUP_XfOrM1481(_), optimize_clone (single_use , lam -> body , info , var_map , 0 ) ); 
  if (! body )

# 8749 "./../src/optimize.c"
  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

  lam2 -> body = body ; 

  if (cl -> arg_types ) {
#   define XfOrM1487_COUNT (0+XfOrM1481_COUNT)
#   define SETUP_XfOrM1487(x) SETUP_XfOrM1481(x)

# 8754 "./../src/optimize.c"
    sz = lam2 -> num_params ; 
    arg_types = ((Scheme_Object * * ) FUNCCALL(SETUP_XfOrM1487(_), GC_malloc (sizeof (Scheme_Object * ) * (sz ) ) )) ; 
    arg_type_contributors = ((short * ) FUNCCALL(SETUP_XfOrM1487(_), GC_malloc_atomic (sizeof (short ) * (sz ) ) )) ; 
    (memcpy (arg_types , cl -> arg_types , sz * sizeof (Scheme_Object * ) ) ) ; 
    (memcpy (arg_type_contributors , cl -> arg_type_contributors , sz * sizeof (short ) ) ) ; 
    cl -> arg_types = arg_types ; 
    cl -> arg_type_contributors = arg_type_contributors ; 
  }

  if (cl -> base_closure && var_map -> count ) {
    int i ; 
#   define XfOrM1482_COUNT (0+XfOrM1481_COUNT)
#   define SETUP_XfOrM1482(x) SETUP_XfOrM1481(x)

# 8765 "./../src/optimize.c"
    ht = FUNCCALL(SETUP_XfOrM1482(_), scheme_make_hash_table (SCHEME_hash_ptr ) ); 
    for (i = 0 ; i < cl -> base_closure -> size ; i ++ ) {
#     define XfOrM1485_COUNT (0+XfOrM1482_COUNT)
#     define SETUP_XfOrM1485(x) SETUP_XfOrM1482(x)

# 8767 "./../src/optimize.c"
      if (cl -> base_closure -> vals [i ] ) {
#       define XfOrM1486_COUNT (0+XfOrM1485_COUNT)
#       define SETUP_XfOrM1486(x) SETUP_XfOrM1485(x)

# 8768 "./../src/optimize.c"
        var = (scheme_eq_hash_tree_get (var_map , cl -> base_closure -> keys [i ] ) ) ; 
        FUNCCALL(SETUP_XfOrM1486(_), scheme_hash_set (ht , (var ? var : cl -> base_closure -> keys [i ] ) , cl -> base_closure -> vals [i ] ) )



        ; 
      }
    }
    cl -> base_closure = ht ; 
  }

  RET_VALUE_START ((Scheme_Object * ) lam2 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 8782 "./../src/optimize.c"
static int lambda_body_size_plus_info (Scheme_Lambda * lam , int check_assign , 
Optimize_Info * info , int * is_leaf ) 
{
  /* No conversion */

# 8785 "./../src/optimize.c"
  int i ; 
  Scheme_IR_Lambda_Info * cl ; 

  cl = lam -> ir_info ; 

  if (check_assign ) {

    for (i = lam -> num_params ; i -- ; ) {
      if (cl -> vars [i ] -> mutated )
        return - 1 ; 
    }
  }

  if (is_leaf )
    * is_leaf = ! cl -> has_nonleaf ; 

  return cl -> body_size + ((info && info -> use_psize ) ? cl -> body_psize : 0 ) ; 
}

static int lambda_has_top_level (Scheme_Lambda * lam ) 
{
  /* No conversion */

# 8806 "./../src/optimize.c"
  return lam -> ir_info -> has_tl ; 
}





static int set_code_closure_flags (Scheme_Object * clones , 
int set_flags , int mask_flags , 
int just_tentative ) 
{
  Scheme_Object * clone , * orig , * first ; 
  int flags = 32 | 4 ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(4);
  BLOCK_SETUP_TOP((PUSH(clones, 0), PUSH(clone, 1), PUSH(orig, 2), PUSH(first, 3)));
# define XfOrM1493_COUNT (4)
# define SETUP_XfOrM1493(x) SETUP(XfOrM1493_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  clone = NULLED_OUT ; 
  orig = NULLED_OUT ; 
  first = NULLED_OUT ; 

# 8825 "./../src/optimize.c"
  while (clones ) {
#   define XfOrM1495_COUNT (0+XfOrM1493_COUNT)
#   define SETUP_XfOrM1495(x) SETUP_XfOrM1493(x)

# 8826 "./../src/optimize.c"
    first = (((Scheme_Simple_Object * ) (clones ) ) -> u . pair_val . car ) ; 
    clone = (((Scheme_Simple_Object * ) (first ) ) -> u . pair_val . car ) ; 
    orig = (((Scheme_Simple_Object * ) (first ) ) -> u . pair_val . cdr ) ; 

    flags = FUNCCALL(SETUP_XfOrM1495(_), set_one_code_flags (orig , flags , orig , clone , set_flags , mask_flags , just_tentative , 0 ) )


    ; 

    clones = (((Scheme_Simple_Object * ) (clones ) ) -> u . pair_val . cdr ) ; 
  }

  RET_VALUE_START (flags ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 8841 "./../src/optimize.c"
static Scheme_Object * is_cross_linklet_inline_candidiate (Scheme_Object * e , Optimize_Info * info , 
int size_override ) 
{
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(e, 1)));
# define XfOrM1496_COUNT (2)
# define SETUP_XfOrM1496(x) SETUP(XfOrM1496_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 8844 "./../src/optimize.c"
  if (((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) )  {
#   define XfOrM1497_COUNT (0+XfOrM1496_COUNT)
#   define SETUP_XfOrM1497(x) SETUP_XfOrM1496(x)

# 8845 "./../src/optimize.c"
    if (size_override || (FUNCCALL(SETUP_XfOrM1497(_), lambda_body_size (e , 1 ) )< 8 ) )
      RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_clone (0 , e , info , empty_eq_hash_tree , 0 ) )) RET_VALUE_EMPTY_END ; 
  }

  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 8852 "./../src/optimize.c"
static int is_general_lambda (Scheme_Object * e , Optimize_Info * info ) 
{
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK(3);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(e, 1)));
# define XfOrM1498_COUNT (2)
# define SETUP_XfOrM1498(x) SETUP(XfOrM1498_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 8855 "./../src/optimize.c"
  if (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) == scheme_sequence_type ) {
    Scheme_Sequence * seq = (Scheme_Sequence * ) e ; 
    BLOCK_SETUP((PUSH(seq, 0+XfOrM1498_COUNT)));
#   define XfOrM1501_COUNT (1+XfOrM1498_COUNT)
#   define SETUP_XfOrM1501(x) SETUP(XfOrM1501_COUNT)

# 8857 "./../src/optimize.c"
    if (seq -> count > 0 ) {
      int i ; 
#     define XfOrM1502_COUNT (0+XfOrM1501_COUNT)
#     define SETUP_XfOrM1502(x) SETUP_XfOrM1501(x)

# 8859 "./../src/optimize.c"
      for (i = seq -> count - 1 ; i -- ; ) {
#       define XfOrM1504_COUNT (0+XfOrM1502_COUNT)
#       define SETUP_XfOrM1504(x) SETUP_XfOrM1502(x)

# 8860 "./../src/optimize.c"
        if (! FUNCCALL(SETUP_XfOrM1504(_), scheme_omittable_expr (seq -> array [i ] , - 1 , 20 , 0 , info , ((void * ) 0 ) ) ))
          RET_VALUE_START (0 ) RET_VALUE_END ; 
      }
    }
    e = seq -> array [seq -> count - 1 ] ; 
  }


  if (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) == scheme_ir_let_header_type ) {
    Scheme_IR_Let_Header * lh = (Scheme_IR_Let_Header * ) e ; 
    if (! ((& lh -> iso ) -> so . keyex & 0x1 ) 
    && (lh -> count == 1 ) 
    && (lh -> num_clauses == 1 ) 
    && ((Scheme_Type ) (((((intptr_t ) (lh -> body ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (lh -> body ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_let_value_type ) ) ) {
      Scheme_IR_Let_Value * lv = (Scheme_IR_Let_Value * ) lh -> body ; 
      if (((Scheme_Type ) (((((intptr_t ) (lv -> value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (lv -> value ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (lv -> value ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (lv -> value ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) 
        RET_VALUE_START (((lv -> body ) == ((Scheme_Object * ) lv -> vars [0 ] ) ) ) RET_VALUE_END ; 
    }
  }

  if (((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) 
    RET_VALUE_START (1 ) RET_VALUE_END ; 

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 8886 "./../src/optimize.c"
void install_definition (Scheme_Object * bodies , int pos , Scheme_Object * old_defn , int name_pos , Scheme_Object * rhs ) 
{
  Scheme_Object * def ; 
  PREPARE_VAR_STACK_ONCE(4);
  BLOCK_SETUP_TOP((PUSH(old_defn, 0), PUSH(rhs, 1), PUSH(bodies, 2), PUSH(def, 3)));
# define XfOrM1505_COUNT (4)
# define SETUP_XfOrM1505(x) SETUP(XfOrM1505_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  def = NULLED_OUT ; 

# 8890 "./../src/optimize.c"
  def = FUNCCALL(SETUP_XfOrM1505(_), scheme_make_vector (2 , ((void * ) 0 ) ) ); 
  ((((Scheme_Vector * ) (def ) ) -> els ) [0 ] ) = rhs ; 
  ((((Scheme_Vector * ) (def ) ) -> els ) [(0 ) + 1 ] ) = ((((Scheme_Vector * ) (old_defn ) ) -> els ) [(name_pos ) + 1 ] ) ; 
  def -> type = scheme_define_values_type ; 

  (((Scheme_Vector * ) (bodies ) ) -> els ) [pos ] = def ; 
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 8898 "./../src/optimize.c"
int split_define_values (Scheme_Object * defn , int n , Scheme_Object * bodies , int offset ) 
{
  Scheme_Object * e = ((((Scheme_Vector * ) (defn ) ) -> els ) [0 ] ) ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK(6);
  BLOCK_SETUP_TOP((PUSH(e, 0), PUSH(defn, 1), PUSH(bodies, 2)));
# define XfOrM1506_COUNT (3)
# define SETUP_XfOrM1506(x) SETUP(XfOrM1506_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 8902 "./../src/optimize.c"
  if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_let_header_type ) )  {




    Scheme_IR_Let_Header * lh = (Scheme_IR_Let_Header * ) e ; 
    BLOCK_SETUP((PUSH(lh, 0+XfOrM1506_COUNT)));
#   define XfOrM1517_COUNT (1+XfOrM1506_COUNT)
#   define SETUP_XfOrM1517(x) SETUP(XfOrM1517_COUNT)

# 8908 "./../src/optimize.c"
    if ((lh -> count == n ) && (lh -> num_clauses == n ) 
    && ! ((& lh -> iso ) -> so . keyex & 0x1 ) ) {
      Scheme_Object * body = lh -> body ; 
      int i ; 
      BLOCK_SETUP((PUSH(body, 0+XfOrM1517_COUNT)));
#     define XfOrM1518_COUNT (1+XfOrM1517_COUNT)
#     define SETUP_XfOrM1518(x) SETUP(XfOrM1518_COUNT)

# 8912 "./../src/optimize.c"
      for (i = 0 ; i < n ; i ++ ) {
#       define XfOrM1532_COUNT (0+XfOrM1518_COUNT)
#       define SETUP_XfOrM1532(x) SETUP_XfOrM1518(x)

# 8913 "./../src/optimize.c"
        if ((Scheme_Type ) (((((intptr_t ) (body ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (body ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_let_value_type ) )  {
          Scheme_IR_Let_Value * lv = (Scheme_IR_Let_Value * ) body ; 
          BLOCK_SETUP((PUSH(lv, 0+XfOrM1532_COUNT)));
#         define XfOrM1533_COUNT (1+XfOrM1532_COUNT)
#         define SETUP_XfOrM1533(x) SETUP(XfOrM1533_COUNT)

# 8915 "./../src/optimize.c"
          if (lv -> count == 1 ) {
#           define XfOrM1534_COUNT (0+XfOrM1533_COUNT)
#           define SETUP_XfOrM1534(x) SETUP_XfOrM1533(x)

# 8916 "./../src/optimize.c"
            if (! FUNCCALL(SETUP_XfOrM1534(_), scheme_omittable_expr (lv -> value , 1 , 5 , 0 , ((void * ) 0 ) , ((void * ) 0 ) ) ))
              RET_VALUE_START (0 ) RET_VALUE_END ; 
            body = lv -> body ; 
          }

# 8919 "./../src/optimize.c"
          else 
          RET_VALUE_START (0 ) RET_VALUE_END ; 
        }

# 8921 "./../src/optimize.c"
        else 
        RET_VALUE_START (0 ) RET_VALUE_END ; 
      }
      if ((n == 2 ) && ((Scheme_Type ) (((((intptr_t ) (body ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (body ) ) -> type ) ) == (Scheme_Type ) (scheme_application3_type ) ) ) {
        Scheme_App3_Rec * app = (Scheme_App3_Rec * ) body ; 
        Scheme_IR_Let_Value * lv = (Scheme_IR_Let_Value * ) lh -> body ; 
        BLOCK_SETUP((PUSH(lv, 0+XfOrM1518_COUNT)));
#       define XfOrM1526_COUNT (1+XfOrM1518_COUNT)
#       define SETUP_XfOrM1526(x) SETUP(XfOrM1526_COUNT)

# 8927 "./../src/optimize.c"
        if (((app -> rator ) == (scheme_values_proc ) ) 
        && ((app -> rand1 ) == ((Scheme_Object * ) lv -> vars [0 ] ) ) 
        && ((app -> rand2 ) == ((Scheme_Object * ) ((Scheme_IR_Let_Value * ) lv -> body ) -> vars [0 ] ) ) ) {
#         define XfOrM1527_COUNT (0+XfOrM1526_COUNT)
#         define SETUP_XfOrM1527(x) SETUP_XfOrM1526(x)

# 8930 "./../src/optimize.c"
          if (bodies ) {
#           define XfOrM1528_COUNT (0+XfOrM1527_COUNT)
#           define SETUP_XfOrM1528(x) SETUP_XfOrM1527(x)
            FUNCCALL(SETUP_XfOrM1528(_), install_definition (bodies , offset , defn , 0 , lv -> value ) )
# 8931 "./../src/optimize.c"
            ; 
            lv = (Scheme_IR_Let_Value * ) lv -> body ; 
            FUNCCALL(SETUP_XfOrM1528(_), install_definition (bodies , offset + 1 , defn , 1 , lv -> value ) ); 
          }
          RET_VALUE_START (1 ) RET_VALUE_END ; 
        }
      }

# 8937 "./../src/optimize.c"
      else if (((Scheme_Type ) (((((intptr_t ) (body ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (body ) ) -> type ) ) == (Scheme_Type ) (scheme_application_type ) ) 
      && ((Scheme_App_Rec * ) body ) -> num_args == n ) {
        Scheme_App_Rec * app = (Scheme_App_Rec * ) body ; 
        Scheme_IR_Let_Value * lv = (Scheme_IR_Let_Value * ) lh -> body ; 
#       define XfOrM1519_COUNT (0+XfOrM1518_COUNT)
#       define SETUP_XfOrM1519(x) SETUP_XfOrM1518(x)

# 8941 "./../src/optimize.c"
        if ((app -> args [0 ] ) == (scheme_values_proc ) )  {
#         define XfOrM1520_COUNT (0+XfOrM1519_COUNT)
#         define SETUP_XfOrM1520(x) SETUP_XfOrM1519(x)

# 8942 "./../src/optimize.c"
          for (i = 0 ; i < n ; i ++ ) {
            if (! ((Scheme_Type ) (((((intptr_t ) (app -> args [i + 1 ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (app -> args [i + 1 ] ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) 
            || ! (((Scheme_Object * ) lv -> vars [0 ] ) == (app -> args [i + 1 ] ) ) )
              RET_VALUE_START (0 ) RET_VALUE_END ; 
            lv = (Scheme_IR_Let_Value * ) lv -> body ; 
          }
          if (bodies ) {
#           define XfOrM1521_COUNT (0+XfOrM1520_COUNT)
#           define SETUP_XfOrM1521(x) SETUP_XfOrM1520(x)

# 8949 "./../src/optimize.c"
            body = lh -> body ; 
            for (i = 0 ; i < n ; i ++ ) {
              Scheme_IR_Let_Value * lv2 = (Scheme_IR_Let_Value * ) body ; 
              BLOCK_SETUP((PUSH(lv2, 0+XfOrM1521_COUNT)));
#             define XfOrM1523_COUNT (1+XfOrM1521_COUNT)
#             define SETUP_XfOrM1523(x) SETUP(XfOrM1523_COUNT)
              FUNCCALL(SETUP_XfOrM1523(_), install_definition (bodies , offset + i , defn , i , lv2 -> value ) )
# 8952 "./../src/optimize.c"
              ; 
              body = lv2 -> body ; 
            }
          }
          RET_VALUE_START (1 ) RET_VALUE_END ; 
        }
      }
    }
  }

# 8960 "./../src/optimize.c"
  else if ((n == 2 ) && ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_application3_type ) ) ) {
    Scheme_App3_Rec * app = (Scheme_App3_Rec * ) e ; 
    BLOCK_SETUP((PUSH(app, 0+XfOrM1506_COUNT)));
#   define XfOrM1514_COUNT (1+XfOrM1506_COUNT)
#   define SETUP_XfOrM1514(x) SETUP(XfOrM1514_COUNT)

# 8962 "./../src/optimize.c"
    if (((app -> rator ) == (scheme_values_proc ) ) 
    && FUNCCALL(SETUP_XfOrM1514(_), scheme_omittable_expr (app -> rand1 , 1 , 5 , 0 , ((void * ) 0 ) , ((void * ) 0 ) ) )
    && FUNCCALL(SETUP_XfOrM1514(_), scheme_omittable_expr (app -> rand2 , 1 , 5 , 0 , ((void * ) 0 ) , ((void * ) 0 ) ) )) {
#     define XfOrM1515_COUNT (0+XfOrM1514_COUNT)
#     define SETUP_XfOrM1515(x) SETUP_XfOrM1514(x)

# 8965 "./../src/optimize.c"
      if (bodies ) {
#       define XfOrM1516_COUNT (0+XfOrM1515_COUNT)
#       define SETUP_XfOrM1516(x) SETUP_XfOrM1515(x)
        FUNCCALL(SETUP_XfOrM1516(_), install_definition (bodies , offset , defn , 0 , app -> rand1 ) )
# 8966 "./../src/optimize.c"
        ; 
        FUNCCALL(SETUP_XfOrM1516(_), install_definition (bodies , offset + 1 , defn , 1 , app -> rand2 ) ); 
      }
      RET_VALUE_START (1 ) RET_VALUE_END ; 
    }
  }

# 8971 "./../src/optimize.c"
  else if (((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_application_type ) ) 
  && ((Scheme_App_Rec * ) e ) -> num_args == n ) {
    Scheme_App_Rec * app = (Scheme_App_Rec * ) e ; 
    BLOCK_SETUP((PUSH(app, 0+XfOrM1506_COUNT)));
#   define XfOrM1507_COUNT (1+XfOrM1506_COUNT)
#   define SETUP_XfOrM1507(x) SETUP(XfOrM1507_COUNT)

# 8974 "./../src/optimize.c"
    if ((app -> args [0 ] ) == (scheme_values_proc ) )  {
      int i ; 
#     define XfOrM1508_COUNT (0+XfOrM1507_COUNT)
#     define SETUP_XfOrM1508(x) SETUP_XfOrM1507(x)

# 8976 "./../src/optimize.c"
      for (i = 0 ; i < n ; i ++ ) {
#       define XfOrM1513_COUNT (0+XfOrM1508_COUNT)
#       define SETUP_XfOrM1513(x) SETUP_XfOrM1508(x)

# 8977 "./../src/optimize.c"
        if (! FUNCCALL(SETUP_XfOrM1513(_), scheme_omittable_expr (app -> args [i + 1 ] , 1 , 5 , 0 , ((void * ) 0 ) , ((void * ) 0 ) ) ))
          RET_VALUE_START (0 ) RET_VALUE_END ; 
      }
      if (bodies ) {
#       define XfOrM1509_COUNT (0+XfOrM1508_COUNT)
#       define SETUP_XfOrM1509(x) SETUP_XfOrM1508(x)

# 8981 "./../src/optimize.c"
        for (i = 0 ; i < n ; i ++ ) {
#         define XfOrM1511_COUNT (0+XfOrM1509_COUNT)
#         define SETUP_XfOrM1511(x) SETUP_XfOrM1509(x)
          FUNCCALL(SETUP_XfOrM1511(_), install_definition (bodies , offset + i , defn , i , app -> args [i + 1 ] ) )
# 8982 "./../src/optimize.c"
          ; 
        }
      }
      RET_VALUE_START (1 ) RET_VALUE_END ; 
    }
  }

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 8992 "./../src/optimize.c"
static Scheme_Hash_Table * set_as_fixed (Scheme_Hash_Table * fixed_table , Optimize_Info * info , int pos ) 
{
  DECL_RET_SAVE (Scheme_Hash_Table * ) PREPARE_VAR_STACK(3);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(fixed_table, 1)));
# define XfOrM1535_COUNT (2)
# define SETUP_XfOrM1535(x) SETUP(XfOrM1535_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 8994 "./../src/optimize.c"
  if (! fixed_table ) {
#   define XfOrM1536_COUNT (0+XfOrM1535_COUNT)
#   define SETUP_XfOrM1536(x) SETUP_XfOrM1535(x)

# 8995 "./../src/optimize.c"
    fixed_table = FUNCCALL(SETUP_XfOrM1536(_), scheme_make_hash_table (SCHEME_hash_ptr ) ); 
    if (! info -> top_level_consts ) {
      Scheme_Hash_Table * consts ; 
      BLOCK_SETUP((PUSH(consts, 0+XfOrM1536_COUNT)));
#     define XfOrM1537_COUNT (1+XfOrM1536_COUNT)
#     define SETUP_XfOrM1537(x) SETUP(XfOrM1537_COUNT)
      consts = NULLED_OUT ; 

# 8998 "./../src/optimize.c"
      consts = FUNCCALL(SETUP_XfOrM1537(_), scheme_make_hash_table (SCHEME_hash_ptr ) ); 
      info -> top_level_consts = consts ; 
    }
    FUNCCALL(SETUP_XfOrM1536(_), scheme_hash_set (info -> top_level_consts , scheme_false , (Scheme_Object * ) fixed_table ) ); 
  }
  FUNCCALL(SETUP_XfOrM1535(_), scheme_hash_set (fixed_table , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (pos ) ) ) << 1 ) | 0x1 ) ) , scheme_true ) )
  ; 

  RET_VALUE_START (fixed_table ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 9009 "./../src/optimize.c"
Scheme_Linklet * scheme_optimize_linklet (Scheme_Linklet * linklet , 
int enforce_const , int can_inline , int unsafe_mode , 
Scheme_Object * * _import_keys , Scheme_Object * get_import ) 
{
  Scheme_Object * e ; 
  int start_simultaneous = 0 , i_m , cnt ; 
  Scheme_Object * cl_first = ((void * ) 0 ) , * cl_last = ((void * ) 0 ) ; 
  Scheme_Hash_Table * consts = ((void * ) 0 ) , * fixed_table = ((void * ) 0 ) , * re_consts = ((void * ) 0 ) ; 
  Scheme_Hash_Table * originals = ((void * ) 0 ) ; 
  int cont , inline_fuel , is_proc_def , any_defns = 0 ; 
  Optimize_Info * info ; 
  Optimize_Info * limited_info ; 
  Optimize_Info_Sequence info_seq ; 
  Scheme_Hash_Tree * * iu ; 


  int support_validation = ! unsafe_mode ; 

# 2490 "./../src/optimize.c"
  Scheme_Object * __funcarg130 = NULLED_OUT ; 

# 1874 "./../src/optimize.c"
  intptr_t __funcarg128 ; 

# 1945 "./../src/optimize.c"
  intptr_t __funcarg127 ; 
  Scheme_Object * __funcarg126 = NULLED_OUT ; 
  Scheme_Object * __funcarg122 = NULLED_OUT ; 
  DECL_RET_SAVE (Scheme_Linklet * ) PREPARE_VAR_STACK(19);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(_import_keys, 1), PUSH(e, 2), PUSH(cl_last, 3), PUSH(limited_info, 4), PUSH(iu, 5), PUSH(cl_first, 6), PUSH(fixed_table, 7), PUSH(linklet, 8), PUSH(originals, 9), PUSH(get_import, 10), PUSH(re_consts, 11), PUSH(consts, 12)));
# define XfOrM1538_COUNT (13)
# define SETUP_XfOrM1538(x) SETUP(XfOrM1538_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  e = NULLED_OUT ; 
  info = NULLED_OUT ; 
  limited_info = NULLED_OUT ; 
  iu = NULLED_OUT ; 

# 9027 "./../src/optimize.c"
  info = FUNCCALL(SETUP_XfOrM1538(_), optimize_info_create (linklet , enforce_const , can_inline , unsafe_mode ) ); 
  info -> context = (Scheme_Object * ) linklet ; 


  if ((((Scheme_Vector * ) (linklet -> bodies ) ) -> size ) > 128 )
    info -> inline_fuel >>= 1 ; 

  if (_import_keys ) {
    Cross_Linklet_Info * cross ; 
    Scheme_Hash_Tree * ht ; 
    int i ; 
    BLOCK_SETUP((PUSH(cross, 0+XfOrM1538_COUNT), PUSH(ht, 1+XfOrM1538_COUNT)));
#   define XfOrM1750_COUNT (2+XfOrM1538_COUNT)
#   define SETUP_XfOrM1750(x) SETUP(XfOrM1750_COUNT)
    cross = NULLED_OUT ; 
    ht = NULLED_OUT ; 

# 9039 "./../src/optimize.c"
    iu = ((Scheme_Hash_Tree * * ) FUNCCALL(SETUP_XfOrM1750(_), GC_malloc (sizeof (Scheme_Hash_Tree * ) * (1 ) ) )) ; 
    * iu = empty_eq_hash_tree ; 
    info -> imports_used = iu ; 

    cross = (Cross_Linklet_Info * ) FUNCCALL(SETUP_XfOrM1750(_), GC_malloc (sizeof (Cross_Linklet_Info ) ) ); 
    info -> cross = cross ; 

    cross -> get_import = get_import ; 

    cross -> import_keys = empty_eq_hash_tree ; 
    cross -> rev_import_keys = empty_eq_hash_tree ; 
    for (i = 0 ; i < (((Scheme_Vector * ) (* _import_keys ) ) -> size ) ; i ++ ) {
#     define XfOrM1752_COUNT (0+XfOrM1750_COUNT)
#     define SETUP_XfOrM1752(x) SETUP_XfOrM1750(x)

# 9051 "./../src/optimize.c"
      ht = FUNCCALL(SETUP_XfOrM1752(_), scheme_hash_tree_set (cross -> import_keys , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (i ) ) ) << 1 ) | 0x1 ) ) , (((Scheme_Vector * ) (* _import_keys ) ) -> els ) [i ] ) )

      ; 
      cross -> import_keys = ht ; 
      ht = FUNCCALL(SETUP_XfOrM1752(_), scheme_hash_tree_set (cross -> rev_import_keys , (((Scheme_Vector * ) (* _import_keys ) ) -> els ) [i ] , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (i ) ) ) << 1 ) | 0x1 ) ) ) )

      ; 
      cross -> rev_import_keys = ht ; 
    }
    cross -> linklets = empty_eq_hash_tree ; 
    cross -> import_next_keys = empty_eq_hash_tree ; 
    cross -> inline_variants = empty_eq_hash_tree ; 
    cross -> import_syms = empty_eq_hash_tree ; 
  }
  FUNCCALL(SETUP_XfOrM1538(_), optimize_info_seq_init (info , & info_seq ) )
  ; 

  cnt = (((Scheme_Vector * ) (linklet -> bodies ) ) -> size ) ; 



  {
    int inc = 0 ; 
#   define XfOrM1734_COUNT (0+XfOrM1538_COUNT)
#   define SETUP_XfOrM1734(x) SETUP_XfOrM1538(x)

# 9074 "./../src/optimize.c"
    for (i_m = 0 ; i_m < cnt ; i_m ++ ) {
#     define XfOrM1747_COUNT (0+XfOrM1734_COUNT)
#     define SETUP_XfOrM1747(x) SETUP_XfOrM1734(x)

# 9075 "./../src/optimize.c"
      e = (((Scheme_Vector * ) (linklet -> bodies ) ) -> els ) [i_m ] ; 
      if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_define_values_type ) )  {
        int n ; 
#       define XfOrM1748_COUNT (0+XfOrM1747_COUNT)
#       define SETUP_XfOrM1748(x) SETUP_XfOrM1747(x)

# 9078 "./../src/optimize.c"
        n = ((((Scheme_Vector * ) (e ) ) -> size ) - 1 ) ; 
        if (n > 1 ) {
#         define XfOrM1749_COUNT (0+XfOrM1748_COUNT)
#         define SETUP_XfOrM1749(x) SETUP_XfOrM1748(x)

# 9080 "./../src/optimize.c"
          if (FUNCCALL(SETUP_XfOrM1749(_), split_define_values (e , n , ((void * ) 0 ) , 0 ) ))
            inc += (n - 1 ) ; 
        }
        any_defns = 1 ; 
      }
    }

    if (inc > 0 ) {
      Scheme_Object * new_bodies ; 
      int j = 0 ; 
      BLOCK_SETUP((PUSH(new_bodies, 0+XfOrM1734_COUNT)));
#     define XfOrM1735_COUNT (1+XfOrM1734_COUNT)
#     define SETUP_XfOrM1735(x) SETUP(XfOrM1735_COUNT)
      new_bodies = NULLED_OUT ; 

# 9090 "./../src/optimize.c"
      new_bodies = FUNCCALL(SETUP_XfOrM1735(_), scheme_make_vector (cnt + inc , scheme_false ) ); 
      for (i_m = 0 ; i_m < cnt ; i_m ++ ) {
#       define XfOrM1740_COUNT (0+XfOrM1735_COUNT)
#       define SETUP_XfOrM1740(x) SETUP_XfOrM1735(x)

# 9092 "./../src/optimize.c"
        e = (((Scheme_Vector * ) (linklet -> bodies ) ) -> els ) [i_m ] ; 
        if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_define_values_type ) )  {
          int n ; 
#         define XfOrM1741_COUNT (0+XfOrM1740_COUNT)
#         define SETUP_XfOrM1741(x) SETUP_XfOrM1740(x)

# 9095 "./../src/optimize.c"
          n = ((((Scheme_Vector * ) (e ) ) -> size ) - 1 ) ; 
          if (n > 1 ) {
#           define XfOrM1742_COUNT (0+XfOrM1741_COUNT)
#           define SETUP_XfOrM1742(x) SETUP_XfOrM1741(x)

# 9097 "./../src/optimize.c"
            if (FUNCCALL(SETUP_XfOrM1742(_), split_define_values (e , n , new_bodies , j ) )) {
              j += n ; 
            }

# 9099 "./../src/optimize.c"
            else 
            (((Scheme_Vector * ) (new_bodies ) ) -> els ) [j ++ ] = e ; 
          }

# 9101 "./../src/optimize.c"
          else 
          (((Scheme_Vector * ) (new_bodies ) ) -> els ) [j ++ ] = e ; 
        }

# 9103 "./../src/optimize.c"
        else 
        (((Scheme_Vector * ) (new_bodies ) ) -> els ) [j ++ ] = e ; 
      }
      cnt += inc ; 
      linklet -> bodies = new_bodies ; 
    }
  }

  if (any_defns ) {
#   define XfOrM1733_COUNT (0+XfOrM1538_COUNT)
#   define SETUP_XfOrM1733(x) SETUP_XfOrM1538(x)


    limited_info = ((Optimize_Info * ) FUNCCALL(SETUP_XfOrM1733(_), GC_malloc_one_small_tagged ((((sizeof (Optimize_Info ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 

    limited_info -> type = scheme_rt_optimize_info ; 

    limited_info -> linklet = info -> linklet ; 
  }

# 9121 "./../src/optimize.c"
  else 
  limited_info = ((void * ) 0 ) ; 

  if (1 && any_defns ) {
#   define XfOrM1722_COUNT (0+XfOrM1538_COUNT)
#   define SETUP_XfOrM1722(x) SETUP_XfOrM1538(x)

# 9125 "./../src/optimize.c"
    if (info -> enforce_const ) {
#     define XfOrM1723_COUNT (0+XfOrM1722_COUNT)
#     define SETUP_XfOrM1723(x) SETUP_XfOrM1722(x)

      for (i_m = 0 ; i_m < cnt ; i_m ++ ) {
#       define XfOrM1729_COUNT (0+XfOrM1723_COUNT)
#       define SETUP_XfOrM1729(x) SETUP_XfOrM1723(x)

# 9130 "./../src/optimize.c"
        e = (((Scheme_Vector * ) (linklet -> bodies ) ) -> els ) [i_m ] ; 
        if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_define_values_type ) )  {
          int n ; 
#         define XfOrM1730_COUNT (0+XfOrM1729_COUNT)
#         define SETUP_XfOrM1730(x) SETUP_XfOrM1729(x)

# 9134 "./../src/optimize.c"
          n = ((((Scheme_Vector * ) (e ) ) -> size ) - 1 ) ; 
          if ((n == 1 ) && (((Scheme_Type ) (((((intptr_t ) (((((Scheme_Vector * ) (e ) ) -> els ) [0 ] ) ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (((((Scheme_Vector * ) (e ) ) -> els ) [0 ] ) ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (((((Scheme_Vector * ) (e ) ) -> els ) [0 ] ) ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (((((Scheme_Vector * ) (e ) ) -> els ) [0 ] ) ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) ) {
            Scheme_IR_Toplevel * var = ((Scheme_IR_Toplevel * ) ((((Scheme_Vector * ) (e ) ) -> els ) [(0 ) + 1 ] ) ) ; 
            BLOCK_SETUP((PUSH(var, 0+XfOrM1730_COUNT)));
#           define XfOrM1731_COUNT (1+XfOrM1730_COUNT)
#           define SETUP_XfOrM1731(x) SETUP(XfOrM1731_COUNT)

# 9138 "./../src/optimize.c"
            if (! ((& (var ) -> iso ) -> so . keyex & 0x4 ) ) {
#             define XfOrM1732_COUNT (0+XfOrM1731_COUNT)
#             define SETUP_XfOrM1732(x) SETUP_XfOrM1731(x)

# 9139 "./../src/optimize.c"
              if (! consts )
                consts = FUNCCALL(SETUP_XfOrM1732(_), scheme_make_hash_table (SCHEME_hash_ptr ) ); 
              (__funcarg130 = FUNCCALL(SETUP_XfOrM1732(_), estimate_closure_size (e ) ), FUNCCALL_AGAIN(scheme_hash_set (consts , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (var -> variable_pos ) ) ) << 1 ) | 0x1 ) ) , __funcarg130 ) )) ; 
            }
          }
        }
      }

      if (consts ) {
        info -> top_level_consts = consts ; 
        consts = ((void * ) 0 ) ; 
      }
    }
  }

  for (i_m = 0 ; i_m < cnt ; i_m ++ ) {
#   define XfOrM1641_COUNT (0+XfOrM1538_COUNT)
#   define SETUP_XfOrM1641(x) SETUP_XfOrM1538(x)

# 9156 "./../src/optimize.c"
    e = (((Scheme_Vector * ) (linklet -> bodies ) ) -> els ) [i_m ] ; 

    is_proc_def = 0 ; 
    if (1 && info -> enforce_const ) {
#     define XfOrM1720_COUNT (0+XfOrM1641_COUNT)
#     define SETUP_XfOrM1720(x) SETUP_XfOrM1641(x)

# 9160 "./../src/optimize.c"
      if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_define_values_type ) )  {
        Scheme_Object * e2 ; 
#       define XfOrM1721_COUNT (0+XfOrM1720_COUNT)
#       define SETUP_XfOrM1721(x) SETUP_XfOrM1720(x)
        e2 = NULLED_OUT ; 

# 9162 "./../src/optimize.c"
        e2 = ((((Scheme_Vector * ) (e ) ) -> els ) [0 ] ) ; 
        if (FUNCCALL(SETUP_XfOrM1721(_), is_general_lambda (e2 , info ) ))
          is_proc_def = 1 ; 
      }
    }

    inline_fuel = info -> inline_fuel ; 
    if (is_proc_def && 1 ) {
      info -> use_psize = 1 ; 
      if (inline_fuel > 2 )
        info -> inline_fuel = 2 ; 
    }
    FUNCCALL(SETUP_XfOrM1641(_), optimize_info_seq_step (info , & info_seq ) ); 
    e = FUNCCALL(SETUP_XfOrM1641(_), optimize_expr (e , info , 0 ) ); 
    if (is_proc_def && 1 ) {
      info -> use_psize = 0 ; 
    }
    info -> inline_fuel = inline_fuel ; 
    (((Scheme_Vector * ) (linklet -> bodies ) ) -> els ) [i_m ] = e ; 

    if (info -> enforce_const ) {
#     define XfOrM1671_COUNT (0+XfOrM1641_COUNT)
#     define SETUP_XfOrM1671(x) SETUP_XfOrM1641(x)

      if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_define_values_type ) )  {
        int n , cnst = 0 , sproc = 0 , sprop = 0 , has_guard = 0 ; 
        Scheme_Object * sstruct = ((void * ) 0 ) , * parent_identity = ((void * ) 0 ) ; 
        Simple_Struct_Type_Info stinfo ; 
        Scheme_Object * defn = e ; 
        BLOCK_SETUP((PUSH(sstruct, 0+XfOrM1671_COUNT), PUSH(parent_identity, 1+XfOrM1671_COUNT), PUSH(defn, 2+XfOrM1671_COUNT)));
#       define XfOrM1673_COUNT (3+XfOrM1671_COUNT)
#       define SETUP_XfOrM1673(x) SETUP(XfOrM1673_COUNT)

# 9192 "./../src/optimize.c"
        n = ((((Scheme_Vector * ) (defn ) ) -> size ) - 1 ) ; 
        e = ((((Scheme_Vector * ) (defn ) ) -> els ) [0 ] ) ; 

        if (support_validation )
          limited_info -> cross = info -> cross ; 
        cont = FUNCCALL(SETUP_XfOrM1673(_), scheme_omittable_expr (e , n , - 1 , (support_validation ? (0x8 | 0x10 ) : 0 ) , (support_validation ? limited_info : info ) , info ) )
# 9213 "./../src/optimize.c"
        ; 
        if (support_validation )
          info -> cross = limited_info -> cross ; 

        if (n == 1 ) {
#         define XfOrM1716_COUNT (0+XfOrM1673_COUNT)
#         define SETUP_XfOrM1716(x) SETUP_XfOrM1673(x)

# 9218 "./../src/optimize.c"
          if (FUNCCALL(SETUP_XfOrM1716(_), ir_propagate_ok (e , info , 0 , ((void * ) 0 ) ) ))
            cnst = 1 ; 
          else if (FUNCCALL(SETUP_XfOrM1716(_), scheme_is_statically_proc (e , info , 0x8 ) )) {
            cnst = 1 ; 
            sproc = 1 ; 
          }
        }

# 9224 "./../src/optimize.c"
        else if (FUNCCALL(SETUP_XfOrM1673(_), scheme_is_simple_make_struct_type (e , n , 0 , ((void * ) 0 ) , & stinfo , & parent_identity , info , ((void * ) 0 ) , ((void * ) 0 ) , 0 , ((void * ) 0 ) , & sstruct , 5 ) )) 




        {
#         define XfOrM1715_COUNT (0+XfOrM1673_COUNT)
#         define SETUP_XfOrM1715(x) SETUP_XfOrM1673(x)

# 9230 "./../src/optimize.c"
          sstruct = FUNCCALL(SETUP_XfOrM1715(_), scheme_make_pair (sstruct , parent_identity ) ); 
          cnst = 1 ; 
        }

# 9232 "./../src/optimize.c"
        else if (FUNCCALL(SETUP_XfOrM1673(_), scheme_is_simple_make_struct_type_property (e , n , 0 , & has_guard , info , ((void * ) 0 ) , ((void * ) 0 ) , 0 , ((void * ) 0 ) , 5 ) )) 



        {
          sprop = 1 ; 
          cnst = 1 ; 
        }

# 9239 "./../src/optimize.c"
        else 
        sstruct = ((void * ) 0 ) ; 

        if (support_validation && (sstruct || sprop ) && ! cont ) {
#         define XfOrM1713_COUNT (0+XfOrM1673_COUNT)
#         define SETUP_XfOrM1713(x) SETUP_XfOrM1673(x)

          cont = FUNCCALL(SETUP_XfOrM1713(_), scheme_omittable_expr (e , n , 5 , 0x8 , limited_info , ((void * ) 0 ) ) ); 
        }

        if (cont ) {

          (((Scheme_Inclhash_Object * ) (defn ) ) -> so . keyex |= 0x2 ) ; 
        }

        if (cnst ) {
          Scheme_IR_Toplevel * var ; 
          int i ; 
          BLOCK_SETUP((PUSH(var, 0+XfOrM1673_COUNT)));
#         define XfOrM1681_COUNT (1+XfOrM1673_COUNT)
#         define SETUP_XfOrM1681(x) SETUP(XfOrM1681_COUNT)
          var = NULLED_OUT ; 

# 9257 "./../src/optimize.c"
          for (i = 0 ; i < n ; i ++ ) {
#           define XfOrM1697_COUNT (0+XfOrM1681_COUNT)
#           define SETUP_XfOrM1697(x) SETUP_XfOrM1681(x)

# 9258 "./../src/optimize.c"
            var = ((Scheme_IR_Toplevel * ) ((((Scheme_Vector * ) (defn ) ) -> els ) [(i ) + 1 ] ) ) ; 

            if (! ((& (var ) -> iso ) -> so . keyex & 0x4 ) ) {
              Scheme_Object * e2 ; 
              BLOCK_SETUP((PUSH(e2, 0+XfOrM1697_COUNT)));
#             define XfOrM1698_COUNT (1+XfOrM1697_COUNT)
#             define SETUP_XfOrM1698(x) SETUP(XfOrM1698_COUNT)
              e2 = NULLED_OUT ; 

# 9263 "./../src/optimize.c"
              if (sstruct ) {
#               define XfOrM1711_COUNT (0+XfOrM1698_COUNT)
#               define SETUP_XfOrM1711(x) SETUP_XfOrM1698(x)

# 9264 "./../src/optimize.c"
                e2 = (__funcarg128 = FUNCCALL(SETUP_XfOrM1711(_), scheme_get_struct_proc_shape (i , & stinfo ) ), FUNCCALL_AGAIN(scheme_make_struct_proc_shape (__funcarg128 , sstruct ) )) 
                ; 
              }

# 9266 "./../src/optimize.c"
              else if (sprop ) {
#               define XfOrM1710_COUNT (0+XfOrM1698_COUNT)
#               define SETUP_XfOrM1710(x) SETUP_XfOrM1698(x)

# 9267 "./../src/optimize.c"
                e2 = (__funcarg127 = FUNCCALL(SETUP_XfOrM1710(_), scheme_get_struct_property_proc_shape (i , has_guard ) ), FUNCCALL_AGAIN(scheme_make_struct_property_proc_shape (__funcarg127 ) )) ; 
              }

# 9268 "./../src/optimize.c"
              else if (sproc ) {
#               define XfOrM1709_COUNT (0+XfOrM1698_COUNT)
#               define SETUP_XfOrM1709(x) SETUP_XfOrM1698(x)

# 9269 "./../src/optimize.c"
                e2 = FUNCCALL(SETUP_XfOrM1709(_), scheme_make_noninline_proc (e ) ); 
              }

# 9270 "./../src/optimize.c"
              else if (((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) )  {
#               define XfOrM1707_COUNT (0+XfOrM1698_COUNT)
#               define SETUP_XfOrM1707(x) SETUP_XfOrM1698(x)

# 9271 "./../src/optimize.c"
                e2 = FUNCCALL(SETUP_XfOrM1707(_), optimize_clone (1 , e , info , empty_eq_hash_tree , 0 ) ); 
                if (e2 ) {
                  Scheme_Object * pr ; 
                  BLOCK_SETUP((PUSH(pr, 0+XfOrM1707_COUNT)));
#                 define XfOrM1708_COUNT (1+XfOrM1707_COUNT)
#                 define SETUP_XfOrM1708(x) SETUP(XfOrM1708_COUNT)
                  pr = NULLED_OUT ; 

# 9274 "./../src/optimize.c"
                  pr = (__funcarg126 = FUNCCALL(SETUP_XfOrM1708(_), scheme_make_raw_pair (e2 , e ) ), FUNCCALL_AGAIN(scheme_make_raw_pair (__funcarg126 , ((void * ) 0 ) ) )) ; 
                  if (cl_last )
                    (((Scheme_Simple_Object * ) (cl_last ) ) -> u . pair_val . cdr ) = pr ; 
                  else 
                  cl_first = pr ; 
                  cl_last = pr ; 
                }

# 9280 "./../src/optimize.c"
                else 
                e2 = FUNCCALL(SETUP_XfOrM1707(_), scheme_make_noninline_proc (e ) ); 
              }

# 9282 "./../src/optimize.c"
              else {
                e2 = e ; 
              }

              if (e2 ) {
#               define XfOrM1700_COUNT (0+XfOrM1698_COUNT)
#               define SETUP_XfOrM1700(x) SETUP_XfOrM1698(x)

# 9287 "./../src/optimize.c"
                consts = info -> top_level_consts ; 
                if (! consts ) {
#                 define XfOrM1705_COUNT (0+XfOrM1700_COUNT)
#                 define SETUP_XfOrM1705(x) SETUP_XfOrM1700(x)

# 9289 "./../src/optimize.c"
                  consts = FUNCCALL(SETUP_XfOrM1705(_), scheme_make_hash_table (SCHEME_hash_ptr ) ); 
                  info -> top_level_consts = consts ; 
                }
                FUNCCALL(SETUP_XfOrM1700(_), scheme_hash_set (consts , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (var -> variable_pos ) ) ) << 1 ) | 0x1 ) ) , e2 ) ); 

                if (sstruct || sprop ) {

                  Scheme_Hash_Table * limited_consts = limited_info -> top_level_consts ; 
                  BLOCK_SETUP((PUSH(limited_consts, 0+XfOrM1700_COUNT)));
#                 define XfOrM1703_COUNT (1+XfOrM1700_COUNT)
#                 define SETUP_XfOrM1703(x) SETUP(XfOrM1703_COUNT)

# 9297 "./../src/optimize.c"
                  if (! limited_consts ) {
#                   define XfOrM1704_COUNT (0+XfOrM1703_COUNT)
#                   define SETUP_XfOrM1704(x) SETUP_XfOrM1703(x)

# 9298 "./../src/optimize.c"
                    limited_consts = FUNCCALL(SETUP_XfOrM1704(_), scheme_make_hash_table (SCHEME_hash_ptr ) ); 
                    limited_info -> top_level_consts = limited_consts ; 
                  }
                  FUNCCALL(SETUP_XfOrM1703(_), scheme_hash_set (limited_consts , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (var -> variable_pos ) ) ) << 1 ) | 0x1 ) ) , e2 ) ); 
                }

                if (sstruct || (((((intptr_t ) (e2 ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e2 ) ) -> type ) > _scheme_ir_values_types_ ) ) {
                }
                else {
#                 define XfOrM1701_COUNT (0+XfOrM1700_COUNT)
#                 define SETUP_XfOrM1701(x) SETUP_XfOrM1700(x)

# 9307 "./../src/optimize.c"
                  if (! re_consts )
                    re_consts = FUNCCALL(SETUP_XfOrM1701(_), scheme_make_hash_table (SCHEME_hash_ptr ) ); 
                  FUNCCALL(SETUP_XfOrM1701(_), scheme_hash_set (re_consts , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (i_m ) ) ) << 1 ) | 0x1 ) ) , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (var -> variable_pos ) ) ) << 1 ) | 0x1 ) ) ) ); 
                }
              }

# 9311 "./../src/optimize.c"
              else {
#               define XfOrM1699_COUNT (0+XfOrM1698_COUNT)
#               define SETUP_XfOrM1699(x) SETUP_XfOrM1698(x)

# 9313 "./../src/optimize.c"
                fixed_table = FUNCCALL(SETUP_XfOrM1699(_), set_as_fixed (fixed_table , info , (((Scheme_IR_Toplevel * ) var ) -> variable_pos ) ) ); 
              }
            }
          }
        }

# 9317 "./../src/optimize.c"
        else if (cont ) {




          int i , n = ((((Scheme_Vector * ) (defn ) ) -> size ) - 1 ) ; 
          Scheme_IR_Toplevel * var ; 
          BLOCK_SETUP((PUSH(var, 0+XfOrM1673_COUNT)));
#         define XfOrM1674_COUNT (1+XfOrM1673_COUNT)
#         define SETUP_XfOrM1674(x) SETUP(XfOrM1674_COUNT)
          var = NULLED_OUT ; 

# 9325 "./../src/optimize.c"
          for (i = 0 ; i < n ; i ++ ) {
#           define XfOrM1678_COUNT (0+XfOrM1674_COUNT)
#           define SETUP_XfOrM1678(x) SETUP_XfOrM1674(x)

# 9326 "./../src/optimize.c"
            var = ((Scheme_IR_Toplevel * ) ((((Scheme_Vector * ) (defn ) ) -> els ) [(i ) + 1 ] ) ) ; 


            if (! ((& (var ) -> iso ) -> so . keyex & 0x4 ) ) {
#             define XfOrM1679_COUNT (0+XfOrM1678_COUNT)
#             define SETUP_XfOrM1679(x) SETUP_XfOrM1678(x)

# 9330 "./../src/optimize.c"
              if (! info -> top_level_consts 
              || ! FUNCCALL(SETUP_XfOrM1679(_), scheme_hash_get (info -> top_level_consts , (Scheme_Object * ) var ) )) {
#               define XfOrM1680_COUNT (0+XfOrM1679_COUNT)
#               define SETUP_XfOrM1680(x) SETUP_XfOrM1679(x)

# 9332 "./../src/optimize.c"
                fixed_table = FUNCCALL(SETUP_XfOrM1680(_), set_as_fixed (fixed_table , info , var -> variable_pos ) ); 
              }
            }
          }
        }
      }

# 9337 "./../src/optimize.c"
      else {
#       define XfOrM1672_COUNT (0+XfOrM1671_COUNT)
#       define SETUP_XfOrM1672(x) SETUP_XfOrM1671(x)

# 9338 "./../src/optimize.c"
        if (i_m + 1 == cnt )
          cont = 0 ; 
        else 
        cont = FUNCCALL(SETUP_XfOrM1672(_), scheme_omittable_expr (e , - 1 , - 1 , 0 , ((void * ) 0 ) , ((void * ) 0 ) ) ); 
      }
    }

# 9343 "./../src/optimize.c"
    else {
      cont = 1 ; 
    }

    if (! cont || (i_m + 1 == cnt ) ) {
      Scheme_Object * prop_later = ((void * ) 0 ) ; 
      BLOCK_SETUP((PUSH(prop_later, 0+XfOrM1641_COUNT)));
#     define XfOrM1650_COUNT (1+XfOrM1641_COUNT)
#     define SETUP_XfOrM1650(x) SETUP(XfOrM1650_COUNT)

# 9350 "./../src/optimize.c"
      if (consts ) {
        int flags ; 
#       define XfOrM1653_COUNT (0+XfOrM1650_COUNT)
#       define SETUP_XfOrM1653(x) SETUP_XfOrM1650(x)


        (void ) FUNCCALL(SETUP_XfOrM1653(_), set_code_closure_flags (cl_first , 32 | 4 | 64 , 0xFFFF , 0 ) )


        ; 

        while (1 ) {

          int old_sz , new_sz , orig_fuel ; 
#         define XfOrM1662_COUNT (0+XfOrM1653_COUNT)
#         define SETUP_XfOrM1662(x) SETUP_XfOrM1653(x)

# 9365 "./../src/optimize.c"
          e = (((Scheme_Vector * ) (linklet -> bodies ) ) -> els ) [start_simultaneous ] ; 

          if (0 || 0 ) {
#           define XfOrM1668_COUNT (0+XfOrM1662_COUNT)
#           define SETUP_XfOrM1668(x) SETUP_XfOrM1662(x)

# 9368 "./../src/optimize.c"
            if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_define_values_type ) )  {
              Scheme_Object * sub_e ; 
#             define XfOrM1669_COUNT (0+XfOrM1668_COUNT)
#             define SETUP_XfOrM1669(x) SETUP_XfOrM1668(x)
              sub_e = NULLED_OUT ; 

# 9370 "./../src/optimize.c"
              sub_e = ((((Scheme_Vector * ) (e ) ) -> els ) [0 ] ) ; 
              old_sz = FUNCCALL(SETUP_XfOrM1669(_), lambda_body_size (sub_e , 0 ) ); 
            }

# 9372 "./../src/optimize.c"
            else 
            old_sz = 0 ; 
          }

# 9374 "./../src/optimize.c"
          else 
          old_sz = 0 ; 
          FUNCCALL(SETUP_XfOrM1662(_), optimize_info_seq_step (info , & info_seq ) )
          ; 
          orig_fuel = info -> inline_fuel ; 
          e = FUNCCALL(SETUP_XfOrM1662(_), optimize_expr (e , info , 0 ) ); 
          info -> inline_fuel = orig_fuel ; 
          (((Scheme_Vector * ) (linklet -> bodies ) ) -> els ) [start_simultaneous ] = e ; 

          if (re_consts ) {


            Scheme_Object * rpos ; 
            BLOCK_SETUP((PUSH(rpos, 0+XfOrM1662_COUNT)));
#           define XfOrM1663_COUNT (1+XfOrM1662_COUNT)
#           define SETUP_XfOrM1663(x) SETUP(XfOrM1663_COUNT)
            rpos = NULLED_OUT ; 

# 9387 "./../src/optimize.c"
            rpos = FUNCCALL(SETUP_XfOrM1663(_), scheme_hash_get (re_consts , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (start_simultaneous ) ) ) << 1 ) | 0x1 ) ) ) ); 
            if (rpos ) {
              Scheme_Object * old_e ; 
              BLOCK_SETUP((PUSH(old_e, 0+XfOrM1663_COUNT)));
#             define XfOrM1664_COUNT (1+XfOrM1663_COUNT)
#             define SETUP_XfOrM1664(x) SETUP(XfOrM1664_COUNT)
              old_e = NULLED_OUT ; 

# 9391 "./../src/optimize.c"
              e = ((((Scheme_Vector * ) (e ) ) -> els ) [0 ] ) ; 

              old_e = FUNCCALL(SETUP_XfOrM1664(_), scheme_hash_get (info -> top_level_consts , rpos ) ); 
              if (old_e && (((Scheme_Type ) (((((intptr_t ) (old_e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (old_e ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (old_e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (old_e ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) && (1 ) ) {
#               define XfOrM1667_COUNT (0+XfOrM1664_COUNT)
#               define SETUP_XfOrM1667(x) SETUP_XfOrM1664(x)

# 9395 "./../src/optimize.c"
                if (! originals )
                  originals = FUNCCALL(SETUP_XfOrM1667(_), scheme_make_hash_table (SCHEME_hash_ptr ) ); 
                FUNCCALL(SETUP_XfOrM1667(_), scheme_hash_set (originals , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (start_simultaneous ) ) ) << 1 ) | 0x1 ) ) , old_e ) ); 
              }

              if (! FUNCCALL(SETUP_XfOrM1664(_), ir_propagate_ok (e , info , 0 , ((void * ) 0 ) ) )
              && FUNCCALL(SETUP_XfOrM1664(_), scheme_is_statically_proc (e , info , 0 ) )) {
#               define XfOrM1666_COUNT (0+XfOrM1664_COUNT)
#               define SETUP_XfOrM1666(x) SETUP_XfOrM1664(x)
                if (((Scheme_Type ) (((((intptr_t ) (old_e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (old_e ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (old_e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (old_e ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) 
                  e = ((void * ) 0 ) ; 
                else 
                e = FUNCCALL(SETUP_XfOrM1666(_), scheme_make_noninline_proc (e ) ); 
              }

              if (e ) {
#               define XfOrM1665_COUNT (0+XfOrM1664_COUNT)
#               define SETUP_XfOrM1665(x) SETUP_XfOrM1664(x)

# 9411 "./../src/optimize.c"
                if (0 || 0 )
                  new_sz = FUNCCALL(SETUP_XfOrM1665(_), lambda_body_size (e , 0 ) ); 
                else 
                new_sz = 0 ; 

                if (! old_sz 
                || (new_sz <= old_sz ) 
                || (! 0 && ! 0 ) )
                  FUNCCALL(SETUP_XfOrM1665(_), scheme_hash_set (info -> top_level_consts , rpos , e ) ); 
                else if (! 0 
                || (new_sz < 4 * old_sz ) )
                  prop_later = (__funcarg122 = FUNCCALL(SETUP_XfOrM1665(_), scheme_make_pair (rpos , e ) ), FUNCCALL_AGAIN(scheme_make_raw_pair (__funcarg122 , prop_later ) )) ; 
              }
            }
          }

          if (start_simultaneous == i_m )
            break ; 
          start_simultaneous ++ ; 
        }

        flags = FUNCCALL(SETUP_XfOrM1653(_), set_code_closure_flags (cl_first , 0 , 0xFFFF , 0 ) ); 
        (void ) FUNCCALL(SETUP_XfOrM1653(_), set_code_closure_flags (cl_first , (flags & (32 | 4 ) ) , ~ (32 | 4 | (64 | 128 ) ) , 1 ) )


        ; 
      }

      cl_last = cl_first = ((void * ) 0 ) ; 
      consts = ((void * ) 0 ) ; 
      re_consts = ((void * ) 0 ) ; 
      start_simultaneous = i_m + 1 ; 

      while (prop_later ) {
#       define XfOrM1652_COUNT (0+XfOrM1650_COUNT)
#       define SETUP_XfOrM1652(x) SETUP_XfOrM1650(x)

# 9445 "./../src/optimize.c"
        e = (((Scheme_Simple_Object * ) (prop_later ) ) -> u . pair_val . car ) ; 
        FUNCCALL(SETUP_XfOrM1652(_), scheme_hash_set (info -> top_level_consts , (((Scheme_Simple_Object * ) (e ) ) -> u . pair_val . car ) , (((Scheme_Simple_Object * ) (e ) ) -> u . pair_val . cdr ) ) ); 
        prop_later = (((Scheme_Simple_Object * ) (prop_later ) ) -> u . pair_val . cdr ) ; 
      }
    }

    if (! cont ) {
#     define XfOrM1642_COUNT (0+XfOrM1641_COUNT)
#     define SETUP_XfOrM1642(x) SETUP_XfOrM1641(x)
      e = (((Scheme_Vector * ) (linklet -> bodies ) ) -> els ) [i_m ] ; 
      if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_define_values_type ) )  {
        int i , n = ((((Scheme_Vector * ) (e ) ) -> size ) - 1 ) ; 
        Scheme_IR_Toplevel * var ; 
        BLOCK_SETUP((PUSH(var, 0+XfOrM1642_COUNT)));
#       define XfOrM1643_COUNT (1+XfOrM1642_COUNT)
#       define SETUP_XfOrM1643(x) SETUP(XfOrM1643_COUNT)
        var = NULLED_OUT ; 

# 9459 "./../src/optimize.c"
        for (i = 0 ; i < n ; i ++ ) {
#         define XfOrM1647_COUNT (0+XfOrM1643_COUNT)
#         define SETUP_XfOrM1647(x) SETUP_XfOrM1643(x)

# 9460 "./../src/optimize.c"
          var = ((Scheme_IR_Toplevel * ) ((((Scheme_Vector * ) (e ) ) -> els ) [(i ) + 1 ] ) ) ; 


          if (! ((& (var ) -> iso ) -> so . keyex & 0x4 ) ) {
#           define XfOrM1648_COUNT (0+XfOrM1647_COUNT)
#           define SETUP_XfOrM1648(x) SETUP_XfOrM1647(x)

# 9464 "./../src/optimize.c"
            if (! info -> top_level_consts 
            || ! FUNCCALL(SETUP_XfOrM1648(_), scheme_hash_get (info -> top_level_consts , (Scheme_Object * ) var ) )) {
#             define XfOrM1649_COUNT (0+XfOrM1648_COUNT)
#             define SETUP_XfOrM1649(x) SETUP_XfOrM1648(x)

# 9466 "./../src/optimize.c"
              fixed_table = FUNCCALL(SETUP_XfOrM1649(_), set_as_fixed (fixed_table , info , var -> variable_pos ) ); 
            }
          }
        }
      }
    }
  }



  if (info -> enforce_const ) {
#   define XfOrM1549_COUNT (0+XfOrM1538_COUNT)
#   define SETUP_XfOrM1549(x) SETUP_XfOrM1538(x)

# 9477 "./../src/optimize.c"
    for (i_m = 0 ; i_m < cnt ; i_m ++ ) {
#     define XfOrM1555_COUNT (0+XfOrM1549_COUNT)
#     define SETUP_XfOrM1555(x) SETUP_XfOrM1549(x)

# 9479 "./../src/optimize.c"
      e = (((Scheme_Vector * ) (linklet -> bodies ) ) -> els ) [i_m ] ; 
      if ((Scheme_Type ) (((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ) == (Scheme_Type ) (scheme_define_values_type ) )  {
        int size_override ; 
#       define XfOrM1556_COUNT (0+XfOrM1555_COUNT)
#       define SETUP_XfOrM1556(x) SETUP_XfOrM1555(x)

# 9482 "./../src/optimize.c"
        size_override = (((Scheme_Inclhash_Object * ) (e ) ) -> so . keyex & 0x1 ) ; 
        if (((((Scheme_Vector * ) (e ) ) -> size ) - 1 ) == 1 ) {
          Scheme_Object * sub_e , * alt_e ; 
          BLOCK_SETUP((PUSH(sub_e, 0+XfOrM1556_COUNT), PUSH(alt_e, 1+XfOrM1556_COUNT)));
#         define XfOrM1557_COUNT (2+XfOrM1556_COUNT)
#         define SETUP_XfOrM1557(x) SETUP(XfOrM1557_COUNT)
          sub_e = NULLED_OUT ; 
          alt_e = NULLED_OUT ; 

# 9485 "./../src/optimize.c"
          sub_e = ((((Scheme_Vector * ) (e ) ) -> els ) [0 ] ) ; 
          alt_e = FUNCCALL(SETUP_XfOrM1557(_), is_cross_linklet_inline_candidiate (sub_e , info , 0 ) ); 
          if (! alt_e && originals && (size_override ) ) {
#           define XfOrM1559_COUNT (0+XfOrM1557_COUNT)
#           define SETUP_XfOrM1559(x) SETUP_XfOrM1557(x)

# 9488 "./../src/optimize.c"
            alt_e = FUNCCALL(SETUP_XfOrM1559(_), scheme_hash_get (originals , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (i_m ) ) ) << 1 ) | 0x1 ) ) ) ); 
            if (((alt_e ) == (sub_e ) ) && ! size_override )
              alt_e = ((void * ) 0 ) ; 
            else if (alt_e )
              alt_e = FUNCCALL(SETUP_XfOrM1559(_), is_cross_linklet_inline_candidiate (alt_e , info , size_override ) ); 
          }
          if (alt_e ) {
            Scheme_Object * iv ; 
            BLOCK_SETUP((PUSH(iv, 0+XfOrM1557_COUNT)));
#           define XfOrM1558_COUNT (1+XfOrM1557_COUNT)
#           define SETUP_XfOrM1558(x) SETUP(XfOrM1558_COUNT)
            iv = NULLED_OUT ; 

# 9496 "./../src/optimize.c"
            iv = FUNCCALL(SETUP_XfOrM1558(_), scheme_make_vector (3 , scheme_false ) ); 
            iv -> type = scheme_inline_variant_type ; 
            (((Scheme_Vector * ) (iv ) ) -> els ) [0 ] = sub_e ; 
            (((Scheme_Vector * ) (iv ) ) -> els ) [1 ] = alt_e ; 
            ((((Scheme_Vector * ) (e ) ) -> els ) [0 ] ) = iv ; 
          }
        }
      }
    }
  }


  {
    int can_omit = 0 ; 
#   define XfOrM1539_COUNT (0+XfOrM1538_COUNT)
#   define SETUP_XfOrM1539(x) SETUP_XfOrM1538(x)

# 9510 "./../src/optimize.c"
    for (i_m = 0 ; i_m < cnt ; i_m ++ ) {
#     define XfOrM1547_COUNT (0+XfOrM1539_COUNT)
#     define SETUP_XfOrM1547(x) SETUP_XfOrM1539(x)

# 9512 "./../src/optimize.c"
      e = (((Scheme_Vector * ) (linklet -> bodies ) ) -> els ) [i_m ] ; 
      if ((i_m < (cnt - 1 ) ) && FUNCCALL(SETUP_XfOrM1547(_), scheme_omittable_expr (e , - 1 , - 1 , 0 , info , ((void * ) 0 ) ) )) {
        can_omit ++ ; 
      }
    }
    if (can_omit ) {
      Scheme_Object * new_bodies ; 
      int j = 0 ; 
      BLOCK_SETUP((PUSH(new_bodies, 0+XfOrM1539_COUNT)));
#     define XfOrM1540_COUNT (1+XfOrM1539_COUNT)
#     define SETUP_XfOrM1540(x) SETUP(XfOrM1540_COUNT)
      new_bodies = NULLED_OUT ; 

# 9520 "./../src/optimize.c"
      new_bodies = FUNCCALL(SETUP_XfOrM1540(_), scheme_make_vector (cnt - can_omit , scheme_false ) ); 
      for (i_m = 0 ; i_m < cnt ; i_m ++ ) {
#       define XfOrM1543_COUNT (0+XfOrM1540_COUNT)
#       define SETUP_XfOrM1543(x) SETUP_XfOrM1540(x)

# 9523 "./../src/optimize.c"
        e = (((Scheme_Vector * ) (linklet -> bodies ) ) -> els ) [i_m ] ; 
        if ((i_m == (cnt - 1 ) ) || ! FUNCCALL(SETUP_XfOrM1543(_), scheme_omittable_expr (e , - 1 , - 1 , 0 , info , ((void * ) 0 ) ) )) {
          (((Scheme_Vector * ) (new_bodies ) ) -> els ) [j ++ ] = e ; 
        }
      }
      linklet -> bodies = new_bodies ; 
    }
    cnt -= can_omit ; 
  }
  FUNCCALL(SETUP_XfOrM1538(_), record_optimize_shapes (info , linklet , _import_keys ) )



  ; 

  RET_VALUE_START (linklet ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

static Scheme_Object * optimize_k (void ) 
{
  /* No conversion */

# 9547 "./../src/optimize.c"
  Scheme_Thread * p = XFORM_OK_ASSIGN ((& scheme_thread_locals ) -> scheme_current_thread_ ) ; 
  Scheme_Object * expr = (Scheme_Object * ) p -> ku . k . p1 ; 
  Optimize_Info * info = (Optimize_Info * ) p -> ku . k . p2 ; 
  int context = p -> ku . k . i1 ; 

  p -> ku . k . p1 = ((void * ) 0 ) ; 
  p -> ku . k . p2 = ((void * ) 0 ) ; 

  return optimize_expr (expr , info , context ) ; 
}

Scheme_Object * optimize_expr (Scheme_Object * expr , Optimize_Info * info , int context ) 
{
  Scheme_Type type = ((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ; 

# 24 "./../src/mzstkchk.h"
  uintptr_t _stk_pos ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(4);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(expr, 1)));
# define XfOrM1754_COUNT (2)
# define SETUP_XfOrM1754(x) SETUP(XfOrM1754_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 26 "./../src/mzstkchk.h"
  _stk_pos = (uintptr_t ) & _stk_pos ; 

  if (((_stk_pos ) < (XFORM_OK_ASSIGN ((& scheme_thread_locals ) -> scheme_stack_boundary_ ) ) ) 
  && ! XFORM_OK_ASSIGN ((& scheme_thread_locals ) -> scheme_no_stack_overflow_ ) ) 
# 9564 "./../src/optimize.c"
  {
    Scheme_Thread * p = XFORM_OK_ASSIGN ((& scheme_thread_locals ) -> scheme_current_thread_ ) ; 
#   define XfOrM1772_COUNT (0+XfOrM1754_COUNT)
#   define SETUP_XfOrM1772(x) SETUP_XfOrM1754(x)

# 9567 "./../src/optimize.c"
    p -> ku . k . p1 = (void * ) expr ; 
    p -> ku . k . p2 = (void * ) info ; 
    p -> ku . k . i1 = context ; 

    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(scheme_handle_stack_overflow (optimize_k ) )) RET_VALUE_EMPTY_END ; 
  }


  info -> preserves_marks = 1 ; 
  info -> single_result = 1 ; 
  info -> escapes = 0 ; 

  switch (type ) {
#   define XfOrM1755_COUNT (0+XfOrM1754_COUNT)
#   define SETUP_XfOrM1755(x) SETUP_XfOrM1754(x)

# 9580 "./../src/optimize.c"
    case scheme_ir_local_type : 
    {
      Scheme_Object * val ; 
      BLOCK_SETUP((PUSH(val, 0+XfOrM1755_COUNT)));
#     define XfOrM1763_COUNT (1+XfOrM1755_COUNT)
#     define SETUP_XfOrM1763(x) SETUP(XfOrM1763_COUNT)
      val = NULLED_OUT ; 

# 9584 "./../src/optimize.c"
      info -> size += 1 ; 

      if (((Scheme_IR_Local * ) expr ) -> mutated ) {
#       define XfOrM1771_COUNT (0+XfOrM1763_COUNT)
#       define SETUP_XfOrM1771(x) SETUP_XfOrM1763(x)

# 9587 "./../src/optimize.c"
        info -> vclock += 1 ; 
        FUNCCALL(SETUP_XfOrM1771(_), register_use (((Scheme_IR_Local * ) expr ) , info ) ); 
        RET_VALUE_START (expr ) RET_VALUE_END ; 
      }

      val = FUNCCALL(SETUP_XfOrM1763(_), optimize_info_propagate_local (expr ) ); 
      if (val ) {
#       define XfOrM1770_COUNT (0+XfOrM1763_COUNT)
#       define SETUP_XfOrM1770(x) SETUP_XfOrM1763(x)

# 9594 "./../src/optimize.c"
        info -> size -= 1 ; 
        RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_expr (val , info , context ) )) RET_VALUE_EMPTY_END ; 
      }

      val = FUNCCALL(SETUP_XfOrM1763(_), collapse_local (expr , info , context ) ); 
      if (val )
        RET_VALUE_START (val ) RET_VALUE_END ; 

      if (! (context & 0x2 ) ) {
#       define XfOrM1764_COUNT (0+XfOrM1763_COUNT)
#       define SETUP_XfOrM1764(x) SETUP_XfOrM1763(x)

# 9603 "./../src/optimize.c"
        val = ((Scheme_IR_Local * ) expr ) -> optimize . known_val ; 

        if (val && ((Scheme_Type ) (((((intptr_t ) (val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (val ) ) -> type ) ) == (Scheme_Type ) (scheme_once_used_type ) ) ) {
          Scheme_Once_Used * o = (Scheme_Once_Used * ) val ; 
          BLOCK_SETUP((PUSH(o, 0+XfOrM1764_COUNT)));
#         define XfOrM1765_COUNT (1+XfOrM1764_COUNT)
#         define SETUP_XfOrM1765(x) SETUP(XfOrM1765_COUNT)

# 9608 "./../src/optimize.c"
          ((void ) sizeof ((! o -> moved ) ? 1 : 0 ) , __extension__ ({
#           define XfOrM1769_COUNT (0+XfOrM1765_COUNT)
#           define SETUP_XfOrM1769(x) SETUP_XfOrM1765(x)

# 9608 "./../src/optimize.c" 3
            if (! o -> moved )

# 9608 "./../src/optimize.c" 3
            ; 

# 9608 "./../src/optimize.c" 3
            else FUNCCALL(SETUP_XfOrM1769(_), __assert_fail ("!o->moved" , "./../src/optimize.c" , 9608 , __extension__ __PRETTY_FUNCTION__ ) ); 
          }
          ) ) 
# 9608 "./../src/optimize.c"
          ; 
          ((void ) sizeof ((! ((Scheme_IR_Local * ) expr ) -> optimize_outside_binding ) ? 1 : 0 ) , __extension__ ({
#           define XfOrM1768_COUNT (0+XfOrM1765_COUNT)
#           define SETUP_XfOrM1768(x) SETUP_XfOrM1765(x)

# 9609 "./../src/optimize.c" 3
            if (! ((Scheme_IR_Local * ) expr ) -> optimize_outside_binding )

# 9609 "./../src/optimize.c" 3
            ; 

# 9609 "./../src/optimize.c" 3
            else FUNCCALL(SETUP_XfOrM1768(_), __assert_fail ("!((Scheme_IR_Local *)expr)->optimize_outside_binding" , "./../src/optimize.c" , 9609 , __extension__ __PRETTY_FUNCTION__ ) ); 
          }
          ) ) 
# 9609 "./../src/optimize.c"
          ; 


          ((Scheme_IR_Local * ) expr ) -> optimize_used = 0 ; 

          if (((o -> vclock == info -> vclock ) 
          && ((o -> aclock == info -> aclock ) 
          || ! o -> spans_k ) 
          && ((context & 0x4 ) 
          || FUNCCALL(SETUP_XfOrM1765(_), single_valued_noncm_expression (o -> expr , info , 5 ) )) ) 
          || FUNCCALL(SETUP_XfOrM1765(_), movable_expression (o -> expr , info , o -> var -> optimize . lambda_depth != info -> lambda_depth , o -> kclock != info -> kclock , o -> sclock != info -> sclock , 0 , 5 ) )) 



          {
            int save_fuel = info -> inline_fuel , save_no_types = info -> no_types ; 
            int save_vclock , save_aclock , save_kclock , save_sclock ; 
#           define XfOrM1766_COUNT (0+XfOrM1765_COUNT)
#           define SETUP_XfOrM1766(x) SETUP_XfOrM1765(x)

# 9626 "./../src/optimize.c"
            info -> size -= 1 ; 
            info -> inline_fuel = - 1 ; 
            info -> no_types = 1 ; 
            save_vclock = info -> vclock ; 
            save_aclock = info -> aclock ; 
            save_kclock = info -> kclock ; 
            save_sclock = info -> sclock ; 

            o -> moved = 1 ; 

            val = FUNCCALL(SETUP_XfOrM1766(_), optimize_expr (o -> expr , info , context ) ); 

            if (info -> maybe_values_argument ) {
#             define XfOrM1767_COUNT (0+XfOrM1766_COUNT)
#             define SETUP_XfOrM1767(x) SETUP_XfOrM1766(x)
              FUNCCALL(SETUP_XfOrM1767(_), advance_clocks_for_optimized (val , & save_vclock , & save_aclock , & save_kclock , & save_sclock , info , 3 ) )



              ; 
            }

            info -> inline_fuel = save_fuel ; 
            info -> no_types = save_no_types ; 
            info -> vclock = save_vclock ; 
            info -> aclock = save_aclock ; 
            info -> kclock = save_kclock ; 
            info -> sclock = save_sclock ; 
            RET_VALUE_START (val ) RET_VALUE_END ; 
          }
        }
      }


      if ((context & (0x3 << 4 ) ) >> 4 ) 
        ((Scheme_IR_Local * ) expr ) -> arg_type = ((context & (0x3 << 4 ) ) >> 4 ) ; 
      if (info -> kclock > ((Scheme_IR_Local * ) expr ) -> optimize . init_kclock )
        ((Scheme_IR_Local * ) expr ) -> escapes_after_k_tick = 1 ; 
      FUNCCALL(SETUP_XfOrM1763(_), register_use (((Scheme_IR_Local * ) expr ) , info ) ); 
      RET_VALUE_START (expr ) RET_VALUE_END ; 
    }
    case scheme_application_type : 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_application (expr , info , context ) )) RET_VALUE_EMPTY_END ; 
    case scheme_application2_type : 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_application2 (expr , info , context ) )) RET_VALUE_EMPTY_END ; 
    case scheme_application3_type : 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_application3 (expr , info , context ) )) RET_VALUE_EMPTY_END ; 
    case scheme_sequence_type : 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_sequence (expr , info , context , 1 ) )) RET_VALUE_EMPTY_END ; 
    case scheme_branch_type : 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_branch (expr , info , context ) )) RET_VALUE_EMPTY_END ; 
    case scheme_with_cont_mark_type : 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_wcm (expr , info , context ) )) RET_VALUE_EMPTY_END ; 
    case scheme_ir_lambda_type : 
    if (context & 0x1 )
      RET_VALUE_START (scheme_true ) RET_VALUE_END ; 
    else 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_lambda (expr , info , context ) )) RET_VALUE_EMPTY_END ; 
    case scheme_ir_let_header_type : 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(optimize_lets (expr , info , context ) )) RET_VALUE_EMPTY_END ; 
    case scheme_ir_toplevel_type : 
    info -> size += 1 ; 
    {
      Scheme_Object * c ; 
      BLOCK_SETUP((PUSH(c, 0+XfOrM1755_COUNT)));
#     define XfOrM1756_COUNT (1+XfOrM1755_COUNT)
#     define SETUP_XfOrM1756(x) SETUP(XfOrM1756_COUNT)
      c = NULLED_OUT ; 

# 9691 "./../src/optimize.c"
      while (1 ) {
#       define XfOrM1762_COUNT (0+XfOrM1756_COUNT)
#       define SETUP_XfOrM1762(x) SETUP_XfOrM1756(x)

# 9692 "./../src/optimize.c"
        c = FUNCCALL(SETUP_XfOrM1762(_), get_import_inline (info , (Scheme_IR_Toplevel * ) expr , - 1 , 0 ) ); 
        if (! c )
          c = FUNCCALL(SETUP_XfOrM1762(_), get_defn_shape (info , (Scheme_IR_Toplevel * ) expr ) ); 
        c = FUNCCALL(SETUP_XfOrM1762(_), no_potential_size (c ) ); 
        if (c && ((Scheme_Type ) (((((intptr_t ) (c ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (c ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_toplevel_type ) ) )
          expr = c ; 
        else 
        break ; 
      }

      if (c ) {
#       define XfOrM1758_COUNT (0+XfOrM1756_COUNT)
#       define SETUP_XfOrM1758(x) SETUP_XfOrM1756(x)

# 9703 "./../src/optimize.c"
        if ((c ) == (scheme_stack_dump_key ) )  {
#         define XfOrM1760_COUNT (0+XfOrM1758_COUNT)
#         define SETUP_XfOrM1760(x) SETUP_XfOrM1758(x)

# 9705 "./../src/optimize.c"
          expr = FUNCCALL(SETUP_XfOrM1760(_), scheme_ir_toplevel_to_flagged_toplevel (expr , 3 ) ); 
          if (context & 0x1 )
            c = scheme_true ; 
          else 
          c = ((void * ) 0 ) ; 
        }

# 9710 "./../src/optimize.c"
        else if ((c ) == (scheme_default_prompt_tag ) )  {
#         define XfOrM1759_COUNT (0+XfOrM1758_COUNT)
#         define SETUP_XfOrM1759(x) SETUP_XfOrM1758(x)

# 9712 "./../src/optimize.c"
          expr = FUNCCALL(SETUP_XfOrM1759(_), scheme_ir_toplevel_to_flagged_toplevel (expr , 2 ) ); 
          c = ((void * ) 0 ) ; 
        }
      }

# 9715 "./../src/optimize.c"
      else 
      info -> vclock += 1 ; 

      if (c ) {
#       define XfOrM1757_COUNT (0+XfOrM1756_COUNT)
#       define SETUP_XfOrM1757(x) SETUP_XfOrM1756(x)

# 9719 "./../src/optimize.c"
        if (context & 0x1 )
          RET_VALUE_START (((((c ) ) == (scheme_false ) ) ? scheme_false : scheme_true ) ) RET_VALUE_END ; 

        if (FUNCCALL(SETUP_XfOrM1757(_), scheme_ir_duplicate_ok (c , 0 ) ))
          RET_VALUE_START (c ) RET_VALUE_END ; 



        expr = FUNCCALL(SETUP_XfOrM1757(_), scheme_ir_toplevel_to_flagged_toplevel (expr , 3 ) ); 
      }
    }
    FUNCCALL(SETUP_XfOrM1755(_), optimize_info_used_top (info ) ); 
    FUNCCALL(SETUP_XfOrM1755(_), register_import_used (info , (Scheme_IR_Toplevel * ) expr ) ); 
    RET_VALUE_START (expr ) RET_VALUE_END ; 
    case scheme_variable_type : FUNCCALL_EMPTY(scheme_signal_error ("got toplevel in wrong place" ) )
    ; 
    RET_VALUE_START (0 ) RET_VALUE_END ; 
    case scheme_define_values_type : 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(define_values_optimize (expr , info , context ) )) RET_VALUE_EMPTY_END ; 
    case scheme_varref_form_type : 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(ref_optimize (expr , info , context ) )) RET_VALUE_EMPTY_END ; 
    case scheme_set_bang_type : 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(set_optimize (expr , info , context ) )) RET_VALUE_EMPTY_END ; 
    case scheme_case_lambda_sequence_type : 
    if (context & 0x1 )
      RET_VALUE_START (scheme_true ) RET_VALUE_END ; 
    else 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(case_lambda_optimize (expr , info , context ) )) RET_VALUE_EMPTY_END ; 
    case scheme_begin0_sequence_type : 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(begin0_optimize (expr , info , context ) )) RET_VALUE_EMPTY_END ; 
    case scheme_apply_values_type : 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(apply_values_optimize (expr , info , context ) )) RET_VALUE_EMPTY_END ; 
    case scheme_with_immed_mark_type : 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(with_immed_mark_optimize (expr , info , context ) )) RET_VALUE_EMPTY_END ; 
    default : 
    info -> size += 1 ; 
    if ((context & 0x1 ) 
    && (((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) > _scheme_ir_values_types_ ) 
    && (! (((expr ) ) == (scheme_false ) ) ) )
      RET_VALUE_START (scheme_true ) RET_VALUE_END ; 
    else 
    RET_VALUE_START (expr ) RET_VALUE_END ; 
  }
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 9764 "./../src/optimize.c"
static void increment_use_count (Scheme_IR_Local * var , int as_rator ) 
{
  /* No conversion */

# 9766 "./../src/optimize.c"
  if (var -> use_count < 7 )
    var -> use_count ++ ; 
  if (! as_rator && (var -> non_app_count < 7 ) )
    var -> non_app_count ++ ; 

  if (var -> optimize . known_val 
  && var -> optimize . clear_known_on_multi_use )
    var -> optimize . known_val = ((void * ) 0 ) ; 
}

static Scheme_Object * optimize_clone_k (void ) 
{
  /* No conversion */

# 9778 "./../src/optimize.c"
  Scheme_Thread * p = XFORM_OK_ASSIGN ((& scheme_thread_locals ) -> scheme_current_thread_ ) ; 
  Scheme_Object * expr = (Scheme_Object * ) p -> ku . k . p1 ; 
  Optimize_Info * info = (Optimize_Info * ) p -> ku . k . p2 ; 
  Scheme_Hash_Tree * var_map = (Scheme_Hash_Tree * ) p -> ku . k . p3 ; 
  int single_use = p -> ku . k . i1 ; 
  int as_rator = p -> ku . k . i2 ; 

  p -> ku . k . p1 = ((void * ) 0 ) ; 
  p -> ku . k . p2 = ((void * ) 0 ) ; 
  p -> ku . k . p3 = ((void * ) 0 ) ; 

  return optimize_clone (single_use , expr , info , var_map , as_rator ) ; 
}

Scheme_Object * optimize_clone (int single_use , Scheme_Object * expr , Optimize_Info * info , Scheme_Hash_Tree * var_map , int as_rator ) 


{
  int t ; 

# 24 "./../src/mzstkchk.h"
  uintptr_t _stk_pos ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(10);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(var_map, 1), PUSH(expr, 2)));
# define XfOrM1775_COUNT (3)
# define SETUP_XfOrM1775(x) SETUP(XfOrM1775_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 26 "./../src/mzstkchk.h"
  _stk_pos = (uintptr_t ) & _stk_pos ; 

  if (((_stk_pos ) < (XFORM_OK_ASSIGN ((& scheme_thread_locals ) -> scheme_stack_boundary_ ) ) ) 
  && ! XFORM_OK_ASSIGN ((& scheme_thread_locals ) -> scheme_no_stack_overflow_ ) ) 
# 9800 "./../src/optimize.c"
  {
    Scheme_Thread * p = XFORM_OK_ASSIGN ((& scheme_thread_locals ) -> scheme_current_thread_ ) ; 
#   define XfOrM1794_COUNT (0+XfOrM1775_COUNT)
#   define SETUP_XfOrM1794(x) SETUP_XfOrM1775(x)

# 9803 "./../src/optimize.c"
    p -> ku . k . i1 = single_use ; 
    p -> ku . k . p1 = (void * ) expr ; 
    p -> ku . k . p2 = (void * ) info ; 
    p -> ku . k . p3 = (void * ) var_map ; 
    p -> ku . k . i2 = as_rator ; 

    RET_VALUE_START (FUNCCALL(SETUP_XfOrM1794(_), scheme_handle_stack_overflow (optimize_clone_k ) )) RET_VALUE_END ; 
  }


  t = ((((intptr_t ) (expr ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (expr ) ) -> type ) ; 

  switch (t ) {
#   define XfOrM1776_COUNT (0+XfOrM1775_COUNT)
#   define SETUP_XfOrM1776(x) SETUP_XfOrM1775(x)

# 9816 "./../src/optimize.c"
    case scheme_ir_local_type : 
    {
      Scheme_Object * v ; 
#     define XfOrM1793_COUNT (0+XfOrM1776_COUNT)
#     define SETUP_XfOrM1793(x) SETUP_XfOrM1776(x)
      v = NULLED_OUT ; 

# 9819 "./../src/optimize.c"
      v = (scheme_eq_hash_tree_get (var_map , expr ) ) ; 
      if (v )
        RET_VALUE_START (v ) RET_VALUE_END ; 
      else if (! single_use )
        FUNCCALL(SETUP_XfOrM1793(_), increment_use_count (((Scheme_IR_Local * ) expr ) , as_rator ) ); 
      RET_VALUE_START (expr ) RET_VALUE_END ; 
    }
    case scheme_application2_type : 
    {
      Scheme_App2_Rec * app = (Scheme_App2_Rec * ) expr , * app2 ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM1776_COUNT), PUSH(app2, 1+XfOrM1776_COUNT)));
#     define XfOrM1792_COUNT (2+XfOrM1776_COUNT)
#     define SETUP_XfOrM1792(x) SETUP(XfOrM1792_COUNT)
      app2 = NULLED_OUT ; 

# 9830 "./../src/optimize.c"
      app2 = ((Scheme_App2_Rec * ) FUNCCALL(SETUP_XfOrM1792(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_App2_Rec ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
      app2 -> iso . so . type = scheme_application2_type ; 

      expr = FUNCCALL(SETUP_XfOrM1792(_), optimize_clone (single_use , app -> rator , info , var_map , 1 ) ); 
      if (! expr )

# 9834 "./../src/optimize.c"
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
      app2 -> rator = expr ; 

      expr = FUNCCALL(SETUP_XfOrM1792(_), optimize_clone (single_use , app -> rand , info , var_map , 0 ) ); 
      if (! expr )

# 9838 "./../src/optimize.c"
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
      app2 -> rand = expr ; 

      (& (app2 ) -> iso ) -> so . keyex |= ((& (app ) -> iso ) -> so . keyex & ((1 << 11 ) | (1 << 12 ) | (1 << 13 ) ) ) ; 
      if (single_use )
        (& (app2 ) -> iso ) -> so . keyex |= ((& (app ) -> iso ) -> so . keyex & 7 ) ; 

      RET_VALUE_START ((Scheme_Object * ) app2 ) RET_VALUE_END ; 
    }
    case scheme_application_type : 
    {
      Scheme_App_Rec * app = (Scheme_App_Rec * ) expr , * app2 ; 
      int i ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM1776_COUNT), PUSH(app2, 1+XfOrM1776_COUNT)));
#     define XfOrM1789_COUNT (2+XfOrM1776_COUNT)
#     define SETUP_XfOrM1789(x) SETUP(XfOrM1789_COUNT)
      app2 = NULLED_OUT ; 

# 9852 "./../src/optimize.c"
      app2 = FUNCCALL(SETUP_XfOrM1789(_), scheme_malloc_application (app -> num_args + 1 ) ); 

      for (i = app -> num_args + 1 ; i -- ; ) {
#       define XfOrM1791_COUNT (0+XfOrM1789_COUNT)
#       define SETUP_XfOrM1791(x) SETUP_XfOrM1789(x)

# 9855 "./../src/optimize.c"
        expr = FUNCCALL(SETUP_XfOrM1791(_), optimize_clone (single_use , app -> args [i ] , info , var_map , ! i ) ); 
        if (! expr )

# 9856 "./../src/optimize.c"
        RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
        app2 -> args [i ] = expr ; 
      }

      (& (app2 ) -> iso ) -> so . keyex |= ((& (app ) -> iso ) -> so . keyex & ((1 << 11 ) | (1 << 12 ) | (1 << 13 ) ) ) ; 
      if (single_use )
        (& (app2 ) -> iso ) -> so . keyex |= ((& (app ) -> iso ) -> so . keyex & 7 ) ; 

      RET_VALUE_START ((Scheme_Object * ) app2 ) RET_VALUE_END ; 
    }
    case scheme_application3_type : 
    {
      Scheme_App3_Rec * app = (Scheme_App3_Rec * ) expr , * app2 ; 
      BLOCK_SETUP((PUSH(app, 0+XfOrM1776_COUNT), PUSH(app2, 1+XfOrM1776_COUNT)));
#     define XfOrM1788_COUNT (2+XfOrM1776_COUNT)
#     define SETUP_XfOrM1788(x) SETUP(XfOrM1788_COUNT)
      app2 = NULLED_OUT ; 

# 9870 "./../src/optimize.c"
      app2 = ((Scheme_App3_Rec * ) FUNCCALL(SETUP_XfOrM1788(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_App3_Rec ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
      app2 -> iso . so . type = scheme_application3_type ; 

      expr = FUNCCALL(SETUP_XfOrM1788(_), optimize_clone (single_use , app -> rator , info , var_map , 1 ) ); 
      if (! expr )

# 9874 "./../src/optimize.c"
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
      app2 -> rator = expr ; 

      expr = FUNCCALL(SETUP_XfOrM1788(_), optimize_clone (single_use , app -> rand1 , info , var_map , 0 ) ); 
      if (! expr )

# 9878 "./../src/optimize.c"
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
      app2 -> rand1 = expr ; 

      expr = FUNCCALL(SETUP_XfOrM1788(_), optimize_clone (single_use , app -> rand2 , info , var_map , 0 ) ); 
      if (! expr )

# 9882 "./../src/optimize.c"
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
      app2 -> rand2 = expr ; 

      (& (app2 ) -> iso ) -> so . keyex |= ((& (app ) -> iso ) -> so . keyex & ((1 << 11 ) | (1 << 12 ) | (1 << 13 ) ) ) ; 
      if (single_use )
        (& (app2 ) -> iso ) -> so . keyex |= ((& (app ) -> iso ) -> so . keyex & 7 ) ; 

      RET_VALUE_START ((Scheme_Object * ) app2 ) RET_VALUE_END ; 
    }
    case scheme_ir_let_header_type : 
    {
      Scheme_IR_Let_Header * head = (Scheme_IR_Let_Header * ) expr , * head2 ; 
      Scheme_Object * body ; 
      Scheme_IR_Let_Value * lv , * lv2 , * prev = ((void * ) 0 ) ; 
      Scheme_IR_Local * * vars ; 
      int i ; 
      BLOCK_SETUP((PUSH(body, 0+XfOrM1776_COUNT), PUSH(prev, 1+XfOrM1776_COUNT), PUSH(vars, 2+XfOrM1776_COUNT), PUSH(lv2, 3+XfOrM1776_COUNT), PUSH(lv, 4+XfOrM1776_COUNT), PUSH(head2, 5+XfOrM1776_COUNT), PUSH(head, 6+XfOrM1776_COUNT)));
#     define XfOrM1783_COUNT (7+XfOrM1776_COUNT)
#     define SETUP_XfOrM1783(x) SETUP(XfOrM1783_COUNT)
      head2 = NULLED_OUT ; 
      body = NULLED_OUT ; 
      lv = NULLED_OUT ; 
      lv2 = NULLED_OUT ; 
      vars = NULLED_OUT ; 

# 9899 "./../src/optimize.c"
      head2 = ((Scheme_IR_Let_Header * ) FUNCCALL(SETUP_XfOrM1783(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_IR_Let_Header ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
      head2 -> iso . so . type = scheme_ir_let_header_type ; 
      head2 -> count = head -> count ; 
      head2 -> num_clauses = head -> num_clauses ; 
      (& head2 -> iso ) -> so . keyex = (& head -> iso ) -> so . keyex ; 


      body = head -> body ; 
      for (i = head -> num_clauses ; i -- ; ) {
#       define XfOrM1787_COUNT (0+XfOrM1783_COUNT)
#       define SETUP_XfOrM1787(x) SETUP_XfOrM1783(x)

# 9908 "./../src/optimize.c"
        lv = (Scheme_IR_Let_Value * ) body ; 

        vars = FUNCCALL(SETUP_XfOrM1787(_), clone_variable_array (lv -> vars , lv -> count , & var_map ) ); 

        lv2 = ((Scheme_IR_Let_Value * ) FUNCCALL(SETUP_XfOrM1787(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_IR_Let_Value ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
        (& (lv2 ) -> iso ) -> so . keyex |= ((& (lv ) -> iso ) -> so . keyex & 0x1 ) ; 
        lv2 -> iso . so . type = scheme_ir_let_value_type ; 
        lv2 -> count = lv -> count ; 
        lv2 -> vars = vars ; 
        lv2 -> value = lv -> value ; 

        if (prev )
          prev -> body = (Scheme_Object * ) lv2 ; 
        else 
        head2 -> body = (Scheme_Object * ) lv2 ; 
        prev = lv2 ; 

        body = lv -> body ; 
      }
      if (prev )
        prev -> body = body ; 
      else 
      head2 -> body = body ; 

      body = head2 -> body ; 
      for (i = head -> num_clauses ; i -- ; ) {
#       define XfOrM1785_COUNT (0+XfOrM1783_COUNT)
#       define SETUP_XfOrM1785(x) SETUP_XfOrM1783(x)

# 9934 "./../src/optimize.c"
        lv2 = (Scheme_IR_Let_Value * ) body ; 

        expr = FUNCCALL(SETUP_XfOrM1785(_), optimize_clone (single_use , lv2 -> value , info , var_map , 0 ) ); 
        if (! expr )

# 9937 "./../src/optimize.c"
        RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
        lv2 -> value = expr ; 

        body = lv2 -> body ; 
      }

      expr = FUNCCALL(SETUP_XfOrM1783(_), optimize_clone (single_use , body , info , var_map , 0 ) ); 
      if (! expr )

# 9944 "./../src/optimize.c"
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

      if (prev )
        prev -> body = expr ; 
      else 
      head2 -> body = expr ; 

      RET_VALUE_START ((Scheme_Object * ) head2 ) RET_VALUE_END ; 
    }
    case scheme_sequence_type : 
    case scheme_begin0_sequence_type : 
    {
      Scheme_Sequence * seq = (Scheme_Sequence * ) expr , * seq2 ; 
      int i ; 
      BLOCK_SETUP((PUSH(seq2, 0+XfOrM1776_COUNT), PUSH(seq, 1+XfOrM1776_COUNT)));
#     define XfOrM1780_COUNT (2+XfOrM1776_COUNT)
#     define SETUP_XfOrM1780(x) SETUP(XfOrM1780_COUNT)
      seq2 = NULLED_OUT ; 

# 9959 "./../src/optimize.c"
      seq2 = FUNCCALL(SETUP_XfOrM1780(_), scheme_malloc_sequence (seq -> count ) ); 
      seq2 -> so . type = seq -> so . type ; 
      seq2 -> count = seq -> count ; 

      for (i = seq -> count ; i -- ; ) {
#       define XfOrM1782_COUNT (0+XfOrM1780_COUNT)
#       define SETUP_XfOrM1782(x) SETUP_XfOrM1780(x)

# 9964 "./../src/optimize.c"
        expr = FUNCCALL(SETUP_XfOrM1782(_), optimize_clone (single_use , seq -> array [i ] , info , var_map , 0 ) ); 
        if (! expr )

# 9965 "./../src/optimize.c"
        RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
        seq2 -> array [i ] = expr ; 
      }

      RET_VALUE_START ((Scheme_Object * ) seq2 ) RET_VALUE_END ; 
    }
    case scheme_branch_type : 
    {
      Scheme_Branch_Rec * b = (Scheme_Branch_Rec * ) expr , * b2 ; 
      BLOCK_SETUP((PUSH(b, 0+XfOrM1776_COUNT), PUSH(b2, 1+XfOrM1776_COUNT)));
#     define XfOrM1779_COUNT (2+XfOrM1776_COUNT)
#     define SETUP_XfOrM1779(x) SETUP(XfOrM1779_COUNT)
      b2 = NULLED_OUT ; 

# 9975 "./../src/optimize.c"
      b2 = ((Scheme_Branch_Rec * ) FUNCCALL(SETUP_XfOrM1779(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_Branch_Rec ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
      b2 -> so . type = scheme_branch_type ; 

      expr = FUNCCALL(SETUP_XfOrM1779(_), optimize_clone (single_use , b -> test , info , var_map , 0 ) ); 
      if (! expr )

# 9979 "./../src/optimize.c"
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
      b2 -> test = expr ; 

      expr = FUNCCALL(SETUP_XfOrM1779(_), optimize_clone (single_use , b -> tbranch , info , var_map , 0 ) ); 
      if (! expr )

# 9983 "./../src/optimize.c"
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
      b2 -> tbranch = expr ; 

      expr = FUNCCALL(SETUP_XfOrM1779(_), optimize_clone (single_use , b -> fbranch , info , var_map , 0 ) ); 
      if (! expr )

# 9987 "./../src/optimize.c"
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
      b2 -> fbranch = expr ; 

      RET_VALUE_START ((Scheme_Object * ) b2 ) RET_VALUE_END ; 
    }
    case scheme_with_cont_mark_type : 
    {
      Scheme_With_Continuation_Mark * wcm = (Scheme_With_Continuation_Mark * ) expr , * wcm2 ; 
      BLOCK_SETUP((PUSH(wcm, 0+XfOrM1776_COUNT), PUSH(wcm2, 1+XfOrM1776_COUNT)));
#     define XfOrM1778_COUNT (2+XfOrM1776_COUNT)
#     define SETUP_XfOrM1778(x) SETUP(XfOrM1778_COUNT)
      wcm2 = NULLED_OUT ; 

# 9996 "./../src/optimize.c"
      wcm2 = ((Scheme_With_Continuation_Mark * ) FUNCCALL(SETUP_XfOrM1778(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_With_Continuation_Mark ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
      wcm2 -> so . type = scheme_with_cont_mark_type ; 

      expr = FUNCCALL(SETUP_XfOrM1778(_), optimize_clone (single_use , wcm -> key , info , var_map , 0 ) ); 
      if (! expr )

# 10000 "./../src/optimize.c"
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
      wcm2 -> key = expr ; 

      expr = FUNCCALL(SETUP_XfOrM1778(_), optimize_clone (single_use , wcm -> val , info , var_map , 0 ) ); 
      if (! expr )

# 10004 "./../src/optimize.c"
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
      wcm2 -> val = expr ; 

      expr = FUNCCALL(SETUP_XfOrM1778(_), optimize_clone (single_use , wcm -> body , info , var_map , 0 ) ); 
      if (! expr )

# 10008 "./../src/optimize.c"
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
      wcm2 -> body = expr ; 

      RET_VALUE_START ((Scheme_Object * ) wcm2 ) RET_VALUE_END ; 
    }
    case scheme_ir_lambda_type : 
    RET_VALUE_START (FUNCCALL(SETUP_XfOrM1776(_), clone_lambda (single_use , expr , info , var_map ) )) RET_VALUE_END ; 
    case scheme_ir_toplevel_type : 
    RET_VALUE_START (expr ) RET_VALUE_END ; 
    case scheme_define_values_type : 
    case scheme_boxenv_type : 
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
    case scheme_varref_form_type : 
    RET_VALUE_START (FUNCCALL(SETUP_XfOrM1776(_), ref_clone (single_use , expr , info , var_map ) )) RET_VALUE_END ; 
    case scheme_set_bang_type : 
    RET_VALUE_START (FUNCCALL(SETUP_XfOrM1776(_), set_clone (single_use , expr , info , var_map ) )) RET_VALUE_END ; 
    case scheme_apply_values_type : 
    RET_VALUE_START (FUNCCALL(SETUP_XfOrM1776(_), apply_values_clone (single_use , expr , info , var_map ) )) RET_VALUE_END ; 
    case scheme_with_immed_mark_type : 
    RET_VALUE_START (FUNCCALL(SETUP_XfOrM1776(_), with_immed_mark_clone (single_use , expr , info , var_map ) )) RET_VALUE_END ; 
    case scheme_case_lambda_sequence_type : 
    RET_VALUE_START (FUNCCALL(SETUP_XfOrM1776(_), case_lambda_clone (single_use , expr , info , var_map ) )) RET_VALUE_END ; 
    default : 
    if (t > _scheme_ir_values_types_ ) {
#     define XfOrM1777_COUNT (0+XfOrM1776_COUNT)
#     define SETUP_XfOrM1777(x) SETUP_XfOrM1776(x)

# 10032 "./../src/optimize.c"
      if (single_use || FUNCCALL(SETUP_XfOrM1777(_), scheme_ir_duplicate_ok (expr , 0 ) ))
        RET_VALUE_START (expr ) RET_VALUE_END ; 
    }
  }

  RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

static Optimize_Info * optimize_info_allocate (Scheme_Linklet * linklet , 
int enforce_const , int can_inline , int unsafe_mode ) 
{
  Optimize_Info * info ; 
  DECL_RET_SAVE (Optimize_Info * ) PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(linklet, 1)));
# define XfOrM1795_COUNT (2)
# define SETUP_XfOrM1795(x) SETUP(XfOrM1795_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  info = NULLED_OUT ; 

# 10049 "./../src/optimize.c"
  info = ((Optimize_Info * ) FUNCCALL(SETUP_XfOrM1795(_), GC_malloc_one_small_tagged ((((sizeof (Optimize_Info ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 

  info -> type = scheme_rt_optimize_info ; 

  info -> inline_fuel = 32 ; 
  info -> flatten_fuel = 16 ; 
  info -> linklet = linklet ; 

  info -> enforce_const = enforce_const ; 
  if (! can_inline )
    info -> inline_fuel = - 1 ; 
  info -> unsafe_mode = unsafe_mode ; 

  RET_VALUE_START (info ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10065 "./../src/optimize.c"
static Optimize_Info * optimize_info_create (Scheme_Linklet * linklet , 
int enforce_const , int can_inline , int unsafe_mode ) 
{
  Optimize_Info * info ; 
  Scheme_Logger * logger ; 
  Scheme_Config * __funcarg132 = NULLED_OUT ; 
  Scheme_Object * __funcarg131 = NULLED_OUT ; 
  DECL_RET_SAVE (Optimize_Info * ) PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(logger, 1)));
# define XfOrM1796_COUNT (2)
# define SETUP_XfOrM1796(x) SETUP(XfOrM1796_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  info = NULLED_OUT ; 
  logger = NULLED_OUT ; 

# 10071 "./../src/optimize.c"
  info = FUNCCALL(SETUP_XfOrM1796(_), optimize_info_allocate (linklet , enforce_const , can_inline , unsafe_mode ) ); 

  logger = (Scheme_Logger * ) (__funcarg132 = FUNCCALL(SETUP_XfOrM1796(_), scheme_current_config () ), FUNCCALL_AGAIN(scheme_get_param (__funcarg132 , MZCONFIG_LOGGER ) )) ; 
  logger = (__funcarg131 = FUNCCALL(SETUP_XfOrM1796(_), scheme_intern_symbol ("optimizer" ) ), FUNCCALL_AGAIN(scheme_make_logger (logger , __funcarg131 ) )) ; 
  info -> logger = logger ; 

  RET_VALUE_START (info ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10080 "./../src/optimize.c"
static void optimize_info_seq_init (Optimize_Info * info , Optimize_Info_Sequence * info_seq ) 
{
  /* No conversion */

# 10082 "./../src/optimize.c"
  info_seq -> init_flatten_fuel = info -> flatten_fuel ; 
  info_seq -> min_flatten_fuel = info -> flatten_fuel ; 
}

static void optimize_info_seq_step (Optimize_Info * info , Optimize_Info_Sequence * info_seq ) 
{
  /* No conversion */

# 10088 "./../src/optimize.c"
  if (info -> flatten_fuel < info_seq -> min_flatten_fuel )
    info_seq -> min_flatten_fuel = info -> flatten_fuel ; 
  info -> flatten_fuel = info_seq -> init_flatten_fuel ; 
}

static void optimize_info_seq_done (Optimize_Info * info , Optimize_Info_Sequence * info_seq ) 
{
  /* No conversion */

# 10095 "./../src/optimize.c"
  if (info -> flatten_fuel > info_seq -> min_flatten_fuel )
    info -> flatten_fuel = info_seq -> min_flatten_fuel ; 
}

static void propagate_used_variables (Optimize_Info * info ) 
{
  Scheme_Hash_Table * ht ; 
  Scheme_IR_Local * tvar ; 
  int j ; 
  PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(tvar, 1), PUSH(ht, 2)));
# define XfOrM1800_COUNT (3)
# define SETUP_XfOrM1800(x) SETUP(XfOrM1800_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  ht = NULLED_OUT ; 
  tvar = NULLED_OUT ; 

# 10105 "./../src/optimize.c"
  if (info -> next -> uses ) {
#   define XfOrM1801_COUNT (0+XfOrM1800_COUNT)
#   define SETUP_XfOrM1801(x) SETUP_XfOrM1800(x)

# 10106 "./../src/optimize.c"
    ht = info -> uses ; 
    for (j = 0 ; j < ht -> size ; j ++ ) {
#     define XfOrM1804_COUNT (0+XfOrM1801_COUNT)
#     define SETUP_XfOrM1804(x) SETUP_XfOrM1801(x)

# 10108 "./../src/optimize.c"
      if (ht -> vals [j ] ) {
#       define XfOrM1805_COUNT (0+XfOrM1804_COUNT)
#       define SETUP_XfOrM1805(x) SETUP_XfOrM1804(x)

# 10109 "./../src/optimize.c"
        tvar = ((Scheme_IR_Local * ) ht -> keys [j ] ) ; 
        if (tvar -> optimize . lambda_depth < info -> next -> lambda_depth )
          FUNCCALL(SETUP_XfOrM1805(_), scheme_hash_set (info -> next -> uses , (Scheme_Object * ) tvar , scheme_true ) ); 
      }
    }
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10117 "./../src/optimize.c"
static int env_uses_toplevel (Optimize_Info * frame ) 
{
  /* No conversion */

# 10119 "./../src/optimize.c"
  int used ; 

  used = frame -> used_toplevel ; 

  if (used ) {

    frame = frame -> next ; 
    while (frame ) {
      if (frame -> flags & 1 ) {
        frame -> used_toplevel = 1 ; 
        break ; 
      }
      frame = frame -> next ; 
    }
  }

  return used ; 
}

static void optimize_info_used_top (Optimize_Info * info ) 
{
  /* No conversion */

# 10140 "./../src/optimize.c"
  while (info ) {
    if (info -> flags & 1 ) {
      info -> used_toplevel = 1 ; 
      break ; 
    }
    info = info -> next ; 
  }
}

static Scheme_Once_Used * make_once_used (Scheme_Object * val , Scheme_IR_Local * var , 
int vclock , int aclock , int kclock , int sclock , int spans_k ) 
{
  Scheme_Once_Used * o ; 
  DECL_RET_SAVE (Scheme_Once_Used * ) PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSH(o, 0), PUSH(var, 1), PUSH(val, 2)));
# define XfOrM1817_COUNT (3)
# define SETUP_XfOrM1817(x) SETUP(XfOrM1817_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  o = NULLED_OUT ; 

# 10158 "./../src/optimize.c"
  ((void ) sizeof ((! (((Scheme_Type ) (((((intptr_t ) (val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (val ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (val ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) ) ? 1 : 0 ) , __extension__ ({
#   define XfOrM1818_COUNT (0+XfOrM1817_COUNT)
#   define SETUP_XfOrM1818(x) SETUP_XfOrM1817(x)

# 10158 "./../src/optimize.c" 3
    if (! (((Scheme_Type ) (((((intptr_t ) (val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (val ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (val ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) )

# 10158 "./../src/optimize.c" 3
    ; 

# 10158 "./../src/optimize.c" 3
    else FUNCCALL(SETUP_XfOrM1818(_), __assert_fail ("!(((Scheme_Type)(((((intptr_t)(val)) & 0x1)?(Scheme_Type)scheme_integer_type:((Scheme_Object *)(val))->type)) == (Scheme_Type)(scheme_ir_lambda_type)) || ((Scheme_Type)(((((intptr_t)(val)) & 0x1)?(Scheme_Type)scheme_integer_type:((Scheme_Object *)(val))->type)) == (Scheme_Type)(scheme_case_lambda_sequence_type)))" , "./../src/optimize.c" , 10158 , __extension__ __PRETTY_FUNCTION__ ) ); 
  }
  ) ) 
# 10158 "./../src/optimize.c"
  ; 

  o = ((Scheme_Once_Used * ) FUNCCALL(SETUP_XfOrM1817(_), GC_malloc_one_small_tagged ((((sizeof (Scheme_Once_Used ) * (1 ) + (1 << 3 ) - 1 ) >> 3 ) << 3 ) ) )) ; 
  o -> so . type = scheme_once_used_type ; 

  o -> expr = val ; 
  o -> var = var ; 
  o -> vclock = vclock ; 
  o -> aclock = aclock ; 
  o -> kclock = kclock ; 
  o -> sclock = sclock ; 
  o -> spans_k = spans_k ; 

  RET_VALUE_START (o ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10174 "./../src/optimize.c"
static int optimize_any_uses (Optimize_Info * info , Scheme_IR_Let_Value * at_irlv , int n ) 
{
  int i , j ; 
  Scheme_IR_Let_Value * irlv = at_irlv ; 
  DECL_RET_SAVE (int ) PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(at_irlv, 0), PUSH(irlv, 1)));
# define XfOrM1819_COUNT (2)
# define SETUP_XfOrM1819(x) SETUP(XfOrM1819_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 10179 "./../src/optimize.c"
  while (n -- ) {
#   define XfOrM1832_COUNT (0+XfOrM1819_COUNT)
#   define SETUP_XfOrM1832(x) SETUP_XfOrM1819(x)

# 10180 "./../src/optimize.c"
    ((void ) sizeof ((((Scheme_Type ) (irlv -> iso . so . type ) == (Scheme_Type ) (scheme_ir_let_value_type ) ) ) ? 1 : 0 ) , __extension__ ({
#     define XfOrM1843_COUNT (0+XfOrM1832_COUNT)
#     define SETUP_XfOrM1843(x) SETUP_XfOrM1832(x)

# 10180 "./../src/optimize.c" 3
      if ((Scheme_Type ) (irlv -> iso . so . type ) == (Scheme_Type ) (scheme_ir_let_value_type ) ) 

# 10180 "./../src/optimize.c" 3
      ; 

# 10180 "./../src/optimize.c" 3
      else FUNCCALL(SETUP_XfOrM1843(_), __assert_fail ("((Scheme_Type)(irlv->iso.so.type) == (Scheme_Type)(scheme_ir_let_value_type))" , "./../src/optimize.c" , 10180 , __extension__ __PRETTY_FUNCTION__ ) ); 
    }
    ) ) 
# 10180 "./../src/optimize.c"
    ; 
    for (i = irlv -> count ; i -- ; ) {
#     define XfOrM1838_COUNT (0+XfOrM1832_COUNT)
#     define SETUP_XfOrM1838(x) SETUP_XfOrM1832(x)

# 10182 "./../src/optimize.c"
      if (irlv -> vars [i ] -> optimize_used )
        RET_VALUE_START (1 ) RET_VALUE_END ; 
      for (j = at_irlv -> count ; j -- ; ) {
#       define XfOrM1841_COUNT (0+XfOrM1838_COUNT)
#       define SETUP_XfOrM1841(x) SETUP_XfOrM1838(x)

# 10185 "./../src/optimize.c"
        if (at_irlv -> vars [j ] -> optimize . transitive_uses ) {
#         define XfOrM1842_COUNT (0+XfOrM1841_COUNT)
#         define SETUP_XfOrM1842(x) SETUP_XfOrM1841(x)

# 10186 "./../src/optimize.c"
          if (FUNCCALL(SETUP_XfOrM1842(_), scheme_hash_get (at_irlv -> vars [j ] -> optimize . transitive_uses , (Scheme_Object * ) irlv -> vars [i ] ) ))

          RET_VALUE_START (1 ) RET_VALUE_END ; 
        }
      }
    }
    irlv = (Scheme_IR_Let_Value * ) irlv -> body ; 
  }

  RET_VALUE_START (0 ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10198 "./../src/optimize.c"
static void optimize_uses_of_mutable_imply_early_alloc (Scheme_IR_Let_Value * at_irlv , int n ) 
{
  int i , j ; 
  Scheme_IR_Let_Value * irlv = at_irlv ; 
  PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(at_irlv, 0), PUSH(irlv, 1)));
# define XfOrM1844_COUNT (2)
# define SETUP_XfOrM1844(x) SETUP(XfOrM1844_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 10209 "./../src/optimize.c"
  while (n -- ) {
#   define XfOrM1860_COUNT (0+XfOrM1844_COUNT)
#   define SETUP_XfOrM1860(x) SETUP_XfOrM1844(x)

# 10210 "./../src/optimize.c"
    for (i = irlv -> count ; i -- ; ) {
#     define XfOrM1868_COUNT (0+XfOrM1860_COUNT)
#     define SETUP_XfOrM1868(x) SETUP_XfOrM1860(x)

# 10211 "./../src/optimize.c"
      if (irlv -> vars [i ] -> mutated ) {
        int used = 0 ; 
#       define XfOrM1869_COUNT (0+XfOrM1868_COUNT)
#       define SETUP_XfOrM1869(x) SETUP_XfOrM1868(x)

# 10213 "./../src/optimize.c"
        if (irlv -> vars [i ] -> optimize_used )
          used = 1 ; 
        else {
#         define XfOrM1870_COUNT (0+XfOrM1869_COUNT)
#         define SETUP_XfOrM1870(x) SETUP_XfOrM1869(x)

# 10216 "./../src/optimize.c"
          for (j = at_irlv -> count ; j -- ; ) {
#           define XfOrM1873_COUNT (0+XfOrM1870_COUNT)
#           define SETUP_XfOrM1873(x) SETUP_XfOrM1870(x)

# 10217 "./../src/optimize.c"
            if (at_irlv -> vars [j ] -> optimize . transitive_uses ) {
#             define XfOrM1874_COUNT (0+XfOrM1873_COUNT)
#             define SETUP_XfOrM1874(x) SETUP_XfOrM1873(x)

# 10218 "./../src/optimize.c"
              if (FUNCCALL(SETUP_XfOrM1874(_), scheme_hash_get (at_irlv -> vars [j ] -> optimize . transitive_uses , (Scheme_Object * ) irlv -> vars [i ] ) ))

              used = 1 ; 
            }
          }
        }
        if (used )
          irlv -> vars [i ] -> must_allocate_immediately = 1 ; 
      }
    }
    irlv = (Scheme_IR_Let_Value * ) irlv -> body ; 
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10232 "./../src/optimize.c"
static void register_use (Scheme_IR_Local * var , Optimize_Info * info ) 
{
  PREPARE_VAR_STACK(3);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(var, 1)));
# define XfOrM1875_COUNT (2)
# define SETUP_XfOrM1875(x) SETUP(XfOrM1875_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 10234 "./../src/optimize.c"
  ((void ) sizeof ((((Scheme_IR_Local * ) var ) -> mode == 3 ) ? 1 : 0 ) , __extension__ ({
#   define XfOrM1880_COUNT (0+XfOrM1875_COUNT)
#   define SETUP_XfOrM1880(x) SETUP_XfOrM1875(x)

# 10234 "./../src/optimize.c" 3
    if (((Scheme_IR_Local * ) var ) -> mode == 3 )

# 10234 "./../src/optimize.c" 3
    ; 

# 10234 "./../src/optimize.c" 3
    else FUNCCALL(SETUP_XfOrM1880(_), __assert_fail ("((Scheme_IR_Local *)var)->mode == 3" , "./../src/optimize.c" , 10234 , __extension__ __PRETTY_FUNCTION__ ) ); 
  }
  ) ) 
# 10234 "./../src/optimize.c"
  ; 
  ((void ) sizeof ((((Scheme_IR_Local * ) var ) -> use_count ) ? 1 : 0 ) , __extension__ ({
#   define XfOrM1879_COUNT (0+XfOrM1875_COUNT)
#   define SETUP_XfOrM1879(x) SETUP_XfOrM1875(x)

# 10235 "./../src/optimize.c" 3
    if (((Scheme_IR_Local * ) var ) -> use_count )

# 10235 "./../src/optimize.c" 3
    ; 

# 10235 "./../src/optimize.c" 3
    else FUNCCALL(SETUP_XfOrM1879(_), __assert_fail ("((Scheme_IR_Local *)var)->use_count" , "./../src/optimize.c" , 10235 , __extension__ __PRETTY_FUNCTION__ ) ); 
  }
  ) ) 
# 10235 "./../src/optimize.c"
  ; 

  if (var -> optimize . lambda_depth < info -> lambda_depth )
    FUNCCALL(SETUP_XfOrM1875(_), scheme_hash_set (info -> uses , (Scheme_Object * ) var , scheme_true ) ); 

  if (! var -> optimize_used ) {
#   define XfOrM1876_COUNT (0+XfOrM1875_COUNT)
#   define SETUP_XfOrM1876(x) SETUP_XfOrM1875(x)

# 10241 "./../src/optimize.c"
    var -> optimize_used = 1 ; 

    if (info -> transitive_use_var 
    && (var -> optimize . lambda_depth 
    <= info -> transitive_use_var -> optimize . lambda_depth ) ) {
      Scheme_Hash_Table * ht = info -> transitive_use_var -> optimize . transitive_uses ; 
      BLOCK_SETUP((PUSH(ht, 0+XfOrM1876_COUNT)));
#     define XfOrM1877_COUNT (1+XfOrM1876_COUNT)
#     define SETUP_XfOrM1877(x) SETUP(XfOrM1877_COUNT)

# 10248 "./../src/optimize.c"
      if (! ht ) {
#       define XfOrM1878_COUNT (0+XfOrM1877_COUNT)
#       define SETUP_XfOrM1878(x) SETUP_XfOrM1877(x)

# 10249 "./../src/optimize.c"
        ht = FUNCCALL(SETUP_XfOrM1878(_), scheme_make_hash_table (SCHEME_hash_ptr ) ); 
        info -> transitive_use_var -> optimize . transitive_uses = ht ; 
      }
      FUNCCALL_EMPTY(scheme_hash_set (ht , (Scheme_Object * ) var , scheme_true ) ); 
    }
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10257 "./../src/optimize.c"
static void register_transitive_uses (Scheme_IR_Local * var , Optimize_Info * info ) 
{
  Scheme_Hash_Table * ht ; 
  Scheme_IR_Local * tvar ; 
  int j ; 
  PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSH(tvar, 0), PUSH(info, 1), PUSH(ht, 2)));
# define XfOrM1881_COUNT (3)
# define SETUP_XfOrM1881(x) SETUP(XfOrM1881_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  ht = NULLED_OUT ; 
  tvar = NULLED_OUT ; 

# 10263 "./../src/optimize.c"
  ht = var -> optimize . transitive_uses ; 

  for (j = 0 ; j < ht -> size ; j ++ ) {
#   define XfOrM1886_COUNT (0+XfOrM1881_COUNT)
#   define SETUP_XfOrM1886(x) SETUP_XfOrM1881(x)

# 10266 "./../src/optimize.c"
    if (ht -> vals [j ] ) {
#     define XfOrM1887_COUNT (0+XfOrM1886_COUNT)
#     define SETUP_XfOrM1887(x) SETUP_XfOrM1886(x)

# 10267 "./../src/optimize.c"
      tvar = ((Scheme_IR_Local * ) ht -> keys [j ] ) ; 

      if (tvar -> optimize . known_val 
      && ((Scheme_Type ) (((((intptr_t ) (tvar -> optimize . known_val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (tvar -> optimize . known_val ) ) -> type ) ) == (Scheme_Type ) (scheme_once_used_type ) ) 
      && ((Scheme_Once_Used * ) tvar -> optimize . known_val ) -> moved ) {
#       define XfOrM1888_COUNT (0+XfOrM1887_COUNT)
#       define SETUP_XfOrM1888(x) SETUP_XfOrM1887(x)
        ((void ) sizeof ((! tvar -> optimize_used ) ? 1 : 0 ) , __extension__ ({
#         define XfOrM1889_COUNT (0+XfOrM1888_COUNT)
#         define SETUP_XfOrM1889(x) SETUP_XfOrM1888(x)

# 10274 "./../src/optimize.c" 3
          if (! tvar -> optimize_used )

# 10274 "./../src/optimize.c" 3
          ; 

# 10274 "./../src/optimize.c" 3
          else FUNCCALL(SETUP_XfOrM1889(_), __assert_fail ("!tvar->optimize_used" , "./../src/optimize.c" , 10274 , __extension__ __PRETTY_FUNCTION__ ) ); 
        }
        ) ) 
# 10274 "./../src/optimize.c"
        ; 
      }

# 10275 "./../src/optimize.c"
      else FUNCCALL(SETUP_XfOrM1887(_), register_use (tvar , info ) )
      ; 
    }
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10281 "./../src/optimize.c"
static Scheme_Object * optimize_info_lookup (Scheme_Object * var ) 
{
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(1);
  BLOCK_SETUP_TOP((PUSH(var, 0)));
# define XfOrM1890_COUNT (1)
# define SETUP_XfOrM1890(x) SETUP(XfOrM1890_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)

# 10283 "./../src/optimize.c"
  ((void ) sizeof ((((Scheme_IR_Local * ) var ) -> mode == 3 ) ? 1 : 0 ) , __extension__ ({
#   define XfOrM1892_COUNT (0+XfOrM1890_COUNT)
#   define SETUP_XfOrM1892(x) SETUP_XfOrM1890(x)

# 10283 "./../src/optimize.c" 3
    if (((Scheme_IR_Local * ) var ) -> mode == 3 )

# 10283 "./../src/optimize.c" 3
    ; 

# 10283 "./../src/optimize.c" 3
    else FUNCCALL(SETUP_XfOrM1892(_), __assert_fail ("((Scheme_IR_Local *)var)->mode == 3" , "./../src/optimize.c" , 10283 , __extension__ __PRETTY_FUNCTION__ ) ); 
  }
  ) ) 
# 10283 "./../src/optimize.c"
  ; 
  ((void ) sizeof ((((Scheme_IR_Local * ) var ) -> use_count ) ? 1 : 0 ) , __extension__ ({
#   define XfOrM1891_COUNT (0+XfOrM1890_COUNT)
#   define SETUP_XfOrM1891(x) SETUP_XfOrM1890(x)

# 10284 "./../src/optimize.c" 3
    if (((Scheme_IR_Local * ) var ) -> use_count )

# 10284 "./../src/optimize.c" 3
    ; 

# 10284 "./../src/optimize.c" 3
    else FUNCCALL(SETUP_XfOrM1891(_), __assert_fail ("((Scheme_IR_Local *)var)->use_count" , "./../src/optimize.c" , 10284 , __extension__ __PRETTY_FUNCTION__ ) ); 
  }
  ) ) 
# 10284 "./../src/optimize.c"
  ; 

  RET_VALUE_START (((Scheme_IR_Local * ) var ) -> optimize . known_val ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10289 "./../src/optimize.c"
static Scheme_Object * optimize_info_propagate_local (Scheme_Object * var ) 
{
  Scheme_Object * last , * val = var ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(3);
  BLOCK_SETUP_TOP((PUSH(last, 0), PUSH(val, 1), PUSH(var, 2)));
# define XfOrM1893_COUNT (3)
# define SETUP_XfOrM1893(x) SETUP(XfOrM1893_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  last = NULLED_OUT ; 

# 10293 "./../src/optimize.c"
  last = val ; 

  while (val && ((Scheme_Type ) (((((intptr_t ) (val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (val ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_local_type ) ) ) {
#   define XfOrM1898_COUNT (0+XfOrM1893_COUNT)
#   define SETUP_XfOrM1898(x) SETUP_XfOrM1893(x)

# 10296 "./../src/optimize.c"
    ((void ) sizeof ((((Scheme_IR_Local * ) val ) -> mode == 3 ) ? 1 : 0 ) , __extension__ ({
#     define XfOrM1900_COUNT (0+XfOrM1898_COUNT)
#     define SETUP_XfOrM1900(x) SETUP_XfOrM1898(x)

# 10296 "./../src/optimize.c" 3
      if (((Scheme_IR_Local * ) val ) -> mode == 3 )

# 10296 "./../src/optimize.c" 3
      ; 

# 10296 "./../src/optimize.c" 3
      else FUNCCALL(SETUP_XfOrM1900(_), __assert_fail ("((Scheme_IR_Local *)val)->mode == 3" , "./../src/optimize.c" , 10296 , __extension__ __PRETTY_FUNCTION__ ) ); 
    }
    ) ) 
# 10296 "./../src/optimize.c"
    ; 
    ((void ) sizeof ((((Scheme_IR_Local * ) val ) -> use_count ) ? 1 : 0 ) , __extension__ ({
#     define XfOrM1899_COUNT (0+XfOrM1898_COUNT)
#     define SETUP_XfOrM1899(x) SETUP_XfOrM1898(x)

# 10297 "./../src/optimize.c" 3
      if (((Scheme_IR_Local * ) val ) -> use_count )

# 10297 "./../src/optimize.c" 3
      ; 

# 10297 "./../src/optimize.c" 3
      else FUNCCALL(SETUP_XfOrM1899(_), __assert_fail ("((Scheme_IR_Local *)val)->use_count" , "./../src/optimize.c" , 10297 , __extension__ __PRETTY_FUNCTION__ ) ); 
    }
    ) ) 
# 10297 "./../src/optimize.c"
    ; 
    last = val ; 
    val = ((Scheme_IR_Local * ) val ) -> optimize . known_val ; 
  }

  if (! val 
  || ((Scheme_Type ) (((((intptr_t ) (val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (val ) ) -> type ) ) == (Scheme_Type ) (scheme_will_be_lambda_type ) ) 
  || (((Scheme_Type ) (((((intptr_t ) (val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (val ) ) -> type ) ) == (Scheme_Type ) (scheme_ir_lambda_type ) ) || ((Scheme_Type ) (((((intptr_t ) (val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (val ) ) -> type ) ) == (Scheme_Type ) (scheme_case_lambda_sequence_type ) ) ) 
  || ((Scheme_Type ) (((((intptr_t ) (val ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (val ) ) -> type ) ) == (Scheme_Type ) (scheme_once_used_type ) ) ) {
#   define XfOrM1894_COUNT (0+XfOrM1893_COUNT)
#   define SETUP_XfOrM1894(x) SETUP_XfOrM1893(x)

# 10306 "./../src/optimize.c"
    if ((last ) == (var ) ) 
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

    if (((Scheme_IR_Local * ) var ) -> use_count != 1 )
      FUNCCALL(SETUP_XfOrM1894(_), increment_use_count (((Scheme_IR_Local * ) last ) , 0 ) ); 

    RET_VALUE_START (last ) RET_VALUE_END ; 
  }

  RET_VALUE_START (val ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10318 "./../src/optimize.c"
Scheme_Object * optimize_get_predicate (Optimize_Info * info , Scheme_Object * var , int ignore_no_types ) 
{
  /* No conversion */

# 10320 "./../src/optimize.c"
  Scheme_Object * pred ; 

  if (info -> no_types && ! ignore_no_types )

# 10322 "./../src/optimize.c"
  return ((void * ) 0 ) ; 

  while (info ) {
    if (info -> types ) {
      pred = scheme_eq_hash_tree_get (info -> types , var ) ; 
      if (pred )
        return pred ; 
    }
    info = info -> next ; 
  }

  return ((void * ) 0 ) ; 
}

static Optimize_Info * optimize_info_add_frame (Optimize_Info * info , int flags ) 
{
  Optimize_Info * naya ; 
  DECL_RET_SAVE (Optimize_Info * ) PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(naya, 0), PUSH(info, 1)));
# define XfOrM1906_COUNT (2)
# define SETUP_XfOrM1906(x) SETUP(XfOrM1906_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  naya = NULLED_OUT ; 

# 10340 "./../src/optimize.c"
  naya = FUNCCALL(SETUP_XfOrM1906(_), optimize_info_allocate (info -> linklet , 0 , 0 , 0 ) ); 
  naya -> flags = (short ) flags ; 
  naya -> next = info ; 
  naya -> inline_fuel = info -> inline_fuel ; 
  naya -> flatten_fuel = info -> flatten_fuel ; 
  naya -> letrec_not_twice = info -> letrec_not_twice ; 
  naya -> enforce_const = info -> enforce_const ; 
  naya -> unsafe_mode = info -> unsafe_mode ; 
  naya -> top_level_consts = info -> top_level_consts ; 
  naya -> context = info -> context ; 
  naya -> vclock = info -> vclock ; 
  naya -> aclock = info -> aclock ; 
  naya -> kclock = info -> kclock ; 
  naya -> sclock = info -> sclock ; 
  naya -> escapes = info -> escapes ; 
  naya -> init_kclock = info -> kclock ; 
  naya -> maybe_values_argument = info -> maybe_values_argument ; 
  naya -> use_psize = info -> use_psize ; 
  naya -> logger = info -> logger ; 
  naya -> no_types = info -> no_types ; 
  naya -> lambda_depth = info -> lambda_depth + ((flags & 1 ) ? 1 : 0 ) ; 
  naya -> uses = info -> uses ; 
  naya -> transitive_use_var = info -> transitive_use_var ; 
  naya -> cross = info -> cross ; 
  naya -> imports_used = info -> imports_used ; 

  RET_VALUE_START (naya ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10369 "./../src/optimize.c"
static void optimize_info_done (Optimize_Info * info , Optimize_Info * parent ) 
{
  /* No conversion */

# 10371 "./../src/optimize.c"
  if (! parent )

# 10371 "./../src/optimize.c"
  parent = info -> next ; 

  parent -> size += info -> size ; 
  parent -> vclock = info -> vclock ; 
  parent -> aclock = info -> aclock ; 
  parent -> kclock = info -> kclock ; 
  parent -> sclock = info -> sclock ; 
  parent -> escapes = info -> escapes ; 
  parent -> psize += info -> psize ; 
  parent -> flatten_fuel = info -> flatten_fuel ; 
  if (info -> has_nonleaf )
    parent -> has_nonleaf = 1 ; 
}






static int is_procedure_expression (Scheme_Object * e ) 
{
  /* No conversion */

# 10392 "./../src/optimize.c"
  Scheme_Type t ; 

  if (! (((intptr_t ) (e ) ) & 0x1 ) && ((((e ) -> type ) >= scheme_prim_type ) && (((e ) -> type ) <= scheme_proc_chaperone_type ) ) ) 
    return 1 ; 

  t = ((((intptr_t ) (e ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (e ) ) -> type ) ; 

  return ((t == scheme_lambda_type ) 
  || (t == scheme_case_lambda_sequence_type ) ) ; 
}

static void linklet_setup_constants (Scheme_Linklet * linklet ) 
{
  int i , cnt , k , defns_start ; 
  Scheme_Object * form , * tl ; 
  Scheme_Hash_Table * ht ; 
  PREPARE_VAR_STACK(7);
  BLOCK_SETUP_TOP((PUSH(tl, 0), PUSH(linklet, 1), PUSH(ht, 2), PUSH(form, 3)));
# define XfOrM1909_COUNT (4)
# define SETUP_XfOrM1909(x) SETUP(XfOrM1909_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  form = NULLED_OUT ; 
  tl = NULLED_OUT ; 
  ht = NULLED_OUT ; 

# 10409 "./../src/optimize.c"
  if (linklet -> constants )
    RET_NOTHING ; 


  ht = FUNCCALL(SETUP_XfOrM1909(_), scheme_make_hash_table (SCHEME_hash_ptr ) ); 
  linklet -> constants = ht ; 

  defns_start = 1 + linklet -> num_total_imports ; 

  cnt = (((Scheme_Vector * ) (linklet -> bodies ) ) -> size ) ; 
  for (i = 0 ; i < cnt ; i ++ ) {
#   define XfOrM1940_COUNT (0+XfOrM1909_COUNT)
#   define SETUP_XfOrM1940(x) SETUP_XfOrM1909(x)

# 10420 "./../src/optimize.c"
    form = (((Scheme_Vector * ) (linklet -> bodies ) ) -> els ) [i ] ; 

    if ((Scheme_Type ) (((((intptr_t ) (form ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (form ) ) -> type ) ) == (Scheme_Type ) (scheme_define_values_type ) )  {
      int checked_st = 0 , is_st_prop = 0 , has_guard = 0 ; 
      Scheme_Object * is_st = ((void * ) 0 ) ; 
      Simple_Struct_Type_Info stinfo ; 
      Scheme_Object * parent_identity ; 
      BLOCK_SETUP((PUSH(parent_identity, 0+XfOrM1940_COUNT), PUSH(is_st, 1+XfOrM1940_COUNT)));
#     define XfOrM1941_COUNT (2+XfOrM1940_COUNT)
#     define SETUP_XfOrM1941(x) SETUP(XfOrM1941_COUNT)
      parent_identity = NULLED_OUT ; 

# 10428 "./../src/optimize.c"
      for (k = ((((Scheme_Vector * ) (form ) ) -> size ) - 1 ) ; k -- ; ) {
#       define XfOrM1956_COUNT (0+XfOrM1941_COUNT)
#       define SETUP_XfOrM1956(x) SETUP_XfOrM1941(x)

# 10429 "./../src/optimize.c"
        tl = (Scheme_Object * ) ((Scheme_IR_Toplevel * ) ((((Scheme_Vector * ) (form ) ) -> els ) [(k ) + 1 ] ) ) ; 
        if ((& ((Scheme_Toplevel * ) (tl ) ) -> iso ) -> so . keyex & 0x1 ) {
          int pos = (((Scheme_Toplevel * ) (tl ) ) -> position ) - defns_start ; 
#         define XfOrM1957_COUNT (0+XfOrM1956_COUNT)
#         define SETUP_XfOrM1957(x) SETUP_XfOrM1956(x)

# 10433 "./../src/optimize.c"
          if (pos < linklet -> num_exports ) {
            Scheme_Object * v ; 
            BLOCK_SETUP((PUSH(v, 0+XfOrM1957_COUNT)));
#           define XfOrM1958_COUNT (1+XfOrM1957_COUNT)
#           define SETUP_XfOrM1958(x) SETUP(XfOrM1958_COUNT)
            v = NULLED_OUT ; 

# 10436 "./../src/optimize.c"
            if (((((Scheme_Vector * ) (form ) ) -> size ) - 1 ) == 1 ) {
#             define XfOrM1965_COUNT (0+XfOrM1958_COUNT)
#             define SETUP_XfOrM1965(x) SETUP_XfOrM1958(x)

# 10437 "./../src/optimize.c"
              if (FUNCCALL(SETUP_XfOrM1965(_), scheme_ir_duplicate_ok (((((Scheme_Vector * ) (form ) ) -> els ) [0 ] ) , 1 ) )) {

                v = ((((Scheme_Vector * ) (form ) ) -> els ) [0 ] ) ; 
              }

# 10440 "./../src/optimize.c"
              else if ((Scheme_Type ) (((((intptr_t ) (((((Scheme_Vector * ) (form ) ) -> els ) [0 ] ) ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (((((Scheme_Vector * ) (form ) ) -> els ) [0 ] ) ) ) -> type ) ) == (Scheme_Type ) (scheme_inline_variant_type ) )  {

                v = ((((Scheme_Vector * ) (form ) ) -> els ) [0 ] ) ; 
              }

# 10443 "./../src/optimize.c"
              else if (FUNCCALL(SETUP_XfOrM1965(_), is_procedure_expression ((((Scheme_Vector * ) (form ) ) -> els ) [0 ] ) )) {
#               define XfOrM1967_COUNT (0+XfOrM1965_COUNT)
#               define SETUP_XfOrM1967(x) SETUP_XfOrM1965(x)

# 10445 "./../src/optimize.c"
                v = FUNCCALL(SETUP_XfOrM1967(_), scheme_make_vector (2 , scheme_false ) ); 
                (((Scheme_Vector * ) (v ) ) -> els ) [0 ] = ((((Scheme_Vector * ) (form ) ) -> els ) [0 ] ) ; 
              }

# 10447 "./../src/optimize.c"
              else {

                v = scheme_default_prompt_tag ; 
              }
            }

# 10451 "./../src/optimize.c"
            else {
#             define XfOrM1959_COUNT (0+XfOrM1958_COUNT)
#             define SETUP_XfOrM1959(x) SETUP_XfOrM1958(x)

# 10452 "./../src/optimize.c"
              if (! checked_st ) {
#               define XfOrM1962_COUNT (0+XfOrM1959_COUNT)
#               define SETUP_XfOrM1962(x) SETUP_XfOrM1959(x)

# 10453 "./../src/optimize.c"
                if (FUNCCALL(SETUP_XfOrM1962(_), scheme_is_simple_make_struct_type (((((Scheme_Vector * ) (form ) ) -> els ) [0 ] ) , ((((Scheme_Vector * ) (form ) ) -> size ) - 1 ) , 0x1 , ((void * ) 0 ) , & stinfo , & parent_identity , ((void * ) 0 ) , ((void * ) 0 ) , ((void * ) 0 ) , 0 , linklet , & is_st , 5 ) )) 





                {
#                 define XfOrM1964_COUNT (0+XfOrM1962_COUNT)
#                 define SETUP_XfOrM1964(x) SETUP_XfOrM1962(x)

# 10460 "./../src/optimize.c"
                  is_st = FUNCCALL(SETUP_XfOrM1964(_), scheme_make_pair (is_st , parent_identity ) ); 
                }

# 10461 "./../src/optimize.c"
                else {
#                 define XfOrM1963_COUNT (0+XfOrM1962_COUNT)
#                 define SETUP_XfOrM1963(x) SETUP_XfOrM1962(x)

# 10462 "./../src/optimize.c"
                  is_st = ((void * ) 0 ) ; 
                  if (FUNCCALL(SETUP_XfOrM1963(_), scheme_is_simple_make_struct_type_property ((((Scheme_Vector * ) (form ) ) -> els ) [0 ] , (((Scheme_Vector * ) (form ) ) -> size ) - 1 , 0x1 , & has_guard , ((void * ) 0 ) , ((void * ) 0 ) , ((void * ) 0 ) , 0 , linklet , 5 ) ))





                  is_st_prop = 1 ; 
                }
                checked_st = 1 ; 
              }
              if (is_st ) {
                intptr_t shape ; 
#               define XfOrM1961_COUNT (0+XfOrM1959_COUNT)
#               define SETUP_XfOrM1961(x) SETUP_XfOrM1959(x)

# 10475 "./../src/optimize.c"
                shape = FUNCCALL(SETUP_XfOrM1961(_), scheme_get_struct_proc_shape (k , & stinfo ) ); 

                v = FUNCCALL(SETUP_XfOrM1961(_), scheme_make_vector (3 , scheme_false ) ); 
                (((Scheme_Vector * ) (v ) ) -> els ) [1 ] = ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (shape ) ) ) << 1 ) | 0x1 ) ) ; 
                (((Scheme_Vector * ) (v ) ) -> els ) [2 ] = is_st ; 
              }

# 10480 "./../src/optimize.c"
              else if (is_st_prop ) {
                intptr_t shape ; 
#               define XfOrM1960_COUNT (0+XfOrM1959_COUNT)
#               define SETUP_XfOrM1960(x) SETUP_XfOrM1959(x)

# 10482 "./../src/optimize.c"
                shape = FUNCCALL(SETUP_XfOrM1960(_), scheme_get_struct_property_proc_shape (k , has_guard ) ); 

                v = FUNCCALL(SETUP_XfOrM1960(_), scheme_make_vector (4 , scheme_false ) ); 
                (((Scheme_Vector * ) (v ) ) -> els ) [1 ] = ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (shape ) ) ) << 1 ) | 0x1 ) ) ; 
              }

# 10486 "./../src/optimize.c"
              else 
              v = ((void * ) 0 ) ; 
            }
            if (v )
              FUNCCALL(SETUP_XfOrM1958(_), scheme_hash_set (ht , (((Scheme_Vector * ) (linklet -> defns ) ) -> els ) [pos ] , v ) ); 
          }
        }
      }
    }
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10498 "./../src/optimize.c"
static Scheme_Object * get_linklet_or_instance_for_import_key (Optimize_Info * info , Scheme_Object * key ) 
{
  Scheme_Object * v , * next_keys , * a [1 ] ; 
  Cross_Linklet_Info * cross = info -> cross ; 
  Scheme_Hash_Tree * ht ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(8);
  BLOCK_SETUP_TOP((PUSH(key, 0), PUSH(next_keys, 1), PUSH(v, 2), PUSH(cross, 3), PUSHARRAY(a, 1, 4), PUSH(ht, 7)));
# define XfOrM1970_COUNT (8)
# define SETUP_XfOrM1970(x) SETUP(XfOrM1970_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  v = NULLED_OUT ; 
  next_keys = NULLED_OUT ; 
  a[0] = NULLED_OUT ; 
  ht = NULLED_OUT ; 

# 10504 "./../src/optimize.c"
  if (! cross || ! cross -> get_import )
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

  v = (scheme_eq_hash_tree_get (cross -> linklets , key ) ) ; 
  if (! v ) {
#   define XfOrM1971_COUNT (0+XfOrM1970_COUNT)
#   define SETUP_XfOrM1971(x) SETUP_XfOrM1970(x)

# 10509 "./../src/optimize.c"
    a [0 ] = key ; 
    v = FUNCCALL(SETUP_XfOrM1971(_), scheme_apply_multi (cross -> get_import , 1 , a ) ); 
    if (((v ) == (((Scheme_Object * ) 0x6 ) ) ) 
    && (XFORM_OK_ASSIGN ((& scheme_thread_locals ) -> scheme_current_thread_ ) -> ku . multiple . count == 2 ) ) {
      v = XFORM_OK_ASSIGN ((& scheme_thread_locals ) -> scheme_current_thread_ ) -> ku . multiple . array [0 ] ; 
      next_keys = XFORM_OK_ASSIGN ((& scheme_thread_locals ) -> scheme_current_thread_ ) -> ku . multiple . array [1 ] ; 
    }

# 10515 "./../src/optimize.c"
    else {
#     define XfOrM1974_COUNT (0+XfOrM1971_COUNT)
#     define SETUP_XfOrM1974(x) SETUP_XfOrM1971(x)
      FUNCCALL(SETUP_XfOrM1974(_), scheme_wrong_return_arity ("compile-linklet" , 2 , (((v ) == (((Scheme_Object * ) 0x6 ) ) ) ? XFORM_OK_ASSIGN ((& scheme_thread_locals ) -> scheme_current_thread_ ) -> ku . multiple . count : 1 ) , (((v ) == (((Scheme_Object * ) 0x6 ) ) ) ? (Scheme_Object * * ) v : XFORM_OK_ASSIGN ((& scheme_thread_locals ) -> scheme_current_thread_ ) -> ku . multiple . array ) , "" ) )





      ; 
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
    }

    ht = FUNCCALL(SETUP_XfOrM1971(_), scheme_hash_tree_set (cross -> linklets , key , v ) ); 
    cross -> linklets = ht ; 

    if (! (((v ) ) == (scheme_false ) ) ) {
#     define XfOrM1972_COUNT (0+XfOrM1971_COUNT)
#     define SETUP_XfOrM1972(x) SETUP_XfOrM1971(x)

# 10532 "./../src/optimize.c"
      if (! ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_linklet_type ) ) 
      && ! ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_instance_type ) ) )
        FUNCCALL(SETUP_XfOrM1972(_), scheme_wrong_contract ("compile-linklet" , "(or/c linklet? instance? #f)" , - 1 , 0 , & v ) ); 

      if (! (((next_keys ) ) == (scheme_false ) ) 
      && (! ((Scheme_Type ) (((((intptr_t ) (next_keys ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (next_keys ) ) -> type ) ) == (Scheme_Type ) (scheme_vector_type ) ) 
      || ! ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_linklet_type ) ) 
      || (((Scheme_Vector * ) (next_keys ) ) -> size ) != (((Scheme_Vector * ) (((Scheme_Linklet * ) v ) -> importss ) ) -> size ) ) )
        FUNCCALL(SETUP_XfOrM1972(_), scheme_contract_error ("compile-linklet" , "result is not #f or a vector of keys that match the result linklet's import count" , (((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_linklet_type ) ) ? "linklet" : "instance" ) , 1 , v , "import count" , 1 , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) ((((Scheme_Vector * ) (((Scheme_Linklet * ) v ) -> importss ) ) -> size ) ) ) ) << 1 ) | 0x1 ) ) , "invalid as vector of keys" , 1 , next_keys , ((void * ) 0 ) ) )




      ; 

      if (! (((next_keys ) ) == (scheme_false ) ) )  {
#       define XfOrM1973_COUNT (0+XfOrM1972_COUNT)
#       define SETUP_XfOrM1973(x) SETUP_XfOrM1972(x)

# 10548 "./../src/optimize.c"
        ht = FUNCCALL(SETUP_XfOrM1973(_), scheme_hash_tree_set (cross -> import_next_keys , key , next_keys ) ); 
        cross -> import_next_keys = ht ; 
      }
    }
  }

  if (((v ) ) == (scheme_false ) ) 
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

  RET_VALUE_START (v ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10560 "./../src/optimize.c"
static Scheme_Object * get_import_inline_or_shape (Optimize_Info * info , Scheme_IR_Toplevel * var , 
int argc , int want_shape , int for_props ) 







{
  Scheme_Object * key , * v , * name , * l_or_i ; 
  Scheme_Hash_Table * iv_ht ; 
  Scheme_Linklet * linklet ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK(9);
  BLOCK_SETUP_TOP((PUSH(name, 0), PUSH(info, 1), PUSH(l_or_i, 2), PUSH(iv_ht, 3), PUSH(v, 4), PUSH(key, 5), PUSH(linklet, 6), PUSH(var, 7)));
# define XfOrM1976_COUNT (8)
# define SETUP_XfOrM1976(x) SETUP(XfOrM1976_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  key = NULLED_OUT ; 
  v = NULLED_OUT ; 
  name = NULLED_OUT ; 
  l_or_i = NULLED_OUT ; 
  iv_ht = NULLED_OUT ; 
  linklet = NULLED_OUT ; 

# 10574 "./../src/optimize.c"
  if (! info -> cross || (var -> instance_pos < 0 ) )
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

  key = (scheme_eq_hash_tree_get (info -> cross -> import_keys , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (var -> instance_pos ) ) ) << 1 ) | 0x1 ) ) ) ) ; 
  if (! key )
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

  l_or_i = FUNCCALL(SETUP_XfOrM1976(_), get_linklet_or_instance_for_import_key (info , key ) ); 

  if (! l_or_i )
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

  if ((var -> instance_pos < (((Scheme_Vector * ) (info -> linklet -> importss ) ) -> size ) ) 
  && (var -> variable_pos < (((Scheme_Vector * ) ((((Scheme_Vector * ) (info -> linklet -> importss ) ) -> els ) [var -> instance_pos ] ) ) -> size ) ) )
    name = (((Scheme_Vector * ) ((((Scheme_Vector * ) (info -> linklet -> importss ) ) -> els ) [var -> instance_pos ] ) ) -> els ) [var -> variable_pos ] ; 
  else {
    Scheme_Hash_Tree * ht ; 
    BLOCK_SETUP((PUSH(ht, 0+XfOrM1976_COUNT)));
#   define XfOrM1997_COUNT (1+XfOrM1976_COUNT)
#   define SETUP_XfOrM1997(x) SETUP(XfOrM1997_COUNT)
    ht = NULLED_OUT ; 

# 10591 "./../src/optimize.c"
    ht = (Scheme_Hash_Tree * ) (scheme_eq_hash_tree_get (info -> cross -> import_syms , 
    ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (var -> instance_pos ) ) ) << 1 ) | 0x1 ) ) ) ) ; 
    ((void ) sizeof ((ht ) ? 1 : 0 ) , __extension__ ({
#     define XfOrM1998_COUNT (0+XfOrM1997_COUNT)
#     define SETUP_XfOrM1998(x) SETUP_XfOrM1997(x)

# 10593 "./../src/optimize.c" 3
      if (ht )

# 10593 "./../src/optimize.c" 3
      ; 

# 10593 "./../src/optimize.c" 3
      else FUNCCALL(SETUP_XfOrM1998(_), __assert_fail ("ht" , "./../src/optimize.c" , 10593 , __extension__ __PRETTY_FUNCTION__ ) ); 
    }
    ) ) 
# 10593 "./../src/optimize.c"
    ; 
    name = (scheme_eq_hash_tree_get (ht , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (var -> variable_pos ) ) ) << 1 ) | 0x1 ) ) ) ) ; 
  }
  ((void ) sizeof ((name ) ? 1 : 0 ) , __extension__ ({
#   define XfOrM1996_COUNT (0+XfOrM1976_COUNT)
#   define SETUP_XfOrM1996(x) SETUP_XfOrM1976(x)

# 10596 "./../src/optimize.c" 3
    if (name )

# 10596 "./../src/optimize.c" 3
    ; 

# 10596 "./../src/optimize.c" 3
    else FUNCCALL(SETUP_XfOrM1996(_), __assert_fail ("name" , "./../src/optimize.c" , 10596 , __extension__ __PRETTY_FUNCTION__ ) ); 
  }
  ) ) 
# 10596 "./../src/optimize.c"
  ; 
  ((void ) sizeof ((((Scheme_Type ) (((((intptr_t ) (name ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (name ) ) -> type ) ) == (Scheme_Type ) (scheme_symbol_type ) ) ) ? 1 : 0 ) , __extension__ ({
#   define XfOrM1995_COUNT (0+XfOrM1976_COUNT)
#   define SETUP_XfOrM1995(x) SETUP_XfOrM1976(x)

# 10597 "./../src/optimize.c" 3
    if ((Scheme_Type ) (((((intptr_t ) (name ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (name ) ) -> type ) ) == (Scheme_Type ) (scheme_symbol_type ) ) 

# 10597 "./../src/optimize.c" 3
    ; 

# 10597 "./../src/optimize.c" 3
    else FUNCCALL(SETUP_XfOrM1995(_), __assert_fail ("((Scheme_Type)(((((intptr_t)(name)) & 0x1)?(Scheme_Type)scheme_integer_type:((Scheme_Object *)(name))->type)) == (Scheme_Type)(scheme_symbol_type))" , "./../src/optimize.c" , 10597 , __extension__ __PRETTY_FUNCTION__ ) ); 
  }
  ) ) 
# 10597 "./../src/optimize.c"
  ; 

  if ((Scheme_Type ) (((((intptr_t ) (l_or_i ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (l_or_i ) ) -> type ) ) == (Scheme_Type ) (scheme_linklet_type ) )  {
#   define XfOrM1979_COUNT (0+XfOrM1976_COUNT)
#   define SETUP_XfOrM1979(x) SETUP_XfOrM1976(x)

# 10600 "./../src/optimize.c"
    linklet = (Scheme_Linklet * ) l_or_i ; 

    if (! linklet -> constants )
      FUNCCALL(SETUP_XfOrM1979(_), linklet_setup_constants (linklet ) ); 

    if (! want_shape && ! for_props && (argc >= 0 ) ) {
#     define XfOrM1990_COUNT (0+XfOrM1979_COUNT)
#     define SETUP_XfOrM1990(x) SETUP_XfOrM1979(x)

# 10607 "./../src/optimize.c"
      iv_ht = (Scheme_Hash_Table * ) (scheme_eq_hash_tree_get (info -> cross -> inline_variants , key ) ) ; 
      if (iv_ht ) {
#       define XfOrM1991_COUNT (0+XfOrM1990_COUNT)
#       define SETUP_XfOrM1991(x) SETUP_XfOrM1990(x)

# 10609 "./../src/optimize.c"
        v = FUNCCALL(SETUP_XfOrM1991(_), scheme_hash_get (iv_ht , name ) ); 
        if (v ) {
#         define XfOrM1992_COUNT (0+XfOrM1991_COUNT)
#         define SETUP_XfOrM1992(x) SETUP_XfOrM1991(x)

# 10612 "./../src/optimize.c"
          if ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_hash_table_type ) )  {
#           define XfOrM1994_COUNT (0+XfOrM1992_COUNT)
#           define SETUP_XfOrM1994(x) SETUP_XfOrM1992(x)

# 10614 "./../src/optimize.c"
            v = FUNCCALL(SETUP_XfOrM1994(_), scheme_hash_get ((Scheme_Hash_Table * ) v , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (argc ) ) ) << 1 ) | 0x1 ) ) ) ); 
            if (v )
              RET_VALUE_START (v ) RET_VALUE_END ; 
          }
          else if (((v ) ) == (scheme_false ) )  {

            RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
          }

# 10621 "./../src/optimize.c"
          else 
          RET_VALUE_START (v ) RET_VALUE_END ; 
        }
      }
    }
    else 
    iv_ht = ((void * ) 0 ) ; 

    v = FUNCCALL(SETUP_XfOrM1979(_), scheme_hash_get (linklet -> constants , name ) ); 

    if (! v )
      RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

    if (((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_vector_type ) ) && ((((Scheme_Vector * ) (v ) ) -> size ) == 2 ) ) {
#     define XfOrM1989_COUNT (0+XfOrM1979_COUNT)
#     define SETUP_XfOrM1989(x) SETUP_XfOrM1979(x)

# 10636 "./../src/optimize.c"
      if (want_shape )
        v = FUNCCALL(SETUP_XfOrM1989(_), scheme_get_or_check_procedure_shape ((((Scheme_Vector * ) (v ) ) -> els ) [0 ] , ((void * ) 0 ) , 0 ) ); 
      else if (for_props )
        RET_VALUE_START ((((Scheme_Vector * ) (v ) ) -> els ) [0 ] ) RET_VALUE_END ; 
      else if (argc < 0 )
        v = scheme_stack_dump_key ; 
      else 
      v = ((void * ) 0 ) ; 
    }

# 10644 "./../src/optimize.c"
    else if ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_inline_variant_type ) )  {
#     define XfOrM1982_COUNT (0+XfOrM1979_COUNT)
#     define SETUP_XfOrM1982(x) SETUP_XfOrM1979(x)

# 10646 "./../src/optimize.c"
      if (for_props ) {
        RET_VALUE_START ((((Scheme_Vector * ) (v ) ) -> els ) [0 ] ) RET_VALUE_END ; 
      }

# 10648 "./../src/optimize.c"
      else if (want_shape ) {
#       define XfOrM1987_COUNT (0+XfOrM1982_COUNT)
#       define SETUP_XfOrM1987(x) SETUP_XfOrM1982(x)

# 10649 "./../src/optimize.c"
        v = FUNCCALL(SETUP_XfOrM1987(_), scheme_get_or_check_procedure_shape (v , ((void * ) 0 ) , 0 ) ); 
        if (v )
          info -> cross -> used_import_shape = 1 ; 
      }

# 10652 "./../src/optimize.c"
      else if (argc >= 0 ) {
        int has_cases = 0 ; 
#       define XfOrM1983_COUNT (0+XfOrM1982_COUNT)
#       define SETUP_XfOrM1983(x) SETUP_XfOrM1982(x)

# 10655 "./../src/optimize.c"
        v = FUNCCALL(SETUP_XfOrM1983(_), scheme_unresolve (v , argc , & has_cases , linklet , key , info ) ); 

        if (! iv_ht ) {
          Scheme_Hash_Tree * ht ; 
          BLOCK_SETUP((PUSH(ht, 0+XfOrM1983_COUNT)));
#         define XfOrM1986_COUNT (1+XfOrM1983_COUNT)
#         define SETUP_XfOrM1986(x) SETUP(XfOrM1986_COUNT)
          ht = NULLED_OUT ; 

# 10659 "./../src/optimize.c"
          iv_ht = FUNCCALL(SETUP_XfOrM1986(_), scheme_make_hash_table (SCHEME_hash_ptr ) ); 
          ht = FUNCCALL_AGAIN(scheme_hash_tree_set (info -> cross -> inline_variants , key , (Scheme_Object * ) iv_ht ) ); 
          info -> cross -> inline_variants = ht ; 
        }


        if (has_cases ) {
          Scheme_Hash_Table * cl_ht ; 
          BLOCK_SETUP((PUSH(cl_ht, 0+XfOrM1983_COUNT)));
#         define XfOrM1984_COUNT (1+XfOrM1983_COUNT)
#         define SETUP_XfOrM1984(x) SETUP(XfOrM1984_COUNT)
          cl_ht = NULLED_OUT ; 

# 10667 "./../src/optimize.c"
          cl_ht = (Scheme_Hash_Table * ) FUNCCALL(SETUP_XfOrM1984(_), scheme_hash_get (iv_ht , name ) ); 
          if (! cl_ht ) {
#           define XfOrM1985_COUNT (0+XfOrM1984_COUNT)
#           define SETUP_XfOrM1985(x) SETUP_XfOrM1984(x)

# 10669 "./../src/optimize.c"
            cl_ht = FUNCCALL(SETUP_XfOrM1985(_), scheme_make_hash_table (SCHEME_hash_ptr ) ); 
            FUNCCALL(SETUP_XfOrM1985(_), scheme_hash_set (iv_ht , name , (Scheme_Object * ) cl_ht ) ); 
          }
          FUNCCALL(SETUP_XfOrM1984(_), scheme_hash_set (cl_ht , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (argc ) ) ) << 1 ) | 0x1 ) ) , v ) ); 
        }

# 10673 "./../src/optimize.c"
        else if (v )
          FUNCCALL(SETUP_XfOrM1983(_), scheme_hash_set (iv_ht , name , v ) ); 
        else FUNCCALL(SETUP_XfOrM1983(_), scheme_hash_set (iv_ht , name , scheme_false ) )
        ; 
      }

# 10677 "./../src/optimize.c"
      else 
      v = scheme_stack_dump_key ; 
    }

# 10679 "./../src/optimize.c"
    else if (((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_vector_type ) ) && ((((Scheme_Vector * ) (v ) ) -> size ) == 3 ) ) {
#     define XfOrM1981_COUNT (0+XfOrM1979_COUNT)
#     define SETUP_XfOrM1981(x) SETUP_XfOrM1979(x)

# 10680 "./../src/optimize.c"
      if (want_shape )
        v = FUNCCALL(SETUP_XfOrM1981(_), scheme_make_struct_proc_shape ((((intptr_t ) ((((Scheme_Vector * ) (v ) ) -> els ) [1 ] ) ) >> 1 ) , (((Scheme_Vector * ) (v ) ) -> els ) [2 ] ) )
      ; 
      else if ((argc < 0 ) || for_props )
        v = scheme_stack_dump_key ; 
      else 
      v = ((void * ) 0 ) ; 
    }

# 10687 "./../src/optimize.c"
    else if (((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_vector_type ) ) && ((((Scheme_Vector * ) (v ) ) -> size ) == 4 ) ) {
#     define XfOrM1980_COUNT (0+XfOrM1979_COUNT)
#     define SETUP_XfOrM1980(x) SETUP_XfOrM1979(x)

# 10688 "./../src/optimize.c"
      if (want_shape )
        v = FUNCCALL(SETUP_XfOrM1980(_), scheme_make_struct_property_proc_shape ((((intptr_t ) ((((Scheme_Vector * ) (v ) ) -> els ) [1 ] ) ) >> 1 ) ) ); 
      else if ((argc < 0 ) || for_props )
        v = scheme_stack_dump_key ; 
      else 
      v = ((void * ) 0 ) ; 
    }
  }

# 10695 "./../src/optimize.c"
  else {
    Scheme_Bucket * b ; 
    int imprecise = (& ((Scheme_Instance * ) l_or_i ) -> iso ) -> so . keyex & 0x1 ; 
    BLOCK_SETUP((PUSH(b, 0+XfOrM1976_COUNT)));
#   define XfOrM1977_COUNT (1+XfOrM1976_COUNT)
#   define SETUP_XfOrM1977(x) SETUP(XfOrM1977_COUNT)
    b = NULLED_OUT ; 

# 10698 "./../src/optimize.c"
    b = FUNCCALL(SETUP_XfOrM1977(_), scheme_instance_variable_bucket_or_null (name , (Scheme_Instance * ) l_or_i ) ); 
    if (b && b -> val && (((Scheme_Bucket_With_Flags * ) b ) -> flags & 2 ) ) {
#     define XfOrM1978_COUNT (0+XfOrM1977_COUNT)
#     define SETUP_XfOrM1978(x) SETUP_XfOrM1977(x)

# 10700 "./../src/optimize.c"
      v = b -> val ; 
      if (want_shape )
        v = FUNCCALL(SETUP_XfOrM1978(_), get_value_shape (v , imprecise ) ); 
      else if (argc < 0 )
        v = scheme_stack_dump_key ; 
      else 
      v = ((void * ) 0 ) ; 
    }

# 10707 "./../src/optimize.c"
    else 
    v = ((void * ) 0 ) ; 
  }

  if (v && (want_shape || (argc < 0 ) ) )
    info -> cross -> used_import_shape = 1 ; 

  RET_VALUE_START (v ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10717 "./../src/optimize.c"
Scheme_Object * scheme_optimize_add_import_variable (Optimize_Info * info , Scheme_Object * linklet_key , Scheme_Object * symbol ) 


{
  Scheme_Object * pos , * var_pos , * vec ; 
  Scheme_Hash_Tree * syms , * ht ; 
  int i ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(7);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(ht, 1), PUSH(symbol, 2), PUSH(var_pos, 3), PUSH(syms, 4), PUSH(pos, 5), PUSH(vec, 6)));
# define XfOrM1999_COUNT (7)
# define SETUP_XfOrM1999(x) SETUP(XfOrM1999_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  pos = NULLED_OUT ; 
  var_pos = NULLED_OUT ; 
  vec = NULLED_OUT ; 
  syms = NULLED_OUT ; 
  ht = NULLED_OUT ; 

# 10725 "./../src/optimize.c"
  if (((linklet_key ) ) == (scheme_false ) ) 
    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 

  pos = (scheme_eq_hash_tree_get (info -> cross -> rev_import_keys , linklet_key ) ) ; 
  ((void ) sizeof ((pos ) ? 1 : 0 ) , __extension__ ({
#   define XfOrM2006_COUNT (0+XfOrM1999_COUNT)
#   define SETUP_XfOrM2006(x) SETUP_XfOrM1999(x)

# 10729 "./../src/optimize.c" 3
    if (pos )

# 10729 "./../src/optimize.c" 3
    ; 

# 10729 "./../src/optimize.c" 3
    else FUNCCALL(SETUP_XfOrM2006(_), __assert_fail ("pos" , "./../src/optimize.c" , 10729 , __extension__ __PRETTY_FUNCTION__ ) ); 
  }
  ) ) 
# 10729 "./../src/optimize.c"
  ; 

  syms = (Scheme_Hash_Tree * ) (scheme_eq_hash_tree_get (info -> cross -> import_syms , pos ) ) ; 
  if (! syms ) {
#   define XfOrM2001_COUNT (0+XfOrM1999_COUNT)
#   define SETUP_XfOrM2001(x) SETUP_XfOrM1999(x)

# 10733 "./../src/optimize.c"
    syms = empty_eq_hash_tree ; 
    if ((((intptr_t ) (pos ) ) >> 1 ) < (((Scheme_Vector * ) (info -> linklet -> importss ) ) -> size ) ) {
#     define XfOrM2003_COUNT (0+XfOrM2001_COUNT)
#     define SETUP_XfOrM2003(x) SETUP_XfOrM2001(x)

# 10736 "./../src/optimize.c"
      vec = (((Scheme_Vector * ) (info -> linklet -> importss ) ) -> els ) [(((intptr_t ) (pos ) ) >> 1 ) ] ; 
      for (i = (((Scheme_Vector * ) (vec ) ) -> size ) ; i -- ; ) {
#       define XfOrM2005_COUNT (0+XfOrM2003_COUNT)
#       define SETUP_XfOrM2005(x) SETUP_XfOrM2003(x)

# 10738 "./../src/optimize.c"
        syms = FUNCCALL(SETUP_XfOrM2005(_), scheme_hash_tree_set (syms , (((Scheme_Vector * ) (vec ) ) -> els ) [i ] , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (i ) ) ) << 1 ) | 0x1 ) ) ) ); 
        syms = FUNCCALL(SETUP_XfOrM2005(_), scheme_hash_tree_set (syms , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (i ) ) ) << 1 ) | 0x1 ) ) , (((Scheme_Vector * ) (vec ) ) -> els ) [i ] ) ); 
      }
    }

# 10741 "./../src/optimize.c"
    else {
    }

    ht = FUNCCALL(SETUP_XfOrM2001(_), scheme_hash_tree_set (info -> cross -> import_syms , pos , (Scheme_Object * ) syms ) ); 
    info -> cross -> import_syms = ht ; 
  }

  var_pos = (scheme_eq_hash_tree_get (syms , symbol ) ) ; 
  if (! var_pos ) {
#   define XfOrM2000_COUNT (0+XfOrM1999_COUNT)
#   define SETUP_XfOrM2000(x) SETUP_XfOrM1999(x)

# 10750 "./../src/optimize.c"
    var_pos = ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (syms -> count >> 1 ) ) ) << 1 ) | 0x1 ) ) ; 
    syms = FUNCCALL(SETUP_XfOrM2000(_), scheme_hash_tree_set (syms , symbol , var_pos ) ); 
    syms = FUNCCALL_AGAIN(scheme_hash_tree_set (syms , var_pos , symbol ) ); 
    ht = FUNCCALL_AGAIN(scheme_hash_tree_set (info -> cross -> import_syms , pos , (Scheme_Object * ) syms ) ); 
    info -> cross -> import_syms = ht ; 
  }


  RET_VALUE_START ((Scheme_Object * ) FUNCCALL_EMPTY(scheme_make_ir_toplevel ((((intptr_t ) (pos ) ) >> 1 ) , (((intptr_t ) (var_pos ) ) >> 1 ) , 1 ) )) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10761 "./../src/optimize.c"
Scheme_Object * scheme_optimize_get_import_key (Optimize_Info * info , Scheme_Object * linklet_key , int instance_pos ) 


{
  Scheme_Object * next_keys , * key , * pos ; 
  Scheme_Hash_Tree * ht ; 
  DECL_RET_SAVE (Scheme_Object * ) PREPARE_VAR_STACK_ONCE(5);
  BLOCK_SETUP_TOP((PUSH(key, 0), PUSH(next_keys, 1), PUSH(info, 2), PUSH(ht, 3), PUSH(pos, 4)));
# define XfOrM2007_COUNT (5)
# define SETUP_XfOrM2007(x) SETUP(XfOrM2007_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  next_keys = NULLED_OUT ; 
  key = NULLED_OUT ; 
  pos = NULLED_OUT ; 
  ht = NULLED_OUT ; 

# 10768 "./../src/optimize.c"
  next_keys = (scheme_eq_hash_tree_get (info -> cross -> import_next_keys , linklet_key ) ) ; 
  if (! next_keys ) {

    RET_VALUE_START (((void * ) 0 ) ) RET_VALUE_END ; 
  }

  ((void ) sizeof ((instance_pos < (((Scheme_Vector * ) (next_keys ) ) -> size ) ) ? 1 : 0 ) , __extension__ ({
#   define XfOrM2009_COUNT (0+XfOrM2007_COUNT)
#   define SETUP_XfOrM2009(x) SETUP_XfOrM2007(x)

# 10774 "./../src/optimize.c" 3
    if (instance_pos < (((Scheme_Vector * ) (next_keys ) ) -> size ) )

# 10774 "./../src/optimize.c" 3
    ; 

# 10774 "./../src/optimize.c" 3
    else FUNCCALL(SETUP_XfOrM2009(_), __assert_fail ("instance_pos < (((Scheme_Vector *)(next_keys))->size)" , "./../src/optimize.c" , 10774 , __extension__ __PRETTY_FUNCTION__ ) ); 
  }
  ) ) 
# 10774 "./../src/optimize.c"
  ; 

  key = (((Scheme_Vector * ) (next_keys ) ) -> els ) [instance_pos ] ; 
  pos = (scheme_eq_hash_tree_get (info -> cross -> rev_import_keys , key ) ) ; 
  if (! pos ) {
#   define XfOrM2008_COUNT (0+XfOrM2007_COUNT)
#   define SETUP_XfOrM2008(x) SETUP_XfOrM2007(x)

# 10780 "./../src/optimize.c"
    pos = ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (info -> cross -> import_keys -> count ) ) ) << 1 ) | 0x1 ) ) ; 

    ht = FUNCCALL(SETUP_XfOrM2008(_), scheme_hash_tree_set (info -> cross -> import_keys , pos , key ) ); 
    info -> cross -> import_keys = ht ; 

    ht = FUNCCALL(SETUP_XfOrM2008(_), scheme_hash_tree_set (info -> cross -> rev_import_keys , key , pos ) ); 
    info -> cross -> rev_import_keys = ht ; 
  }

  RET_VALUE_START (key ) RET_VALUE_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10792 "./../src/optimize.c"
static Scheme_Object * get_import_shape (Optimize_Info * info , Scheme_IR_Toplevel * var ) 
{
  /* No conversion */

# 10794 "./../src/optimize.c"
  return get_import_inline_or_shape (info , var , - 1 , 1 , 0 ) ; 
}

static Scheme_Object * get_import_inline (Optimize_Info * info , Scheme_IR_Toplevel * var , int argc , int for_props ) 

{
  /* No conversion */

# 10800 "./../src/optimize.c"
  return get_import_inline_or_shape (info , var , argc , 0 , for_props ) ; 
}

static void register_import_used (Optimize_Info * info , Scheme_IR_Toplevel * var ) 
{
  PREPARE_VAR_STACK(3);
  BLOCK_SETUP_TOP((PUSH(info, 0), PUSH(var, 1)));
# define XfOrM2013_COUNT (2)
# define SETUP_XfOrM2013(x) SETUP(XfOrM2013_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)

# 10805 "./../src/optimize.c"
  if ((var -> instance_pos >= 0 ) && info -> imports_used ) {


    Scheme_Hash_Tree * ht ; 
    BLOCK_SETUP((PUSH(ht, 0+XfOrM2013_COUNT)));
#   define XfOrM2014_COUNT (1+XfOrM2013_COUNT)
#   define SETUP_XfOrM2014(x) SETUP(XfOrM2014_COUNT)
    ht = NULLED_OUT ; 

# 10809 "./../src/optimize.c"
    ht = (Scheme_Hash_Tree * ) (scheme_eq_hash_tree_get (* info -> imports_used , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (var -> instance_pos ) ) ) << 1 ) | 0x1 ) ) ) ) ; 
    if (! ht )
      ht = empty_eq_hash_tree ; 
    if (! (scheme_eq_hash_tree_get (ht , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (var -> variable_pos ) ) ) << 1 ) | 0x1 ) ) ) ) ) {
#     define XfOrM2015_COUNT (0+XfOrM2014_COUNT)
#     define SETUP_XfOrM2015(x) SETUP_XfOrM2014(x)

# 10813 "./../src/optimize.c"
      ht = FUNCCALL(SETUP_XfOrM2015(_), scheme_hash_tree_set (ht , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (var -> variable_pos ) ) ) << 1 ) | 0x1 ) ) , scheme_true ) ); 
      ht = FUNCCALL(SETUP_XfOrM2015(_), scheme_hash_tree_set (* info -> imports_used , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (var -> instance_pos ) ) ) << 1 ) | 0x1 ) ) , (Scheme_Object * ) ht ) ); 
      (* info -> imports_used ) = ht ; 
    }
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10820 "./../src/optimize.c"
static void record_optimize_shapes (Optimize_Info * info , Scheme_Linklet * linklet , Scheme_Object * * _import_keys ) 
{
  int i , j , k , used , total , added_imports = 0 , dropped_imports = 0 , total_used ; 
  Scheme_Object * shapes , * v , * name ; 
  Scheme_Linklet * in_linklet ; 
  Scheme_Instance * in_instance ; 
  Scheme_Hash_Tree * ht ; 
  Scheme_Bucket * b ; 
  Scheme_Object * __funcarg133 = NULLED_OUT ; 
  PREPARE_VAR_STACK(11);
  BLOCK_SETUP_TOP((PUSH(name, 0), PUSH(b, 1), PUSH(info, 2), PUSH(_import_keys, 3), PUSH(ht, 4), PUSH(in_linklet, 5), PUSH(v, 6), PUSH(linklet, 7), PUSH(shapes, 8), PUSH(in_instance, 9)));
# define XfOrM2016_COUNT (10)
# define SETUP_XfOrM2016(x) SETUP(XfOrM2016_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_each(x)
# define FUNCCALL(s, x) FUNCCALL_each(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_each(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_each(x)
  shapes = NULLED_OUT ; 
  v = NULLED_OUT ; 
  name = NULLED_OUT ; 
  in_linklet = NULLED_OUT ; 
  in_instance = NULLED_OUT ; 
  ht = NULLED_OUT ; 
  b = NULLED_OUT ; 

# 10829 "./../src/optimize.c"
  if (info -> cross ) {
#   define XfOrM2105_COUNT (0+XfOrM2016_COUNT)
#   define SETUP_XfOrM2105(x) SETUP_XfOrM2016(x)

# 10831 "./../src/optimize.c"
    if (info -> cross -> import_keys -> count > (((Scheme_Vector * ) (linklet -> importss ) ) -> size ) ) {
#     define XfOrM2118_COUNT (0+XfOrM2105_COUNT)
#     define SETUP_XfOrM2118(x) SETUP_XfOrM2105(x)

# 10832 "./../src/optimize.c"
      added_imports = (((Scheme_Vector * ) (linklet -> importss ) ) -> size ) - info -> cross -> import_keys -> count ; 
      v = (__funcarg133 = FUNCCALL(SETUP_XfOrM2118(_), scheme_make_vector (0 , ((void * ) 0 ) ) ), FUNCCALL_AGAIN(scheme_make_vector (info -> cross -> import_keys -> count , __funcarg133 ) )) ; 
      for (i = 0 ; i < (((Scheme_Vector * ) (linklet -> importss ) ) -> size ) ; i ++ ) {
        (((Scheme_Vector * ) (v ) ) -> els ) [i ] = (((Scheme_Vector * ) (linklet -> importss ) ) -> els ) [i ] ; 
      }
      linklet -> importss = v ; 
    }


    for (i = 0 ; i < (((Scheme_Vector * ) (linklet -> importss ) ) -> size ) ; i ++ ) {
#     define XfOrM2112_COUNT (0+XfOrM2105_COUNT)
#     define SETUP_XfOrM2112(x) SETUP_XfOrM2105(x)

# 10842 "./../src/optimize.c"
      ht = (Scheme_Hash_Tree * ) (scheme_eq_hash_tree_get (info -> cross -> import_syms , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (i ) ) ) << 1 ) | 0x1 ) ) ) ) ; 
      if (ht && ((ht -> count >> 1 ) > (((Scheme_Vector * ) ((((Scheme_Vector * ) (linklet -> importss ) ) -> els ) [i ] ) ) -> size ) ) ) {
        Scheme_Object * sym ; 
        BLOCK_SETUP((PUSH(sym, 0+XfOrM2112_COUNT)));
#       define XfOrM2113_COUNT (1+XfOrM2112_COUNT)
#       define SETUP_XfOrM2113(x) SETUP(XfOrM2113_COUNT)
        sym = NULLED_OUT ; 

# 10845 "./../src/optimize.c"
        v = FUNCCALL(SETUP_XfOrM2113(_), scheme_make_vector ((ht -> count >> 1 ) , ((void * ) 0 ) ) ); 
        (((Scheme_Vector * ) (linklet -> importss ) ) -> els ) [i ] = v ; 

        for (j = ht -> count >> 1 ; j -- ; ) {
#         define XfOrM2116_COUNT (0+XfOrM2113_COUNT)
#         define SETUP_XfOrM2116(x) SETUP_XfOrM2113(x)

# 10849 "./../src/optimize.c"
          sym = (scheme_eq_hash_tree_get (ht , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (j ) ) ) << 1 ) | 0x1 ) ) ) ) ; 
          ((void ) sizeof ((sym ) ? 1 : 0 ) , __extension__ ({
#           define XfOrM2117_COUNT (0+XfOrM2116_COUNT)
#           define SETUP_XfOrM2117(x) SETUP_XfOrM2116(x)

# 10850 "./../src/optimize.c" 3
            if (sym )

# 10850 "./../src/optimize.c" 3
            ; 

# 10850 "./../src/optimize.c" 3
            else FUNCCALL(SETUP_XfOrM2117(_), __assert_fail ("sym" , "./../src/optimize.c" , 10850 , __extension__ __PRETTY_FUNCTION__ ) ); 
          }
          ) ) 
# 10850 "./../src/optimize.c"
          ; 
          (((Scheme_Vector * ) (v ) ) -> els ) [j ] = sym ; 
        }
      }
    }
  }


  total_used = 0 ; 
  total = 0 ; 
  for (i = 0 ; i < (((Scheme_Vector * ) (linklet -> importss ) ) -> size ) ; i ++ ) {
#   define XfOrM2098_COUNT (0+XfOrM2016_COUNT)
#   define SETUP_XfOrM2098(x) SETUP_XfOrM2016(x)

# 10861 "./../src/optimize.c"
    used = 0 ; 
    k = (((Scheme_Vector * ) ((((Scheme_Vector * ) (linklet -> importss ) ) -> els ) [i ] ) ) -> size ) ; 
    total += k ; 
    if (info -> imports_used ) {
#     define XfOrM2100_COUNT (0+XfOrM2098_COUNT)
#     define SETUP_XfOrM2100(x) SETUP_XfOrM2098(x)

# 10865 "./../src/optimize.c"
      ht = (Scheme_Hash_Tree * ) (scheme_eq_hash_tree_get (* info -> imports_used , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (i ) ) ) << 1 ) | 0x1 ) ) ) ) ; 
      if (! ht )

# 10866 "./../src/optimize.c"
      ht = empty_eq_hash_tree ; 
      for (j = 0 ; j < k ; j ++ ) {
#       define XfOrM2103_COUNT (0+XfOrM2100_COUNT)
#       define SETUP_XfOrM2103(x) SETUP_XfOrM2100(x)

# 10868 "./../src/optimize.c"
        if (! (scheme_eq_hash_tree_get (ht , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (j ) ) ) << 1 ) | 0x1 ) ) ) ) ) {

          (((Scheme_Vector * ) ((((Scheme_Vector * ) (linklet -> importss ) ) -> els ) [i ] ) ) -> els ) [j ] = scheme_false ; 
        }

# 10871 "./../src/optimize.c"
        else 
        used ++ ; 
      }
    }

# 10874 "./../src/optimize.c"
    else 
    used += k ; 
    total_used += used ; 
    if (! used && _import_keys 

    && ((i >= (((Scheme_Vector * ) (* _import_keys ) ) -> size ) ) 
    || ((! ((((((Scheme_Vector * ) (* _import_keys ) ) -> els ) [i ] ) ) == (scheme_false ) ) ) 
    && ! ((Scheme_Type ) (scheme_instance_type ) == (Scheme_Type ) (((((intptr_t ) ((((Scheme_Vector * ) (* _import_keys ) ) -> els ) [i ] ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) ((((Scheme_Vector * ) (* _import_keys ) ) -> els ) [i ] ) ) -> type ) ) ) ) ) ) {
      dropped_imports ++ ; 



      (((Scheme_Vector * ) (linklet -> importss ) ) -> els ) [i ] = ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (k ) ) ) << 1 ) | 0x1 ) ) ; 
    }
  }
  linklet -> num_total_imports = total ; 

  if (dropped_imports || added_imports ) {
#   define XfOrM2083_COUNT (0+XfOrM2016_COUNT)
#   define SETUP_XfOrM2083(x) SETUP_XfOrM2016(x)

# 10893 "./../src/optimize.c"
    v = FUNCCALL(SETUP_XfOrM2083(_), scheme_make_vector ((((Scheme_Vector * ) (linklet -> importss ) ) -> size ) - dropped_imports , ((void * ) 0 ) ) ); 
    * _import_keys = v ; 
    used = 0 ; 
    for (i = 0 ; i < (((Scheme_Vector * ) (linklet -> importss ) ) -> size ) ; i ++ ) {
#     define XfOrM2088_COUNT (0+XfOrM2083_COUNT)
#     define SETUP_XfOrM2088(x) SETUP_XfOrM2083(x)

# 10897 "./../src/optimize.c"
      if (! (((intptr_t ) ((((Scheme_Vector * ) (linklet -> importss ) ) -> els ) [i ] ) ) & 0x1 ) ) {
#       define XfOrM2089_COUNT (0+XfOrM2088_COUNT)
#       define SETUP_XfOrM2089(x) SETUP_XfOrM2088(x)

# 10898 "./../src/optimize.c"
        v = (scheme_eq_hash_tree_get (info -> cross -> import_keys , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (i ) ) ) << 1 ) | 0x1 ) ) ) ) ; 
        ((void ) sizeof ((v ) ? 1 : 0 ) , __extension__ ({
#         define XfOrM2090_COUNT (0+XfOrM2089_COUNT)
#         define SETUP_XfOrM2090(x) SETUP_XfOrM2089(x)

# 10899 "./../src/optimize.c" 3
          if (v )

# 10899 "./../src/optimize.c" 3
          ; 

# 10899 "./../src/optimize.c" 3
          else FUNCCALL(SETUP_XfOrM2090(_), __assert_fail ("v" , "./../src/optimize.c" , 10899 , __extension__ __PRETTY_FUNCTION__ ) ); 
        }
        ) ) 
# 10899 "./../src/optimize.c"
        ; 
        (((Scheme_Vector * ) ((* _import_keys ) ) ) -> els ) [used ++ ] = v ; 
      }
    }
    ((void ) sizeof ((used == ((((Scheme_Vector * ) (linklet -> importss ) ) -> size ) - dropped_imports ) ) ? 1 : 0 ) , __extension__ ({
#     define XfOrM2084_COUNT (0+XfOrM2083_COUNT)
#     define SETUP_XfOrM2084(x) SETUP_XfOrM2083(x)

# 10903 "./../src/optimize.c" 3
      if (used == ((((Scheme_Vector * ) (linklet -> importss ) ) -> size ) - dropped_imports ) )

# 10903 "./../src/optimize.c" 3
      ; 

# 10903 "./../src/optimize.c" 3
      else FUNCCALL(SETUP_XfOrM2084(_), __assert_fail ("used == ((((Scheme_Vector *)(linklet->importss))->size) - dropped_imports)" , "./../src/optimize.c" , 10903 , __extension__ __PRETTY_FUNCTION__ ) ); 
    }
    ) ) 
# 10903 "./../src/optimize.c"
    ; 
  }

  if (info -> cross && info -> cross -> used_import_shape ) {
#   define XfOrM2017_COUNT (0+XfOrM2016_COUNT)
#   define SETUP_XfOrM2017(x) SETUP_XfOrM2016(x)

# 10908 "./../src/optimize.c"
    shapes = FUNCCALL(SETUP_XfOrM2017(_), scheme_make_vector (total_used , scheme_false ) ); 
    linklet -> import_shapes = shapes ; 
    k = 0 ; 
    for (i = 0 ; i < (((Scheme_Vector * ) (linklet -> importss ) ) -> size ) ; i ++ ) {
#     define XfOrM2051_COUNT (0+XfOrM2017_COUNT)
#     define SETUP_XfOrM2051(x) SETUP_XfOrM2017(x)

# 10912 "./../src/optimize.c"
      if (! (((intptr_t ) ((((Scheme_Vector * ) (linklet -> importss ) ) -> els ) [i ] ) ) & 0x1 ) ) {
#       define XfOrM2052_COUNT (0+XfOrM2051_COUNT)
#       define SETUP_XfOrM2052(x) SETUP_XfOrM2051(x)

# 10913 "./../src/optimize.c"
        v = (scheme_eq_hash_tree_get (info -> cross -> import_keys , ((Scheme_Object * ) (void * ) (intptr_t ) ((((uintptr_t ) ((intptr_t ) (i ) ) ) << 1 ) | 0x1 ) ) ) ) ; 
        if (v )
          v = (scheme_eq_hash_tree_get (info -> cross -> linklets , v ) ) ; 
        in_linklet = ((v && ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_linklet_type ) ) ) ? (Scheme_Linklet * ) v : ((void * ) 0 ) ) ; 
        in_instance = ((v && ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_instance_type ) ) ) ? (Scheme_Instance * ) v : ((void * ) 0 ) ) ; 
        ((void ) sizeof ((! in_linklet || ((Scheme_Type ) (in_linklet -> so . type ) == (Scheme_Type ) (scheme_linklet_type ) ) ) ? 1 : 0 ) , __extension__ ({
#         define XfOrM2082_COUNT (0+XfOrM2052_COUNT)
#         define SETUP_XfOrM2082(x) SETUP_XfOrM2052(x)

# 10918 "./../src/optimize.c" 3
          if (! in_linklet || ((Scheme_Type ) (in_linklet -> so . type ) == (Scheme_Type ) (scheme_linklet_type ) ) )

# 10918 "./../src/optimize.c" 3
          ; 

# 10918 "./../src/optimize.c" 3
          else FUNCCALL(SETUP_XfOrM2082(_), __assert_fail ("!in_linklet || ((Scheme_Type)(in_linklet->so.type) == (Scheme_Type)(scheme_linklet_type))" , "./../src/optimize.c" , 10918 , __extension__ __PRETTY_FUNCTION__ ) ); 
        }
        ) ) 
# 10918 "./../src/optimize.c"
        ; 
        ((void ) sizeof ((! in_instance || ((Scheme_Type ) (in_instance -> iso . so . type ) == (Scheme_Type ) (scheme_instance_type ) ) ) ? 1 : 0 ) , __extension__ ({
#         define XfOrM2081_COUNT (0+XfOrM2052_COUNT)
#         define SETUP_XfOrM2081(x) SETUP_XfOrM2052(x)

# 10919 "./../src/optimize.c" 3
          if (! in_instance || ((Scheme_Type ) (in_instance -> iso . so . type ) == (Scheme_Type ) (scheme_instance_type ) ) )

# 10919 "./../src/optimize.c" 3
          ; 

# 10919 "./../src/optimize.c" 3
          else FUNCCALL(SETUP_XfOrM2081(_), __assert_fail ("!in_instance || ((Scheme_Type)(in_instance->iso.so.type) == (Scheme_Type)(scheme_instance_type))" , "./../src/optimize.c" , 10919 , __extension__ __PRETTY_FUNCTION__ ) ); 
        }
        ) ) 
# 10919 "./../src/optimize.c"
        ; 
        for (j = 0 ; j < (((Scheme_Vector * ) ((((Scheme_Vector * ) (linklet -> importss ) ) -> els ) [i ] ) ) -> size ) ; j ++ ) {
#         define XfOrM2067_COUNT (0+XfOrM2052_COUNT)
#         define SETUP_XfOrM2067(x) SETUP_XfOrM2052(x)

# 10921 "./../src/optimize.c"
          name = (((Scheme_Vector * ) ((((Scheme_Vector * ) (linklet -> importss ) ) -> els ) [i ] ) ) -> els ) [j ] ; 
          if (! (((name ) ) == (scheme_false ) ) )  {
#           define XfOrM2068_COUNT (0+XfOrM2067_COUNT)
#           define SETUP_XfOrM2068(x) SETUP_XfOrM2067(x)

# 10923 "./../src/optimize.c"
            if (in_linklet && in_linklet -> constants ) {
#             define XfOrM2072_COUNT (0+XfOrM2068_COUNT)
#             define SETUP_XfOrM2072(x) SETUP_XfOrM2068(x)

# 10924 "./../src/optimize.c"
              v = FUNCCALL(SETUP_XfOrM2072(_), scheme_hash_get (in_linklet -> constants , name ) ); 
              if (v ) {
#               define XfOrM2073_COUNT (0+XfOrM2072_COUNT)
#               define SETUP_XfOrM2073(x) SETUP_XfOrM2072(x)

# 10926 "./../src/optimize.c"
                if (((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_vector_type ) ) && ((((Scheme_Vector * ) (v ) ) -> size ) == 3 ) ) {
#                 define XfOrM2080_COUNT (0+XfOrM2073_COUNT)
#                 define SETUP_XfOrM2080(x) SETUP_XfOrM2073(x)

# 10927 "./../src/optimize.c"
                  v = FUNCCALL(SETUP_XfOrM2080(_), scheme_intern_struct_proc_shape ((((intptr_t ) ((((Scheme_Vector * ) (v ) ) -> els ) [1 ] ) ) >> 1 ) ) ); 
                  (((Scheme_Vector * ) (shapes ) ) -> els ) [k ] = v ; 
                }

# 10929 "./../src/optimize.c"
                else if (((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_vector_type ) ) && ((((Scheme_Vector * ) (v ) ) -> size ) == 4 ) ) {
#                 define XfOrM2079_COUNT (0+XfOrM2073_COUNT)
#                 define SETUP_XfOrM2079(x) SETUP_XfOrM2073(x)

# 10930 "./../src/optimize.c"
                  v = FUNCCALL(SETUP_XfOrM2079(_), scheme_intern_struct_prop_proc_shape ((((intptr_t ) ((((Scheme_Vector * ) (v ) ) -> els ) [1 ] ) ) >> 1 ) ) ); 
                  (((Scheme_Vector * ) (shapes ) ) -> els ) [k ] = v ; 
                }

# 10932 "./../src/optimize.c"
                else if ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_vector_type ) )  {
#                 define XfOrM2077_COUNT (0+XfOrM2073_COUNT)
#                 define SETUP_XfOrM2077(x) SETUP_XfOrM2073(x)

# 10933 "./../src/optimize.c"
                  ((void ) sizeof (((((Scheme_Vector * ) (v ) ) -> size ) == 2 ) ? 1 : 0 ) , __extension__ ({
#                   define XfOrM2078_COUNT (0+XfOrM2077_COUNT)
#                   define SETUP_XfOrM2078(x) SETUP_XfOrM2077(x)

# 10933 "./../src/optimize.c" 3
                    if ((((Scheme_Vector * ) (v ) ) -> size ) == 2 )

# 10933 "./../src/optimize.c" 3
                    ; 

# 10933 "./../src/optimize.c" 3
                    else FUNCCALL(SETUP_XfOrM2078(_), __assert_fail ("(((Scheme_Vector *)(v))->size) == 2" , "./../src/optimize.c" , 10933 , __extension__ __PRETTY_FUNCTION__ ) ); 
                  }
                  ) ) 
# 10933 "./../src/optimize.c"
                  ; 
                  v = FUNCCALL(SETUP_XfOrM2077(_), scheme_get_or_check_procedure_shape ((((Scheme_Vector * ) (v ) ) -> els ) [0 ] , ((void * ) 0 ) , 0 ) ); 
                  (((Scheme_Vector * ) (shapes ) ) -> els ) [k ] = v ; 
                }

# 10936 "./../src/optimize.c"
                else if ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_inline_variant_type ) )  {
#                 define XfOrM2076_COUNT (0+XfOrM2073_COUNT)
#                 define SETUP_XfOrM2076(x) SETUP_XfOrM2073(x)

# 10937 "./../src/optimize.c"
                  v = FUNCCALL(SETUP_XfOrM2076(_), scheme_get_or_check_procedure_shape (v , ((void * ) 0 ) , 0 ) ); 
                  (((Scheme_Vector * ) (shapes ) ) -> els ) [k ] = v ; 
                }

# 10939 "./../src/optimize.c"
                else if ((v ) == (scheme_default_prompt_tag ) )  {
                  (((Scheme_Vector * ) (shapes ) ) -> els ) [k ] = scheme_void ; 
                }

# 10941 "./../src/optimize.c"
                else {
                }
              }
            }
            else if (in_instance ) {
#             define XfOrM2069_COUNT (0+XfOrM2068_COUNT)
#             define SETUP_XfOrM2069(x) SETUP_XfOrM2068(x)

# 10946 "./../src/optimize.c"
              b = FUNCCALL(SETUP_XfOrM2069(_), scheme_instance_variable_bucket_or_null (name , in_instance ) ); 
              if (b && b -> val && (((Scheme_Bucket_With_Flags * ) b ) -> flags & 2 ) ) {
                int imprecise = (& (in_instance ) -> iso ) -> so . keyex & 0x1 ; 
#               define XfOrM2070_COUNT (0+XfOrM2069_COUNT)
#               define SETUP_XfOrM2070(x) SETUP_XfOrM2069(x)

# 10949 "./../src/optimize.c"
                v = FUNCCALL(SETUP_XfOrM2070(_), get_value_shape (b -> val , imprecise ) ); 
                if (v ) {
#                 define XfOrM2071_COUNT (0+XfOrM2070_COUNT)
#                 define SETUP_XfOrM2071(x) SETUP_XfOrM2070(x)

# 10951 "./../src/optimize.c"
                  if ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_proc_shape_type ) ) 
                    v = FUNCCALL(SETUP_XfOrM2071(_), scheme_intern_struct_proc_shape (((Scheme_Struct_Proc_Shape * ) v ) -> mode ) ); 
                  else if ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_prop_proc_shape_type ) ) 
                    v = FUNCCALL(SETUP_XfOrM2071(_), scheme_intern_struct_prop_proc_shape (((Scheme_Small_Object * ) v ) -> u . int_val ) ); 
                  (((Scheme_Vector * ) (shapes ) ) -> els ) [k ] = v ; 
                }

# 10956 "./../src/optimize.c"
                else 
                (((Scheme_Vector * ) (shapes ) ) -> els ) [k ] = scheme_void ; 
              }
            }
            k ++ ; 
          }
        }
      }
    }
    ((void ) sizeof ((k == total_used ) ? 1 : 0 ) , __extension__ ({
#     define XfOrM2018_COUNT (0+XfOrM2017_COUNT)
#     define SETUP_XfOrM2018(x) SETUP_XfOrM2017(x)

# 10965 "./../src/optimize.c" 3
      if (k == total_used )

# 10965 "./../src/optimize.c" 3
      ; 

# 10965 "./../src/optimize.c" 3
      else FUNCCALL_EMPTY(__assert_fail ("k == total_used" , "./../src/optimize.c" , 10965 , __extension__ __PRETTY_FUNCTION__ ) ); 
    }
    ) ) 
# 10965 "./../src/optimize.c"
    ; 
  }
  RET_NOTHING_AT_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 10969 "./../src/optimize.c"
static Scheme_Object * get_value_shape (Scheme_Object * v , int imprecise ) 
{
  intptr_t s ; 
  Scheme_Object * identity ; 
  PREPARE_VAR_STACK_ONCE(2);
  BLOCK_SETUP_TOP((PUSH(identity, 0), PUSH(v, 1)));
# define XfOrM2121_COUNT (2)
# define SETUP_XfOrM2121(x) SETUP(XfOrM2121_COUNT)
# define BLOCK_SETUP(x) BLOCK_SETUP_once(x)
# define FUNCCALL(s, x) FUNCCALL_once(s, x)
# define FUNCCALL_EMPTY(x) FUNCCALL_EMPTY_once(x)
# define FUNCCALL_AGAIN(x) FUNCCALL_AGAIN_once(x)
  identity = NULLED_OUT ; 

# 10974 "./../src/optimize.c"
  s = FUNCCALL(SETUP_XfOrM2121(_), scheme_get_or_check_structure_shape (v , ((void * ) 0 ) ) ); 
  if (s != - 1 ) {
#   define XfOrM2122_COUNT (0+XfOrM2121_COUNT)
#   define SETUP_XfOrM2122(x) SETUP_XfOrM2121(x)

# 10976 "./../src/optimize.c"
    if ((Scheme_Type ) (((((intptr_t ) (v ) ) & 0x1 ) ? (Scheme_Type ) scheme_integer_type : ((Scheme_Object * ) (v ) ) -> type ) ) == (Scheme_Type ) (scheme_struct_type_type ) ) 
      identity = v ; 
    else 
    identity = ((Scheme_Primitive_Closure * ) v ) -> val [0 ] ; 
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(scheme_make_struct_proc_shape (s , identity ) )) RET_VALUE_EMPTY_END ; 
  }

  s = FUNCCALL(SETUP_XfOrM2121(_), scheme_get_or_check_structure_property_shape (v , ((void * ) 0 ) ) ); 
  if (s != - 1 )
    RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(scheme_make_struct_property_proc_shape (s ) )) RET_VALUE_EMPTY_END ; 

  RET_VALUE_EMPTY_START (FUNCCALL_EMPTY(scheme_get_or_check_procedure_shape (v , ((void * ) 0 ) , imprecise ) )) RET_VALUE_EMPTY_END ; 
# undef BLOCK_SETUP
# undef FUNCCALL
# undef FUNCCALL_EMPTY
# undef FUNCCALL_AGAIN
}

# 3 "./../src/mzmark_optimize.inc"
static int mark_optimize_info_SIZE (void * p , struct NewGC * gc ) {



  return 0 ; 
}


static int mark_optimize_info_MARK (void * p , struct NewGC * gc ) {

  Optimize_Info * i = (Optimize_Info * ) p ; 

  GC_mark2 (& (i -> next ) , gc ) ; 
  GC_mark2 (& (i -> linklet ) , gc ) ; 
  GC_mark2 (& (i -> cross ) , gc ) ; 
  GC_mark2 (& (i -> imports_used ) , gc ) ; 
  GC_mark2 (& (i -> top_level_consts ) , gc ) ; 
  GC_mark2 (& (i -> transitive_use_var ) , gc ) ; 
  GC_mark2 (& (i -> context ) , gc ) ; 
  GC_mark2 (& (i -> logger ) , gc ) ; 
  GC_mark2 (& (i -> types ) , gc ) ; 
  GC_mark2 (& (i -> uses ) , gc ) ; 


  return 0 ; 
}






static int mark_optimize_info_FIXUP (void * p , struct NewGC * gc ) {

  Optimize_Info * i = (Optimize_Info * ) p ; 

  GC_fixup2 (& (i -> next ) , gc ) ; 
  GC_fixup2 (& (i -> linklet ) , gc ) ; 
  GC_fixup2 (& (i -> cross ) , gc ) ; 
  GC_fixup2 (& (i -> imports_used ) , gc ) ; 
  GC_fixup2 (& (i -> top_level_consts ) , gc ) ; 
  GC_fixup2 (& (i -> transitive_use_var ) , gc ) ; 
  GC_fixup2 (& (i -> context ) , gc ) ; 
  GC_fixup2 (& (i -> logger ) , gc ) ; 
  GC_fixup2 (& (i -> types ) , gc ) ; 
  GC_fixup2 (& (i -> uses ) , gc ) ; 


  return 0 ; 
}

# 63 "./../src/mzmark_optimize.inc"
static int mark_once_used_SIZE (void * p , struct NewGC * gc ) {



  return 0 ; 
}


static int mark_once_used_MARK (void * p , struct NewGC * gc ) {

  Scheme_Once_Used * o = (Scheme_Once_Used * ) p ; 
  GC_mark2 (& (o -> expr ) , gc ) ; 
  GC_mark2 (& (o -> var ) , gc ) ; 

  return 0 ; 
}






static int mark_once_used_FIXUP (void * p , struct NewGC * gc ) {

  Scheme_Once_Used * o = (Scheme_Once_Used * ) p ; 
  GC_fixup2 (& (o -> expr ) , gc ) ; 
  GC_fixup2 (& (o -> var ) , gc ) ; 

  return 0 ; 
}

# 11000 "./../src/optimize.c"
static void register_traversers (void ) 
{
  GC_register_traversers2 (scheme_once_used_type , mark_once_used_SIZE , mark_once_used_MARK , mark_once_used_FIXUP , 1 , 0 ) ; 
  GC_register_traversers2 (scheme_rt_optimize_info , mark_optimize_info_SIZE , mark_optimize_info_MARK , mark_optimize_info_FIXUP , 1 , 0 ) ; 
}
